/* file context.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef CONTEXT_H
#define CONTEXT_H 1

#include <time.h>
#include <libdg87/daemon.h>
#include <libdg87/vector.h>
#include <libdg87/conf.h>
#include <libdg87/storage.h>
#include "global.h"
#include "ref.h"
#include "context.h"
#include "target.h"

/*
 * dg87_context object
 */
typedef struct dg87_context_struct
{
	/* TODO ipPort is stored in dg87_context.ipPort and dg87_server.ipPort
	 * when a SIGHUP is sended, configuration file is re-read, and 
	 * dg87_context.ipPort value can changed.
	 * In this case, dg87_server tries new IP Port and if an error occurres
	 * (port already used), dg87_server.ipPort keeps old value.
	 */
	int ipPort;
	struct dg87_engine_struct* pEngine;
	dg87_vector* pTargetVector;
	dg87_vector* pRefVector;
	struct dg87_server_struct* pServer;
	dg87_daemon* pDaemon;
	int dg87x_ipPort;
	char* dg87x_address;
	char* pPidFile;
	dg87_conf *pConf; /* configuration file */
	dg87_storage *pStorage; 
	char* pDiogenePassword;
	time_t saveTime;
	dg87uint saveTempo;
} dg87_context;

/*
 * dg87_context_new
 * Create a dg87_context object and return a pointer on it.
 * pConfFile is the path of the configuration file (/etc/diogene87/dg87?.conf)
 */
dg87_context* dg87_context_new(char* pConfFile);

/*
 * dg87_context_delete
 */
BOOL dg87_context_delete(dg87_context* pContext);

/*
 * dg87_context_setServer
 */
BOOL dg87_context_setServer(dg87_context* pContext, 
		struct dg87_server_struct* pServer);

/*
 * dg87_context_makePeriodicSave
 */
BOOL dg87_context_makePeriodicSave(dg87_context* pContext);

/*
 * dg87_context_readConfig
 * This function reads (or re-reads) the configuration file variables
 */
BOOL dg87_context_readConfig(dg87_context* pContext);

/*
 * dg87_log
 * write the message in the system log.
 */
void dg87_context_log(char* pMessage);

/*************************************************************************/
/*  functions for targets                                                */
/*************************************************************************/

/*
 * dg87_context_targetAdd
 * create a Target object, add this object in the TargetVector 
 * of the context and return a pointer on this object.
 * Return NULL if the target object can't be create.
 */
dg87_target* dg87_context_targetAdd(dg87_context* pContext, char* pTargetName);

/*
 * dg87_context_TargetCount
 * return the number of Targets.
 * return -1 if pRef is NULL.
 */
int dg87_context_targetCount(dg87_context* pContext);

/*
 * dg87_context_targetGetByName
 * return a pointer on the target which has the name
 * given in parameter. Return NULL if it can't be find.
 */
dg87_target* dg87_context_targetGetByName(dg87_context* pContext, char* pName);

/*
 * dg87_context_targetGetByIndex
 * return a pointer on the target at the given position.
 */
dg87_target* dg87_context_targetGetByIndex(dg87_context* pContext, int index);

/*
 * dg87_context_targetDeleteByName
 * delete the target which has the name given in parameter.
 */
BOOL dg87_context_targetDeleteByName(dg87_context* pContext,char* pName);

/*
 * dg87_context_targetDeleteByIndex
 * delete the target at the given index.
 */
BOOL dg87_context_targetDeleteByIndex(dg87_context* pContext,int index);

/*
 * dg87_context_targetDelete
 */
BOOL dg87_context_targetDelete(dg87_context* pContext, dg87_target* pTarget);

/*************************************************************************/
/*  functions for refs */
/*************************************************************************/

/*
 * dg87_context_refAdd
 * create a Ref object, add this object in the RefVector 
 * of the context and return a pointer on this object.
 * Return NULL if the Ref object can't be create.
 */
dg87_ref* dg87_context_refAdd(dg87_context* pContext, char* pRefName);

/*
 * dg87_context_refDuplicate
 * create a new Ref by copying an other. 
 */
dg87_ref* dg87_context_refDuplicate(dg87_context* pContext, 
		dg87_ref* pOriginalRef, char* pNewRefName);

/*
 * dg87_context_refDeleteByName
 * delete the ref which has the name given in parameter.
 */
BOOL dg87_context_refDeleteByName(dg87_context* pContext,char* pName);

/*
 * dg87_context_refGetByName
 * return a pointer on the ref which has the name
 * given in parameter. Return NULL if it can't be find.
 */
dg87_ref* dg87_context_refGetByName(dg87_context* pContext, char* pName);

/*
 * dg87_context_save
 */
BOOL dg87_context_save(dg87_context* pContext);

/*
 * dg87_context_load
 */
BOOL dg87_context_load(dg87_context* pContext);

#ifdef DEBUG
/*
 * dg87_context_ckeck
 */
int dg87_context_check(dg87_context* pContext, int (*print)(char*));
#endif /*DEBUG*/

#endif /*CONTEXT_H*/
/* End of file */
