/* file engine.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <assert.h>
#include "engine.h"
#include "variable.h"
#include <config.h>

/******************************************************************************/
/* dg87_replaceVariables
 * this functions parses InBuffer to replace variables $<VarName> by
 * its value.
 * If a variable is not declared, the $<VarName> is copied.
 */
/*TODO this function is duplicated in dg87d and dg87x */
static char* dg87_replaceVariables(char* pInBuffer,dg87_vector* pVariableVector)
{
	char *pSubBeg, *pSubEnd;
	char *pVarBeg, *pVarEnd;
	char *pBuffer;
	char *pVarName;
	dg87_variable *pVariable;
	dg87_element* pElement;
	dg87_strbuf* pOutBuf;
	int varSize;
	assert(pInBuffer != NULL && pVariableVector != NULL);
	if (pInBuffer == NULL || pVariableVector == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}

	/* optimization if no variables */
	if (strstr(pInBuffer, "$<") == NULL)
		return dg87_strdup(pInBuffer, DG87_CHAR);

	/* initialize out buffer */
	pOutBuf = dg87_strbuf_new();
	if (pOutBuf == NULL)
		return NULL;

	/* substring points on begin of string */
	pSubBeg = pInBuffer;
	pSubEnd = pSubBeg;

	while( *pSubBeg != '\0') {
		pVariable = NULL;
		pVarBeg = NULL;
		pVarEnd = NULL;

		/* find begin of variable */
		if ((pVarBeg = strstr(pSubEnd, "$<")) == NULL){
			/* skip to end of string */
			pSubEnd = strchr(pSubEnd, '\0');
		} 
		/* find end of variable */
		else if ((pVarEnd = strchr(pVarBeg, '>')) == NULL){
			/* skip to end of string */
			pSubEnd = strchr(pSubEnd, '\0');
		} else {
			varSize = pVarEnd - pVarBeg - 2;
			/* extract variable name */
			pVarName = dg87_malloc(varSize + 1, DG87_CHAR);
			if (pVarName == NULL){
				dg87_strbuf_delete(pOutBuf);
				return NULL;
			}
			strncpy(pVarName, pVarBeg+2, pVarEnd-pVarBeg-2);
			pVarName[varSize] = '\0';

			/* find variable */
			pElement = pVariableVector->pFirst;
			while (pElement != NULL){
				pVariable = pElement->pObject;
				if (strcmp(pVariable->name, pVarName) == 0)
					break;
				pVariable = NULL;
				pElement = pElement->pNext;
			}
			dg87_free(pVarName, DG87_CHAR);

			/* unknown variable */
			if (pVariable == NULL){
				/* search next variable */
				pSubEnd = pVarEnd;
				continue;
			}
			/* end of substring is start of variable */
			pSubEnd = pVarBeg;
		}

		/* copy substring */
		if (dg87_strbuf_ncat(pOutBuf, pSubBeg,
					pSubEnd - pSubBeg) == FALSE){
			dg87_strbuf_delete(pOutBuf);
			return NULL;
		}
		pSubBeg = pSubEnd;

		/* continue if no variable value to copy */
		if (pVariable == NULL)
			continue;

		/* copy variable value */
		if (dg87_strbuf_cat(pOutBuf, 
					pVariable->pValue) == FALSE){
			dg87_strbuf_delete(pOutBuf);
			return NULL;
		}

		/* next sub string starts after end of variable */
		pSubBeg = pVarEnd + 1;
		pSubEnd = pSubBeg;

	}

	/*pBuffer = dg87_strbuf_getBuffer(pOutBuf);*/
	pBuffer = dg87_strdup(pOutBuf->pBuffer, DG87_CHAR);
	dg87_strbuf_delete(pOutBuf);
	return pBuffer;
}
/****************************************************************************/
static BOOL dg87_engine_clearLog(dg87_engine* pEngine)
{
	assert(pEngine != NULL);
	if (pEngine == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_strbuf_cpy(pEngine->pLog , "") == FALSE)
		return FALSE;
	return TRUE;
}
/****************************************************************************/
dg87_engine* dg87_engine_new(dg87_context* pContext)
{
	dg87_engine* pEngine;
	assert(pContext != NULL);
	if (pContext == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
       
	pEngine = dg87_malloc(sizeof(dg87_engine), DG87_ENGINE);
	if (pEngine == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pEngine->pVariableVector = dg87_vector_new();
	if (pEngine->pVariableVector == NULL){
		dg87_engine_delete(pEngine);
		return NULL;
	}
	pEngine->pConnection = NULL;
	pEngine->pContext = pContext;
	pEngine->enabled = FALSE;
	pEngine->pRefToPlan = NULL;
	pEngine->pTaskToPlanVector = NULL;
	pEngine->pTaskDepToPlanList = NULL;
	pEngine->step = dg87_engine_step_disabled;
	pEngine->pTaskElement = NULL;
	pEngine->dayCreated = FALSE;
	pEngine->pLog = dg87_strbuf_new();
	if (pEngine->pLog == NULL){
		dg87_engine_delete(pEngine);
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pEngine->pNewDayName = dg87_strdup("", DG87_CHAR);
	if (pEngine->pNewDayName == NULL) {
		dg87_engine_delete(pEngine);
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pEngine->pDate = dg87_strdup("", DG87_CHAR);
	if (pEngine->pDate == NULL) {
		dg87_engine_delete(pEngine);
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}

	return pEngine;
}
/******************************************************************************/
BOOL dg87_engine_delete(dg87_engine* pEngine)
{
	dg87_variable *pVariable;
	assert(pEngine != NULL);
	if (pEngine == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pEngine->pRefToPlan != NULL)
		pEngine->pRefToPlan->locked = FALSE;
	if (pEngine->pTaskToPlanVector != NULL)
		dg87_vector_delete(pEngine->pTaskToPlanVector);
	if (pEngine->pTaskDepToPlanList != NULL)
		dg87_list_delete(pEngine->pTaskDepToPlanList);
	if (pEngine->pNewDayName != NULL)
		dg87_free(pEngine->pNewDayName, DG87_CHAR);
	if (pEngine->pDate != NULL)
		dg87_free(pEngine->pDate , DG87_CHAR);
	if (pEngine->pConnection != NULL)
		dg87_connection_delete(pEngine->pConnection);
	if (pEngine->pLog != NULL)
		dg87_strbuf_delete(pEngine->pLog);
	if (pEngine->pVariableVector != NULL){
		while (pEngine->pVariableVector->count > 0){
			pVariable = dg87_vector_removeObjectByIndex(
					pEngine->pVariableVector, 0);
			/* force variable deletion */
			pVariable->linkedRules = 0;
			pVariable->linkedConditions = 0;
			dg87_variable_delete(pVariable);
		}
		dg87_vector_delete(pEngine->pVariableVector);
	}
	dg87_free(pEngine, DG87_ENGINE);
	return TRUE;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_engine_check(dg87_engine* pEngine, int (*print)(char*))
{
	if (pEngine->pNewDayName == NULL){
		print("Engine : newDayName is null !\n");
		return 1;
	}
	if (pEngine->pDate == NULL){
		print("Engine : date is null !\n");
		return 1;
	}
	if (pEngine->pContext == NULL){
		print("Engine : context is null !\n");
		return 1;
	}
	if (pEngine->pLog == NULL){
		print ("Engine : Log is null !\n");
		return 1;
	}
	if (pEngine->pRefToPlan != NULL){
		if (pEngine->pRefToPlan->locked == FALSE){
			print("Engine : Ref is not locked !\n");
			return 1;
		}
		if (pEngine->pTaskToPlanVector == NULL){
			print("Engine : TaskToPlanVector is null !\n");
			return 1;
		}
		if (pEngine->pTaskDepToPlanList == NULL){
			print("Engine : TaskDepToPlanList is null !\n");
			return 1;
		}
	} else {
		if (pEngine->pTaskToPlanVector != NULL){
			print("Engine : TaskToPlanVector is null !\n");
			return 1;
		}
		if (pEngine->pTaskDepToPlanList != NULL){
			print("Engine : TaskDepToPlanList is null !\n");
			return 1;
		}
	}
	/* TODO check variables */
	return 0;
}
#endif /*DEBUG*/

/****************************************************************************/
/* dg87_engine_verifyBanner
 * This function verify that the banner sended by the remote daemon
 * is OK. 
 * Banner sample : diogene87-dg87x-8.8.8-(P01)
 * The name of the daemon must be dg87x.
 * The protocol version must be the same as DG87_PROTOCOL_VERSION 
 */
static BOOL dg87_engine_verifyBanner(char* pString)
{
	char *i, *j;
	assert(pString != NULL);
	if (pString == NULL) return FALSE;

	/* control daemon name is dg87x */
	if ((i = strchr(pString,'-')) == NULL)
		return FALSE;
	if ((j = strchr(i+1,'-')) == NULL)
		return FALSE;
	if (strncmp(i+1, DG87X_NAME, j - i - 1) != 0)
		return FALSE;

	/* control protocol version (P01) */
	if (pString == NULL) return FALSE;
	if ((j = strrchr(pString,')')) == NULL)
		return FALSE;
	if ((i = strrchr(pString,'(')) == NULL)
		return FALSE;
	if (i>=j) return FALSE;
	if (strncmp(i+1, DG87_PROTOCOL_VERSION , j - i - 1) != 0)
		return FALSE;

	return TRUE;
}
/****************************************************************************/
/*
 * dg87_engine_translateDependence
 * This function creates a dependence between DownStreamTask and UpStreamTask
 * if upstreamTask is planned.
 * Otherwise, the function tries to create dependences between the 
 * downStreamTask and all upstream tasks of the UpStreamTask.
 * The function returns FALSE if an error occures.
 */
static BOOL dg87_engine_translateDependence(dg87_engine* pEngine,
		dg87_task* pDownStreamTask,
		dg87_task* pUpStreamTask,
		dg87_taskdep_type type)
{
	dg87_item* pItem;
	dg87_taskdep* pTaskDep;
	assert(pEngine!=NULL && pDownStreamTask!=NULL && pUpStreamTask!=NULL);
	if (pEngine == NULL || pDownStreamTask == NULL || pUpStreamTask==NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	dg87_strbuf_printf(pEngine->pLog,
			"Translate dependence : %s -> %s \n",
			pUpStreamTask->name,
			pDownStreamTask->name);

	/* if UpStreamTask isn't planned */
	if (dg87_vector_getElementByKey(pEngine->pTaskToPlanVector,
			pUpStreamTask->name)== NULL)
	{

		/* the UpStreamTask isn't planned */
		/* for each upstream task */
		pItem = pUpStreamTask->pTaskDepBeforeList->pFirst;
		while (pItem != NULL){
			pTaskDep = pItem->pObject;
			pItem = pItem->pNext;
			/* skip dependence which type is onError */
			if (pTaskDep->type == dg87_taskdep_onError)
				continue;
			/* translate dependence on the upstream task */
			if (dg87_engine_translateDependence(pEngine, 
					pDownStreamTask, 
					pTaskDep->pTaskBefore,type) == FALSE)
				return FALSE;
			}
	} else {
		/* create a virtual task dependence*/
		pTaskDep = dg87_taskdep_new(pUpStreamTask, pDownStreamTask);
		if (pTaskDep == NULL)
			return FALSE;
		if (dg87_taskdep_setType(pTaskDep, type)==FALSE)
			return FALSE;

		/* Add the new taskdep to the list */
		if (dg87_list_addObject(pEngine->pTaskDepToPlanList, 
					pTaskDep) == FALSE){
			dg87_taskdep_delete(pTaskDep);
			return FALSE;
		}
	}
	return TRUE;
}
/****************************************************************************/
static void dg87_engine_run_stepPrepare(dg87_engine* pEngine)
{
	dg87_element* pTaskElement;
	dg87_task* pTask;
	dg87_item* pTaskDepItem;
	dg87_taskdep* pTaskDep;
	char* pTempDate;
	time_t day;
	BOOL taskBeforeSelected;
	BOOL taskAfterSelected;
	BOOL result;
	assert(pEngine != NULL);
	dg87_strbuf_printf(pEngine->pLog,"Day plan : %s\n",
			pEngine->pNewDayName);
	dg87_strbuf_printf(pEngine->pLog,"Date : %s\n", pEngine->pDate);

	/* interpret date */
	/* YYYY-MM-DD HH:MM -> YYYY-MM-DD */
	pTempDate = dg87_strdup(pEngine->pDate, DG87_CHAR);
	pTempDate[10] = '\0';
	day = dg87_rule_asciiToDate(pTempDate);
	if (day == -1){
		dg87_free(pTempDate, DG87_CHAR);
		pEngine->step = dg87_engine_step_error;
		return;
	}
	dg87_free(pTempDate, DG87_CHAR);

	/* create the vector of tasks to plan */
	pEngine->pTaskToPlanVector = dg87_vector_new();
	if (pEngine->pTaskToPlanVector == NULL){
		pEngine->step = dg87_engine_step_error;
		return;
	}

	/* for each task of the referential */
	pTaskElement = pEngine->pRefToPlan->pTaskVector->pFirst;
	while (pTaskElement != NULL){
		pTask = pTaskElement->pObject;
		pTaskElement = pTaskElement->pNext;

		/* check if task must be planed */
		result = FALSE;
		if (dg87_operationlist_calculate(pTask->pRuleOperationList,
					day, &result,-1) == FALSE){
			pEngine->step = dg87_engine_step_error;
			return;
		}
		/* skip tasks which are not selected */
		if (result == FALSE)
			continue;
		/* Add task to plan to the list */
		if (dg87_vector_addObject(pEngine->pTaskToPlanVector, 
					pTask, pTask->name) == FALSE){
			pEngine->step = dg87_engine_step_error;
			return;
		}
	}

	/* remove tasks that can't be planed due to 'onError' dependence */
	pTaskElement = pEngine->pTaskToPlanVector->pFirst;
	while (pTaskElement != NULL){
		pTask = pTaskElement->pObject;
		pTaskElement = pTaskElement->pNext;

		/* no problem with task which don't have upstream dependences*/
		if (pTask->pTaskDepBeforeList->count == 0)
			continue;

		/* for each upstream dependence of the task */
		result = TRUE;
		pTaskDepItem= pTask->pTaskDepBeforeList->pFirst; 
		while (pTaskDepItem != NULL){
			pTaskDep = pTaskDepItem->pObject;
			pTaskDepItem= pTaskDepItem->pNext;
			/* a task which has one (or more) 'onSuccess' or
			 * 'always' dependence musn't be deleted.*/
			if (pTaskDep->type == dg87_taskdep_onSuccess ||
					pTaskDep->type == dg87_taskdep_always){
				result = FALSE;
				break;
			}
			/* a task which has one (or more) 'onerror' dependence
			 * which upstream task is selected musn't be deleted */
			if (pTaskDep->type == dg87_taskdep_onError &&
				dg87_vector_getElementByKey(
					pEngine->pTaskToPlanVector,
					pTaskDep->pTaskBefore->name)!= NULL){
				result = FALSE;
				break;
			}
		}
		if (result == FALSE)
			continue;

		/* task must be removed */
		dg87_strbuf_printf(pEngine->pLog,
			"WARNING : the task '%s' can't be planed, "\
			" due to 'onerror' dependence problem.",
			pTask->name);
		if (dg87_vector_removeObject(pEngine->pTaskToPlanVector, 
					pTask) == FALSE){
			pEngine->step = dg87_engine_step_error;
			return;
		}

	}


	/* create the list of task dependences to plan */
	pEngine->pTaskDepToPlanList = dg87_list_new();
	if (pEngine->pTaskDepToPlanList == NULL){
		pEngine->step = dg87_engine_step_error;
		return;
	}
	/* for each taskdep of the referential */
	pTaskDepItem= pEngine->pRefToPlan->pTaskdepList->pFirst;
	while (pTaskDepItem != NULL){
		pTaskDep = pTaskDepItem->pObject;
		pTaskDepItem= pTaskDepItem->pNext;
		/* test if upstream and downstream task are selected */
		taskBeforeSelected = 
			(dg87_vector_getElementByKey(pEngine->pTaskToPlanVector,
			pTaskDep->pTaskBefore->name)== NULL)?FALSE:TRUE;
		taskAfterSelected = 
			(dg87_vector_getElementByKey(pEngine->pTaskToPlanVector,
			pTaskDep->pTaskAfter->name)== NULL)?FALSE:TRUE;

		/* skip dependence if downstream task isn't selected */
		if (taskAfterSelected == FALSE)
			continue;

		/* onError dependence can't be translated */
		if (taskBeforeSelected == FALSE && 
				pTaskDep->type == dg87_taskdep_onError){
			dg87_strbuf_printf(pEngine->pLog,
				"WARNING : There is a '%s' dependence between "\
				"task '%s' and '%s', and the upstream task "\
				"is not selected !\n",
				dg87_taskdep_typeToAscii(pTaskDep->type),
				pTaskDep->pTaskBefore->name,
				pTaskDep->pTaskAfter->name);
			continue;
		}

		/* create dependence with upstream tasks */
		if (dg87_engine_translateDependence(pEngine,
				pTaskDep->pTaskAfter,
				pTaskDep->pTaskBefore,
				pTaskDep->type) == FALSE)
		{
			pEngine->step = dg87_engine_step_error;
			return;
		}
	}

	pEngine->pTaskElement = pEngine->pTaskToPlanVector->pFirst;
	pEngine->pTaskDepItem = pEngine->pTaskDepToPlanList->pFirst;
	pEngine->step = dg87_engine_step_connection;
	dg87_strbuf_printf(pEngine->pLog,"* Connection\n");
}
/****************************************************************************/
static void dg87_engine_run_stepConnection(dg87_engine* pEngine)
{
	char* pString;
 	assert(pEngine != NULL); 
	if (pEngine->pConnection->state != 
			dg87_connection_state_messageAvailable)
		return;
	pString = dg87_connection_read(pEngine->pConnection);
	/* verify banner : protocol version and daemon name */
	if (dg87_engine_verifyBanner(pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		dg87_strbuf_printf(pEngine->pLog, 
				"ERROR : Can't communicate with dg87x !");
		pEngine->step = dg87_engine_step_error;
		return;
	}
	dg87_free(pString, DG87_CHAR);

	/* next step : user identification */ 
	pEngine->step = dg87_engine_step_identification;
	dg87_strbuf_printf(pEngine->pLog,"* Create day\n");

}

/****************************************************************************/
static void dg87_engine_run_stepIdentification(dg87_engine* pEngine)
{
	char* pString;
 	assert(pEngine != NULL); 
	/* send 'user' command */ 
	if (pEngine->pConnection->state == dg87_connection_state_waitMessage){
		dg87_strbuf_printf(pEngine->pLog, 
				"Identification\n");
		dg87_connection_printf(pEngine->pConnection,
			"user \"%s\" \"%s\"", 
			DG87_USER_DIOGENE_NAME,
			pEngine->pContext->pDiogenePassword);
		if (dg87_connection_send(pEngine->pConnection) == FALSE){
			pEngine->step = dg87_engine_step_error;
			return;
		}
		pEngine->step = dg87_engine_step_identification;
	}

	/* verify response for user identification */
	else if (pEngine->pConnection->state ==  
			dg87_connection_state_messageAvailable)
	{
		pString = dg87_connection_read(pEngine->pConnection);
		if (strcmp(pString, "OK") != 0){
			dg87_strbuf_printf(pEngine->pLog, 
					"ERROR : %s\n",pString); 
			dg87_free(pString, DG87_CHAR);
			pEngine->step = dg87_engine_step_error;
			return ;
		}
		dg87_free(pString, DG87_CHAR);
		/* next step : send command to create a day */
		pEngine->step = dg87_engine_step_createDay;
		dg87_strbuf_printf(pEngine->pLog,"* Create day\n");
	}
}
/****************************************************************************/
static void dg87_engine_run_stepCreateDay(dg87_engine* pEngine)
{
	char* pString;
 	assert(pEngine != NULL); 
	/* send command to create a day */
	if (pEngine->pConnection->state == dg87_connection_state_waitMessage){
		dg87_strbuf_printf(pEngine->pLog, 
				"Create day '%s' at '%s'\n",
				pEngine->pNewDayName, pEngine->pDate);
		dg87_connection_printf(pEngine->pConnection,
			"xda \"%s\" \"%s\"", 
			pEngine->pNewDayName, pEngine->pDate);
		if (dg87_connection_send(pEngine->pConnection) == FALSE){
			pEngine->step = dg87_engine_step_error;
			return;
		}
		pEngine->step = dg87_engine_step_createDay;
	}

	/* verify response for day creation */
	else if (pEngine->pConnection->state ==  
			dg87_connection_state_messageAvailable)
	{
		pString = dg87_connection_read(pEngine->pConnection);
		if (strcmp(pString, "OK") != 0){
			dg87_strbuf_printf(pEngine->pLog, 
					"ERROR : %s\n",pString); 
			dg87_free(pString, DG87_CHAR);
			pEngine->step = dg87_engine_step_error;
			return ;
		}
		dg87_free(pString, DG87_CHAR);
		pEngine->dayCreated = TRUE;
		/* next step : copy tasks */
		pEngine->step = dg87_engine_step_copyTask;
		dg87_strbuf_printf(pEngine->pLog,"* Copy task\n");
	}
}
/****************************************************************************/
static void dg87_engine_run_stepCopyTasks(dg87_engine* pEngine)
{
	char* pString;
	dg87_task* pTask;
 	assert(pEngine != NULL); 
	/* next step if no task to plan */
	if (pEngine->pTaskElement == NULL){
		dg87_strbuf_printf(pEngine->pLog,"* Copy task dependences\n");
		pEngine->step = dg87_engine_step_copyTaskDep;
		return;
	}
	pTask = pEngine->pTaskElement->pObject;
	/* send command to create task */
	if (pEngine->pConnection->state == dg87_connection_state_waitMessage){
		dg87_connection_printf(pEngine->pConnection,
			"xta \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" %s %s %s",
			pEngine->pNewDayName, pTask->name, 
			(pTask->pQueue == NULL) ? "" :
				pTask->pQueue->pTarget->name,
			(pTask->pQueue == NULL) ? "" :
				pTask->pQueue->name,
			pTask->description,
			dg87_boolToAscii(pTask->waitOldestTask),
			dg87_boolToAscii(pTask->waitAllDependences),
			dg87_boolToAscii(pTask->onErrorContinue));
		if (dg87_connection_send(pEngine->pConnection) == FALSE){
			pEngine->step = dg87_engine_step_error;
			return;
		}
	}
	/* verify response for task creation */
	else if (pEngine->pConnection->state == 
			dg87_connection_state_messageAvailable){
		pString = dg87_connection_read(pEngine->pConnection);
		if (strcmp(pString, "OK") != 0){
			dg87_strbuf_printf(pEngine->pLog,
				"ERROR : Can't create the task '%s'\n",
				pTask->name, pString); 
			dg87_free(pString, DG87_CHAR);
			pEngine->step = dg87_engine_step_error;
			return ;
		}
		dg87_free(pString, DG87_CHAR);
		/* copy each conditions of the task */
		pEngine->pConditionItem = 
			pTask->pConditionList->pFirst;
		pEngine->step = dg87_engine_step_copyCondition;
	}

}
/****************************************************************************/
static void dg87_engine_run_stepCopyCondition(dg87_engine* pEngine)
{
	char* pData;
	char* pString;
	dg87_task* pTask;
	dg87_condition* pCondition;
 	assert(pEngine != NULL); 
	if (pEngine->pConditionItem == NULL){
		/* next step : copy each jobs of the task */
		pTask = pEngine->pTaskElement->pObject;
		pEngine->pJobElement = pTask->pJobVector->pFirst;
		pEngine->step = dg87_engine_step_copyJob;
		return;
	}
	pCondition = pEngine->pConditionItem->pObject;
	pTask = pEngine->pTaskElement->pObject;

	/* send command to create condition */
	if (pEngine->pConnection->state == 
			dg87_connection_state_waitMessage){

		/* TODO optimization of this switch */
		switch (pCondition->type){
			case condition_hmin:
				pData = dg87_condition_minuteToHour(
				      pCondition->data.pConditionHmin->minutes);
				if (pData == NULL){
					pEngine->step = dg87_engine_step_error;
					return ;
				}
				dg87_connection_printf(pEngine->pConnection,
					"xtca \"%s\" \"%s\" \"%s\" \"%s\"",
					pEngine->pNewDayName, pTask->name, 
					dg87_condition_typeToAscii(
						pCondition->type), pData);
				dg87_free(pData, DG87_CHAR);
				break;
			case condition_hmax:
				pData = dg87_condition_minuteToHour(
				      pCondition->data.pConditionHmax->minutes);
				if (pData == NULL){
					pEngine->step = dg87_engine_step_error;
					return ;
				}
				dg87_connection_printf(pEngine->pConnection,
					"xtca \"%s\" \"%s\" \"%s\" \"%s\"",
					pEngine->pNewDayName, pTask->name, 
					dg87_condition_typeToAscii(
						pCondition->type), pData);
				dg87_free(pData, DG87_CHAR);
				break;
			case condition_hlate:
				pData = dg87_condition_minuteToHour(
				     pCondition->data.pConditionHlate->minutes);
				if (pData == NULL){
					pEngine->step = dg87_engine_step_error;
					return ;
				}
				dg87_connection_printf(pEngine->pConnection,
					"xtca \"%s\" \"%s\" \"%s\" \"%s\"",
					pEngine->pNewDayName, pTask->name, 
					dg87_condition_typeToAscii(
						pCondition->type), pData);
				dg87_free(pData, DG87_CHAR);
				break;

			case condition_file:
				pData = dg87_strdup(
					pCondition->data.pConditionFile->pPath, 
					DG87_CHAR);
				if (pData == NULL){
					pEngine->step = dg87_engine_step_error;
					return ;
				}
				dg87_connection_printf(pEngine->pConnection,
					"xtca \"%s\" \"%s\" \"%s\" \"%s\"",
					pEngine->pNewDayName, pTask->name, 
					dg87_condition_typeToAscii(
						pCondition->type), pData);
				dg87_free(pData, DG87_CHAR);
				break;
			case condition_manual:
				pData = dg87_strdup(
					pCondition->data.pConditionManual
					->pMessage,
					DG87_CHAR);
				if (pData == NULL){
					pEngine->step = dg87_engine_step_error;
					return ;
				}
				dg87_connection_printf(pEngine->pConnection,
					"xtca \"%s\" \"%s\" \"%s\" \"%s\"",
					pEngine->pNewDayName, pTask->name, 
					dg87_condition_typeToAscii(
						pCondition->type), pData);
				dg87_free(pData, DG87_CHAR);
				break;
			case condition_ifequal:
				dg87_connection_printf(pEngine->pConnection,
				"xtca \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
					pEngine->pNewDayName, 
					pTask->name, 
					dg87_condition_typeToAscii(
						pCondition->type), 
					pCondition->data.pConditionIfEqual
						->pVariable->name,
					pCondition->data.pConditionIfEqual
						->pValue);
				break;
			case condition_ifnequal:
				dg87_connection_printf(pEngine->pConnection,
				"xtca \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
					pEngine->pNewDayName, 
					pTask->name, 
					dg87_condition_typeToAscii(
						pCondition->type), 
					pCondition->data.pConditionIfNotEqual
						->pVariable->name,
					pCondition->data.pConditionIfNotEqual
						->pValue);

				break;
			default:
				/* TODO explicit message in log*/
				pEngine->step = dg87_engine_step_error;
				return;
		}
		if (dg87_connection_send(pEngine->pConnection) == FALSE){
			pEngine->step = dg87_engine_step_error;
			return;
		}
	}
	/* verify response for condition creation */
	if (pEngine->pConnection->state ==
		dg87_connection_state_messageAvailable){
		pString = dg87_connection_read(pEngine->pConnection);
		if (strcmp(pString, "OK") != 0){
			dg87_strbuf_printf(pEngine->pLog,
				"Can't create a condition for task '%s' : %s\n",
				pTask->name, pString); 
			dg87_free(pString, DG87_CHAR);
			pEngine->step = dg87_engine_step_error;
			return ;
		}
		dg87_free(pString, DG87_CHAR);
		/* copy next condition */
		pEngine->pConditionItem= pEngine->pConditionItem->pNext;
		pEngine->step = dg87_engine_step_copyCondition;
	}

}
/****************************************************************************/
static void dg87_engine_run_stepCopyJob(dg87_engine* pEngine)
{
	dg87_task* pTask;
	dg87_job* pJob;
	char* pString;
	char* pArgBuffer;
	char* pPatBuffer;
	assert(pEngine != NULL);
	if (pEngine->pJobElement == NULL){
		/* next step : copy job dependences */
		pTask = pEngine->pTaskElement->pObject;
		pEngine->pJobDepItem= pTask->pJobDepList->pFirst;
		pEngine->step = dg87_engine_step_copyJobDep;
		return;
	}
	pJob = pEngine->pJobElement->pObject;
	pTask = pJob->pTask;
	/* send command to create job */
	if (pEngine->pConnection->state == dg87_connection_state_waitMessage){

		/* replace argument variables by values */
		pArgBuffer = dg87_replaceVariables(
				(pJob->pArgument==NULL)?"":pJob->pArgument, 
				pEngine->pVariableVector);
		if (pArgBuffer == NULL){
			pEngine->step = dg87_engine_step_error;
			return;
		}
		/* replace path variables by values */
		pPatBuffer = dg87_replaceVariables(
				(pJob->pPath==NULL)?"":pJob->pPath, 
				pEngine->pVariableVector);
		if (pPatBuffer == NULL){
			dg87_free(pArgBuffer, DG87_CHAR);
			pEngine->step = dg87_engine_step_error;
			return;
		}

		dg87_connection_printf(pEngine->pConnection,
		"xja \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" %i %i %s %s",
			pEngine->pNewDayName,pJob->pTask->name, 
			pJob->name, pJob->description,
			pPatBuffer, pArgBuffer,	
			pJob->startTempo, pJob->endTempo,
			dg87_boolToAscii(pJob->waitAllDependences),
			dg87_boolToAscii(pJob->onErrorContinue));

		dg87_free(pArgBuffer, DG87_CHAR);
		dg87_free(pPatBuffer, DG87_CHAR);

		if (dg87_connection_send(pEngine->pConnection) == FALSE){
			pEngine->step = dg87_engine_step_error;
			return;
		}

	}
	/* verify response for job creation */
	if (pEngine->pConnection->state ==
		dg87_connection_state_messageAvailable){
		pString = dg87_connection_read(pEngine->pConnection);
		if (strcmp(pString, "OK") != 0){
			dg87_strbuf_printf(pEngine->pLog,
				"ERROR : Can't create the job '%s' in the "\
				"task '%s' : %s\n",
				pJob->name, pJob->pTask->name, pString); 
			dg87_free(pString, DG87_CHAR);
			pEngine->step = dg87_engine_step_error;
			return;
		}
		dg87_free(pString, DG87_CHAR);
		/* copy next job */
		pEngine->pJobElement= 
			pEngine->pJobElement->pNext;
		pEngine->step = dg87_engine_step_copyJob;

	}
}
/****************************************************************************/
static void dg87_engine_run_stepCopyJobDep(dg87_engine* pEngine)
{
	char* pString;	
	dg87_jobdep* pJobDep;
	assert(pEngine != NULL);
	if (pEngine->pJobDepItem == NULL){
		/* copy next task */
		pEngine->pTaskElement = 
			pEngine->pTaskElement->pNext;
		pEngine->step = dg87_engine_step_copyTask;
		return;
	}
	pJobDep = pEngine->pJobDepItem->pObject;

	/* send command to create jobdep */
	if (pEngine->pConnection->state == 
			dg87_connection_state_waitMessage){
		dg87_connection_printf(pEngine->pConnection,
		"xjda \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
			pEngine->pNewDayName,
			pJobDep->pJobBefore->pTask->name, 
			pJobDep->pJobBefore->name, 
			pJobDep->pJobAfter->name, 
			dg87_jobdep_typeToAscii(pJobDep->type));
		if (dg87_connection_send(pEngine->pConnection) == FALSE){
			pEngine->step = dg87_engine_step_error;
			return;
		}
	}
	/* verify response for job dependence creation */
	if (pEngine->pConnection->state ==
		dg87_connection_state_messageAvailable){
		pString = dg87_connection_read(pEngine->pConnection);
		if (strcmp(pString, "OK") != 0){
			dg87_strbuf_printf(pEngine->pLog,
				"ERROR : Can't create a job dependence in the "\
				"task '%s' between jobs '%s' and '%s' : %s\n",
				pJobDep->pJobBefore->pTask->name,
				pJobDep->pJobBefore->name,
				pJobDep->pJobAfter->name,
				pString); 
			dg87_free(pString, DG87_CHAR);
			pEngine->step = dg87_engine_step_error;
			return ;
		}
		dg87_free(pString, DG87_CHAR);
		/* copy next jobdep */
		pEngine->pJobDepItem= pEngine->pJobDepItem->pNext;
		pEngine->step = dg87_engine_step_copyJobDep;
	}

}
/****************************************************************************/
static void dg87_engine_run_stepCopyTaskDep(dg87_engine* pEngine)
{
	char* pString;	
	dg87_taskdep* pTaskDep;
	assert(pEngine != NULL);
	if (pEngine->pTaskDepItem == NULL){
		pEngine->step = dg87_engine_step_validateDay;
		return;
	}
	pTaskDep = pEngine->pTaskDepItem->pObject;
	/* send command to create task dep */
	if (pEngine->pConnection->state == 
			dg87_connection_state_waitMessage){
		dg87_connection_printf(pEngine->pConnection,
			"xtda \"%s\" \"%s\" \"%s\" \"%s\"",
			pEngine->pNewDayName,
			pTaskDep->pTaskBefore->name,
			pTaskDep->pTaskAfter->name,
			dg87_taskdep_typeToAscii(
				pTaskDep->type));
		if (dg87_connection_send(pEngine->pConnection) == FALSE){
			pEngine->step = dg87_engine_step_error;
			return;
		}
	}
	/* verify response for task creation */
	if (pEngine->pConnection->state ==
		dg87_connection_state_messageAvailable){
		pString = dg87_connection_read(pEngine->pConnection);
		if (strcmp(pString, "OK") != 0){
			dg87_strbuf_printf(pEngine->pLog,
				"ERROR : Can't create a dependence between "\
				"tasks '%s' and '%s' : %s\n",
				pTaskDep->pTaskBefore->name,
				pTaskDep->pTaskAfter->name,
				pString); 
			dg87_free(pString, DG87_CHAR);
			pEngine->step = dg87_engine_step_error;
			return ;
		}
		dg87_free(pString, DG87_CHAR);
		/* copy next taskdep */
		pEngine->pTaskDepItem= 
			pEngine->pTaskDepItem->pNext;
		pEngine->step = dg87_engine_step_copyTaskDep;
	}

}
/****************************************************************************/
static void dg87_engine_run_stepValidateDay(dg87_engine* pEngine)
{
	char* pString;	
	assert(pEngine != NULL);
	/* send command to validate the day */
	if (pEngine->pConnection->state == 
			dg87_connection_state_waitMessage){
		dg87_connection_printf(pEngine->pConnection,
			"xdv \"%s\"", pEngine->pNewDayName);
		if (dg87_connection_send(pEngine->pConnection) == FALSE){
			pEngine->step = dg87_engine_step_error;
			return ;
		}
		pEngine->step = dg87_engine_step_validateDay;
	}
	/* verify response for day validation*/
	if (pEngine->pConnection->state ==
			dg87_connection_state_messageAvailable){
		pString = dg87_connection_read(pEngine->pConnection);
		if (strcmp(pString, "OK") != 0){
			dg87_strbuf_printf(pEngine->pLog,
				"ERROR : Can't validate day  : %s\n",
				pString); 
			dg87_free(pString, DG87_CHAR);
			pEngine->step = dg87_engine_step_error;
			return ;
		}
		dg87_free(pString, DG87_CHAR);
		pEngine->step = dg87_engine_step_terminate;
	}

}
/****************************************************************************/
static void dg87_engine_run_stepError(dg87_engine* pEngine)
{
	char* pString;	
	assert(pEngine != NULL);
	dg87_strbuf_printf(pEngine->pLog,"ERROR !\n");
	/* don't delete the day if it was not created */
	if (pEngine->dayCreated == FALSE){
		pEngine->step = dg87_engine_step_terminate;
		return;
	}
	/* if connection is closed don't try to delete day */
	if (pEngine->pConnection->state == dg87_connection_state_error){
		pEngine->step = dg87_engine_step_terminate;
		return;
	}
	/* send command to delete the day */
	if (pEngine->dayCreated == TRUE && pEngine->pConnection->state ==
			dg87_connection_state_waitMessage){
		dg87_connection_printf(pEngine->pConnection,
			"xdk \"%s\"", pEngine->pNewDayName);
		/* don't call dg87_engine_setCommand */
		printf("->%s\n", pEngine->pConnection->pBuffer);
		if (dg87_connection_send(pEngine->pConnection) == FALSE){
			pEngine->step = dg87_engine_step_terminate;
			return ;
		}
		pEngine->step = dg87_engine_step_error;
	}
	/* verify response for day kill */
	if (pEngine->dayCreated == TRUE && pEngine->pConnection->state ==
			dg87_connection_state_messageAvailable){
		pString = dg87_connection_read(pEngine->pConnection);
		/* ignore error */
		/* don't call dg87_engine_addToLog */
		printf("->%s\n", pString);
		dg87_free(pString, DG87_CHAR);
		pEngine->step = dg87_engine_step_terminate;
	}

}
/****************************************************************************/
static void dg87_engine_run_stepTerminate(dg87_engine* pEngine)
{
	dg87_taskdep* pTaskDepCopy;
	assert(pEngine != NULL);
	dg87_strbuf_printf(pEngine->pLog,"Termination\n");
	/* free connection with dg87x */
	dg87_connection_close(pEngine->pConnection);
	dg87_connection_delete(pEngine->pConnection);
	pEngine->pConnection = NULL;

	/* unlock the ref */
	pEngine->pRefToPlan->locked = FALSE;
	pEngine->pRefToPlan = NULL;

	/* free task vector */
	if (pEngine->pTaskToPlanVector != NULL){
		dg87_vector_delete(pEngine->pTaskToPlanVector);
		pEngine->pTaskToPlanVector = NULL;
	}
	pEngine->pTaskElement = NULL;

	/* free task dependence list */
	if (pEngine->pTaskDepToPlanList != NULL){
		while (pEngine->pTaskDepToPlanList->count > 0){
			pTaskDepCopy = dg87_list_removeObjectByIndex(
					pEngine->pTaskDepToPlanList,0);
			dg87_taskdep_delete(pTaskDepCopy);

		}
		dg87_list_delete(pEngine->pTaskDepToPlanList);
		pEngine->pTaskDepToPlanList = NULL;
	}
	pEngine->pTaskDepItem = NULL;

	pEngine->dayCreated = FALSE;
	pEngine->step = dg87_engine_step_disabled;
}

/****************************************************************************/
BOOL dg87_engine_run(dg87_engine* pEngine, dg87_server* pServer)
{
	dg87_context* pContext;
	dg87_connection* pConnection;
	assert(pEngine != NULL && pServer != NULL);
	if (pEngine == NULL || pServer == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* save database */
	dg87_context_makePeriodicSave(pEngine->pContext);

	/* exit if engine is not enabled */
	if (pEngine->enabled == FALSE)
		return FALSE;
	if (pEngine->pConnection == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* exit if nothing to plan */
	if (pEngine->pRefToPlan == NULL)
		return FALSE;
	if (pEngine->pRefToPlan->locked == FALSE)
		pEngine->pRefToPlan->locked = TRUE;

	pContext = pEngine->pContext;
	pConnection = pEngine->pConnection;
	dg87_connection_update(pEngine->pConnection);
	if (pConnection->state == dg87_connection_state_error){
		dg87_strbuf_printf(pEngine->pLog, "Connection lost\n");
		pEngine->step = dg87_engine_step_terminate;
	}

	switch (pEngine->step){
		case dg87_engine_step_prepare:
			dg87_engine_run_stepPrepare(pEngine);
			break;

		case dg87_engine_step_connection:
			dg87_engine_run_stepConnection(pEngine);
			break;

		case dg87_engine_step_identification:
			dg87_engine_run_stepIdentification(pEngine);
			break;

		case dg87_engine_step_createDay:
			dg87_engine_run_stepCreateDay(pEngine);
			break;

		case dg87_engine_step_copyTask:
			dg87_engine_run_stepCopyTasks(pEngine);
			break;

		case dg87_engine_step_copyCondition:
			dg87_engine_run_stepCopyCondition(pEngine);
			break;
			
		case dg87_engine_step_copyJob:
			dg87_engine_run_stepCopyJob(pEngine);
			break;

		case dg87_engine_step_copyJobDep:
			dg87_engine_run_stepCopyJobDep(pEngine);
			break;

		case dg87_engine_step_copyTaskDep:
			dg87_engine_run_stepCopyTaskDep(pEngine);
			break;

		case dg87_engine_step_validateDay:
			dg87_engine_run_stepValidateDay(pEngine);
			break;

		case dg87_engine_step_error:
			dg87_engine_run_stepError(pEngine);
			break;

		case dg87_engine_step_terminate:
			dg87_engine_run_stepTerminate(pEngine);
			break;
		default:
			/* nothing to do */
			break;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_engine_plan(dg87_engine* pEngine, dg87_ref* pRef, 
		char* pNewDayName, char* pDate,
		char* pHost, int ipPort ,
		int (*print)(char*))
{
	assert(pEngine != NULL && pRef != NULL && pNewDayName != NULL
			&& pDate != NULL && pHost != NULL && print != NULL);
	if (pEngine == NULL || pRef == NULL || pNewDayName == NULL 
			|| pDate == NULL || pHost == NULL || print == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}

	/* copy day name */
	if (pEngine->pNewDayName != NULL)
		dg87_free(pEngine->pNewDayName, DG87_CHAR);
	if ((pEngine->pNewDayName = 
				dg87_strdup(pNewDayName, DG87_CHAR)) == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	/* copy date */
	if (pEngine->pDate != NULL)
		dg87_free(pEngine->pDate, DG87_CHAR);
	if ((pEngine->pDate = 
			dg87_strdup(pDate, DG87_CHAR)) == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pEngine->pNewDayName, DG87_CHAR);
		pEngine->pNewDayName = NULL;
		return FALSE;
	}

	/* verify that another planification is not in progress */
	if (pEngine->pRefToPlan != NULL){
		dg87_err_setError(DG87_ERR_PLANINPROGRESS);
		return FALSE;
	}

	/* create connection to dg87x */
	if (pEngine->pConnection != NULL)
		dg87_connection_delete(pEngine->pConnection);
	pEngine->pConnection = dg87_connection_out_new(pHost, ipPort);
	if (pEngine->pConnection == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pEngine->pNewDayName, DG87_CHAR);
		dg87_free(pEngine->pDate, DG87_CHAR);
		return FALSE;
	}
	/* initialize engine to plan the referential */
	dg87_engine_clearLog(pEngine);
	pRef->locked = TRUE;
	pEngine->pRefToPlan = pRef;
	pEngine->step = dg87_engine_step_prepare;
	dg87_strbuf_printf(pEngine->pLog,"* Prepare\n");
	pEngine->dayCreated = FALSE;

	return TRUE;
}

/****************************************************************************/
BOOL dg87_engine_cancelPlan(dg87_engine* pEngine)
{
	assert(pEngine != NULL);
	if (pEngine == NULL ){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pEngine->pRefToPlan == NULL){
		dg87_err_setError(DG87_ERR_NOPLANINPROGRESS);
		return FALSE;
	}
	dg87_strbuf_printf(pEngine->pLog, "Canceled by operator");
	pEngine->step = dg87_engine_step_error;
	return TRUE;
}
/****************************************************************************/
dg87_variable* dg87_engine_variableAdd(dg87_engine* pEngine, 
		char* pVariableName, char* pValue)
{
	dg87_variable* pVariable;
	assert(pEngine != NULL && pVariableName != NULL && pValue != NULL);
	if (pEngine == NULL || pVariableName == NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pVariable = dg87_variable_new(pVariableName, pValue);
	if (pVariable == NULL)
		return NULL;

	/* vector generate error if an other variable has the same name */
	/* key must point on pVariable->name */
	if (dg87_vector_addObject(pEngine->pVariableVector, 
				pVariable, pVariable->name) == FALSE)
	{
		dg87_variable_delete(pVariable);
		return NULL;
	}
	return pVariable;
}
/****************************************************************************/
int dg87_engine_variableCount(dg87_engine* pEngine)
{
	assert(pEngine != NULL);
	if (pEngine == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_vector_count(pEngine->pVariableVector);
}
/****************************************************************************/
dg87_variable* dg87_engine_variableGetByName(dg87_engine* pEngine, char* pName)
{
	assert(pEngine != NULL && pName != NULL);
	if (pEngine == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByKey(pEngine->pVariableVector, pName);
}
/****************************************************************************/
dg87_variable* dg87_engine_variableGetByIndex(dg87_engine* pEngine, int index)
{
	assert(pEngine != NULL);
	if (pEngine == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByIndex(pEngine->pVariableVector, index);
}

/****************************************************************************/
BOOL dg87_engine_variableDeleteByName(dg87_engine* pEngine,char* pName)
{
	dg87_variable* pVariable;
	assert(pEngine != NULL && pName != NULL);
	if (pEngine == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pVariable = dg87_vector_removeObjectByKey
		(pEngine->pVariableVector, pName);
	if (pVariable == NULL) 
		return FALSE;
	if (dg87_variable_delete(pVariable) == FALSE){
		/* add the variable in vector again */
		dg87_vector_addObject(pEngine->pVariableVector,
				pVariable, pVariable->name);
		return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_engine_save(dg87_engine* pEngine, dg87_storage* pStorage)
{
	assert(pEngine != NULL && pStorage != NULL);
	if (pEngine == NULL || pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* 'enabled' property */
	if (dg87_storage_writeByte(pStorage, pEngine->enabled) == FALSE)
		return FALSE;

	return TRUE;
}
/****************************************************************************/
BOOL dg87_engine_load(dg87_engine* pEngine, dg87_storage* pStorage)
{
	BOOL value;
	assert(pEngine != NULL && pStorage != NULL);
	if (pEngine == NULL || pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* 'enabled' property */
	if (dg87_storage_readByte(pStorage, &value) == FALSE)
		return FALSE;
	pEngine->enabled = value;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_engine_variableDeleteByIndex(dg87_engine* pEngine,int index)
{
	dg87_variable* pVariable;
	assert(pEngine != NULL);
	if (pEngine == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pVariable = dg87_vector_removeObjectByIndex
		(pEngine->pVariableVector, index);
	if (pVariable == NULL) 
		return FALSE;
	dg87_variable_delete(pVariable);
	return TRUE;
}
/****************************************************************************/
/* End of file */
