/* file job.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "job.h"

/****************************************************************************/
dg87_job* dg87_job_new(dg87_task* pTask, char* pName)
{
	dg87_job* pJob;
	assert(pTask != NULL && pName != NULL);
	if (pTask == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	if (strlen(pName) >= DG87_JOB_NAME_LENGTH){
		dg87_err_setError(DG87_ERR_NAMETOLONG);
		return FALSE;
	}
	pJob = dg87_malloc(sizeof(dg87_job), DG87_JOB);
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	strncpy(pJob->name, pName, DG87_JOB_NAME_LENGTH); 
	pJob->pTask = pTask;
	pJob->description[0] = 0;
	pJob->pPath = NULL;
	pJob->pArgument = NULL;
	pJob->startTempo = 0; 
	pJob->endTempo = 0;
	pJob->waitAllDependences = TRUE;
	pJob->onErrorContinue = TRUE;
	return pJob;
}

/****************************************************************************/
dg87_job* dg87_job_duplicate(dg87_task* pNewTask, dg87_job* pOriginalJob)
{	
	dg87_job* pNewJob;
	assert(pNewTask != NULL && pOriginalJob != NULL);
	if (pNewTask == NULL || pOriginalJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	/* create a job with the same name */
	pNewJob = dg87_job_new(pNewTask, pOriginalJob->name);
	if (pNewJob == NULL) return NULL;
	/* copy job properties */
	strncpy(pNewJob->description, pOriginalJob->description, 
			DG87_JOB_DESC_LENGTH);
	pNewJob->startTempo = pOriginalJob->startTempo;
	pNewJob->endTempo = pOriginalJob->endTempo;
	pNewJob->waitAllDependences= pOriginalJob->waitAllDependences;
	pNewJob->onErrorContinue = pOriginalJob->onErrorContinue;
	/* copy path */
	if (pOriginalJob->pPath != NULL){
		pNewJob->pPath = dg87_strdup(pOriginalJob->pPath, DG87_CHAR);
		if (pNewJob->pPath == NULL){
			dg87_job_delete(pNewJob);
			return NULL;
		}
	}
	/* copy argument */
	if (pOriginalJob->pArgument != NULL){
		pNewJob->pArgument = dg87_strdup(pOriginalJob->pArgument, 
				DG87_CHAR);
		if (pNewJob->pArgument == NULL){
			dg87_job_delete(pNewJob);
			return NULL;
		}
	}
	return pNewJob;
}
/****************************************************************************/
BOOL dg87_job_delete(dg87_job* pJob)
{
	assert(pJob != NULL);
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pJob->pPath != NULL)
		dg87_free(pJob->pPath, DG87_CHAR);
	if (pJob->pArgument != NULL)
		dg87_free(pJob->pArgument, DG87_CHAR);
	dg87_free(pJob, DG87_JOB);
	return TRUE;
}

/****************************************************************************/
BOOL dg87_job_load(dg87_job* pJob, dg87_storage* pStorage)
{	
	char* pString;
	dg87byte b, b2;
	assert(pJob != NULL && pStorage != NULL);
	if (pJob == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read job description*/
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_job_setDescription(pJob, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);
	/* read job path */
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_job_setPath(pJob, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);
	/* read job arugment */
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_job_setArgument(pJob, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);

	/* read start tempo and end tempo */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_storage_readByte(pStorage, &b2) == FALSE)
		return FALSE;
	if (dg87_job_setTempo(pJob, b, b2) == FALSE)
		return FALSE;

	/* read waitAllDependences */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_job_setWaitAllDependences(pJob, b) == FALSE)
		return FALSE;

	/* read onErrorContinue */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_job_setOnErrorContinue(pJob, b) == FALSE)
		return FALSE;

	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_save(dg87_job* pJob, dg87_storage* pStorage)
{	
	assert(pJob != NULL && pStorage != NULL);
	if (pJob == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write job description */
	if (dg87_storage_writeString(pStorage, pJob->description) == FALSE)
		return FALSE;
	/* write job path */
	if (dg87_storage_writeString(pStorage, 
			(pJob->pPath == NULL)?"":pJob->pPath) == FALSE)
		return FALSE;
	/* write job argument */
	if (dg87_storage_writeString(pStorage, 
			(pJob->pArgument == NULL)?"":pJob->pArgument) == FALSE)
		return FALSE;
	/* write start tempo */
	if (dg87_storage_writeByte(pStorage, pJob->startTempo)== FALSE)
		return FALSE;
	/* write end tempo */
	if (dg87_storage_writeByte(pStorage, pJob->endTempo)== FALSE)
		return FALSE;
	/* write waitAllDependences */
	if (dg87_storage_writeByte(pStorage, pJob->waitAllDependences)== FALSE)
		return FALSE;
	/* write onErrorContinue */
	if (dg87_storage_writeByte(pStorage, pJob->onErrorContinue)== FALSE)
		return FALSE;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setDescription(dg87_job* pJob, char* pDescription)
{
	assert(pJob != NULL && pDescription != NULL);
	if (pJob == NULL || pDescription == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (strlen(pDescription) >= DG87_JOB_DESC_LENGTH){
		dg87_err_setError(DG87_ERR_DESCRIPTIONTOLONG);
		return FALSE;
	}
	strncpy(pJob->description, pDescription, DG87_JOB_DESC_LENGTH);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setTempo(dg87_job* pJob, int startTempo, int endTempo)
{
	assert(pJob != NULL);
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (startTempo < 0 || startTempo > JOB_MAX_TEMPO
			|| endTempo < 0 || endTempo > JOB_MAX_TEMPO){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return FALSE;
	}
	pJob->startTempo = startTempo;
	pJob->endTempo = endTempo;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setPath(dg87_job* pJob, char* pPath)
{
	assert(pJob != NULL);
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	
	if (pJob->pPath != NULL) dg87_free(pJob->pPath, DG87_CHAR);
	pJob->pPath = NULL;
	if (pPath == NULL || strlen(pPath) == 0) 
		return TRUE;

	pJob->pPath = dg87_malloc(strlen(pPath)+1, DG87_CHAR);
	if (pJob->pPath == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	strcpy(pJob->pPath, pPath); 
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setArgument(dg87_job* pJob, char* pArgument)
{
	assert(pJob != NULL);
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	
	if (pJob->pArgument != NULL) dg87_free(pJob->pArgument, DG87_CHAR);
	pJob->pArgument = NULL;
	if (pArgument == NULL || strlen(pArgument) == 0) 
		return TRUE;

	pJob->pArgument = dg87_malloc(strlen(pArgument)+1, DG87_CHAR);
	if (pJob->pArgument == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	strcpy(pJob->pArgument, pArgument); 
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setWaitAllDependences(dg87_job* pJob, BOOL value)
{
	assert(pJob != NULL);
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob->waitAllDependences = value;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setOnErrorContinue(dg87_job* pJob, BOOL value)
{
	assert(pJob != NULL);
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob->onErrorContinue = value;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_findUpStreamJobs(dg87_job* pJob, dg87_list* pJobList)
{
	dg87_jobdep* pJobDep;
	dg87_item* pJobDepItem;
	assert(pJob != NULL && pJobList != NULL);
	if (pJob== NULL || pJobList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_list_addObject(pJobList, pJob) == FALSE)
		return FALSE;
	pJobDepItem = pJob->pTask->pJobDepList->pFirst;
	while (pJobDepItem != NULL){
		pJobDep = pJobDepItem->pObject;
		pJobDepItem = pJobDepItem->pNext;
		if (pJobDep->pJobAfter != pJob)
			continue;
		if (dg87_job_findUpStreamJobs(pJobDep->pJobBefore,
					pJobList) == FALSE)
			return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_findDownStreamJobs(dg87_job* pJob, dg87_list* pJobList)
{
	dg87_jobdep* pJobDep;
	dg87_item* pJobDepItem;
	assert(pJob != NULL && pJobList != NULL);
	if (pJob== NULL || pJobList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_list_addObject(pJobList, pJob) == FALSE)
		return FALSE;
	pJobDepItem = pJob->pTask->pJobDepList->pFirst;
	while (pJobDepItem != NULL){
		pJobDep = pJobDepItem->pObject;
		pJobDepItem = pJobDepItem->pNext;
		if (pJobDep->pJobBefore != pJob)
			continue;
		if (dg87_job_findDownStreamJobs(pJobDep->pJobAfter,
					pJobList) == FALSE)
			return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_job_check(dg87_job* pJob, int (*print)(char*))
{
	if (strlen(pJob->name) == 0){
		print("Job : name length is zero !\n");
		return 1;
	}
	if (pJob->pTask == NULL){
		print("Job : Task pointer is null !\n");
		return 1;
	}
	if (pJob->startTempo > JOB_MAX_TEMPO){
		print("Job : incorrect value for startTempo !\n");
		return 1;
	}
	if (pJob->endTempo > JOB_MAX_TEMPO){
		print("Job : incorrect value for endTempo\n");
		return 1;
	}
	/* pPath and pArgument could be null -> no test */

	return 0;
}
#endif /*DEBUG*/


/****************************************************************************/
/* End of file */
