/* file jobdep.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "job.h"
#include "jobdep.h"
/*****************************************************************************/
dg87_jobdep* dg87_jobdep_new(struct dg87_job_struct* pJobBefore, struct dg87_job_struct* pJobAfter)
{
	dg87_jobdep* pJobDep;
	dg87_list* pUpStreamJobList;
	dg87_list* pDownStreamJobList;
	assert(pJobBefore != NULL && pJobAfter != NULL);
	if (pJobBefore == NULL || pJobAfter == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	/* jobs must be in the same task to have a dependance */
	if (pJobBefore->pTask != pJobAfter->pTask){
		dg87_err_setError(DG87_ERR_JOBSNOTINSAMETASK);
		return NULL;
	}

	/* find all upstream jobs */
	pUpStreamJobList = dg87_list_new();
	if (pUpStreamJobList == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	dg87_job_findUpStreamJobs(pJobBefore, pUpStreamJobList);
	/* find all downstream tasks */
	pDownStreamJobList = dg87_list_new();
	if (pDownStreamJobList == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	dg87_job_findDownStreamJobs(pJobAfter, pDownStreamJobList);
	/* detect circular dependences*/ 
	if (dg87_list_intersection(pDownStreamJobList, 
				pUpStreamJobList) == TRUE){
		dg87_list_delete(pDownStreamJobList);
		dg87_list_delete(pUpStreamJobList);
		dg87_err_setError(DG87_ERR_CIRCULARDEPENDENCE);
		return NULL;
	}
	dg87_list_delete(pDownStreamJobList);
	dg87_list_delete(pUpStreamJobList);

	pJobDep = dg87_malloc(sizeof(dg87_jobdep), DG87_JOBDEP);
	if (pJobDep == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pJobDep->pJobBefore = pJobBefore;
	pJobDep->pJobAfter = pJobAfter;
	pJobDep->type = dg87_jobdep_always;
	return pJobDep;
}
/*****************************************************************************/
BOOL dg87_jobdep_delete(dg87_jobdep* pJobDep)
{
	assert(pJobDep != NULL);
	if (pJobDep == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	dg87_free(pJobDep, DG87_JOBDEP);
	return TRUE;
}
/*****************************************************************************/
BOOL dg87_jobdep_load(dg87_jobdep* pJobDep, dg87_storage* pStorage)
{
	dg87byte b;
	assert(pJobDep != NULL && pStorage != NULL);
	if (pJobDep == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read dependence type */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_jobdep_setType(pJobDep, b) == FALSE)
		return FALSE;

	return TRUE;
}
/*****************************************************************************/
BOOL dg87_jobdep_save(dg87_jobdep* pJobDep, dg87_storage* pStorage)
{
	assert(pJobDep != NULL && pStorage != NULL);
	if (pJobDep == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write dependence type */
	if (dg87_storage_writeByte(pStorage, pJobDep->type)== FALSE)
		return FALSE;
	return TRUE;
}
/*****************************************************************************/
BOOL dg87_jobdep_setType(dg87_jobdep* pJobDep, dg87_jobdep_type type)
{
	assert(pJobDep != NULL);
	if (pJobDep == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJobDep->type = type;
	return TRUE;
}

/*****************************************************************************/
dg87_jobdep_type dg87_jobdep_asciiToType(char* pStrType)
{
	assert(pStrType != NULL);
	if (pStrType == NULL)
		return dg87_jobdep_unknown;
	if (strcasecmp(pStrType, "always") == 0)
		return dg87_jobdep_always;
	else if (strcasecmp(pStrType, "onsuccess") == 0)
		return dg87_jobdep_onSuccess;
	else if (strcasecmp(pStrType, "onerror") == 0)
		return dg87_jobdep_onError;
	return dg87_jobdep_unknown;
}
/*****************************************************************************/
char* dg87_jobdep_typeToAscii(dg87_jobdep_type deptype)
{
	switch (deptype){
		case dg87_jobdep_always: return "always";
		case dg87_jobdep_onSuccess: return "onsuccess";
		case dg87_jobdep_onError: return "onerror";
		default: return "unknown"; /* to remove warning */
	}
	return NULL;
}
/*****************************************************************************/
#ifdef DEBUG
int dg87_jobdep_check(dg87_jobdep* pJobdep, int (*print)(char*))
{
	if (pJobdep->pJobBefore == NULL){
		print("JobDep : JobBefore is NULL !\n");
		return 1;
	}
	if (pJobdep->pJobAfter == NULL){
		print("JobDep : JobAfter is NULL !\n");
		return 1;
	}
	return 0;
}
#endif /*DEBUG*/
/*****************************************************************************/

/* End of file */
