/* file queue.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "context.h"
#include "queue.h"

/****************************************************************************/
dg87_queue* dg87_queue_new(dg87_target* pTarget, char* pName)
{
	dg87_queue* pQueue;
	assert(pTarget != NULL && pName != NULL);
	if (pTarget == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	if (strlen(pName) >= DG87_QUEUE_NAME_LENGTH){
		dg87_err_setError(DG87_ERR_NAMETOLONG);
		return FALSE;
	}
	pQueue = dg87_malloc(sizeof(dg87_queue), DG87_QUEUE);
	if (pQueue == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* create the RefTaskList */
	pQueue->pRefTaskList = dg87_list_new();
	if (pQueue->pRefTaskList == NULL){
		dg87_queue_delete(pQueue);
		return NULL;
	}
	strncpy(pQueue->name, pName, DG87_QUEUE_NAME_LENGTH); 
	pQueue->pTarget = pTarget;
	pQueue->description[0] = 0;
	pQueue->max = 1;
	return pQueue;
}

/****************************************************************************/
BOOL dg87_queue_delete(dg87_queue* pQueue)
{
	dg87_task* pTask;
	
	assert(pQueue != NULL);
	if (pQueue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* set Task's Queue to NULL for each task */
	while (pQueue->pRefTaskList->count > 0){
		pTask = dg87_list_removeObjectByIndex
			(pQueue->pRefTaskList, 0);
		pTask->pQueue = NULL;
	}

	dg87_list_delete(pQueue->pRefTaskList);
	dg87_free(pQueue, DG87_QUEUE);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_queue_load(dg87_queue* pQueue, dg87_storage* pStorage)
{
	dg87uint value;
	char* pString;
	assert(pQueue != NULL && pStorage != NULL);
	if (pQueue == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read target description */
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_queue_setDescription(pQueue, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);
	/* read max */
	if (dg87_storage_readUInt(pStorage, &value) == FALSE)
		return FALSE;
	pQueue->max = value;

	return TRUE;
}
/****************************************************************************/
BOOL dg87_queue_save(dg87_queue* pQueue, dg87_storage* pStorage)
{
	assert(pQueue != NULL && pStorage != NULL);
	if (pQueue == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write target description */
	if (dg87_storage_writeString(pStorage, pQueue->description) == FALSE)
		return FALSE;
	/* write max */
	if (dg87_storage_writeUInt(pStorage, pQueue->max) == FALSE)
		return FALSE;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_queue_setDescription(dg87_queue* pQueue, char* pDescription)
{
	assert(pQueue != NULL && pDescription != NULL);
	if (pQueue == NULL || pDescription == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (strlen(pDescription) >= DG87_QUEUE_DESC_LENGTH){
		dg87_err_setError(DG87_ERR_DESCRIPTIONTOLONG);
		return FALSE;
	}
	strncpy(pQueue->description, pDescription, DG87_QUEUE_DESC_LENGTH);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_queue_setMax(dg87_queue* pQueue, int value)
{
	assert(pQueue != NULL);
	if (pQueue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (value < 1){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return FALSE;
	}
	pQueue->max = value;
	return TRUE;
}

/****************************************************************************/
#ifdef DEBUG
int dg87_queue_check(dg87_queue* pQueue, int (*print)(char*))
{
	dg87_item *pItem;
	dg87_task *pTask;
	if (strlen(pQueue->name) == 0){
		print("Queue : name length is zero !\n");
		return 1;
	}
	if (pQueue->pTarget == NULL) {
		print("Queue : target pointer is NULL !\n");
		return 1;
	}

	/* check RefTaskList */
	if (pQueue->pRefTaskList == NULL){
		print("Queue : RefTaskList is NULL !\n");
		return 1;
	}
	if (dg87_list_check(pQueue->pRefTaskList,print) != 0)
		return 1;
	pItem = pQueue->pRefTaskList->pFirst;
	while (pItem != NULL){
		pTask = pItem->pObject;
		if (pTask->pQueue != pQueue){
			print("Queue : Task's queue incorrect !\n");
			return 1;
		}
		pItem = pItem->pNext;
	}

	return 0;
}
#endif /*DEBUG*/
/****************************************************************************/

/* End of file */
