/* file ref.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <libdg87/element.h>
#include <libdg87/connection.h>
#include "dgerr.h"
#include "ref.h"
#include "global.h"

/****************************************************************************/
dg87_ref* dg87_ref_new(char* pName)
{
	dg87_ref* pRef;
	assert(pName != NULL);
	if (pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}

	/* allocate memory for ref */
	pRef = dg87_malloc(sizeof(dg87_ref), DG87_REF);
	if (pRef == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}

	/* allocate memory for the TaskVector */
	pRef->pTaskVector = dg87_vector_new();
	if (pRef->pTaskVector == NULL) {
		dg87_free(pRef, DG87_REF);
		return NULL;
	}
	/* allocate memory for the TaskdepList */
	pRef->pTaskdepList = dg87_list_new();
	if (pRef->pTaskdepList == NULL) {
		dg87_vector_delete(pRef->pTaskVector);
		dg87_free(pRef, DG87_REF);
		return NULL;
	}	
	/* allocate memory for the RuleVector */
	pRef->pRuleVector = dg87_vector_new();
	if (pRef->pRuleVector == NULL) {
		dg87_vector_delete(pRef->pTaskVector);
		dg87_list_delete(pRef->pTaskdepList);
		dg87_free(pRef, DG87_REF);
		return NULL;
	}
	/* initialize the structure of the object*/
	strncpy(pRef->name,pName, DG87_REF_NAME_LENGTH);
	pRef->locked = FALSE;
	return pRef;
}
/****************************************************************************/
dg87_ref* dg87_ref_duplicate(dg87_ref* pOriginalRef, char* pNewRefName)
{
	dg87_ref* pNewRef;
	dg87_element* pElement,*pElement2;
	dg87_item* pItem;
	dg87_task *pOriginalTask, *pNewTask;
	dg87_task *pTaskBefore, *pTaskAfter;
	dg87_rule *pOriginalRule, *pNewRule;
	dg87_rule_operation *pOriginalOperation, *pNewOperation;
	dg87_taskdep *pNewTaskDep, *pOriginalTaskDep;
	assert(pOriginalRef != NULL && pNewRefName != NULL);
	if (pOriginalRef == NULL || pNewRefName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pNewRef = dg87_ref_new(pNewRefName);
	if (pNewRef == NULL)
		return NULL;
	/* copy all rules */
	/* first step is to copy only rules (without operations)*/
	pElement = pOriginalRef->pRuleVector->pFirst;
	while (pElement != NULL){
		pOriginalRule = pElement->pObject;
		pElement = pElement->pNext;
		/* duplicate rule */
		pNewRule = dg87_rule_duplicate(pOriginalRule);
		if (pNewRule == NULL){
			dg87_ref_delete(pNewRef);
			return NULL;
		}
		/* add it to rule vector of the new ref */
		if (dg87_vector_addObject(pNewRef->pRuleVector, 
					pNewRule, pNewRule->name) == FALSE)
		{
			dg87_rule_delete(pNewRule);
			dg87_ref_delete(pNewRef);
			return NULL;
		}
	}
	/* second step is to copy each operation of each rule */
	pElement = pOriginalRef->pRuleVector->pFirst;
	pElement2 = pNewRef->pRuleVector->pFirst;
	while (pElement != NULL && pElement2 != NULL){
		pOriginalRule = pElement->pObject;
		pElement = pElement->pNext;
		pNewRule = pElement2->pObject;
		pElement2 = pElement2->pNext;
		/* for each operation of the rule */
		pItem = pOriginalRule->pOperationList->pFirst;
		while (pItem != NULL){
			pOriginalOperation = pItem->pObject;
			pItem = pItem->pNext;
			/* duplicate operation */
			pNewOperation = dg87_operation_duplicate(
					pOriginalOperation, 
					pNewRef->pRuleVector);
			if (pNewOperation == NULL){
				dg87_ref_delete(pNewRef);
				return NULL;
			}
			/* add it to the operation list of the rule */
			if (dg87_rule_operationAdd(pNewRule, pNewOperation)
					== FALSE){
				dg87_ref_delete(pNewRef);
				return NULL;
			}
		}
	}

	/* copy all tasks */
	pElement = pOriginalRef->pTaskVector->pFirst;
	while (pElement != NULL){
		pOriginalTask = pElement->pObject;
		pElement = pElement->pNext;
		/* duplicate task */
		pNewTask = dg87_task_duplicate(pOriginalTask, 
				pNewRef->pRuleVector);
		if (pNewTask == NULL){
			dg87_ref_delete(pNewRef);
			return NULL;
		}
		/* add it to task vector of the new ref */
		if (dg87_vector_addObject(pNewRef->pTaskVector, 
					pNewTask, pNewTask->name) == FALSE)
		{
			dg87_task_delete(pNewTask);
			dg87_ref_delete(pNewRef);
			return NULL;
		}
		pNewTask->pRef = pNewRef;

	}
	/* copy TaskDepList */
	pItem = pOriginalRef->pTaskdepList->pFirst;
	while (pItem != NULL){
		pOriginalTaskDep = pItem->pObject;
		pItem = pItem->pNext;
		pTaskBefore = dg87_vector_getObjectByKey(pNewRef->pTaskVector, 
				pOriginalTaskDep->pTaskBefore->name);
		pTaskAfter = dg87_vector_getObjectByKey(pNewRef->pTaskVector, 
				pOriginalTaskDep->pTaskAfter->name);
		if (pTaskBefore == NULL || pTaskAfter == NULL){
			dg87_ref_delete(pNewRef);
			return NULL;
		}
		pNewTaskDep = dg87_ref_taskdepAdd(pNewRef,
				pTaskBefore,pTaskAfter);
		if (pNewTaskDep == NULL){
			dg87_ref_delete(pNewRef);
			return NULL;
		}
		pNewTaskDep->type = pOriginalTaskDep->type;
	}
	return pNewRef;
}
/****************************************************************************/
BOOL dg87_ref_delete(dg87_ref* pRef)
{
	dg87_task *pTask;
	dg87_taskdep *pTaskDep;
	dg87_rule *pRule;
	assert(pRef != NULL);
	if (pRef == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_err_setError(DG87_ERR_PLANINPROGRESS);
		return FALSE;
	}
	/* delete all taskdep objects */
	while (pRef->pTaskdepList->count > 0){
		pTaskDep = dg87_list_removeObjectByIndex(pRef->pTaskdepList, 0);
		dg87_taskdep_delete(pTaskDep);
	}
	dg87_list_delete(pRef->pTaskdepList);

	/* delete all task object */
	while (pRef->pTaskVector->count > 0){
		pTask = dg87_vector_removeObjectByIndex(pRef->pTaskVector,0);
		dg87_list_clear(pTask->pTaskDepBeforeList);
		dg87_list_clear(pTask->pTaskDepAfterList);
		dg87_task_delete(pTask);
	}
	dg87_vector_delete(pRef->pTaskVector);

	/* delete all rule object */
	while (pRef->pRuleVector->count > 0){
		pRule = dg87_vector_removeObjectByIndex(pRef->pRuleVector,0);
		pRule->linkedRules = 0; /* force rule deletion */
		dg87_rule_delete(pRule);
	}
	dg87_vector_delete(pRef->pRuleVector);

	dg87_free(pRef, DG87_REF);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_ref_load(dg87_ref* pRef, dg87_storage* pStorage)
{
	/* nothing to load */
	return TRUE;
}
/****************************************************************************/
BOOL dg87_ref_save(dg87_ref* pRef, dg87_storage* pStorage)
{
	/* nothing to save */
	return TRUE;
}
/****************************************************************************/
dg87_task* dg87_ref_taskAdd(dg87_ref* pRef, char* pTaskName)
{
	dg87_task* pTask = NULL;
	assert(pRef != NULL && pTaskName != NULL);
	if (pRef == NULL || pTaskName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pTask = dg87_task_new(pTaskName);
	if (pTask == NULL) 
		return NULL;

	/* add the task in the TaskList */
	/* vector generate error if an other task has not the same name*/
	/* key must point on pTask->name */
	if (dg87_vector_addObject(pRef->pTaskVector, 
				pTask, pTask->name) == FALSE)
	{
		dg87_task_delete(pTask);
		return NULL;
	}
	pTask->pRef = pRef;

	return pTask;
}
/****************************************************************************/
int dg87_ref_taskCount(dg87_ref* pRef)
{
	assert(pRef != NULL);
	if (pRef == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_vector_count(pRef->pTaskVector);
}
/****************************************************************************/
dg87_task* dg87_ref_taskGetByName(dg87_ref* pRef,char* pName)
{
	dg87_element *pElement;
	assert(pRef != NULL && pName != NULL);
	if (pRef == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}

	/* browse the list to find an item with the same name*/
	pElement = pRef->pTaskVector->pFirst;
	while (pElement != NULL){
		if (strcmp(pElement->pKey, pName) == 0)
			return pElement->pObject;
		pElement = pElement->pNext;
	}	
	dg87_err_setError(DG87_ERR_CANTFINDOBJECT);
	return NULL;
}
/****************************************************************************/
dg87_task* dg87_ref_taskGetByIndex(dg87_ref* pRef,int index)
{
	assert(pRef != NULL);
	if (pRef == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByIndex(pRef->pTaskVector, index);
}
/****************************************************************************/
BOOL dg87_ref_taskDelete(dg87_ref* pRef,dg87_task* pTask)
{
	assert(pRef != NULL && pTask != NULL);
	if (pRef == NULL || pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_vector_removeObject(pRef->pTaskVector, pTask) == FALSE)
		return FALSE;
	if (dg87_task_delete(pTask) == FALSE){
		/* add task in vector again */
		dg87_vector_addObject(pRef->pTaskVector, pTask, pTask->name); 
		return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_ref_taskDeleteByName(dg87_ref* pRef,char* pName)
{
	dg87_task* pTask;
	assert(pRef != NULL && pName != NULL);
	if (pRef == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTask = dg87_vector_getObjectByKey(pRef->pTaskVector, pName);
	if (pTask == NULL) return FALSE;
	return dg87_ref_taskDelete(pRef, pTask);
}
/****************************************************************************/
BOOL dg87_ref_taskDeleteByIndex(dg87_ref* pRef,int index)
{
	dg87_task* pTask;
	assert(pRef != NULL);
	if (pRef == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTask = dg87_vector_getObjectByIndex(pRef->pTaskVector, index);
	if (pTask == NULL) return FALSE;
	return dg87_ref_taskDelete(pRef, pTask);
}
/****************************************************************************/
dg87_taskdep* dg87_ref_taskdepAdd (dg87_ref* pRef, 
	 struct dg87_task_struct* pTaskBefore, 
	 struct dg87_task_struct* pTaskAfter)
{
	dg87_taskdep* pTaskDep;
	assert(pRef != NULL && pTaskBefore != NULL && pTaskAfter != NULL);
	if (pRef == NULL || pTaskBefore == NULL || pTaskAfter == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}

	/* create and add the taskdep */
	pTaskDep = dg87_taskdep_new(pTaskBefore, pTaskAfter);
	if (pTaskDep == NULL) 
		return NULL;

	dg87_list_addObject(pRef->pTaskdepList, pTaskDep);
	dg87_list_addObject(pTaskBefore->pTaskDepAfterList, pTaskDep);
	dg87_list_addObject(pTaskAfter->pTaskDepBeforeList, pTaskDep);

	return pTaskDep;
}
/****************************************************************************/
int dg87_ref_taskdepCount(dg87_ref* pRef)
{
	assert(pRef != NULL);
	if (pRef == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_list_count(pRef->pTaskdepList);
}
/****************************************************************************/
dg87_taskdep* dg87_ref_taskdepGet(dg87_ref* pRef, int index)
{
	assert(pRef != NULL);
	if (pRef == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_list_getObjectByIndex(pRef->pTaskdepList, index);
}
/****************************************************************************/
BOOL dg87_ref_taskdepDelete(dg87_ref* pRef,int index)
{
	dg87_taskdep* pTaskDep;
	assert(pRef != NULL);
	if (pRef == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTaskDep = dg87_list_removeObjectByIndex(pRef->pTaskdepList, index);
	if (pTaskDep == NULL) return FALSE;
	dg87_taskdep_delete(pTaskDep);
	return TRUE;
}
/****************************************************************************/
dg87_rule* dg87_ref_ruleAdd(dg87_ref* pRef, char* pRuleName)
{
	dg87_rule* pRule = NULL;
	assert(pRef != NULL && pRuleName != NULL);
	if (pRef == NULL || pRuleName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pRule = dg87_rule_new(pRuleName);
	if (pRule == NULL) 
		return NULL;

	/* add the rule in the RuleVector */
	/* vector generate error if an other rule has not the same name*/
	/* key must point on pRule->name */
	if (dg87_vector_addObject(pRef->pRuleVector, 
				pRule, pRule->name) == FALSE)
	{
		dg87_rule_delete(pRule);
		return NULL;
	}
	return pRule;
}
/****************************************************************************/
int dg87_ref_ruleCount(dg87_ref* pRef)
{
	assert(pRef != NULL);
	if (pRef == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_vector_count(pRef->pRuleVector);
}
/****************************************************************************/
dg87_rule* dg87_ref_ruleGetByName(dg87_ref* pRef,char* pName)
{
	dg87_element *pElement;
	assert(pRef != NULL && pName != NULL);
	if (pRef == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}

	/* browse the list to find an item with the same name*/
	pElement = pRef->pRuleVector->pFirst;
	while (pElement != NULL){
		if (strcmp(pElement->pKey, pName) == 0)
			return pElement->pObject;
		pElement = pElement->pNext;
	}	
	dg87_err_setError(DG87_ERR_CANTFINDOBJECT);
	return NULL;
}
/****************************************************************************/
dg87_rule* dg87_ref_ruleGetByIndex(dg87_ref* pRef,int index)
{
	assert(pRef != NULL);
	if (pRef == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByIndex(pRef->pRuleVector, index);
}
/****************************************************************************/
BOOL dg87_ref_ruleDeleteByName(dg87_ref* pRef,char* pName)
{
	dg87_rule* pRule;
	assert(pRef != NULL && pName != NULL);
	if (pRef == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pRule = dg87_vector_removeObjectByKey(pRef->pRuleVector, pName);
	if (pRule == NULL)
		return FALSE;
	if (dg87_rule_delete(pRule) == FALSE){
		/* add the rule in the vector again */
		dg87_vector_addObject(pRef->pRuleVector, pRule, pRule->name); 
		return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_ref_ruleDeleteByIndex(dg87_ref* pRef,int index)
{
	dg87_rule* pRule;
	assert(pRef != NULL);
	if (pRef == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pRule = dg87_vector_removeObjectByIndex(pRef->pRuleVector, index);
	if (pRule == NULL) 
		return FALSE;
	dg87_rule_delete(pRule);
	return TRUE;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_ref_check(dg87_ref* pRef, int (*print)(char*))
{
	dg87_element* pElement;
	dg87_item* pItem;
	dg87_taskdep* pTaskDep;

	/* check TaskVector*/
	if (pRef->pTaskVector == NULL){
		print("Ref : TaskVector is null !\n");
		return 1;
	}
	if (dg87_vector_check(pRef->pTaskVector, print) != 0)
		return 1;
	pElement = pRef->pTaskVector->pFirst;
	while (pElement != NULL){
		if (dg87_task_check(pElement->pObject, print) != 0)
			return 1;
		pElement = pElement->pNext;
	}
	
	/* check TaskdepList */
	if (pRef->pTaskdepList == NULL){
		print("Ref : TaskdepList is null !\n");
		return 1;
	}
	if (dg87_list_check(pRef->pTaskdepList, print) != 0)
		return 1;
	pItem = pRef->pTaskdepList->pFirst;
	while (pItem != NULL){
		pTaskDep = pItem->pObject;
		if (dg87_taskdep_check(pTaskDep, print) != 0)
			return 1;
		pItem = pItem->pNext;
	}
	return 0;
}
#endif /*DEBUG*/


/****************************************************************************/
/* End of file */
