/* file rule.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "global.h"
#include "variable.h"
#include "dgerr.h"
#include "rule.h"
#include <libdg87/element.h>
#include <libdg87/connection.h>
#include "global.h"

/****************************************************************************/
dg87_rule* dg87_rule_new(char* pName)
{
	dg87_rule* pRule;
	assert(pName != NULL);
	if (pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}

	/* allocate memory for rule */
	pRule = dg87_malloc(sizeof(dg87_rule), DG87_RULE);
	if (pRule == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* create operation list */
	pRule->pOperationList = dg87_list_new();
	if (pRule->pOperationList == NULL) {
		dg87_free(pRule, DG87_RULE);
		return NULL;
	}	
	/* initialize the structure of the object*/
	strncpy(pRule->name,pName, DG87_RULE_NAME_LENGTH);
	pRule->linkedRules = 0;
	pRule->linkedTasks = 0;
	return pRule;
}
/****************************************************************************/
dg87_rule* dg87_rule_duplicate(dg87_rule* pOriginalRule)
{
	dg87_rule* pNewRule;
	assert(pOriginalRule != NULL);
	if (pOriginalRule == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	/* create a rule with the same name */
	pNewRule = dg87_rule_new(pOriginalRule->name);
	if (pNewRule == NULL) return NULL;
	return pNewRule;
}
/****************************************************************************/
BOOL dg87_rule_delete(dg87_rule* pRule)
{	
	assert(pRule != NULL);
	if (pRule == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* verify that rule isn't used by another rule or by a task */
	if (pRule->linkedRules > 0){
		dg87_err_setError(DG87_ERR_RULEUSEDBYANOTERRULE);
		return FALSE;
	}
	if (pRule->linkedTasks > 0){
		dg87_err_setError(DG87_ERR_RULEUSEDBYTASK);
		return FALSE;
	}
	/* delete operation */
	while (pRule->pOperationList->count > 0){
		if (dg87_rule_operationDelete(pRule, 0) == FALSE)
			return FALSE;
	}
	dg87_list_delete(pRule->pOperationList);

	dg87_free(pRule, DG87_RULE);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_rule_load(dg87_rule* pRule, dg87_storage* pStorage)
{
	return TRUE;
}
/****************************************************************************/
BOOL dg87_rule_save(dg87_rule* pRule, dg87_storage* pStorage)
{
	return TRUE;
}
/****************************************************************************/
/* dg87_operation_dom_calculate
 * This function calculates day_of_month operation.
 */
BOOL dg87_operation_dom_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	time_t limit;
	struct tm dc;
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	
	dc = *localtime(&day);
	if (pOperation->rule.dom.dayOfMonth > 0){
		/* find first day of the month */
		dc.tm_mday = 1;
		limit = mktime(&dc);
		/* calculate limit date */
		limit += (pOperation->rule.dom.dayOfMonth-1)*(24*3600);
	}else{
		/* find first day of next month */
		if (dc.tm_mon == 11){
			/* december : next month is january of next year */
			dc.tm_year += 1;
			dc.tm_mon = 0;
			dc.tm_mday = 1;
		} else {
			dc.tm_mon += 1;
			dc.tm_mday = 1;
		}
		limit = mktime(&dc);
		/* calculate limit date */
		limit += pOperation->rule.dom.dayOfMonth*(24*3600);

	}
	switch (pOperation->oper){
		case dg87_rule_operator_shiftAfter:
		case dg87_rule_operator_shiftBefore:
		case dg87_rule_operator_add:
			if (day >= limit && day < limit+(24*3600) )
				*result = TRUE; 
			break;
		case dg87_rule_operator_addfrom:
			if (day >= limit)
				*result = TRUE; 
			break;
		case dg87_rule_operator_adduntil:
			if (day <= limit)
				*result = TRUE; 
			break;
		case dg87_rule_operator_remove:
			if (day >= limit && day < limit+(24*3600) )
				*result = FALSE; 
			break;
		case dg87_rule_operator_removefrom:
			if (day >= limit)
				*result = FALSE; 
			break;
		case dg87_rule_operator_removeuntil:
			if (day <= limit)
				*result = FALSE; 
			break;
	}
	return TRUE;
}
/****************************************************************************/
/* dg87_operation_dow_calculate
 * This function calculates day_of_week operation.
 */
BOOL dg87_operation_dow_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	struct tm dc;
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	
	dc = *localtime(&day);
	switch (pOperation->oper){
		case dg87_rule_operator_shiftAfter:
		case dg87_rule_operator_shiftBefore:
		case dg87_rule_operator_add:
			if (dc.tm_wday == pOperation->rule.dow.dayOfWeek)
				*result = TRUE; 
			break;
		case dg87_rule_operator_addfrom:
			if (dc.tm_wday >= pOperation->rule.dow.dayOfWeek)
				*result = TRUE; 
			break;
		case dg87_rule_operator_adduntil:
			if (dc.tm_wday <= pOperation->rule.dow.dayOfWeek)
				*result = TRUE; 
			break;
		case dg87_rule_operator_remove:
			if (dc.tm_wday == pOperation->rule.dow.dayOfWeek)
				*result = FALSE; 
			break;
		case dg87_rule_operator_removefrom:
			if (dc.tm_wday >= pOperation->rule.dow.dayOfWeek)
				*result = FALSE; 
			break;
		case dg87_rule_operator_removeuntil:
			if (dc.tm_wday <= pOperation->rule.dow.dayOfWeek)
				*result = FALSE; 
			break;
	}
	return TRUE;
}
/****************************************************************************/
/* dg87_operation_doy_calculate
 * This function calculates day_of_year operation.
 */
BOOL dg87_operation_doy_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	time_t limit;
	struct tm dc;
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	
	dc = *localtime(&day);
	if (pOperation->rule.doy.dayOfYear > 0){
		/* find first day of year */
		dc.tm_mon = 0;
		dc.tm_mday = 1;
		limit = mktime(&dc);
		/* calculate limit date */
		limit += (pOperation->rule.doy.dayOfYear-1)*(24*3600);
	}else{
		/* find first day of next year */
		dc.tm_year += 1;
		dc.tm_mon = 0;
		dc.tm_mday = 1;
		limit = mktime(&dc);
		/* calculate limit date */
		limit += pOperation->rule.doy.dayOfYear*(24*3600);

	}
	switch (pOperation->oper){
		case dg87_rule_operator_shiftAfter:
		case dg87_rule_operator_shiftBefore:
		case dg87_rule_operator_add:
			if (day >= limit && day < limit+(24*3600) )
				*result = TRUE; 
			break;
		case dg87_rule_operator_addfrom:
			if (day >= limit)
				*result = TRUE; 
			break;
		case dg87_rule_operator_adduntil:
			if (day <= limit)
				*result = TRUE; 
			break;
		case dg87_rule_operator_remove:
			if (day >= limit && day < limit+(24*3600) )
				*result = FALSE; 
			break;
		case dg87_rule_operator_removefrom:
			if (day >= limit)
				*result = FALSE; 
			break;
		case dg87_rule_operator_removeuntil:
			if (day <= limit)
				*result = FALSE; 
			break;
	}
	return TRUE;
}
/****************************************************************************/
/* dg87_operation_day_calculate
 * This function calculates day operation (dd/mm).
 */
BOOL dg87_operation_day_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	struct tm dc;
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	
	dc = *localtime(&day);
	switch (pOperation->oper){
		case dg87_rule_operator_shiftAfter:
		case dg87_rule_operator_shiftBefore:
		case dg87_rule_operator_add:
			if (dc.tm_mday == pOperation->rule.day.day &&
				dc.tm_mon == pOperation->rule.day.month)
				*result = TRUE; 
			break;
		case dg87_rule_operator_addfrom:
			if (dc.tm_mon > pOperation->rule.day.month ||
				(dc.tm_mon == pOperation->rule.day.month &&
				dc.tm_mday >= pOperation->rule.day.day))
				*result = TRUE; 
			break;
		case dg87_rule_operator_adduntil:
			if (dc.tm_mon < pOperation->rule.day.month ||
				(dc.tm_mon == pOperation->rule.day.month &&
				dc.tm_mday <= pOperation->rule.day.day)) 
				*result = TRUE; 
			break;
		case dg87_rule_operator_remove:
			if (dc.tm_mday == pOperation->rule.day.day &&
				dc.tm_mon == pOperation->rule.day.month)
				*result = FALSE; 
			break;
		case dg87_rule_operator_removefrom:
			if (dc.tm_mon > pOperation->rule.day.month ||
				(dc.tm_mon == pOperation->rule.day.month &&
				dc.tm_mday >= pOperation->rule.day.day)) 
				*result = FALSE; 
			break;
		case dg87_rule_operator_removeuntil:
			if (dc.tm_mon < pOperation->rule.day.month ||
				(dc.tm_mon == pOperation->rule.day.month &&
				dc.tm_mday <= pOperation->rule.day.day)) 
				*result = FALSE; 
			break;
	}
	return TRUE;
}

/****************************************************************************/
/* dg87_operation_date_calculate
 * This function calculates date operation (yyyy-mm-dd).
 */
BOOL dg87_operation_date_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	struct tm dc;
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	
	dc = *localtime(&day);
	switch (pOperation->oper){
		case dg87_rule_operator_shiftAfter:
		case dg87_rule_operator_shiftBefore:
		case dg87_rule_operator_add:
			if (day == pOperation->rule.date.date)
				*result = TRUE; 
			break;
		case dg87_rule_operator_addfrom:
			if (day >= pOperation->rule.date.date)
				*result = TRUE; 
			break;
		case dg87_rule_operator_adduntil:
			if (day <= pOperation->rule.date.date)
				*result = TRUE; 
			break;
		case dg87_rule_operator_remove:
			if (day == pOperation->rule.date.date)
				*result = FALSE; 
			break;
		case dg87_rule_operator_removefrom:
			if (day >= pOperation->rule.date.date)
				*result = FALSE; 
			break;
		case dg87_rule_operator_removeuntil:
			if (day <= pOperation->rule.date.date)
				*result = FALSE; 
			break;
	}
	return TRUE;
}

/****************************************************************************/
/* dg87_operation_always_calculate
 * This function calculates always operation
 */
BOOL dg87_operation_always_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pOperation->oper != dg87_rule_operator_add){
		dg87_err_setError(DG87_ERR_RULEWRONGOPERATORALWAYS );
		return FALSE;
	}
	*result = TRUE;
	return TRUE;
}

/****************************************************************************/
/* dg87_operation_complex_calculate
 * This function calculates complex operation
 */
BOOL dg87_operation_complex_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	BOOL res;
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_rule_calculate(pOperation->rule.pRule, day,&res) == FALSE)
		return FALSE;

	switch (pOperation->oper){

		case dg87_rule_operator_shiftAfter:
		case dg87_rule_operator_shiftBefore:
		case dg87_rule_operator_add:
			if (res == TRUE) *result = TRUE; 
			break;

		case dg87_rule_operator_remove:
			if (res == TRUE) *result = FALSE; 
			break;

		case dg87_rule_operator_adduntil:
		case dg87_rule_operator_addfrom:
		case dg87_rule_operator_removefrom:
		case dg87_rule_operator_removeuntil:
			dg87_err_setError(DG87_ERR_RULEWRONGOPERATORCOMPLEX);
			return FALSE;
			break;

	}
	return TRUE;
}
/****************************************************************************/
/* dg87_operation_ifequal_calculate
 * This function calculates ifequal operation
 */
BOOL dg87_operation_ifequal_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	dg87_variable* pVariable;
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pVariable = pOperation->rule.ifequal.pVariable;
	if (strcmp(pOperation->rule.ifequal.pVariable->pValue,
				pOperation->rule.ifequal.pValue) == 0){
		if (pOperation->oper == dg87_rule_operator_add)
			*result = TRUE;
		else if (pOperation->oper == dg87_rule_operator_remove)
			*result = FALSE;

	}
	return TRUE;
}
/****************************************************************************/
/* dg87_operation_ifnequal_calculate
 * This function calculates ifnequal operation
 */
BOOL dg87_operation_ifnequal_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	dg87_variable* pVariable;
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pVariable = pOperation->rule.ifequal.pVariable;
	if (strcmp(pOperation->rule.ifequal.pVariable->pValue,
				pOperation->rule.ifequal.pValue) != 0){
		if (pOperation->oper == dg87_rule_operator_add)
			*result = TRUE;
		else if (pOperation->oper == dg87_rule_operator_remove)
			*result = FALSE;

	}
	return TRUE;
}
/****************************************************************************/
/* dg87_operation_moy_calculate
 * This function calculates month_of_year operation
 */
BOOL dg87_operation_moy_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	struct tm dc;
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	
	dc = *localtime(&day);
	switch (pOperation->oper){
		case dg87_rule_operator_shiftAfter:
		case dg87_rule_operator_shiftBefore:
		case dg87_rule_operator_add:
			if (dc.tm_mon == pOperation->rule.moy.monthOfYear)
				*result = TRUE; 
			break;
		case dg87_rule_operator_addfrom:
			if (dc.tm_mon >= pOperation->rule.moy.monthOfYear)
				*result = TRUE; 
			break;
		case dg87_rule_operator_adduntil:
			if (dc.tm_mon <= pOperation->rule.moy.monthOfYear)
				*result = TRUE; 
			break;
		case dg87_rule_operator_remove:
			if (dc.tm_mon == pOperation->rule.moy.monthOfYear)
				*result = FALSE; 
			break;
		case dg87_rule_operator_removefrom:
			if (dc.tm_mon >= pOperation->rule.moy.monthOfYear)
				*result = FALSE; 
			break;
		case dg87_rule_operator_removeuntil:
			if (dc.tm_mon <= pOperation->rule.moy.monthOfYear)
				*result = FALSE; 
			break;
	}
	return TRUE;
}

/****************************************************************************/
/* dg87_operation_wom_calculate
 * This function calculates week_of_month operation
 */
BOOL dg87_operation_wom_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	struct tm dc;
	time_t firstDay;
	time_t fdow; /* first day of week */
	time_t ldow; /* last day of week */
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	
	dc = *localtime(&day);
	if (pOperation->rule.wom.weekOfMonth > 0){
		/* find first day of month */
		dc.tm_mday = 1;
		firstDay = mktime(&dc);
		/* come back to begin of week */
		dc =*localtime(&firstDay);
		firstDay -= dc.tm_wday * 24 * 3600;
		/* calculate first and last day of week */
		fdow = firstDay + 7*24*3600 
			* (pOperation->rule.wom.weekOfMonth -1);
		ldow = fdow + 7 * 24 * 3600;
	} else {
		/* find last day of month */
		dc.tm_mon++;
		dc.tm_mday = 1;
		firstDay = mktime(&dc) - 24 * 3600;
		/* go to first day of next week */
		dc =*localtime(&firstDay);
		firstDay += (7 - dc.tm_wday ) * 24 * 3600;
		/* calculate first and last day of week */
		fdow = firstDay + 7*24*3600 * pOperation->rule.wom.weekOfMonth;
		ldow = fdow + 7 * 24 * 3600;
	}
	switch (pOperation->oper){
		case dg87_rule_operator_shiftAfter:
		case dg87_rule_operator_shiftBefore:
		case dg87_rule_operator_add:
			if (day >= fdow && day < ldow )
				*result = TRUE; 
			break;
		case dg87_rule_operator_addfrom:
			if (day >= fdow)
				*result = TRUE;
			break;
		case dg87_rule_operator_adduntil:
			if (day < ldow)
				*result = TRUE;
			break;
		case dg87_rule_operator_remove:
			if (day >= fdow && day < ldow )
				*result = FALSE; 
			break;
		case dg87_rule_operator_removefrom:
			if (day >= fdow)
				*result = FALSE;
			break;
		case dg87_rule_operator_removeuntil:
			if (day < ldow)
				*result = FALSE;
			break;
	}
	return TRUE;
}
/****************************************************************************/
/* dg87_operation_woy_calculate
 * This function calculates week_of_year operation
 */
BOOL dg87_operation_woy_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	struct tm dc;
	time_t firstDay;
	time_t fdow; /* first day of week */
	time_t ldow; /* last day of week */
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	
	dc = *localtime(&day);
	if (pOperation->rule.woy.weekOfYear > 0){
		/* find first day of year */
		dc.tm_mon = 0;
		dc.tm_mday = 1;
		firstDay = mktime(&dc);
		/* come back to begin of week */
		dc =*localtime(&firstDay);
		firstDay -= dc.tm_wday * 24 * 3600;
		/* calculate first and last day of week */
		fdow = firstDay + 7*24*3600 
			* (pOperation->rule.woy.weekOfYear -1);
		ldow = fdow + 7 * 24 * 3600;
	} else {
		/* find last day of year */
		dc.tm_mon = 0;
		dc.tm_mday = 1;
		dc.tm_year++;
		firstDay = mktime(&dc) - 24 * 3600;
		/* go to first day of next week */
		dc =*localtime(&firstDay);
		firstDay += (7 - dc.tm_wday ) * 24 * 3600;
		/* calculate first and last day of week */
		fdow = firstDay + 7*24*3600 * pOperation->rule.woy.weekOfYear;
		ldow = fdow + 7 * 24 * 3600;
	}
	switch (pOperation->oper){
		case dg87_rule_operator_shiftAfter:
		case dg87_rule_operator_shiftBefore:
		case dg87_rule_operator_add:
			if (day >= fdow && day < ldow )
				*result = TRUE; 
			break;
		case dg87_rule_operator_addfrom:
			if (day >= fdow)
				*result = TRUE;
			break;
		case dg87_rule_operator_adduntil:
			if (day < ldow)
				*result = TRUE;
			break;
		case dg87_rule_operator_remove:
			if (day >= fdow && day < ldow )
				*result = FALSE; 
			break;
		case dg87_rule_operator_removefrom:
			if (day >= fdow)
				*result = FALSE;
			break;
		case dg87_rule_operator_removeuntil:
			if (day < ldow)
				*result = FALSE;
			break;
	}
	return TRUE;
}

/****************************************************************************/
/* 
 * dg87_operation_calculate
 * This function set to true the value of the parameter 'result' if the day 
 * given in parameter is selected by the operation.
 * If the day isn't selected, the value of *result is not changed.
 * Operation witch operator is shiftbefore of shiftafter are not calculated
 * by this function.
 * The function returns FALSE if an error occurres. 
 */
static BOOL dg87_operation_calculate(dg87_rule_operation* pOperation, 
		time_t day, BOOL *result)
{
	assert(pOperation != NULL && result != NULL);
	if (pOperation == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	

	switch(pOperation->type){
		case dg87_rule_type_dow: /* day of week */
			if (dg87_operation_dow_calculate(pOperation, 
					day,result) == FALSE)
				return FALSE;
			break;
		case dg87_rule_type_dom: /* day of month */
			if (dg87_operation_dom_calculate(pOperation, 
					day,result) == FALSE)
				return FALSE;
			break;
		case dg87_rule_type_doy: /* day of year */
			if (dg87_operation_doy_calculate(pOperation, 
					day,result) == FALSE)
				return FALSE;
			break;
		case dg87_rule_type_wom: /* week of month */
			if (dg87_operation_wom_calculate(pOperation, 
					day,result) == FALSE)
				return FALSE;
			break;

		case dg87_rule_type_woy: /* week of year */
			if (dg87_operation_woy_calculate(pOperation, 
					day,result) == FALSE)
				return FALSE;
			break;
		case dg87_rule_type_moy: /* month of year */
			if (dg87_operation_moy_calculate(pOperation, 
					day,result) == FALSE)
				return FALSE;
			break;
		case dg87_rule_type_day: /* date dd/mm */
			if (dg87_operation_day_calculate(pOperation, 
					day,result) == FALSE)
				return FALSE;
			break;
		case dg87_rule_type_date: /* date dd/mm/yyyy */
			if (dg87_operation_date_calculate(pOperation, 
					day,result) == FALSE)
				return FALSE;
			break;
		case dg87_rule_type_always: /* all days */
			if (dg87_operation_always_calculate(pOperation, 
					day,result) == FALSE)
				return FALSE;
			break;
		case dg87_rule_type_complex:/* composed rule */
			if (dg87_operation_complex_calculate(
					pOperation,day,result) == FALSE)
				return FALSE;
			break;
		case dg87_rule_type_ifequal: /* if variable equal value */
			if (dg87_operation_ifequal_calculate(
					pOperation,day,result) == FALSE)
				return FALSE;
			break;
		case dg87_rule_type_ifnequal: /* if variable not equal value */
			if (dg87_operation_ifnequal_calculate(
					pOperation,day,result) == FALSE)
				return FALSE;
			break;

	}

	return TRUE;
}
/****************************************************************************/
BOOL dg87_operationlist_calculate(dg87_list* pOperationList, 
		time_t day, BOOL *result, int nOperations)
{
	int iOperation;
	int delta, limit;
	time_t day_delta;
	dg87_item* pItem;
	dg87_rule_operation* pOperation;
	BOOL res;
	assert(pOperationList != NULL && result != NULL);
	if (pOperationList == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	
	if (nOperations < -1){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return FALSE;
	}
	if (nOperations == -1)
		nOperations = pOperationList->count;
	*result = FALSE;

	/* for each operation of the rule */
	iOperation = 0;
	pItem = pOperationList->pFirst;
	while (pItem != NULL && iOperation < nOperations){
		pOperation = pItem->pObject;
		pItem = pItem->pNext;
		iOperation++;

		/* optimization : nothing to do if
		 * - operator is add and day is already selected
		 * - operator is remove and day is already unselected */
		/* TODO move optimization in dg87_operation_calculate ? */
		/* TODO optimization with shift_after and shif_before */
		if (*result == TRUE && (
			pOperation->oper == dg87_rule_operator_add ||
			pOperation->oper == dg87_rule_operator_adduntil ||
			pOperation->oper == dg87_rule_operator_addfrom ))
			continue;
		if (*result == FALSE && (
			pOperation->oper == dg87_rule_operator_remove ||
			pOperation->oper == dg87_rule_operator_removeuntil ||
			pOperation->oper == dg87_rule_operator_removefrom ))
			continue;

		/* for operator different from shiftafter or shiftbefore */
		if (pOperation->oper != dg87_rule_operator_shiftBefore &&
				pOperation->oper!=dg87_rule_operator_shiftAfter)
		{
			if (dg87_operation_calculate(pOperation, day, result) 
					== FALSE){
				return FALSE;
			}
			continue;
		}

		/* 
		 * here, we are sure that operator is shiftAfter or shiftBefore.
		 *
		 * - if the day is selected by the shift operation, then
		 *   the day is unselected. (shift)
		 *
		 * - if day is just outside of a range of days which are 
		 *   selected by the shift operation, then day is selected
		 *   if one (or more) days of the range should be selected
		 *   by previous operations in the list.
		 *
		 * - else no change
		 */

		/* 
		 * si day dans la plage shift 
		 * 	le deselectionner 
		 * 	operation suivante
		 * fin si
		 */
		res = FALSE;
		/* TODO changer l'operateur pour dg87_operator_add ? */
		if (dg87_operation_calculate(pOperation, day, &res) 
				== FALSE){
			/* TODO erreur calcul : sortir */
			return FALSE;
		}
		/* if day is selected by the operation, */
		if (res == TRUE){
			*result = FALSE;
			/* next operation */
			continue;
		}

		if (pOperation->oper == dg87_rule_operator_shiftBefore)
			delta = 24*3600; /* shift before <<< */
		else
			delta = -24*3600; /* shift after   >>> */

		/* si jour_teste+delta n'est pas dans plage shift alors
		 * 	operation suivante
		 * fin si
		 */

		/* the day is just at the bound of range of days which are 
		 * selected by the shift operation. 
		 * If one or more days in this range are 
		 * selected by previous operations of the list, then
		 * the day must be selected.
		 * (selected days by previous operations are shifted at the
		 * bound of the range)
		 */
		limit = 0;
		day_delta = day;
		while (limit < 365){
			/* next day in the range of selected days */
			day_delta += delta;
			limit++;

			/* break if outside of range of selected days */
			/* TODO change operator for operator_add ? */
			res = FALSE;
			if (dg87_operation_calculate(pOperation, 
						day_delta, &res) == FALSE){
				return FALSE;
			}
			if (res == FALSE){
				break;
			}

			res = FALSE;
			
			if (dg87_operationlist_calculate(pOperationList, 
				day_delta, &res, nOperations -1) == FALSE){
				return FALSE;
			}

			if (res == TRUE){
				*result = TRUE;
				continue;
			}
		}

	} /* next operation */
	return TRUE;

}
/****************************************************************************/
BOOL dg87_rule_calculate(dg87_rule* pRule, time_t day, BOOL *result)
{	
	assert(pRule != NULL && result != NULL);
	if (pRule == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	
	return dg87_operationlist_calculate(pRule->pOperationList, 
			day, result, -1);
}
/****************************************************************************/
/*
 * dg87_rule_useRule
 * This function is recursive. The function return TRUE if the searchRule
 * is used by the rule.
 */
BOOL dg87_rule_useRule(dg87_rule* pRule, dg87_rule* pSearchRule)
{
	dg87_item* pItem;
	dg87_rule_operation* pOperation;
	assert(pRule != NULL && pSearchRule != NULL);
	if (pRule == NULL || pSearchRule == NULL)
		return FALSE;
	if (pRule == pSearchRule)
		return TRUE;
	/* for each operation in the rule */
	pItem = pRule->pOperationList->pFirst;
	while (pItem != NULL){
		pOperation = pItem->pObject;
		/* if operation uses another rule */
		if (pOperation->type == dg87_rule_type_complex){
			/* determine if this rule uses the search rule */
			if (dg87_rule_useRule(pOperation->rule.pRule, 
					pSearchRule) == TRUE)
				return TRUE;
		}
		pItem = pItem->pNext;
	}
	return FALSE;
}
/****************************************************************************/
BOOL dg87_rule_operationAdd(dg87_rule* pRule, dg87_rule_operation* pOperation)
{
	assert(pRule != NULL && pOperation != NULL);
	if (pRule == NULL || pOperation == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pOperation->type == dg87_rule_type_complex){
		/* avoid circular reference */
		if (dg87_rule_useRule(pOperation->rule.pRule, pRule) == TRUE){
			dg87_err_setError(DG87_ERR_CIRCULARRULEREFERENCE);
			return FALSE;
		}
	}
	/* add the operation to the list */
	if (dg87_list_addObject(pRule->pOperationList, pOperation) == FALSE){
		return FALSE;
	}
	/* increment linkedRules counter for complex rule */
	if (pOperation->type == dg87_rule_type_complex)
		pOperation->rule.pRule->linkedRules++;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_rule_operationDelete(dg87_rule* pRule,int index)
{
	dg87_rule_operation* pOperation;
	assert(pRule != NULL);
	if (pRule == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* remove the operation from the list */
	pOperation = dg87_list_removeObjectByIndex(pRule->pOperationList, 
			index);
	if (pOperation == NULL) 
		return FALSE;
	/* if complex rule, decrement linkedRules counter */
	if (pOperation->type == dg87_rule_type_complex)
		pOperation->rule.pRule->linkedRules--;
	/* delete operation object */
	if (dg87_operation_delete(pOperation) == FALSE){
		dg87_list_addObject(pRule->pOperationList, pOperation);
		if (pOperation->type == dg87_rule_type_complex)
			pOperation->rule.pRule->linkedRules++;
		return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_rule_operationMove(dg87_rule* pRule,int index, int delta)
{
	assert(pRule != NULL);
	if (pRule == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	return dg87_list_moveItem(pRule->pOperationList, index, delta);

}
/****************************************************************************/
dg87_rule_operation* dg87_operation_dayofweek_new(
		dg87_rule_operator oper, int day)
{
	dg87_rule_operation *pOperation;
	if (day < 0 || day > 6){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pOperation->oper = oper;
	pOperation->type = dg87_rule_type_dow; 
	pOperation->rule.dow.dayOfWeek = day;
	return pOperation;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_dayofmonth_new(
		dg87_rule_operator oper, int day)
{
	dg87_rule_operation *pOperation;
	if (day < -31 || day > 31 || day == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pOperation->oper = oper;
	pOperation->type = dg87_rule_type_dom; 
	pOperation->rule.dom.dayOfMonth = day;
	return pOperation;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_dayofyear_new(
		dg87_rule_operator oper, int day)
{
	dg87_rule_operation *pOperation;
	if (day < -366 || day == 0 || day > 366 ){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pOperation->oper = oper;
	pOperation->type = dg87_rule_type_doy; 
	pOperation->rule.doy.dayOfYear = day;
	return pOperation;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_weekofmonth_new(
		dg87_rule_operator oper, int week)
{
	dg87_rule_operation *pOperation;
	if (week < -5 || week == 0 || week > 5 ){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pOperation->oper = oper;
	pOperation->type = dg87_rule_type_wom; 
	pOperation->rule.wom.weekOfMonth = week;
	return pOperation;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_weekofyear_new(
		dg87_rule_operator oper, int week)
{
	dg87_rule_operation *pOperation;
	if (week < -53 || week == 0 || week > 53 ){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pOperation->oper = oper;
	pOperation->type = dg87_rule_type_woy; 
	pOperation->rule.woy.weekOfYear = week;
	return pOperation;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_monthofyear_new(
		dg87_rule_operator oper, int month)
{
	dg87_rule_operation *pOperation;
	if (month < 0 || month > 11 ){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pOperation->oper = oper;
	pOperation->type = dg87_rule_type_moy; 
	pOperation->rule.moy.monthOfYear = month;
	return pOperation;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_day_new(
		dg87_rule_operator oper, int month, int day)
{
	dg87_rule_operation *pOperation;
	if (day < 1 || day > 31 || month < 0 || month > 11 ){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pOperation->oper = oper;
	pOperation->type = dg87_rule_type_day; 
	pOperation->rule.day.day = day;
	pOperation->rule.day.month = month;
	return pOperation;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_date_new(
		dg87_rule_operator oper, time_t date)
{
	dg87_rule_operation *pOperation;
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pOperation->oper = oper;
	pOperation->type = dg87_rule_type_date; 
	pOperation->rule.date.date = date;
	return pOperation;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_always_new(void)
{
	dg87_rule_operation *pOperation;
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pOperation->oper = dg87_rule_operator_add;
	pOperation->type = dg87_rule_type_always; 
	return pOperation;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_complex_new(
		dg87_rule_operator oper, dg87_rule* pRule)
{
	dg87_rule_operation *pOperation;
	if (pRule == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pOperation->oper = oper;
	pOperation->type = dg87_rule_type_complex; 
	pOperation->rule.pRule = pRule;
	return pOperation;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_ifequal_new(
		dg87_rule_operator oper, 
		struct dg87_variable_struct* pVariable, char* pValue)
{
	dg87_rule_operation *pOperation;
	char* pVariableValue;
	assert(pVariable != NULL && pValue != NULL);
	if (pVariable== NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pVariableValue = dg87_strdup(pValue, DG87_CHAR);
	if (pVariableValue == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pOperation, DG87_OPERATION);
		return NULL;
	}
	pOperation->oper = oper;
	pOperation->type = dg87_rule_type_ifequal; 
	pOperation->rule.ifequal.pVariable = pVariable;
	pOperation->rule.ifequal.pValue = pVariableValue;
	/* increment variable use counter */
	pVariable->linkedRules++;
	return pOperation;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_ifnequal_new(
		dg87_rule_operator oper, 
		struct dg87_variable_struct* pVariable, char* pValue)
{
	dg87_rule_operation *pOperation;
	char* pVariableValue;
	assert(pVariable != NULL && pValue != NULL);
	if (pVariable== NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pOperation = dg87_malloc(sizeof(dg87_rule_operation), DG87_OPERATION);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pVariableValue = dg87_strdup(pValue, DG87_CHAR);
	if (pVariableValue == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pOperation, DG87_OPERATION);
		return NULL;
	}
	pOperation->oper = oper;
	pOperation->type = dg87_rule_type_ifnequal; 
	pOperation->rule.ifequal.pVariable = pVariable;
	pOperation->rule.ifequal.pValue = pVariableValue;
	/* increment variable use counter */
	pVariable->linkedRules++;
	return pOperation;
}
/****************************************************************************/
BOOL dg87_operation_delete(dg87_rule_operation* pOperation)
{
	struct dg87_variable_struct *pVariable;
	assert(pOperation != NULL);
	if (pOperation == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* variable decrement counter */
	if (pOperation->type == dg87_rule_type_ifequal){
		pVariable = pOperation->rule.ifequal.pVariable;
		pVariable->linkedRules--;
		dg87_free(pOperation->rule.ifequal.pValue, DG87_CHAR);
	}
	if (pOperation->type == dg87_rule_type_ifnequal){
		pVariable = pOperation->rule.ifnequal.pVariable;
		pVariable->linkedRules--;
		dg87_free(pOperation->rule.ifnequal.pValue, DG87_CHAR);
	}
	dg87_free(pOperation, DG87_OPERATION);
	return TRUE;
}
/****************************************************************************/
dg87_rule_operation* dg87_operation_duplicate(
		dg87_rule_operation* pOriginalOperation, 
		dg87_vector *pNewRuleVector)	
{	
	dg87_rule_operation* pNewOperation;
	dg87_rule *pRule;
	pNewOperation = NULL;
	assert(pOriginalOperation != NULL && pNewRuleVector != NULL);
	switch(pOriginalOperation->type)
	{
		case dg87_rule_type_dow:
			pNewOperation = dg87_operation_dayofweek_new(
					pOriginalOperation->oper, 
					pOriginalOperation->rule.dow.dayOfWeek);
			break;
		case dg87_rule_type_dom:
			pNewOperation = dg87_operation_dayofmonth_new(
					pOriginalOperation->oper, 
				pOriginalOperation->rule.dom.dayOfMonth);
			break;
		case dg87_rule_type_doy:
			pNewOperation = dg87_operation_dayofyear_new(
					pOriginalOperation->oper, 
					pOriginalOperation->rule.doy.dayOfYear);
			break;
		case dg87_rule_type_wom:
			pNewOperation = dg87_operation_weekofmonth_new(
					pOriginalOperation->oper, 
				pOriginalOperation->rule.wom.weekOfMonth);
			break;
		case dg87_rule_type_woy:
			pNewOperation = dg87_operation_weekofyear_new(
					pOriginalOperation->oper, 
				pOriginalOperation->rule.woy.weekOfYear);
			break;
		case dg87_rule_type_moy:
			pNewOperation = dg87_operation_monthofyear_new(
					pOriginalOperation->oper, 
				pOriginalOperation->rule.moy.monthOfYear);
			break;
		case dg87_rule_type_day:
			pNewOperation = dg87_operation_day_new(
					pOriginalOperation->oper, 
					pOriginalOperation->rule.day.month,
					pOriginalOperation->rule.day.day);
			break;
		case dg87_rule_type_date:
			pNewOperation = dg87_operation_date_new(
					pOriginalOperation->oper, 
					pOriginalOperation->rule.date.date);
			break;
		case dg87_rule_type_always:
			pNewOperation = dg87_operation_always_new();
			break;
		case dg87_rule_type_complex:
			/* find the rule in the vector of new rules */
			pRule = dg87_vector_getObjectByKey(pNewRuleVector,
					pOriginalOperation->rule.pRule->name);
			if (pRule == NULL)
				return NULL;
			pNewOperation = dg87_operation_complex_new(
					pOriginalOperation->oper, 
					pRule);
			break;
		case dg87_rule_type_ifequal:
			pNewOperation = dg87_operation_ifequal_new(
				pOriginalOperation->oper,
				pOriginalOperation->rule.ifequal.pVariable,
				pOriginalOperation->rule.ifequal.pValue);
			break;
		case dg87_rule_type_ifnequal:
			pNewOperation = dg87_operation_ifnequal_new(
				pOriginalOperation->oper,
				pOriginalOperation->rule.ifnequal.pVariable,
				pOriginalOperation->rule.ifnequal.pValue);
			break;

	}
	return pNewOperation;
}
/****************************************************************************/
dg87_rule_type dg87_rule_asciiToType(char* pRuleType)
{
	assert(pRuleType != NULL);
	if (pRuleType == NULL) return -1; 

	if ( strcasecmp(pRuleType, "day_of_week") == 0)
		return dg87_rule_type_dow;
	if ( strcasecmp(pRuleType, "day_of_month") == 0)
		return dg87_rule_type_dom;
	if ( strcasecmp(pRuleType, "day_of_year") == 0)
		return dg87_rule_type_doy;
	if ( strcasecmp(pRuleType, "week_of_month") == 0)
		return dg87_rule_type_wom;
	if ( strcasecmp(pRuleType, "week_of_year") == 0)
		return dg87_rule_type_woy;
	if ( strcasecmp(pRuleType, "month_of_year") == 0)
		return dg87_rule_type_moy;
	if ( strcasecmp(pRuleType, "day") == 0)
		return dg87_rule_type_day;
	if ( strcasecmp(pRuleType, "date") == 0)
		return dg87_rule_type_date;
	if ( strcasecmp(pRuleType, "always") == 0)
		return dg87_rule_type_always;
	if ( strcasecmp(pRuleType, "rule") == 0)
		return dg87_rule_type_complex;
	if ( strcasecmp(pRuleType, "if_equal") == 0)
		return dg87_rule_type_ifequal;
	if ( strcasecmp(pRuleType, "if_not_equal") == 0)
		return dg87_rule_type_ifnequal;
		
	return -1;
}
/****************************************************************************/
char* dg87_rule_typeToAscii(dg87_rule_type ruleType)
{
	switch(ruleType)
	{
		case dg87_rule_type_dow: return "day_of_week";
		case dg87_rule_type_dom: return "day_of_month";
		case dg87_rule_type_doy: return "day_of_year";
		case dg87_rule_type_wom: return "week_of_month";
		case dg87_rule_type_woy: return "week_of_year";
		case dg87_rule_type_moy: return "month_of_year";
		case dg87_rule_type_day: return "day";
		case dg87_rule_type_date: return "date";
		case dg87_rule_type_always: return "always";
		case dg87_rule_type_complex: return "rule";
		case dg87_rule_type_ifequal: return "if_equal";
		case dg87_rule_type_ifnequal: return "if_not_equal";
	}
	return "error";
}
/****************************************************************************/
dg87_rule_operator dg87_rule_asciiToOperator(char* pOperator)
{
	assert(pOperator != NULL);
	if (pOperator == NULL) return -1;

	if ( strcasecmp(pOperator , "add") == 0)
		return dg87_rule_operator_add;
	if ( strcasecmp(pOperator , "adduntil") == 0)
		return dg87_rule_operator_adduntil;
	if ( strcasecmp(pOperator , "addfrom") == 0)
		return dg87_rule_operator_addfrom;
	if ( strcasecmp(pOperator , "remove") == 0)
		return dg87_rule_operator_remove;
	if ( strcasecmp(pOperator , "removeuntil") == 0)
		return dg87_rule_operator_removeuntil;
	if ( strcasecmp(pOperator , "removefrom") == 0)
		return dg87_rule_operator_removefrom;
	if ( strcasecmp(pOperator , "shift_after") == 0)
		return dg87_rule_operator_shiftAfter;
	if ( strcasecmp(pOperator , "shift_before") == 0)
		return dg87_rule_operator_shiftBefore;
	return -1;
}
/****************************************************************************/
char* dg87_rule_operatorToAscii(dg87_rule_operator oper)
{
	switch(oper)
	{
		case dg87_rule_operator_add: return "add";
		case dg87_rule_operator_adduntil: return "adduntil";
		case dg87_rule_operator_addfrom: return "addfrom";
		case dg87_rule_operator_remove: return "remove";
		case dg87_rule_operator_removeuntil: return "removeuntil";
		case dg87_rule_operator_removefrom: return "removefrom";
		case dg87_rule_operator_shiftAfter: return "shift_after";
		case dg87_rule_operator_shiftBefore: return "shift_before";
	}
	return "error";
}
/******************************************************************************/
BOOL dg87_rule_asciiToDay(char* pString, int *month, int *day)
{
	char buffer[6];
	assert(pString != NULL && month != NULL);
	if (pString == NULL || strlen(pString) != 5)
		return FALSE;
	if (pString[2] != '-')
		return FALSE;
	strncpy(buffer, pString,5);
	buffer[2] = '\0';
	*month = atoi(pString) -1;
	*day = atoi(pString+3);
	if (*month < 0 || *month > 11 || *day < 1 || *day > 31)
		return FALSE;
	return TRUE;
}
/******************************************************************************/
char* dg87_rule_dayToAscii(int month, int day)
{
	char* pString = dg87_malloc(sizeof(char) * 6, DG87_CHAR);
	if (pString == NULL) return NULL;
	sprintf(pString, "%02i-%02i", month+1, day);
	return pString;

	return FALSE;
}
/******************************************************************************/
time_t dg87_rule_asciiToDate(char* pString)
{
	time_t td;
	struct tm dc;
	int year, month, day;
	char buffer[11];
	assert(pString != NULL);
	if (pString == NULL || strlen(pString) != 10)
		return -1;
	if (pString[4] != '-' || pString[7] != '-')
		return -1;
	strncpy(buffer, pString,10);
	buffer[4] = '\0'; buffer[7] = '\0'; 
	
	year = atoi(buffer); month = atoi(buffer+5); day = atoi(buffer+8); 

	if (year < 1900 || month < 1 || month > 12 || day < 1 || day > 31){
		return -1;
	}
	dc.tm_year = year - 1900;
	dc.tm_mon = month - 1;
	dc.tm_mday = day;
	dc.tm_hour = 0;
	dc.tm_min = 0;
	dc.tm_sec = 0;
	dc.tm_isdst = 0; /* don't change tm_isdst */
	td = mktime(&dc);
	return td;
}
/******************************************************************************/
char* dg87_rule_dateToAscii(time_t time)
{
	struct tm dc;
	char format[] = "%Y-%m-%d";
	char* pString = dg87_malloc(sizeof(char) * 11, DG87_CHAR);
	if (pString == NULL) return NULL;
	dc = *localtime(&time);
	strftime(pString,11, format,&dc);
	return pString;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_rule_check(dg87_rule* pRule, int (*print)(char*))
{
	return 0;
}
#endif /*DEBUG*/


/****************************************************************************/
/* End of file */
