/* file rule.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef RULE_H 
#define RULE_H 1

#include "libdg87/vector.h"
#include "libdg87/list.h"
#include "libdg87/storage.h"
#include <time.h>

 
typedef enum {
	dg87_rule_type_dow, /* day of week */
	dg87_rule_type_dom, /* day of month */
	dg87_rule_type_doy, /* day of year */
	dg87_rule_type_wom, /* week of month */
	dg87_rule_type_woy, /* week of year */
	dg87_rule_type_moy, /* month of year */
	dg87_rule_type_day, /* date mm-dd */
	dg87_rule_type_date, /* date dd/mm/yyyy */
	dg87_rule_type_always, /* all days */
	dg87_rule_type_complex, /* composed rule */
	dg87_rule_type_ifequal, /* if variable equal value */
	dg87_rule_type_ifnequal  /* if variable equal value */
} dg87_rule_type;

typedef struct dg87_rule_struct dg87_rule;

/* rule to select a day of week (1-7) */
typedef struct {
	int dayOfWeek;
} dg87_rule_dow;

/* rule to select a day of month (1 to 31 and -31 to -1) */
typedef struct {
	int dayOfMonth;
} dg87_rule_dom;

/* rule to select a day of year (1-365) */
typedef struct {
	int dayOfYear;
} dg87_rule_doy;

/* rule to select a week of month (1-4) */
typedef struct {
	int weekOfMonth;
} dg87_rule_wom;

/* rule to select a week of year (1-52)*/
typedef struct {
	int weekOfYear;
} dg87_rule_woy;

/* rule to select a month of year (1-12) */
typedef struct {
	int monthOfYear;
} dg87_rule_moy;

/* rule to select a day of the year (dd/mm) */
typedef struct {
	int day;
	int month;
} dg87_rule_day;

/* rule to select a date */
typedef struct {
	time_t date;
} dg87_rule_date;


/* rule always */
typedef struct {
	/* empty */
	int dummy;
} dg87_rule_always;

/* rule ifequal */
typedef struct {
	struct dg87_variable_struct* pVariable;
	char* pValue;
} dg87_rule_ifequal;

/* rule ifnequal (if not equal) */
typedef struct {
	struct dg87_variable_struct* pVariable;
	char* pValue;
} dg87_rule_ifnequal;


/* generic rule */
typedef union {
	dg87_rule_dow dow;          /* day of week */
	dg87_rule_dom dom;          /* day of month */
	dg87_rule_doy doy;          /* day of year */
	dg87_rule_wom wom;          /* week of month */
	dg87_rule_woy woy;          /* week of year */
	dg87_rule_moy moy;          /* month of year */
	dg87_rule_day day;          /* day of year dd/mm */
	dg87_rule_date date;        /* date dd/mm/yyyy */
	dg87_rule_always always;    /* all days */
	dg87_rule* pRule;           /* composed rule */
	dg87_rule_ifequal  ifequal;  /* if variable equal value */
	dg87_rule_ifnequal ifnequal; /* if variable equal value */
} dg87_genericRule;

/* enum operator */
typedef enum {
	dg87_rule_operator_add,
	dg87_rule_operator_adduntil,
	dg87_rule_operator_addfrom,
	dg87_rule_operator_remove,
	dg87_rule_operator_removeuntil,
	dg87_rule_operator_removefrom,
	dg87_rule_operator_shiftAfter,
	dg87_rule_operator_shiftBefore
} dg87_rule_operator;

/* struct operation */
typedef struct {
	dg87_rule_operator oper;
	dg87_rule_type type;
	dg87_genericRule rule;
} dg87_rule_operation; 


/* struct rule */
struct dg87_rule_struct{
	char name[DG87_RULE_NAME_LENGTH];
	dg87_list* pOperationList;
	dg87uint linkedRules;
	dg87uint linkedTasks;
};

/*
 * dg87_rule_new
 * create a new rule and return a pointer on this.
 */
dg87_rule* dg87_rule_new(char* name);

/*
 * dg87_rule_delete
 */
BOOL dg87_rule_delete(dg87_rule* pRule);

/*
 * dg87_rule_load
 */
BOOL dg87_rule_load(dg87_rule* pRule, dg87_storage* pStorage);

/*
 * dg87_rule_save
 */
BOOL dg87_rule_save(dg87_rule* pRule, dg87_storage* pStorage);

/*
 * dg87_rule_calculate
 */
BOOL dg87_rule_calculate(dg87_rule* pRule, time_t day, BOOL *result);

/*
 * dg87_rule_duplicate
 */
dg87_rule* dg87_rule_duplicate(dg87_rule* pOrignialRule);

/*
 * dg87_rule_opearationAdd
 */
BOOL dg87_rule_operationAdd(dg87_rule* pRule, dg87_rule_operation* pOperation);

/*
 * dg87_rule_operationDelete
 */
BOOL dg87_rule_operationDelete(dg87_rule* pRule,int index);

/*
 * dg87_rule_operationMove
 */
BOOL dg87_rule_operationMove(dg87_rule* pRule, int index, int delta);

/******************************************************************************/
/*   functions for operations                                                 */
/******************************************************************************/

/*
 * dg87_operation_dayofweek_new (1 to 7)
 */
dg87_rule_operation* dg87_operation_dayofweek_new(
		dg87_rule_operator oper, int day);

/*
 * dg87_operation_dayofmonth_new (1 to 31 and -31 to -1)
 */
dg87_rule_operation* dg87_operation_dayofmonth_new(
		dg87_rule_operator oper, int day);

/*
 * dg87_operation_dayofyear_new (1 to 365)
 */
dg87_rule_operation* dg87_operation_dayofyear_new(
		dg87_rule_operator oper, int day);

/*
 * dg87_operation_weekofmonth_new (1 to 4)
 */
dg87_rule_operation* dg87_operation_weekofmonth_new(
		dg87_rule_operator oper, int week);

/*
 * dg87_operation_weekofyear_new (1 to 52)
 */
dg87_rule_operation* dg87_operation_weekofyear_new(
		dg87_rule_operator oper, int week);

/*
 * dg87_operation_monthofyear_new (1 to 12)
 */
dg87_rule_operation* dg87_operation_monthofyear_new(
		dg87_rule_operator oper, int month);

/*
 * dg87_operation_day_new (dd/mm)
 */
dg87_rule_operation* dg87_operation_day_new(
		dg87_rule_operator oper, int month, int day);

/*
 * dg87_operation_date_new ()
 */
dg87_rule_operation* dg87_operation_date_new(
		dg87_rule_operator oper, time_t date);

/*
 * dg87_operation_always_new (add all days)
 */
dg87_rule_operation* dg87_operation_always_new(void);

/*
 * dg87_operation_complex_new 
 */
dg87_rule_operation* dg87_operation_complex_new(
		dg87_rule_operator oper, dg87_rule* pRule);

/*
 * dg87_operation_ifequal_new
 */
dg87_rule_operation* dg87_operation_ifequal_new(
		dg87_rule_operator oper, 
		struct dg87_variable_struct* pVariableName, char* pValue);

/*
 * dg87_operation_ifnequal_new
 */
dg87_rule_operation* dg87_operation_ifnequal_new(
		dg87_rule_operator oper, 
		struct dg87_variable_struct* pVariableName, char* pValue);


/*
 * dg87_operation_delete
 */
BOOL dg87_operation_delete(dg87_rule_operation* pOperation);

/*
 * dg87_operation_duplicate
 */
dg87_rule_operation* dg87_operation_duplicate(dg87_rule_operation* pOperation, 
		dg87_vector* pNewRuleVector);

/******************************************************************************/
/*   utilities                                                                */
/******************************************************************************/

/*
 * dg87_rule_asciiToType
 */
dg87_rule_type dg87_rule_asciiToType(char* pRuleType);

/*
 * dg87_rule_typeToAscii
 */
char* dg87_rule_typeToAscii(dg87_rule_type ruleType);

/*
 * dg87_rule_asciiToOperator
 */
dg87_rule_operator dg87_rule_asciiToOperator(char* pOperator);

/*
 * dg87_rule_operatorToAscii
 */
char* dg87_rule_operatorToAscii(dg87_rule_operator oper);


/*
 * dg87_rule_operatorToAscii
 */
BOOL dg87_operationlist_calculate(dg87_list* pOperationList, 
		time_t day, BOOL *result, int nOperations);
/******************************************************************************/
/*   utilities                                                                */
/******************************************************************************/

/*
 * dg87_rule_asciiToDay
 * This function extracts the month and the day from a string 
 * with a format hh:mm.
 * returns FALSE on error.
 */
BOOL dg87_rule_asciiToDay(char* pString, int *month, int *day);

/* dg87_rule_dayToAscii
 * return a string witch represent the time given in parameter. 
 * The format of the string is MM-DD where MM is month, DD is month day.
 * The function return -1 on error.
 * The string must be free with a call to dg87_free
 */
char* dg87_rule_dayToAscii(int month, int day);


/* dg87_rule_asciiToDate
 * convert a string to a time_t.
 * The format of the string is YYYY-MM-DD where YYYY is year,
 * MM is month, DD is month day.
 * The function return -1 on error.
 */
time_t dg87_rule_asciiToDate(char* pString);

/* dg87_rule_dateToAscii
 * return a string witch represent the time given in parameter. 
 * The format of the string is YYYY-MM-DD where YYYY is year,
 * MM is month, DD is month day.
 * The function return -1 on error.
 * The string must be free with a call to dg87_free
 */
char* dg87_rule_dateToAscii(time_t time);

#ifdef DEBUG
/*
 * dg87_rule_ckeck
 */
int dg87_rule_check(dg87_rule* pRule, int (*print)(char*));
#endif /*DEBUG*/

#endif /*RULE_H*/

/* End of file */
