/* file target.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <config.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <libdg87/list.h>
#include <libdg87/vector.h>
#include "target.h"
#include "queue.h"


/****************************************************************************/
dg87_target* dg87_target_new(char* pName)
{
	dg87_target* pTarget;
	assert(pName != NULL);
	if (pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	if (strlen(pName) >= DG87_TARGET_NAME_LENGTH){
		dg87_err_setError(DG87_ERR_NAMETOLONG);
		return FALSE;
	}
	/* create the object */
	pTarget = dg87_malloc(sizeof(dg87_target), DG87_TARGET);
	if (pTarget == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* create the QueueVector*/
	pTarget->pQueueVector = dg87_vector_new();
	if (pTarget->pQueueVector == NULL){
		dg87_target_delete(pTarget);
		return NULL;
	}
	/* initialize properties */
	strncpy(pTarget->name, pName, DG87_TARGET_NAME_LENGTH); 
	pTarget->description[0] = 0;
	pTarget->ipPort = atoi(DG87R_IPPORT);
	pTarget->pHostName = NULL;
	return pTarget;
}

/****************************************************************************/
BOOL dg87_target_delete(dg87_target* pTarget)
{
	dg87_queue* pQueue;
	assert(pTarget != NULL);
	if (pTarget == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	while (pTarget->pQueueVector->count > 0){
		pQueue = dg87_vector_removeObjectByIndex
			(pTarget->pQueueVector,0);
		dg87_queue_delete(pQueue);
	}
	dg87_vector_delete(pTarget->pQueueVector);

	if (pTarget->pHostName != NULL)
		dg87_free(pTarget->pHostName, DG87_CHAR);

	dg87_free(pTarget, DG87_TARGET);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_target_load(dg87_target* pTarget, dg87_storage* pStorage)
{
	char* pString;
	dg87uint value;
	assert(pTarget != NULL && pStorage != NULL);
	if (pTarget == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read target description */
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_target_setDescription(pTarget, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);
	/* read long host name (can be NULL)*/
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_target_setHostName(pTarget, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);
	/* read IP port */
	if (dg87_storage_readUInt(pStorage, &value) == FALSE)
		return FALSE;
	pTarget->ipPort = value;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_target_save(dg87_target* pTarget, dg87_storage* pStorage)
{
	dg87int value;
	assert(pTarget != NULL && pStorage != NULL);
	if (pTarget == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write target description */
	if (dg87_storage_writeString(pStorage, pTarget->description) == FALSE)
		return FALSE;
	/* write long host name (can be NULL)*/
	if (dg87_storage_writeString(pStorage, 
				(pTarget->pHostName == NULL)?"":
				pTarget->pHostName) == FALSE)
		return FALSE;
	/* write IP port */
	value = pTarget->ipPort;
	if (dg87_storage_writeUInt(pStorage, value) == FALSE)
		return FALSE;

	return TRUE;
}

/****************************************************************************/
BOOL dg87_target_setHostName(dg87_target* pTarget, char* pHostName)
{
	/* pHostName can be NULL */
	assert(pTarget != NULL);
	if (pTarget == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* free previous string */
	if (pTarget->pHostName != NULL){
		dg87_free(pTarget->pHostName, DG87_CHAR);
		pTarget->pHostName = NULL;
	}
	/* copy new host name */
	if (pHostName != NULL && strlen(pHostName) > 0){
		pTarget->pHostName = dg87_strdup(pHostName, DG87_CHAR);
		if (pTarget->pHostName == NULL)
			return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_target_setDescription(dg87_target* pTarget, char* pDescription)
{
	assert(pTarget != NULL && pDescription != NULL);
	if (pTarget == NULL || pDescription == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (strlen(pDescription) >= DG87_TARGET_DESC_LENGTH){
		dg87_err_setError(DG87_ERR_DESCRIPTIONTOLONG);
		return FALSE;
	}
	strncpy(pTarget->description, pDescription, DG87_TARGET_DESC_LENGTH);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_target_setIpPort(dg87_target* pTarget, dg87int value)
{
	assert(pTarget != NULL);
	if (pTarget == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/*TODO add best test for ipPort */
	if (value < 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return FALSE;
	}
	pTarget->ipPort = value;
	return TRUE;
}

/****************************************************************************/
dg87_queue* dg87_target_queueAdd(dg87_target* pTarget, char* pQueueName)
{
	dg87_queue* pQueue;
	assert(pTarget != NULL && pQueueName != NULL);
	if (pTarget == NULL || pQueueName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pQueue = dg87_queue_new(pTarget, pQueueName);
	if (pQueue == NULL)
		return NULL;

	/* vector generate error if an other queue has the same name */
	/* key must point on pQueue->name */
	if (dg87_vector_addObject(pTarget->pQueueVector, pQueue, pQueue->name) 
			== FALSE)
	{
		dg87_queue_delete(pQueue);
		return NULL;
	}
	return pQueue;
}
/****************************************************************************/
int dg87_target_queueCount(dg87_target* pTarget)
{
	assert(pTarget != NULL);
	if (pTarget == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_vector_count(pTarget->pQueueVector);
}
/****************************************************************************/
dg87_queue* dg87_target_queueGetByName(dg87_target* pTarget, char* pName)
{
	assert(pTarget != NULL && pName != NULL);
	if (pTarget == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByKey(pTarget->pQueueVector, pName);
}
/****************************************************************************/
dg87_queue* dg87_target_queueGetByIndex(dg87_target* pTarget, int index)
{
	assert(pTarget != NULL);
	if (pTarget == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByIndex(pTarget->pQueueVector, index);
}

/****************************************************************************/
BOOL dg87_target_queueDelete(dg87_target* pTarget,dg87_queue* pQueue)
{
	assert(pTarget != NULL && pQueue != NULL);
	if (pTarget == NULL || pQueue == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_vector_removeObject(pTarget->pQueueVector, pQueue) == FALSE)
		return FALSE;
	if (dg87_queue_delete(pQueue) == FALSE){
		/* add queue in vector again */
		dg87_vector_addObject(pTarget->pQueueVector, pQueue,
				pQueue->name);
		return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_target_queueDeleteByName(dg87_target* pTarget,char* pName)
{
	/*TODO : remove all dependances */
	dg87_queue* pQueue;
	assert(pTarget != NULL && pName != NULL);
	if (pTarget == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pQueue = dg87_vector_getObjectByKey(pTarget->pQueueVector, pName);
	if (pQueue == NULL) 
		return FALSE;
	return dg87_target_queueDelete(pTarget, pQueue);
}
/****************************************************************************/
BOOL dg87_target_queueDeleteByIndex(dg87_target* pTarget,int index)
{
	/*TODO : remove all dependances */
	dg87_queue* pQueue;
	assert(pTarget != NULL);
	if (pTarget == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pQueue = dg87_vector_getObjectByIndex(pTarget->pQueueVector, index);
	if (pQueue == NULL) 
		return FALSE;
	return dg87_target_queueDelete(pTarget, pQueue);
}

/****************************************************************************/
#ifdef DEBUG
int dg87_target_check(dg87_target* pTarget, int (*print)(char*))
{
	dg87_element* pElement;
	dg87_queue* pQueue;

	if (strlen(pTarget->name) == 0){
		print("Target : Name is zero length !\n");
		return 1;
	}
	/* check QueueVector */
	if (pTarget->pQueueVector == NULL){
		print("Target : QueueVector is NULL !\n");
		return 1;
	}
	if (dg87_vector_check(pTarget->pQueueVector, print) != 0)
		return 1;
	pElement = pTarget->pQueueVector->pFirst;
	while (pElement != NULL){
		pQueue = pElement->pObject;
		if (dg87_queue_check(pQueue, print) != 0)
			return 1;
		if (pQueue->pTarget != pTarget){
			print("Target : Queue's Target pointer incorrect !\n");
			return 1;
		}
		pElement = pElement->pNext;
	}
	return 0;
}
#endif /*DEBUG*/

/****************************************************************************/
/* End of file */
