/* file task.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <libdg87/list.h>
#include <libdg87/vector.h>

#include "context.h"
#include "target.h"
#include "job.h"
#include "jobdep.h"
#include "task.h"
#include "ref.h"
#include "target.h"
#include "condition.h"


/****************************************************************************/
dg87_task* dg87_task_new(char* pName)
{
	dg87_task* pTask;
	assert(pName != NULL);
	if (pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	if (strlen(pName) >= DG87_TASK_NAME_LENGTH){
		dg87_err_setError(DG87_ERR_NAMETOLONG);
		return FALSE;
	}
	pTask = dg87_malloc(sizeof(dg87_task), DG87_TASK);
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* allocate memory for JobVector */
	pTask->pJobVector = dg87_vector_new();
	if (pTask->pJobVector == NULL){
		dg87_free(pTask, DG87_TASK);
		return NULL;
	}
	/* allocate memory for JobDepList */
	pTask->pJobDepList = dg87_list_new();
	if (pTask->pJobDepList == NULL){
		dg87_vector_delete(pTask->pJobVector);
		dg87_free(pTask, DG87_TASK);
		return NULL;
	}
	/* allocate memory for TaskDepBefore */
	pTask->pTaskDepBeforeList = dg87_list_new();
	if (pTask->pTaskDepBeforeList == NULL){
		dg87_vector_delete(pTask->pJobVector);
		dg87_list_delete(pTask->pJobDepList);
		dg87_free(pTask, DG87_TASK);
		return NULL;
	}
	/* allocate memory for TaskDepAfter */
	pTask->pTaskDepAfterList = dg87_list_new();
	if (pTask->pTaskDepAfterList == NULL){
		dg87_vector_delete(pTask->pJobVector);
		dg87_list_delete(pTask->pJobDepList);
		dg87_list_delete(pTask->pTaskDepBeforeList);
		dg87_free(pTask, DG87_TASK);
		return NULL;
	}
	/* allocate memory for ConditionList */
	pTask->pConditionList = dg87_list_new();
	if (pTask->pConditionList == NULL){
		dg87_vector_delete(pTask->pJobVector);
		dg87_list_delete(pTask->pJobDepList);
		dg87_list_delete(pTask->pTaskDepBeforeList);
		dg87_list_delete(pTask->pTaskDepAfterList);
		dg87_free(pTask, DG87_TASK);
		return NULL;
	}
	/* allocate memory for RuleOperationList */
	pTask->pRuleOperationList = dg87_list_new();
	if (pTask->pRuleOperationList == NULL){
		dg87_vector_delete(pTask->pJobVector);
		dg87_list_delete(pTask->pJobDepList);
		dg87_list_delete(pTask->pTaskDepBeforeList);
		dg87_list_delete(pTask->pTaskDepAfterList);
		dg87_list_delete(pTask->pConditionList);
		dg87_free(pTask, DG87_TASK);
		return NULL;
	}
	strncpy(pTask->name, pName, DG87_TASK_NAME_LENGTH); 
	pTask->description[0] = 0;
	pTask->waitOldestTask = TRUE;
	pTask->pQueue= NULL;
	pTask->pRef = NULL;
	pTask->waitAllDependences = TRUE;
	pTask->onErrorContinue = TRUE;
	return pTask;
}

/****************************************************************************/
dg87_task* dg87_task_duplicate(dg87_task* pOriginalTask,
		dg87_vector *pNewRuleVector)
{
	dg87_task *pNewTask;
	dg87_element* pElement;
	dg87_item* pItem;
	dg87_job *pOriginalJob, *pNewJob;
	dg87_job *pJobBefore, *pJobAfter;
	dg87_jobdep *pOriginalJobDep, *pNewJobDep;
	dg87_condition *pOriginalCondition, *pNewCondition;
	dg87_rule_operation *pOriginalOperation, *pNewOperation;
	assert(pOriginalTask != NULL && pNewRuleVector != NULL);
	if (pOriginalTask == NULL || pNewRuleVector == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	/* create a task with the same name */
	pNewTask = dg87_task_new(pOriginalTask->name);
	if (pNewTask == NULL) return NULL;
	/* copy task properties */
	strncpy(pNewTask->description, pOriginalTask->description, 
			DG87_TASK_DESC_LENGTH);
	pNewTask->waitOldestTask = pOriginalTask->waitOldestTask;
	pNewTask->waitAllDependences = pOriginalTask->waitAllDependences;
	pNewTask->onErrorContinue = pOriginalTask->onErrorContinue;
	pNewTask->pRef = NULL;
	/* set task queue */
	if (dg87_task_setQueue(pNewTask, pOriginalTask->pQueue) == FALSE){
		dg87_task_delete(pNewTask);
		return NULL;

	}
	/* copy JobVector */
	pElement = pOriginalTask->pJobVector->pFirst;
	while (pElement != NULL){
		pOriginalJob = pElement->pObject;
		pElement = pElement->pNext;
		/* duplicate job */
		pNewJob = dg87_job_duplicate(pNewTask, pOriginalJob);
		if (pNewJob == NULL){
			dg87_task_delete(pNewTask);
			return NULL;
		}
		/* add it to job vector of the new task */
		if (dg87_vector_addObject(pNewTask->pJobVector, 
					pNewJob, pNewJob->name) == FALSE)
		{
			dg87_job_delete(pNewJob);
			dg87_task_delete(pNewTask);
			return NULL;
		}
	}
	/* copy JobDepList */
	pItem = pOriginalTask->pJobDepList->pFirst;
	while (pItem != NULL){
		pOriginalJobDep = pItem->pObject;
		pItem = pItem->pNext;
		pJobBefore = dg87_vector_getObjectByKey(pNewTask->pJobVector, 
				pOriginalJobDep->pJobBefore->name);
		pJobAfter = dg87_vector_getObjectByKey(pNewTask->pJobVector, 
				pOriginalJobDep->pJobAfter->name);
		if (pJobBefore == NULL || pJobAfter == NULL){
			dg87_task_delete(pNewTask);
			return NULL;
		}
		pNewJobDep = dg87_task_jobdepAdd(pNewTask,pJobBefore,pJobAfter);
		if (pNewJobDep == NULL){
			dg87_task_delete(pNewTask);
			return NULL;
		}
		pNewJobDep->type = pOriginalJobDep->type;
	}

	/* copy ConditionList */
	pItem = pOriginalTask->pConditionList->pFirst;
	while (pItem != NULL){
		pOriginalCondition = pItem->pObject;
		pItem = pItem->pNext;
		/* duplicate condition */
		pNewCondition = dg87_condition_duplicate(pOriginalCondition);
		if (pNewCondition == NULL){
			dg87_task_delete(pNewTask);
			return NULL;
		}
		/* add it to condition list of the new task */
		if (dg87_list_addObject(pNewTask->pConditionList, 
					pNewCondition) == FALSE)
		{
			dg87_condition_delete(pNewCondition);
			dg87_task_delete(pNewTask);
			return NULL;
		}
	}
	/* copy ruleOperationList */
	pItem = pOriginalTask->pRuleOperationList->pFirst;
	while (pItem != NULL){
		pOriginalOperation = pItem->pObject;
		pItem = pItem->pNext;
		/* duplicate operation */
		pNewOperation = dg87_operation_duplicate
			(pOriginalOperation, pNewRuleVector);
		if (pNewOperation == NULL){
			dg87_task_delete(pNewTask);
			return NULL;
		}
		/* add it to ruleoperation list of the new task */
		if (dg87_task_operationAdd(pNewTask, pNewOperation) == FALSE)
		{
			dg87_operation_delete(pNewOperation);
			dg87_task_delete(pNewTask);
			return NULL;
		}
	}
	return pNewTask;
}
/****************************************************************************/
BOOL dg87_task_delete(dg87_task* pTask)
{
	dg87_job *pJob;
	dg87_jobdep *pJobDep;
	dg87_condition *pCondition;
	assert(pTask != NULL);
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* can't delete task if task has dependences whith other task */
	if (pTask->pTaskDepBeforeList->count > 0 ||
			pTask->pTaskDepAfterList->count > 0){
		dg87_err_setError(DG87_ERR_TASKHASDEPENDENCES);
		return FALSE;
	}
	dg87_list_delete(pTask->pTaskDepBeforeList);
	dg87_list_delete(pTask->pTaskDepAfterList);

	/* remove Task from Queue */
	if (pTask->pQueue != NULL)
		dg87_list_removeObject(pTask->pQueue->pRefTaskList, pTask);	

	/* delete all JobDep object */
	while (pTask->pJobDepList->count > 0){
		pJobDep = dg87_list_removeObjectByIndex(pTask->pJobDepList,0);
		dg87_jobdep_delete(pJobDep);
	}
	dg87_list_delete(pTask->pJobDepList);

	/* delete all Job object */
	while (pTask->pJobVector->count > 0){
		pJob = dg87_vector_removeObjectByIndex(pTask->pJobVector,0);
		dg87_job_delete(pJob);
	}
	dg87_vector_delete(pTask->pJobVector);

	/* delete all Condition object */
	while (pTask->pConditionList->count > 0){
		pCondition = dg87_list_removeObjectByIndex
			(pTask->pConditionList,0);
		dg87_condition_delete(pCondition);
	}
	dg87_list_delete(pTask->pConditionList);

	/* delete all RuleOperationList object */
	while (pTask->pRuleOperationList->count > 0){
		if (dg87_task_operationDelete(pTask,0) == FALSE)
			return FALSE;
	}
	dg87_list_delete(pTask->pRuleOperationList);


	dg87_free(pTask, DG87_TASK);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_load(dg87_task* pTask, 
		dg87_storage* pStorage, struct dg87_context_struct *pContext)
{
	dg87byte b;
	char* pString;
	dg87_target *pTarget;
	dg87_queue *pQueue;
	assert(pTask != NULL && pStorage != NULL && pContext != NULL);
	if (pTask == NULL || pStorage == NULL || pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read task description*/
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_task_setDescription(pTask, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);

	/* read waitAllDependences */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_task_setWaitAllDependences(pTask, b) == FALSE)
		return FALSE;

	/* read onErrorContinue */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_task_setOnErrorContinue(pTask, b) == FALSE)
		return FALSE;

	/* read target and queue */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (b == TRUE){ /* queue is not NULL */
		/* find target */
		if (dg87_storage_readString(pStorage, &pString) == FALSE)
			return FALSE;
		if ((pTarget = dg87_context_targetGetByName(
				pContext, pString)) == NULL){
			dg87_free(pString, DG87_CHAR);
			return FALSE;
		}
		dg87_free(pString, DG87_CHAR);
		/* find queue */
		if (dg87_storage_readString(pStorage, &pString) == FALSE)
			return FALSE;
		if ((pQueue = dg87_target_queueGetByName(
				pTarget, pString)) == NULL){
			dg87_free(pString, DG87_CHAR);
			return FALSE;
		}
		dg87_free(pString, DG87_CHAR);
		/* set queue */
		if (dg87_task_setQueue(pTask, pQueue) == FALSE)
			return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_save(dg87_task* pTask, dg87_storage* pStorage)
{
	assert(pTask != NULL && pStorage != NULL);
	if (pTask == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write task description */
	if (dg87_storage_writeString(pStorage, pTask->description) == FALSE)
		return FALSE;
	/* write waitAllDependences */
	if (dg87_storage_writeByte(pStorage, pTask->waitAllDependences)== FALSE)
		return FALSE;
	/* write onErrorContinue */
	if (dg87_storage_writeByte(pStorage, pTask->onErrorContinue)== FALSE)
		return FALSE;
	/* save target and queue */
	if (dg87_storage_writeByte(pStorage, 
				(pTask->pQueue==NULL)?FALSE:TRUE) == FALSE)
		return FALSE;
	if (pTask->pQueue != NULL){
		if (dg87_storage_writeString(pStorage, 
					pTask->pQueue->pTarget->name) == FALSE)
			return FALSE;
		if (dg87_storage_writeString(pStorage, 
					pTask->pQueue->name) == FALSE)
			return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_operationMove(dg87_task* pTask,int index, int delta)
{
	assert(pTask == NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	return dg87_list_moveItem(pTask->pRuleOperationList, index, delta);

}
/****************************************************************************/
BOOL dg87_task_calculate(dg87_task* pTask, time_t day, BOOL *result)
{	
	assert(pTask != NULL && result != NULL);
	if (pTask == NULL || result == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}	
	return dg87_operationlist_calculate(
			pTask->pRuleOperationList, day, result, -1);
}
/****************************************************************************/
BOOL dg87_task_setDescription(dg87_task* pTask, char* pDescription)
{
	assert(pTask != NULL && pDescription != NULL);
	if (pTask == NULL || pDescription == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (strlen(pDescription) >= DG87_TASK_DESC_LENGTH){
		dg87_err_setError(DG87_ERR_DESCRIPTIONTOLONG);
		return FALSE;
	}
	strncpy(pTask->description, pDescription, DG87_TASK_DESC_LENGTH);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_setQueue(dg87_task *pTask, dg87_queue *pQueue)
{
	dg87_list* pList;

	assert(pTask != NULL);
	if (pTask == NULL){  /* no test on pQueue, it can be null ! */
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}

	/* nothing to do, already done */
	if (pTask->pQueue == pQueue)
		return TRUE; 
	/* remove task of previous target */
	if (pTask->pQueue != NULL){
		pList = pTask->pQueue->pRefTaskList;
		dg87_list_removeObject(pList,pTask);
	}
	/* add this task to refTaskVector of queue */
	pTask->pQueue = pQueue;
	/* add task of actual Queue */
	if (pTask->pQueue != NULL){
		pList = pTask->pQueue->pRefTaskList;
		dg87_list_addObject(pList, pTask);
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_setWaitOldestTask(dg87_task *pTask,BOOL waitOldestTask)
{
	assert(pTask != NULL);
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTask->waitOldestTask = waitOldestTask;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_setWaitAllDependences(dg87_task* pTask, BOOL value)
{
	assert(pTask != NULL);
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTask->waitAllDependences = value;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_setOnErrorContinue(dg87_task* pTask, BOOL value)
{
	assert(pTask != NULL);
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTask->onErrorContinue = value;
	return TRUE;
}
/****************************************************************************/
dg87_job* dg87_task_jobAdd(dg87_task* pTask, char* pJobName)
{
	dg87_job* pJob;
	assert(pTask != NULL && pJobName != NULL);
	if (pTask == NULL || pJobName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob = dg87_job_new(pTask, pJobName);
	if (pJob == NULL) 
		return NULL;

	/* vector generate error if an other job has the same name */
	/* key must point on pJob->name */
	if (dg87_vector_addObject(pTask->pJobVector, pJob, pJob->name) == FALSE)
	{
		dg87_job_delete(pJob);
		return NULL;
	}
	return pJob;
}
/****************************************************************************/
int dg87_task_jobCount(dg87_task* pTask)
{
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_vector_count(pTask->pJobVector);
}
/****************************************************************************/
dg87_job* dg87_task_jobGetByName(dg87_task* pTask, char* pName)
{
	assert(pTask != NULL && pName != NULL);
	if (pTask == NULL || pName == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByKey(pTask->pJobVector, pName);
}
/****************************************************************************/
dg87_job* dg87_task_jobGetByIndex(dg87_task* pTask, int index)
{
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByIndex(pTask->pJobVector, index);
}
/****************************************************************************/
BOOL dg87_task_jobDelete(dg87_task* pTask,dg87_job* pJob)
{
	dg87_jobdep* pJobDep;
	dg87_item* pItem;
	assert(pTask != NULL && pJob != NULL);
	if (pTask == NULL || pJob == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* remove all dependences link to this job */
	pItem = pTask->pJobDepList->pFirst;
	while (pItem != NULL){
		pJobDep = pItem->pObject;
		if (pJobDep->pJobBefore == pJob || pJobDep->pJobAfter == pJob)
		{
			dg87_item* pTemp = pItem;
			pItem = pItem->pNext;
			dg87_list_removeItem(pTask->pJobDepList, pTemp);
			dg87_jobdep_delete(pJobDep);
		} else {
			pItem = pItem->pNext;
		}
	}
	if (dg87_vector_removeObject(pTask->pJobVector, pJob) == FALSE)
		return FALSE;
	if (dg87_job_delete(pJob) == FALSE){
		/* add job in vector again */
		/* TODO les dpendances supprimes au dbut sont perdues */
		dg87_vector_addObject(pTask->pJobVector, pJob, pJob->name);
		return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_jobDeleteByName(dg87_task* pTask,char* pName)
{
	dg87_job* pJob;
	assert(pTask != NULL && pName != NULL);
	if (pTask == NULL || pName == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob = dg87_vector_getObjectByKey(pTask->pJobVector, pName);
	if (pJob == NULL) return FALSE;
	return dg87_task_jobDelete(pTask,pJob);
}
/****************************************************************************/
BOOL dg87_task_jobDeleteByIndex(dg87_task* pTask,int index)
{
	dg87_job* pJob;
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob = dg87_vector_getObjectByIndex(pTask->pJobVector, index);
	if (pJob == NULL) 
		return FALSE;
	return dg87_task_jobDelete(pTask,pJob);
}
/****************************************************************************/
dg87_jobdep* dg87_task_jobdepAdd (dg87_task* pTask, 
	 struct dg87_job_struct* pJobBefore, 
	 struct dg87_job_struct* pJobAfter)
{
	dg87_jobdep* pJobDep;

	assert(pTask != NULL && pJobBefore !=NULL && pJobAfter != NULL);
	if (pTask == NULL || pJobBefore == NULL || pJobAfter == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (pJobBefore->pTask != pTask || pJobAfter->pTask != pTask){
		dg87_err_setError(DG87_ERR_JOBSNOTINSAMETASK);
		return NULL;
	}

	/* create and add the job dep */
	pJobDep = dg87_jobdep_new(pJobBefore, pJobAfter);
	if (pJobDep == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pJobDepList, pJobDep);
	
	return pJobDep;
}
/****************************************************************************/
int dg87_task_jobdepCount(dg87_task* pTask)
{
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_list_count(pTask->pJobDepList);
}
/****************************************************************************/
dg87_jobdep* dg87_task_jobdepGet(dg87_task* pTask, int index)
{
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_list_getObjectByIndex(pTask->pJobDepList, index);
}
/****************************************************************************/
BOOL dg87_task_jobdepDeleteByIndex(dg87_task* pTask,int index)
{
	dg87_jobdep* pJobDep;
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJobDep = dg87_list_removeObjectByIndex(pTask->pJobDepList, index);
	if (pJobDep == NULL) 
		return FALSE;
	dg87_jobdep_delete(pJobDep);
	return TRUE;
}
/****************************************************************************/
int dg87_task_taskdepBeforeCount(dg87_task* pTask)
{
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_list_count(pTask->pTaskDepBeforeList);
}
/****************************************************************************/
dg87_taskdep* dg87_task_taskdepBeforeGet(dg87_task* pTask, int index)
{
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_list_getObjectByIndex(pTask->pTaskDepBeforeList, index);
}
/****************************************************************************/
int dg87_task_taskdepAfterCount(dg87_task* pTask)
{
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_list_count(pTask->pTaskDepAfterList);
}
/****************************************************************************/
dg87_taskdep* dg87_task_taskdepAfterGet(dg87_task* pTask, int index)
{
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_list_getObjectByIndex(pTask->pTaskDepAfterList, index);
}
/****************************************************************************/
dg87_condition* dg87_task_conditionHminAdd(dg87_task* pTask, int minutes)
{
	dg87_condition* pCondition;
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionHmin_new(minutes);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionHmaxAdd(dg87_task* pTask, int minutes)
{
	dg87_condition* pCondition;
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionHmax_new(minutes);
	if (pCondition == NULL) return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionHlateAdd(dg87_task* pTask, int minutes)
{
	dg87_condition* pCondition;
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
       	pCondition = dg87_conditionHlate_new(minutes);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionFileAdd(dg87_task* pTask, char* pFilePath)
{
	dg87_condition* pCondition;
	assert(pTask != NULL);
	/* TODO control pFilePath = NULL ?*/
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionFile_new(pFilePath);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionManualAdd(dg87_task* pTask, char* pMessage)
{
	dg87_condition* pCondition;
	assert(pTask != NULL);
	/*TODO control pMessage = NULL ?*/
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionManual_new(pMessage);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionIfEqualAdd(
		dg87_task* pTask, 
		struct dg87_variable_struct* pVariable, 
		char* pValue)
{
	dg87_condition* pCondition;
	assert(pTask != NULL && pVariable != NULL && pValue != NULL);
	if (pTask == NULL || pVariable == NULL || pValue == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionIfEqual_new(pVariable, pValue);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionIfNotEqualAdd(
		dg87_task* pTask, 
		struct dg87_variable_struct* pVariable, 
		char* pValue)
{
	dg87_condition* pCondition;
	assert(pTask != NULL && pVariable != NULL && pValue !=NULL);
	if (pTask == NULL || pVariable == NULL || pValue == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionIfNotEqual_new(pVariable, pValue);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
int dg87_task_conditionCount(dg87_task* pTask)
{
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_list_count(pTask->pConditionList);
}
/****************************************************************************/
dg87_condition* dg87_task_conditionGet(dg87_task* pTask, int index)
{
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_list_getObjectByIndex(pTask->pConditionList, index);
}
/****************************************************************************/
BOOL dg87_task_conditionDelete(dg87_task* pTask,int index)
{
	dg87_condition* pCondition;
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pCondition = dg87_list_removeObjectByIndex
		(pTask->pConditionList, index);
	if (pCondition == NULL) 
		return FALSE;
	return dg87_condition_delete(pCondition); 
}
/****************************************************************************/
BOOL dg87_task_operationAdd(dg87_task* pTask, dg87_rule_operation* pOperation)
{
	assert(pTask != NULL && pOperation != NULL);
	if (pTask == NULL || pOperation == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* add the operation to the list */
	if (dg87_list_addObject(pTask->pRuleOperationList, 
				pOperation) == FALSE){
		return FALSE;
	}
	/* increment linkedRules counter for complex rule */
	if (pOperation->type == dg87_rule_type_complex)
		pOperation->rule.pRule->linkedTasks++;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_operationDelete(dg87_task* pTask,int index)
{
	dg87_rule_operation* pOperation;
	assert(pTask != NULL);
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* remove the operation from the list */
	pOperation = dg87_list_removeObjectByIndex(pTask->pRuleOperationList, 
			index);
	if (pOperation == NULL) 
		return FALSE;
	/* if complex rule, decrement linkedTasks counter */
	if (pOperation->type == dg87_rule_type_complex)
		pOperation->rule.pRule->linkedTasks--;
	/* delete operation object */
	if (dg87_operation_delete(pOperation) == FALSE){
		dg87_list_addObject(pTask->pRuleOperationList, pOperation);
		if (pOperation->type == dg87_rule_type_complex)
			pOperation->rule.pRule->linkedTasks++;
		return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_findUpStreamTasks(dg87_task* pTask, dg87_list* pTaskList)
{
	dg87_taskdep* pTaskDep;
	dg87_item* pTaskDepItem;
	assert(pTask != NULL && pTaskList != NULL);
	if (pTask== NULL || pTaskList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_list_addObject(pTaskList, pTask) == FALSE)
		return FALSE;
	pTaskDepItem = pTask->pTaskDepBeforeList->pFirst;
	while (pTaskDepItem != NULL){
		pTaskDep = pTaskDepItem->pObject;
		pTaskDepItem = pTaskDepItem->pNext;
		if (dg87_task_findUpStreamTasks(pTaskDep->pTaskBefore,
					pTaskList) == FALSE)
			return FALSE;
	}
	return TRUE;
}
/*****************************************************************************/
BOOL dg87_task_findDownStreamTasks(dg87_task* pTask, dg87_list* pTaskList)
{
	dg87_taskdep* pTaskDep;
	dg87_item* pTaskDepItem;
	assert(pTask != NULL && pTaskList != NULL);
	if (pTask== NULL || pTaskList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_list_addObject(pTaskList, pTask) == FALSE)
		return FALSE;
	pTaskDepItem = pTask->pTaskDepAfterList->pFirst;
	while (pTaskDepItem != NULL){
		pTaskDep = pTaskDepItem->pObject;
		pTaskDepItem = pTaskDepItem->pNext;
		if (dg87_task_findDownStreamTasks(pTaskDep->pTaskAfter,
					pTaskList) == FALSE)
			return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_task_check(dg87_task* pTask, int (*print)(char*))
{
	dg87_element* pElement;
	dg87_item* pItem;
	dg87_job* pJob;

	/* check name */
	if (strlen(pTask->name) == 0){
		print("Task : name length is zero !\n");
		return 1;
	}
	/* check waitOldestTask */
	if (pTask->waitOldestTask != TRUE && pTask->waitOldestTask != FALSE){
		print("Task : incorrect value for waitOldestTask !\n");
		return 1;
	}
	/* check JobVector*/
	if (pTask->pJobVector == NULL){
		print("Task : JobVector is null !\n");
		return 1;
	}
	if (dg87_vector_check(pTask->pJobVector, print) != 0)
		return 1;
	pElement = pTask->pJobVector->pFirst;
	while (pElement != NULL){
		pJob = pElement->pObject;
		if (dg87_job_check(pJob, print) != 0)
			return 1;
		if (pJob->pTask != pTask){
			print("Task : Job's Task pointer incorrect !\n");
			return 1;
		}
		pElement = pElement->pNext;
	}
	
	/* check JobdepList */
	if (pTask->pJobDepList == NULL){
		print("Task : JobdepList is null !\n");
		return 1;
	}
	if (dg87_list_check(pTask->pJobDepList, print) != 0)
		return 1;
	pItem = pTask->pJobDepList->pFirst;
	while (pItem != NULL){
		if (dg87_jobdep_check(pItem->pObject, print) != 0)
			return 1;
		pItem = pItem->pNext;
	}

	/* check TaskDepBeforeList */
	if (pTask->pTaskDepBeforeList == NULL){
		print("Task : TaskDepBeforeList is null !\n");
		return 1;
	}
	if (dg87_list_check(pTask->pTaskDepBeforeList, print) != 0)
		return 1;

	/* check TaskDepAfterList */
	if (pTask->pTaskDepAfterList == NULL){
		print("Task : TaskDepAfterList is null !\n");
		return 1;
	}
	if (dg87_list_check(pTask->pTaskDepAfterList, print) != 0)
		return 1;
	
	/* check Queue */
	if (pTask->pQueue != NULL){
		if (dg87_list_findItem(pTask->pQueue->pRefTaskList,
					pTask) == NULL){
			print("Task : task not in Queue.RefTaskList !\n");
			return 1;
		}
	}
	/* check ConditionList */
	if (pTask->pConditionList == NULL){
		print("Task : ConditionList is null !\n");
		return 1;
	}
	if (dg87_list_check(pTask->pConditionList, print) != 0)
		return 1;
	pItem = pTask->pConditionList->pFirst;
	while (pItem != NULL){
		if (dg87_condition_check(pItem->pObject, print) != 0)
			return 1;
		pItem = pItem->pNext;
	}
	
	return 0;
}
#endif /*DEBUG*/


/****************************************************************************/
/* End of file */
