/* file task.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef TASK_H 
#define TASK_H 1

#include "global.h"
#include "job.h"
#include "jobdep.h"
#include "taskdep.h"
#include "condition.h"
#include "rule.h"
#include "queue.h"
#include <libdg87/vector.h>
#include <libdg87/list.h>


typedef struct dg87_task_struct
{
	char name[DG87_TASK_NAME_LENGTH];
	char description[DG87_TASK_DESC_LENGTH];
	BOOL waitOldestTask;
	dg87_vector *pJobVector;    /* jobs of the task */
	dg87_list *pJobDepList; /* dependances between jobs of the task */
	dg87_list *pTaskDepBeforeList;
	dg87_list *pTaskDepAfterList;
	dg87_list *pConditionList;
	dg87_list *pRuleOperationList;
	struct dg87_queue_struct *pQueue; /* queue where to execute the TASK */
	struct dg87_ref_struct *pRef;
	BOOL waitAllDependences;
	BOOL onErrorContinue;
} dg87_task;

/*
 * dg87_task_new 
 * create a new task and initialize the name property
 */
dg87_task* dg87_task_new(char* name);

/*
 * dg87_task_duplicate
 * create a new task by copying an other 
 */
dg87_task* dg87_task_duplicate(dg87_task* pOriginalTask, 
		dg87_vector *pNewRuleVector);

/*
 * dg87_task_delete
 */
BOOL dg87_task_delete(dg87_task* pTask);

/*
 * dg87_task_load
 */
BOOL dg87_task_load(dg87_task* pTask, dg87_storage* pStorage, 
		struct dg87_context_struct *pContext);

/*
 * dg87_task_save
 */
BOOL dg87_task_save(dg87_task* pTask, dg87_storage* pStorage);

/*
 * dg87_task_operationMove
 */
BOOL dg87_task_operationMove(dg87_task* pTask, int index, int delta);

/*
 * dg87_task_calculate
 */
BOOL dg87_task_calculate(dg87_task* pTask, time_t day, BOOL *result);
/*
 * dg87_task_setDescription
 * set the description of the task.
 */
BOOL dg87_task_setDescription(dg87_task* pTask, char* pDescription);

/*
 * dg87_task_setQueue
 * set the queue where the job of the TASK will be execute.
 */
BOOL dg87_task_setQueue(dg87_task *pTask, struct dg87_queue_struct *pQueue);

/*
 * dg87_task_setWaitOldestTask
 */
BOOL dg87_task_setWaitOldestTask(dg87_task *pTask,BOOL waitOldestTask);

/*
 * dg87_task_setWaitAllDependences
 */
BOOL dg87_task_setWaitAllDependences(dg87_task* pTask, BOOL value);

/*
 * dg87_task_setOnErrorContinue
 */
BOOL dg87_task_setOnErrorContinue(dg87_task* pTask, BOOL value);

/*************************************************************************/
/*  functions for job                                                    */
/*************************************************************************/

/*
 * dg87_task_jobAdd
 * create a job object, add this object in the JobVector
 * of the task and return a pointer on this object.
 * Return NULL if the Job object can't be create.
 */
struct dg87_job_struct* dg87_task_jobAdd(dg87_task* pTask, char* pJobName);

/*
 * dg87_task_jobCount
 * return the number of job in the task.
 * return -1 if pTask is NULL.
 */
int dg87_task_jobCount(dg87_task* pTask);


/*
 * dg87_task_jobGetByName
 * return a pointer on the job which has the name
 * given in parameter. Return NULL if it can't be find.
 */
struct dg87_job_struct* dg87_task_jobGetByName(dg87_task* pTask, char* pName);

/*
 * dg87_task_jobGetByIndex
 * return a pointer on the job at the given index.
 */
struct dg87_job_struct* dg87_task_jobGetByIndex(dg87_task* pTask, int index);


/*
 * dg87_task_jobDeleteByName
 * delete the job which has the name given in parameter.
 */
BOOL dg87_task_jobDeleteByName(dg87_task* pTask,char* pName);

/*
 * dg87_task_jobDeleteByIndex
 * delete the job at the given index.
 */
BOOL dg87_task_jobDeleteByIndex(dg87_task* pTask,int index);

/*
 * dg87_task_jobDelete
 */
BOOL dg87_task_jobDelete(dg87_task* pTask,struct dg87_job_struct* pJob);

/*************************************************************************/
/*  functions for job dependances                                        */
/*************************************************************************/

/*
 * dg87_task_jobDepAdd
 * add a dependance between two jobs of the TASK. The name of the jobs must
 * be pass in parameters.
 */
dg87_jobdep* dg87_task_jobdepAdd (dg87_task* pTask, 
	 struct dg87_job_struct* pJobBefore, 
	 struct dg87_job_struct* pJobAfter);

/*
 * dg87_task_jobdepCount
 * return the number of jobdep the task.
 * return -1 if pTask is NULL.
 */
int dg87_task_jobdepCount(dg87_task* pTask);

/*
 * dg87_task_jobdepGet
 * return a pointer on the jobdep at the given index.
 */
dg87_jobdep* dg87_task_jobdepGet(dg87_task* pTask, int index);

/*
 * dg87_task_jobdepDeleteByIndex
 * delete the jobdep at the given index.
 */
BOOL dg87_task_jobdepDeleteByIndex(dg87_task* pTask,int index);


/*************************************************************************/
/*  functions for taskdep dependances                                      */
/*************************************************************************/

/*
 * dg87_task_taskdepBeforeCount
 * return the number of taskdep "before" the task.
 * return -1 if pTask is NULL.
 */
int dg87_task_taskdepBeforeCount(dg87_task* pTask);

/*
 * dg87_task_taskdepBeforeGet
 * return a pointer on the taskdep "before" at the given index.
 */
dg87_taskdep* dg87_task_taskdepBeforeGet(dg87_task* pTask, int index);

/*
 * dg87_task_taskdepAfterCount
 * return the number of taskdep "after" the task.
 * return -1 if pTask is NULL.
 */
int dg87_task_taskdepAfterCount(dg87_task* pTask);

/*
 * dg87_task_taskdepAfterGet
 * return a pointer on the taskdep "after" at the given index.
 */
dg87_taskdep* dg87_task_taskdepAfterGet(dg87_task* pTask, int index);

/*************************************************************************/
/*  functions for condition list                                         */
/*************************************************************************/

/*
 * dg87_task_ConditionHminAdd
 */
dg87_condition* dg87_task_conditionHminAdd(dg87_task* pTask, int minutes);

/*
 * dg87_task_ConditionHmaxAdd
 */
dg87_condition* dg87_task_conditionHmaxAdd(dg87_task* pTask, int minutes);

/*
 * dg87_task_ConditionHlateAdd
 */
dg87_condition* dg87_task_conditionHlateAdd(dg87_task* pTask, int minutes);

/*
 * dg87_task_ConditionFileAdd
 */
dg87_condition* dg87_task_conditionFileAdd(dg87_task* pTask, char* pFilePath);

/*
 * dg87_task_ConditionManualAdd
 */
dg87_condition* dg87_task_conditionManualAdd(dg87_task* pTask, char* pMessage);

/*
 * dg87_task_ConditionIfEqualAdd
 */
dg87_condition* dg87_task_conditionIfEqualAdd(
		dg87_task* pTask, 
		struct dg87_variable_struct* pVariable, char* pValue);
/*
 * dg87_task_ConditionIfNotEqualAdd
 */
dg87_condition* dg87_task_conditionIfNotEqualAdd(
		dg87_task* pTask, 
		struct dg87_variable_struct* pVariable, char* pValue);

/*
 * dg87_task_ConditionCount
 * return the number of condition in the task.
 * return -1 if pTask is NULL.
 */
int dg87_task_conditionCount(dg87_task* pTask);

/*
 * dg87_task_conditionGet
 * return a pointer on the condition at the given index.
 */
dg87_condition* dg87_task_conditionGet(dg87_task* pTask, int index);


/*
 * dg87_task_conditionDelete
 * delete the condition object at the given index.
 */
BOOL dg87_task_conditionDelete(dg87_task* pTask,int index);


/*************************************************************************/
/*  functions for ruleOperation list                                     */
/*************************************************************************/

/*
 * dg87_task_opearationAdd
 */
BOOL dg87_task_operationAdd(dg87_task* pTask, dg87_rule_operation* pOperation);

/*
 * dg87_task_operationDelete
 */
BOOL dg87_task_operationDelete(dg87_task* pTask,int index);

/*
 * dg87_task_findUpStreamTasks
 * This function adds to the list given in parameter all upstream tasks
 * of a task.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_task_findUpStreamTasks(dg87_task* pTask, dg87_list* pTaskList);

/*
 * dg87_task_findDownStreamTasks
 * This function adds to the list given in parameter all downstream tasks
 * of a task.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_task_findDownStreamTasks(dg87_task* pTask, dg87_list* pTaskList);

#ifdef DEBUG
/*
 * dg87_task_ckeck
 */
int dg87_task_check(dg87_task* pTask, int (*print)(char*));
#endif /*DEBUG*/

#endif
/* End of file */
