/* file taskdep.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "task.h"
#include "taskdep.h"

/*****************************************************************************/
dg87_taskdep* dg87_taskdep_new(struct dg87_task_struct* pTaskBefore, 
		struct dg87_task_struct* pTaskAfter)
{
	dg87_taskdep* pTaskDep;
	dg87_list* pUpStreamTaskList;
	dg87_list* pDownStreamTaskList;
	assert(pTaskBefore != NULL && pTaskAfter != NULL);
	if (pTaskBefore == NULL || pTaskAfter == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}

	/* find all upstream tasks */
	pUpStreamTaskList = dg87_list_new();
	if (pUpStreamTaskList == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	dg87_task_findUpStreamTasks(pTaskBefore, pUpStreamTaskList);
	/* find all downstream tasks */
	pDownStreamTaskList = dg87_list_new();
	if (pDownStreamTaskList == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	dg87_task_findDownStreamTasks(pTaskAfter, pDownStreamTaskList);
	/* detect circular dependences*/ 
	if (dg87_list_intersection(pDownStreamTaskList, 
				pUpStreamTaskList) == TRUE){
		dg87_list_delete(pDownStreamTaskList);
		dg87_list_delete(pUpStreamTaskList);
		dg87_err_setError(DG87_ERR_CIRCULARDEPENDENCE);
		return NULL;
	}
	dg87_list_delete(pDownStreamTaskList);
	dg87_list_delete(pUpStreamTaskList);

	/* create task dependence */
	pTaskDep = dg87_malloc(sizeof(dg87_taskdep), DG87_TASKDEP);
	if (pTaskDep == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pTaskDep->pTaskBefore = pTaskBefore;
	pTaskDep->pTaskAfter = pTaskAfter;
	pTaskDep->type = dg87_taskdep_always;
	return pTaskDep;
}
/*****************************************************************************/
BOOL dg87_taskdep_delete(dg87_taskdep* pTaskDep)
{
	assert(pTaskDep != NULL);
	if (pTaskDep == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* remove taskDep from vectors */
	dg87_list_removeObject(
			pTaskDep->pTaskBefore->pTaskDepAfterList, pTaskDep);
	dg87_list_removeObject(
			pTaskDep->pTaskAfter->pTaskDepBeforeList, pTaskDep);

	dg87_free(pTaskDep, DG87_TASKDEP);
	return TRUE;
}
/*****************************************************************************/
BOOL dg87_taskdep_load(dg87_taskdep* pTaskDep, dg87_storage* pStorage)
{
	dg87byte b;
	assert(pTaskDep != NULL && pStorage != NULL);
	if (pTaskDep == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read dependence type */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_taskdep_setType(pTaskDep, b) == FALSE)
		return FALSE;

	return TRUE;
}
/*****************************************************************************/
BOOL dg87_taskdep_save(dg87_taskdep* pTaskDep, dg87_storage* pStorage)
{
	assert(pTaskDep != NULL && pStorage != NULL);
	if (pTaskDep == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write dependence type */
	if (dg87_storage_writeByte(pStorage, pTaskDep->type)== FALSE)
		return FALSE;
	return TRUE;
}
/*****************************************************************************/
BOOL dg87_taskdep_setType(dg87_taskdep* pTaskDep, dg87_taskdep_type type)
{
	assert(pTaskDep != NULL);
	if (pTaskDep == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTaskDep->type = type;
	return TRUE;
}

/*****************************************************************************/
dg87_taskdep_type dg87_taskdep_asciiToType(char* pStrType)
{
	assert(pStrType != NULL);
	if (pStrType == NULL)
		return dg87_taskdep_unknown;
	if (strcasecmp(pStrType, "always") == 0)
		return dg87_taskdep_always;
	else if (strcasecmp(pStrType, "onsuccess") == 0)
		return dg87_taskdep_onSuccess;
	else if (strcasecmp(pStrType, "onerror") == 0)
		return dg87_taskdep_onError;
	return dg87_taskdep_unknown;
}
/*****************************************************************************/
char* dg87_taskdep_typeToAscii(dg87_taskdep_type deptype)
{
	switch (deptype){
		case dg87_taskdep_always: return "always";
		case dg87_taskdep_onSuccess: return "onsuccess";
		case dg87_taskdep_onError: return "onerror";
		default: return "unknown"; /* to remove warning */
	}
	return NULL;
}

/*****************************************************************************/
#ifdef DEBUG
int dg87_taskdep_check(dg87_taskdep* pTaskdep, int (*print)(char*))
{
	/* check TaskBefore */
	if (pTaskdep->pTaskBefore == NULL){
		print("TaskDep : TaskBefore is NULL !\n");
		return 1;
	}
	if (dg87_list_findItem(pTaskdep->pTaskBefore->pTaskDepAfterList,
			pTaskdep) == NULL) {
		print("Taskdep : not found in TaskBefore->TaskdepAfter !\n");
		return 1;
	}
	/* check TaskAfter */
	if (pTaskdep->pTaskAfter == NULL){
		print("TaskDep : TaskAfter is NULL !\n");
		return 1;
	}
	if (dg87_list_findItem(pTaskdep->pTaskAfter->pTaskDepBeforeList,
			pTaskdep) == NULL) {
		print("Taskdep : not found in TaskAfter->TaskdepBefore !\n");
		return 1;
	}
	return 0;
}
#endif /*DEBUG*/

/*****************************************************************************/
/* End of file */
