/* file context.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <config.h>
#include <libdg87/message.h>
#include "context.h"
#include "engine.h"

#define DG87R_DATA_VERSION 1
/*
 * dg87_context_fieldArray
 */
dg87_conf_field dg87_context_fieldArray[]=
{
	{"IPPORT", DG87R_IPPORT, NULL},
	{"PID_FILE", NULL, NULL}, 
	{"USER_ACCOUNT", NULL, NULL},
	{"DATA_FILE", DG87R_DATA_FILE, NULL},
	{"BACKUP_FILE", DG87R_BACKUP_FILE, NULL},
	{"SAVE_TEMPO", NULL, NULL},
	{"DIOGENE_PASSWORD", DG87_USER_DIOGENE_PASSWORD, NULL},
	{"DG87X_ADDRESS", "localhost", NULL},
	{"DG87X_IPPORT", DG87X_IPPORT, NULL},
	{NULL, NULL, NULL}
};

/******************************************************************************/
dg87_context* dg87_context_new(char* pConfFile)
{
	dg87_context* pContext;
	if (pConfFile == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pContext = dg87_malloc(sizeof(dg87_context), DG87_CONTEXT);
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* create dg87_conf object */
	pContext->pConf = dg87_conf_new(pConfFile, dg87_context_fieldArray);
	if (pContext->pConf == NULL){
		dg87_free(pContext, DG87_CONTEXT);
		return NULL;
	}
	if (dg87_context_readConfig(pContext) == FALSE){
		printf("Error while reading configuration file :%s\n", 
				pConfFile);
		dg87_conf_delete(pContext->pConf);
		dg87_free(pContext, DG87_CONTEXT);
		return NULL;
	}
	/* create the engine */
	pContext->pEngine = dg87_engine_new(pContext);
	if (pContext->pEngine == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pContext, DG87_CONTEXT);
		return NULL;
	}
	/* create job list */
	pContext->pJobList = dg87_list_new();
	if (pContext->pJobList == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_engine_delete(pContext->pEngine);
		dg87_free(pContext, DG87_CONTEXT);
		return NULL;
	}
	/* create storage */
	pContext->pStorage = dg87_storage_new( 
			dg87_conf_getFieldValue(pContext->pConf, "DATA_FILE"),
			dg87_conf_getFieldValue(pContext->pConf, "BACKUP_FILE"),
			DG87R_NAME, DG87R_DATA_VERSION );
	if (pContext->pEngine == NULL){
		dg87_engine_delete(pContext->pEngine);
		dg87_list_delete(pContext->pJobList);
		dg87_free(pContext, DG87_CONTEXT);
		return NULL;
	}
	pContext->pServer = NULL;
	return pContext;
}
/******************************************************************************/
BOOL dg87_context_delete(dg87_context* pContext)
{
	dg87_job* pJob;

	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* delete all job's object */
	while (pContext->pJobList->count > 0){
		pJob = dg87_list_removeObjectByIndex(pContext->pJobList,0);
		dg87_job_delete(pJob);
	}
	dg87_list_delete(pContext->pJobList);
	dg87_free(pContext->dg87x_address, DG87_CHAR);

	dg87_storage_delete(pContext->pStorage);
	dg87_engine_delete(pContext->pEngine);
	dg87_free(pContext, DG87_CONTEXT);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_context_setServer(dg87_context* pContext, dg87_server* pServer)
{
	if (pContext == NULL || pServer == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pContext->pServer = pServer;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_context_makePeriodicSave(dg87_context* pContext)
{
	time_t currentTime;
	currentTime = time(NULL);
	if (currentTime > pContext->saveTime + pContext->saveTempo){
		printf("periodic save\n");
		if (dg87_context_save(pContext) == FALSE)
			dg87_daemon_log("Save Error !\n");
		pContext->saveTime = time(NULL);
	}
	return TRUE;
}
/******************************************************************************/
BOOL dg87_context_save(dg87_context* pContext)
{
	dg87_storage *pStorage;
	dg87_item *pItem;
	dg87_job *pJob;
	dg87_message *pMessage;
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* open file */
	pStorage = pContext->pStorage;
	if (dg87_storage_open(pStorage, dg87_storage_mode_write) == FALSE)
		return FALSE;

	/* --------- Jobs ---------------- */
	/* save jobs count */
	if (dg87_storage_writeUInt(pStorage, 
			pContext->pJobList->count) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	/* for each job */
	pItem = pContext->pJobList->pFirst; 
	while (pItem != NULL){
		pJob = pItem->pObject;
		pItem = pItem->pNext;
		/* save task name */
		if (dg87_storage_writeString(pStorage, pJob->taskName) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* save job name */
		if (dg87_storage_writeString(pStorage, pJob->jobName) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* save day name */
		if (dg87_storage_writeString(pStorage, pJob->dayName) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* save path */
		if (dg87_storage_writeString(pStorage, pJob->pPath) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* save argument */
		if (dg87_storage_writeString(pStorage,pJob->pArgument)== FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_job_save(pJob, pStorage) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
	} /* next job */

	/* --------- Messages -------------------- */
	/* save messages count */
	if (dg87_storage_writeUInt(pStorage, 
			pContext->pServer->pListMessage->count) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	pItem = pContext->pServer->pListMessage->pFirst;
	while (pItem != NULL){
		pMessage = pItem->pObject;
		pItem = pItem->pNext;
		if (dg87_storage_writeString(pStorage, 
				pMessage->pHost) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_storage_writeUInt(pStorage, 
				pMessage->ipPort) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_storage_writeString(pStorage, 
				pMessage->pMsgString) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
	}
	/* close file */
	dg87_storage_close(pStorage);
	return TRUE;
}
/******************************************************************************/
BOOL dg87_context_load(dg87_context* pContext)
{
	dg87uint count, ipport;
	dg87_storage *pStorage;
	char *pTaskName, *pJobName, *pDayName, *pPath, *pArgument;
	char *pString, *pString2;
	dg87_job *pJob;
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* open file */
	pStorage = pContext->pStorage;
	if (dg87_storage_open(pStorage, dg87_storage_mode_read) == FALSE)
		return FALSE;

	/* --------- Jobs ----------- */
	/* read job count */
	if (dg87_storage_readUInt(pStorage, &count) == FALSE)
		return FALSE;
	while (count > 0){
		count --;
		/* load task name */
		if (dg87_storage_readString(pStorage, &pTaskName) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* load job name */
		if (dg87_storage_readString(pStorage, &pJobName) == FALSE){
			dg87_storage_close(pStorage);
			dg87_free(pTaskName, DG87_CHAR);
			return FALSE;
		}
		/* load day name */
		if (dg87_storage_readString(pStorage, &pDayName) == FALSE){
			dg87_storage_close(pStorage);
			dg87_free(pTaskName, DG87_CHAR);
			dg87_free(pJobName, DG87_CHAR);
			return FALSE;
		}
		/* load path */
		if (dg87_storage_readString(pStorage, &pPath) == FALSE){
			dg87_storage_close(pStorage);
			dg87_free(pTaskName, DG87_CHAR);
			dg87_free(pJobName, DG87_CHAR);
			dg87_free(pDayName, DG87_CHAR);
			return FALSE;
		}
		/* load argument */
		if (dg87_storage_readString(pStorage, &pArgument) == FALSE){
			dg87_storage_close(pStorage);
			dg87_free(pTaskName, DG87_CHAR);
			dg87_free(pJobName, DG87_CHAR);
			dg87_free(pDayName, DG87_CHAR);
			dg87_free(pPath, DG87_CHAR);
			return FALSE;
		}
		/* create job*/
		pJob = dg87_context_jobAdd(pContext, 
					pTaskName, pJobName, pDayName,
					pPath, pArgument);
		dg87_free(pTaskName, DG87_CHAR);
		dg87_free(pJobName, DG87_CHAR);
		dg87_free(pDayName, DG87_CHAR);
		dg87_free(pPath, DG87_CHAR);
		dg87_free(pArgument, DG87_CHAR);
		if (pJob == NULL){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_job_load(pJob, pStorage) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
	} /* next job */

	/* --------- Messages -------------------- */
	/* read messages count */
	if (dg87_storage_readUInt(pStorage, &count) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	while (count > 0){
		count --;
		/* load host name */
		if (dg87_storage_readString(pStorage, &pString) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* load IP port */
		if (dg87_storage_readUInt(pStorage, &ipport) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* load message */
		if (dg87_storage_readString(pStorage, &pString2) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* post message */
		if (dg87_server_postMessage(pContext->pServer,
					pString, ipport, pString2) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
	}

	/* close file */
	dg87_storage_close(pStorage);
	return TRUE;
}
/******************************************************************************/
BOOL dg87_context_readConfig(dg87_context* pContext)
{
	char* pValue;
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_conf_read(pContext->pConf) == FALSE)
		return FALSE;
	/* interpret values */
	/* TODO control values */
	pContext->ipPort = atoi(dg87_conf_getFieldValue(pContext->pConf, 
				"IPPORT"));
	/* TODO use a function like dg87_server_changeIpPort */
	if (pContext->pServer != NULL)
		pContext->pServer->ipPort = pContext->ipPort;
	pContext->pPidFile = dg87_conf_getFieldValue(pContext->pConf, 
				"PID_FILE");
	pContext->dg87x_address= dg87_conf_getFieldValue(pContext->pConf, 
				"DG87X_ADDRESS");
	pContext->dg87x_ipPort = atoi(dg87_conf_getFieldValue(pContext->pConf,
				"DG87X_IPPORT"));
	pContext->pDiogenePassword = dg87_conf_getFieldValue(pContext->pConf,
				"DIOGENE_PASSWORD");
	if (pContext->pStorage != NULL) /* pStorage is null at daemon startup */
		dg87_storage_changeDataFile(pContext->pStorage,
			dg87_conf_getFieldValue(pContext->pConf,"DATA_FILE"),
			dg87_conf_getFieldValue(pContext->pConf,"BACKUP_FILE"));
	if (pContext->pServer != NULL)
		dg87_server_reload(pContext->pServer, pContext->pPidFile,
				pContext->ipPort,
				pContext->pDiogenePassword);
	/* read save tempo */
	pValue = dg87_conf_getFieldValue(pContext->pConf, "SAVE_TEMPO");
	pContext->saveTempo = (pValue == NULL)?DG87_SAVE_TEMPO:atoi(pValue);
	if (pContext->saveTempo <= 0)
		pContext->saveTempo = DG87_SAVE_TEMPO;
	return TRUE;
}
/****************************************************************************/
struct dg87_job_struct*
dg87_context_jobAdd(dg87_context* pContext, char* pDayName, char* pTaskName, 
		char* pJobName, char* pPath, char* pArgument)
{
	dg87_item* pItem;
	dg87_job* pJob;
	if (pContext == NULL || pTaskName == NULL || pJobName == NULL 
		|| pDayName == NULL || pPath == NULL || pArgument == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}

	/* error if a same job is already in the list */
	pItem = pContext->pJobList->pFirst;
	while (pItem != NULL){
		pJob = pItem->pObject;
		pItem = pItem->pNext;
		if (strcmp(pDayName, pJob->dayName) == 0 &&
			strcmp(pTaskName, pJob->taskName) == 0 &&
			strcmp(pJobName, pJob->jobName) == 0)
		{
			dg87_err_setError(DG87_ERR_SAMEJOBALREADYEXIST);
			return NULL;
		}

	}
	
	pJob = dg87_job_new(pTaskName, pJobName, pDayName, pPath, pArgument);
	if (pJob == NULL)
		return NULL;
	if (dg87_list_addObject(pContext->pJobList, pJob) == FALSE){
		dg87_job_delete(pJob);
		return NULL;
	}
	return pJob;
}
/****************************************************************************/
BOOL dg87_context_jobDelete(dg87_context* pContext, dg87_job *pJob)
{
	if (pContext == NULL || pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* TODO more think to do ! if the job run, kill it */
	if (dg87_list_removeObject(pContext->pJobList, pJob) == FALSE)
		return FALSE;
	dg87_job_delete(pJob);
	return TRUE;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_context_check(dg87_context* pContext, int (*print)(char*))
{
	dg87_item* pItem;
	/* check JobList */
	if (pContext->pJobList == NULL){
		print("Context : JobList is null !\n");
		return 1;
	}
	if (dg87_list_check(pContext->pJobList, print) != 0)
		return 1;
	pItem = pContext->pJobList->pFirst;
	while (pItem != NULL){
		if (dg87_job_check(pItem->pObject, print) != 0)
			return 1;
		pItem = pItem->pNext;
	}
	/* TODO complete verification */
	return 0;
}
#endif /*DEBUG*/

/****************************************************************************/
/* End of file */
