/* file job.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "caller.h"
#include "job.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>


extern char **environ; /* environment array */


/****************************************************************************/
BOOL dg87_job_changeState(dg87_job* pJob, dg87_job_state state)
{
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob->state = state;
	pJob->transmitState = TRUE;
	return TRUE;
}

/****************************************************************************/
BOOL dg87_job_transmitState(dg87_context* pContext, dg87_job* pJob)
{
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pJob->transmitState == TRUE){
		if (dg87_caller_transmitState(pContext,pJob) == TRUE){
			pJob->transmitState = FALSE;
			if (pJob->state == dg87_job_state_exit ||
				pJob->state ==dg87_job_state_deleted )
				pJob->state=dg87_job_state_old;
		}
	}
	return TRUE;
}

/****************************************************************************/
void dg87_job_arrayCleanup(char** argv)
{
	char** arg;
	if (argv == NULL) return;
	arg = argv;
	while (*arg != NULL){
		dg87_free(*arg, DG87_CHAR);
		arg++;
	}
	dg87_free(argv, DG87_CHAR);
}
/****************************************************************************/
char** dg87_job_splitArgument(char* pPath, char* pArgument)
{
	char ** argv = NULL;
	int argc = 0;
	int newSize;
	char** newBuffer;
	char* pBeg;
	char* pEnd;
	int argSize;

	if (pPath == NULL) return NULL;

	/* first element in array is job path */
	argv = dg87_malloc(sizeof(char*) * 2, DG87_CHAR);
	if (argv == NULL)
		return NULL;
	argv[0] = dg87_malloc(sizeof(char) * strlen(pPath) + 1, DG87_CHAR);
	if (argv[0] == NULL) {
		dg87_free(argv, DG87_CHAR);
		return NULL;
	}
	strcpy(argv[0], pPath);
	argv[1] = NULL; /* terminator */
	argc = 1;

	if (pArgument == NULL)
		return argv;

	pEnd = pArgument;
	while (1){

		/* skip separators */
		pBeg = pEnd; 
		while (*pBeg == ' ' && *pBeg != '\0')
			pBeg++;
		if (*pBeg == '\0') break;

		/* if character is quote */
		pEnd = pBeg + 1;
		if (*pBeg == '\"'){
			/* skip until next quote */
			while (*pEnd != '\"' && *pEnd != '\0')
				pEnd++;
			if (*pEnd == '\0'){
				dg87_job_arrayCleanup(argv);
				dg87_err_setError(DG87_ERR_ARGUMENTSYTANXERROR);
				return NULL;
			}
			/* remove first quote */
			pBeg++; pEnd++;
			argSize = pEnd - pBeg - 1;
		} else {
			/* else skip word */
			while (*pEnd != ' ' && *pEnd != '\0')
				pEnd++;
			argSize = pEnd - pBeg;
		}

		/* increase argument array size */
		newSize = argc + 1;
		newBuffer = realloc(argv, sizeof(char*) * (newSize + 1));
		if (newBuffer == NULL){
			dg87_job_arrayCleanup(argv);
			dg87_err_setError(DG87_ERR_NULLPARAMETER);
			return NULL;
		}
		argv = newBuffer;
		argc = newSize;
		argv[argc] = NULL; /* terminator */

		/* add argument in array */
		argv[argc - 1] = 
			dg87_malloc(sizeof(char) * (argSize + 1), DG87_CHAR);
		if (argv[argc - 1] == NULL){
			dg87_job_arrayCleanup(argv);
			dg87_err_setError(DG87_ERR_NULLPARAMETER);
			return NULL;
		}
		memset(argv[argc-1],0, sizeof(char) * (argSize + 1));
		strncpy(argv[argc - 1], pBeg, argSize);
	}

	return argv;
}
/****************************************************************************/
BOOL dg87_job_run(dg87_job* pJob)
{
	char** argv = NULL;

	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}

	argv = dg87_job_splitArgument(pJob->pPath, pJob->pArgument);
	if (argv == NULL)
		return FALSE;

	pJob->pid = fork();
	if (pJob->pid == -1) {
		/* can't fork */
		dg87_err_setError(DG87_ERR_CANTFORK);
		return FALSE;
	}
	
	if (pJob->pid == 0) {
		/* this code is executed by the child */

		/* define environment variables */
		setenv("DG87_DAY", pJob->dayName,1);
		setenv("DG87_TASK", pJob->taskName,1);
		setenv("DG87_JOB", pJob->jobName,1);

		/* execute job */
		execve(pJob->pPath, argv, environ);
		/* execve return only if an error occures */
		/* kill the child process */
		abort();
		return FALSE;
	}

	dg87_job_arrayCleanup(argv);
	dg87_job_changeState(pJob, dg87_job_state_running);

	return TRUE;
}
/****************************************************************************/
dg87_job* dg87_job_new(char* pTaskName, char* pJobName, char* pDayName,
		char* pPath, char* pArgument)
{
	dg87_job* pJob;
	if (pTaskName == NULL || pJobName == NULL || pDayName == NULL 
			|| pPath == NULL || pArgument == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	/* argument's length could be zero length */
	if (strlen(pTaskName)==0 || strlen(pJobName) == 0 
			|| strlen(pDayName) == 0 || strlen(pPath) == 0)
	{
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	if (strlen(pTaskName) >= DG87_TASK_NAME_LENGTH ||
			strlen(pJobName) >= DG87_JOB_NAME_LENGTH ||
			strlen(pDayName) >= DG87_DAY_NAME_LENGTH)
	{
		dg87_err_setError(DG87_ERR_NAMETOLONG);
		return FALSE;
	}
	/* allocate memory for job */
	pJob = dg87_malloc(sizeof(dg87_job), DG87_JOB);
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	strncpy(pJob->jobName, pJobName, DG87_JOB_NAME_LENGTH); 
	strncpy(pJob->taskName, pTaskName, DG87_TASK_NAME_LENGTH); 
	strncpy(pJob->dayName, pDayName, DG87_DAY_NAME_LENGTH); 
	/* copy path */
	pJob->pPath = dg87_strdup(pPath, DG87_CHAR);
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pJob, DG87_JOB);
		return NULL;
	}
	/* copy argument */
	pJob->pArgument = dg87_strdup(pArgument, DG87_CHAR);
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pJob->pPath, DG87_CHAR);
		dg87_free(pJob, DG87_JOB);
		return NULL;
	}
	pJob->state = dg87_job_state_uninitialize;
	pJob->pid = 0;
	pJob->status = -1;

	return pJob;
}

/****************************************************************************/
BOOL dg87_job_delete(dg87_job* pJob)
{
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pJob->pPath != NULL)
		dg87_free(pJob->pPath, DG87_CHAR);
	if (pJob->pArgument != NULL)
		dg87_free(pJob->pArgument, DG87_CHAR);
	dg87_free(pJob, DG87_JOB);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_load(dg87_job* pJob, dg87_storage* pStorage)
{
	dg87byte b;
	dg87uint ui;
	dg87int i;
	/* TODO load state, pid, status, transmitState */
	/* read 'state' property */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	pJob->state = b;
	/* read 'pid' property */
	if (dg87_storage_readUInt(pStorage, &ui) == FALSE)
		return FALSE;
	pJob->pid = ui;
	/* read 'status' property */
	if (dg87_storage_readInt(pStorage, &i) == FALSE)
		return FALSE;
	pJob->status = i;
	/* read 'transmitState' property */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	pJob->transmitState = b;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_save(dg87_job* pJob, dg87_storage* pStorage)
{
	/* TODO save state, pid, status, transmitState */
	/* 'state' property */
	if (dg87_storage_writeByte(pStorage, pJob->state) == FALSE)
		return FALSE;
	/* 'pid' property */
	if (dg87_storage_writeUInt(pStorage, pJob->pid) == FALSE)
		return FALSE;
	/* 'status' property */
	if (dg87_storage_writeUInt(pStorage, pJob->status) == FALSE)
		return FALSE;
	/* 'transmitState' property */
	if (dg87_storage_writeByte(pStorage, pJob->transmitState) == FALSE)
		return FALSE;
	return TRUE;
}
/****************************************************************************/
char* dg87_job_stateToAscii(dg87_job_state state)
{
	switch (state){
		case dg87_job_state_readytorun:
			return "ready to run";
		case dg87_job_state_deleted:
			return "deleted";
		case dg87_job_state_running:
			return "running";
		case dg87_job_state_exit:
			return "exit";
		case dg87_job_state_old:
			return "old";
		default:
			return "?";
	}
	return "?";
}
/****************************************************************************/
#ifdef DEBUG
int dg87_job_check(dg87_job* pJob, int (*print)(char*))
{
	if (strlen(pJob->jobName) == 0){
		print("Job : job's name is zero length !\n");
		return 1;
	}
	if (strlen(pJob->taskName) == 0){
		print("Job : task's name is zero length !\n");
		return 1;
	}
	if (strlen(pJob->dayName) == 0){
		print("Job : day's name is zero length !\n");
		return 1;
	}
	if (pJob->pPath == NULL){
		print("Job : path is NULL !\n");
		return 1;
	}
	if (strlen(pJob->pPath) == 0){
		print("Job : path's is zero length !\n");
		return 1;
	}
	if (pJob->pArgument == NULL){
		print("Job : argument is NULL !\n");
		return 1;
	}

	return 0;
}
#endif /*DEBUG*/


/****************************************************************************/
/* End of file */
