/* file main.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include <config.h>
#include "engine.h"
#include <libdg87/server.h>
#include <libdg87/daemon.h>
#include <libdg87/interpret.h>
#include "cmdjob.h"
#include "cmdengine.h"

extern char *optarg;
extern int optind;
extern int opterr;

/* local prototypes */
int cmd_shutdown(dg87_connection* pConnection);
#ifdef DEBUG	
int cmd_check(dg87_connection* pConnection);
#endif

/* array for Diogene's commands */
static dg87_interpret_cmd pCmdArray[] = {

#ifdef DEBUG
	{"check", cmd_check,"Check data (debug release only)", 
		"",dg87_command_generic, dg87_command_access_consult},
#endif

	{"ei", cmd_ei, "Engine Information","B:S:E:",
		dg87_command_engine, dg87_command_access_consult},
	{"ja", cmd_ja, "Job Add", 
		"",dg87_command_job, dg87_command_access_admin},
	{"jc", cmd_jc, "Job Count", "",
		dg87_command_job, dg87_command_access_consult},
	{"ji", cmd_ji, "Job Information", "B:S:E:",
		dg87_command_job, dg87_command_access_consult},
	{"jk", cmd_jk, "Job Kill", "",
		dg87_command_job, dg87_command_access_admin},
	{"jl", cmd_jl, "Job List", "sB:S:E:I:N:",
		dg87_command_job, dg87_command_access_consult},
	
	{"save", cmd_save,"Save", "",
		dg87_command_engine, dg87_command_access_manage},
	{"shutdown", cmd_shutdown,"Stop the execution of dg87r", "",
		dg87_command_engine, dg87_command_access_manage},

	{NULL,NULL,NULL, NULL,0, 0}
};
/******************************************************************************/
static void usage(const char* name){
	char* msg =
		"Diogene87 daemon for remote jobs execution.\n\n"\
		"Usage : %s [options]\n\n" \
		"Options :\n" \
		" -C path   : path of the configuration file\n" \
		" -d        : start process as daemon\n" \
		" -N        : don't try to load database at startup\n" \
		" -I        : create database file and exit\n" \
		"\n"; 
	fprintf(stderr,msg, name);

}
/******************************************************************************/
#ifdef DEBUG	
/* TODO find a way to remove this local variable */
static dg87_connection *pLocalConnection;
/* callback function for messages printing */
int callBack(char* message){
	dg87_connection_printf(pLocalConnection, message);
	return 0;
}
int cmd_check(dg87_connection* pConnection)
{
	dg87_context* pContext;
	

	if (pConnection == NULL) return 1;
	if (pConnection->pServer->pData == NULL){
		dg87_connection_printf(pConnection,
				"ERROR : pData is NULL !\n");
		return 1;
	}
	pLocalConnection = pConnection;
	pContext = pConnection->pServer->pData;
	if (dg87_context_check(pContext, callBack) != 0){
		dg87_connection_printf(pConnection,"ERROR !\n");
		return 1;
	}
	
	dg87_connection_printf(pConnection,"All is right ;-)\n", PACKAGE);
	return 0;
}
#endif
/******************************************************************************/
int cmd_shutdown(dg87_connection* pConnection)
{
	/*TODO what to do when job are running : 
	 * abort them or refuse to quit ? */
	dg87_connection_printf(pConnection,
			"%s will stop...\n", DG87R_NAME);
	return -1;
}
/******************************************************************************/
static char* getBaseName(const char* path){
	char *baseName = 0;
	if ((baseName = strrchr(path,'/')) != 0)
		++baseName;
	else
		baseName = (char*) path;
	return baseName;
}
/******************************************************************************/
int signal_function(dg87_server* pServer,int signo)
{
	dg87_context *pContext;
	if (pServer == NULL)
		return FALSE;
	pContext = pServer->pData;
	switch (signo)
	{
		case SIGHUP:
			dg87_context_readConfig(pContext);
			dg87_daemon_log("SIGNAL SIGHUP");
			break;
		case SIGTERM:
			dg87_daemon_log("SIGNAL SIGTERM");
			break;
		case SIGINT:
			dg87_daemon_log("SIGNAL SIGINT");
			break;
		case SIGPIPE:
			dg87_daemon_log("SIGNAL SIGPIPE");
			break;
		case SIGUSR1: /* save database */
			dg87_daemon_log("Signal SIGUSR1 : save database");
			if (dg87_context_save(pContext) == FALSE)
				dg87_daemon_log("Save error !");
			else 
				dg87_daemon_log("Save done !");
			break;
		default:
			dg87_daemon_log("SIGNAL NOT HANDLED");
			break;
	}
	return FALSE;
}
/******************************************************************************/
int request_function(dg87_interpret* pInterpret, dg87_connection* pConnection)
{
	if (pConnection == NULL || pInterpret == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE; /* TODO what to return */
	}
	return dg87_interpret_execute(pInterpret, pConnection);
}

/******************************************************************************/
BOOL work_function(dg87_server* pServer)
{
	dg87_context* pContext;
	if (pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pContext = pServer->pData;
	return dg87_engine_run(pContext->pEngine, pServer);
}
/******************************************************************************/
int main(int argc, char* argv[])
{
	int optch;
	static char optstring[] = "C:hvdNI";
	char* baseName= getBaseName(argv[0]);
	dg87_daemon* pDaemon = NULL;
	int res=1;
	BOOL startDaemon = FALSE;
	BOOL newDatabase = FALSE;
	BOOL initDatabase = FALSE;
	char* pConfFile = NULL; 

	dg87_context* pContext;
	dg87_server* pServer;
	void* pData;
	struct stat sb;
	
	/* interpret options */
	while ((optch = getopt(argc, argv, optstring)) != -1){
		switch (optch){
			case 'I': /* init database */
				initDatabase = TRUE;
				break;
			case 'N': /* new database */
				newDatabase = TRUE;
				break;
			case 'C': /* configuration file */
				pConfFile = dg87_strdup(optarg,DG87_CHAR);
				break;
			case 'h': /* help */
				usage(baseName);
				return 1;
			case 'v': /* version */
				fprintf(stderr,"%s-%s\n", PACKAGE, VERSION);
				return 1;
			case 'd': /* start as daemon */
				startDaemon = TRUE;
				break;

			default:
				fprintf(stderr,"Use -h for help.\n");
				return 1;
		}
	}

	/* create the context object */
	if (pConfFile == NULL)
		pConfFile = dg87_strdup(DG87R_CONFIGFILE, DG87_CHAR);
	if ((pContext = dg87_context_new(pConfFile)) == NULL){
		dg87_daemon_log("Can't create context !");
		exit(1);
	}
	pData = pContext;
	dg87_free(pConfFile, DG87_CHAR);

	if (startDaemon == TRUE){
		pDaemon = dg87_daemon_new(DG87X_NAME, 
				dg87_conf_getFieldValue(pContext->pConf,
					"USER_ACCOUNT"));
		if (pDaemon == NULL){
			fprintf(stderr,"Can't create daemon object\n");
			dg87_context_delete(pContext);
			return 1;
		}
	
		res = dg87_daemon_start(pDaemon);
		if (res < 0){
			fprintf(stderr,"Can't daemonize !\n");
			dg87_daemon_delete(pDaemon);
			dg87_context_delete(pContext);
			return 1;
		}
			
		if (res == 0){
			/* the parent process terminate here */
			dg87_daemon_delete(pDaemon);
			dg87_context_delete(pContext);
			return 0;
		}
	}


	/* create the server object */
	pServer = dg87_server_new(PACKAGE, DG87R_NAME, VERSION, 
			pContext->ipPort,
			work_function, signal_function, pData, pCmdArray,
			pContext->pPidFile,
			pContext->pDiogenePassword);
	if (pServer == NULL){
		fprintf(stderr,"Can't create server object\n");
		dg87_context_delete(pContext);
		return 1;
	}
	if (dg87_context_setServer(pContext, pServer) == FALSE){
		fprintf(stderr, "Can't set server object\n");
		return 1;
	}
	/* option -I : init database and exit */
	if (initDatabase == TRUE){
		if (stat(pContext->pStorage->pDataFile, &sb) != -1 ||
				errno != ENOENT){
			fprintf(stderr, "Database already exists : %s\n",
				pContext->pStorage->pDataFile);
			dg87_server_delete(pServer);
			dg87_context_delete(pContext);
			return 1;
		}
		fprintf(stderr, "Create database :%s\n", 
				pContext->pStorage->pDataFile);
		if (dg87_context_save(pContext) == FALSE){
			fprintf(stderr,"Can't save database '%s' ! %s\n",
					pContext->pStorage->pDataFile,
					dg87_err_getErrorStr());
			dg87_server_delete(pServer);
			dg87_context_delete(pContext);
			return 1;
		}
		dg87_server_delete(pServer);
		dg87_context_delete(pContext);
		return 0;
	}

	/* load database */
	if (newDatabase == TRUE) {
		fprintf(stderr, "Don't load database, because -N option "
				"was specified.\n");
	} else {
		fprintf(stderr, "Load database :%s\n", 
				pContext->pStorage->pDataFile);
		if (dg87_context_load(pContext) == FALSE){
			fprintf(stderr,"Can't load database '%s' ! %s\n",
					pContext->pStorage->pDataFile,
					dg87_err_getErrorStr());
			dg87_context_delete(pContext);
			return 1;
		}
	}

	fprintf(stderr, "server ready.\n");
	if (dg87_server_run(pServer) == TRUE){
		fprintf(stderr, "Save database :%s\n", 
				pContext->pStorage->pDataFile);
		if (dg87_context_save(pContext) == FALSE){
			fprintf(stderr,"Can't save database '%s' ! %s\n",
					pContext->pStorage->pDataFile,
					dg87_err_getErrorStr());
			dg87_server_delete(pServer);
			dg87_context_delete(pContext);
			return 1;
		}
	}

	dg87_server_delete(pServer);
	dg87_context_delete(pContext);
	if (startDaemon == TRUE){
		dg87_daemon_stop(pDaemon);
		dg87_daemon_delete(pDaemon);
	}

	return 0;
}
/******************************************************************************/
/* End of file */
