/* file cmdjob.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "cmdjob.h"
#include "util.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>

/******************************************************************************/
/* xja
 * eXp Job Add 
 * Usage : xja DayName TaskName JobName Description \
 *             Path Argument startTempo endTempo \
 *             waitAllDependences onErrorContinue
 */
int cmd_xja(dg87_connection* pConnection)
{
	dg87_command* pCommand;
	dg87_day* pDay;
	dg87_task* pTask;
	dg87_job* pJob;
	BOOL value;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 10){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xja DayName TaskName JobName Description Path " \
			"Argument startTempo endTempo " \
			"waitAllDependences onErrorContinue\n");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}	
	/* find the day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;
	/* find the task */
	pTask = dg87_util_findTask(pConnection, pDay, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* control locked property */
	if (pDay->locked == FALSE){
		dg87_connection_printf(pConnection, 
			"Can't add a job to the task '%s' because " \
			"day '%s' is unlocked !\n",
			pTask->name, pDay->name);
		return 1;
	}
	/* create the job */
	pJob = dg87_task_jobAdd(pTask, pCommand->argv[2]);
	if (pJob == NULL){
		dg87_connection_printf(pConnection,
				"Can't add a job '%s' for the task '%s' ! %s\n",
				pCommand->argv[2], pTask->name,
				dg87_err_getErrorStr());
		return 1;
	}
	/* set description */
	if (dg87_job_setDescription(pJob, pCommand->argv[3]) == FALSE){
		dg87_connection_printf(pConnection,
				"Can't add a job '%s' for the task '%s' ! %s\n",
				pJob->name, pTask->name);
		dg87_task_jobDelete(pTask, pJob);
		return 1;
	}
	/* set path */
	if (dg87_job_setPath(pJob, pCommand->argv[4]) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set path for job '%s' of task '%s' !\n",
			pJob->name, pTask->name);
		dg87_task_jobDelete(pTask, pJob);
		return 1;
	}
	/* set argument */
	if (dg87_job_setArgument(pJob, pCommand->argv[5]) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set argument for job '%s' of task '%s' !\n",
			pJob->name, pTask->name);
		dg87_task_jobDelete(pTask, pJob);
		return 1;
	}
	/* set tempo */
	if (dg87_job_setTempo(pJob, atoi(pCommand->argv[6]),
				atoi(pCommand->argv[7])) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set tempo for job '%s' of task '%s' !\n",
			pJob->name, pTask->name);
		dg87_task_jobDelete(pTask, pJob);
		return 1;
	}
	/* change waitAllDependences property */
	if ((value = dg87_asciiToBool(pCommand->argv[8])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s' for WaitAllDependences, "\
		        "use true or false !\n",
			pCommand->argv[8]);
		return 1;
	}
	if (dg87_job_setWaitAllDependences(pJob, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set WaitAllDependences for "\
			"job '%s' of task '%s' !\n", pJob->name, pTask->name);
		dg87_task_jobDelete(pTask, pJob);
		return 1;
	}

	/* change OnErrorContinue property */
	if ((value = dg87_asciiToBool(pCommand->argv[9])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s' for OnErrorContinue, "\
		        "use true or false !\n", pCommand->argv[9]);
		return 1;
	}
	if (dg87_job_setOnErrorContinue(pJob, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set OnErrorContinue for "\
			"job '%s' of task '%s' !\n", pJob->name, pTask->name);
		dg87_task_jobDelete(pTask, pJob);
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* cmd_xjc
 * Usage : xjc DayName TaskName
 * eXp Task Job Count
 * return the number of jobs in the TASK.
 */
int cmd_xjc(dg87_connection* pConnection)
{
	dg87_task* pTask;
	dg87_day* pDay;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xjc DayName TaskName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findTask(pConnection, pDay, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* return jobs count */
	dg87_connection_printf(pConnection,"%i\n", pTask->pJobVector->count);
	return 0;
}
/******************************************************************************/
/* cmd_xjl
 * xjl [options] DayName TaskName
 * eXp Job List
 * Options
 * -d : print description of job 
 * -t : print task name of job
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first job to display (first index is 0)
 * -Nnumber : number of job to display (no limit if -N < 1)
 * -s : state of job
 */
int cmd_xjl(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "sdtB:S:E:I:N:";*/
	int optch, iopt;
	char* optarg;
	int start = 0; /* index of the first task to display */
	int number = 0;/* number of TASKs to display */
	int count = 0; /* number of TASK actually display */
	dg87_element* pElement = NULL;
	dg87_task* pTask;
	dg87_job* pJob;
	BOOL showDescription = FALSE;
	BOOL showTaskName = FALSE;
	BOOL showState = FALSE;
	dg87_day* pDay;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xjl [options] DayName TaskName");
		return 1;
	}
	/* find the day */
	if ((pDay = dg87_util_findDay(pConnection, 
				pCommand->argv[0])) == NULL) 
		return 1;
	/* find the task */
	if ((pTask = dg87_util_findTask(pConnection, pDay,
				pCommand->argv[1])) == NULL)
		return 1;
	
	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg); 
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg); 
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg); 
				break;
			case 'I': /* index of the first task to display */
				start = atoi(optarg); 
				break;
			case 'N': /* number of task to display */
				number = atoi(optarg); 
				break;
			case 'd': /* description */
				showDescription = TRUE; 
				break;
			case 't': /* Task */
				showTaskName = TRUE; 
				break;
			case 's': /* show state*/
				showState = TRUE;
				break;
				
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option !\n");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* find the first job to display */
	pElement = NULL;
	if (start > 0)
		pElement = dg87_vector_getElementByIndex(
			pTask->pJobVector, start);
	else
		pElement = pTask->pJobVector->pFirst;

	/* create the list */
	count = 0;
	while (pElement != NULL){
		/* reach the limit of job to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pJob = pElement->pObject;

		/* response */
		dg87_connection_printf(pConnection, pBeg);
		if (showTaskName ==  TRUE){
			dg87_connection_printf(pConnection, "%s%s",
				pJob->pTask->name, pSep);
		}
		dg87_connection_printf(pConnection, pJob->name);
		/* print state */
		if (showState == TRUE)
			dg87_connection_printf(pConnection, "%s%s", pSep,
				dg87_job_stateToAscii(pJob->state));
		/* print description */
		if (showDescription == TRUE) {
			dg87_connection_printf(pConnection, "%s%s",
				pSep, pJob->description);
		} 
		dg87_connection_printf(pConnection, "%s\n",pEnd);
		pElement = pElement->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}

/******************************************************************************/
/* xji
 * exploit job information 
 * get information for a job
 * Usage : xji [options] DayName TaskName JobName
 * Options :
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int cmd_xji(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:";*/
	int optch, iopt;
	char* optarg;
	dg87_task *pTask;
	dg87_job *pJob;
	dg87_day *pDay;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	
	/* control number of arguments */
	if (pCommand->argc != 3){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xji [options] DayName TaskName JobName");
		return 1;
	}
	/* find the day */
	if ((pDay = dg87_util_findDay(pConnection, 
				pCommand->argv[0])) == NULL)
		return 1;

	/* find the task */
	if ((pTask = dg87_util_findTask(pConnection, pDay,
				pCommand->argv[1])) == NULL)
		return 1;
	
	/* find the job */
	pJob = dg87_util_findJob(pConnection,
			pTask, pCommand->argv[2]);
	if (pJob == NULL)
		return 1;

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg); 
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg); 
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg); 
				break;
				
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option !\n");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* job name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Job Name", pSep, pJob->name, pEnd);
	/* Task name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Task Name", pSep, pJob->pTask->name, pEnd);
	/* day name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Day Name", pSep, pJob->pTask->pDay->name, pEnd);
	/* job desription */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Description", pSep, pJob->description, pEnd);
	/* Task Target */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Task Target", pSep, 
		(pJob->pTask->pQueue==NULL) ? "none" :
	       		pJob->pTask->pQueue->pTarget->name, pEnd);
	/* Target Queue */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Target Queue", pSep, 
		(pJob->pTask->pQueue==NULL) ? "none" :
	       		pJob->pTask->pQueue->name, pEnd);
	/* job Path */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Path", pSep, 
		(pJob->pPath==NULL) ? "none" : pJob->pPath, pEnd);
	/* argument */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Argument", pSep, 
		(pJob->pArgument==NULL) ? "none" : pJob->pArgument, pEnd);
	/* start tempo */
	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg,"Start Tempo",pSep, pJob->startTempo, pEnd);
	/* end tempo */
	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg,"End Tempo",pSep, pJob->endTempo, pEnd);
	/* waitAllDependences*/
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Wait All Dependences", pSep,
		dg87_boolToAscii(pJob->waitAllDependences), pEnd);
	/* onErrorContinue */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "On Error Continue", pSep,
		dg87_boolToAscii(pJob->onErrorContinue), pEnd);
	/* enabled */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Enabled", pSep,
		dg87_boolToAscii(pJob->enabled), pEnd);
	/* Task's state */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "State", pSep, 
		dg87_job_stateToAscii(pJob->state), pEnd);
	/* exit code */
	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg,"Exit code",pSep, pJob->exitCode, pEnd);
	
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}
/******************************************************************************/
/* xjse
 * exploit job set enabled 
 * change the enabled property of the job
 * Usage : xjse DayName TaskName JobName BooleanValue 
 */
int cmd_xjse(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_task* pTask;
	dg87_job* pJob;
	dg87_command* pCommand;
	BOOL value;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xjse DayName TaskName JobName BooleanValue\n");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the day */
	if ((pDay = dg87_util_findDay(pConnection, pCommand->argv[0])) == NULL)
		return 1;
	/* find the task */
	if ((pTask = dg87_util_findTask(pConnection, pDay,
				pCommand->argv[1])) == NULL)
		return 1;
	/* find the job */
	if ((pJob = dg87_util_findJob(pConnection,
			pTask, pCommand->argv[2])) == NULL)
		return 1;
	/* control locked property */
	if (pDay->locked == TRUE){
		dg87_connection_printf(pConnection, 
			"Can't update the job '%s' of the task '%s' because " \
		        "day '%s' is locked !\n",
			pJob->name, pTask->name, pDay->name);
		return 1;
	}
	/* interpret boolean value */
	if ((value = dg87_asciiToBool(pCommand->argv[3])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s', use true or false !\n",
			pCommand->argv[3]);
		return 1;
	}
	if (dg87_job_setEnabled(pJob, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't change Enabled property for job '%s' "\
			"of the task '%s' !\n",
			pJob->name, pJob->pTask->name);
		return 1;
	}

	return 0;
}

/******************************************************************************/
/* xju
 * exploit job update 
 * change the state of the job
 * Usage : xju DayName TaskName JobName State [exitCode]
 */
int cmd_xju(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_task* pTask;
	dg87_job* pJob;
	dg87_job_state state;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc < 4 || pCommand->argc > 5){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xju DayName TaskName JobName State [exitCode]\n");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the day */
	if ((pDay = dg87_util_findDay(pConnection, pCommand->argv[0])) == NULL)
		return 1;
	/* find the task */
	if ((pTask = dg87_util_findTask(pConnection, pDay,
				pCommand->argv[1])) == NULL)
		return 1;
	/* find the job */
	if ((pJob = dg87_util_findJob(pConnection,
			pTask, pCommand->argv[2])) == NULL)
		return 1;
	/* control locked property */
	if (pDay->locked == TRUE){
		dg87_connection_printf(pConnection, 
			"Can't update the job '%s' of the task '%s' because " \
		        "day '%s' is locked !\n",
			pJob->name, pTask->name, pDay->name);
		return 1;
	}
	/* TODO control actual state before changing it :
	 * the state of a job that is terminated could not be changed
	 * because all dependences after it are finished */
	state = dg87_job_asciiToState(pCommand->argv[3]);

	switch (state){
		case dg87_job_state_ignore :
		case dg87_job_state_ignored :
		case dg87_job_state_canceled :
		case dg87_job_state_readytorun :
		case dg87_job_state_running :
		case dg87_job_state_deleted :
			dg87_job_setState(pJob, state, -1);
			break;
		case dg87_job_state_terminated : 
			dg87_job_setState(pJob, dg87_job_state_terminated, 0);
			break;
		case dg87_job_state_exit:
			if (pCommand->argc != 5){
				dg87_connection_printf(pConnection,
					"exitCode must be specified.\n");
				return 1;
			}
			printf("exit\n");
			dg87_job_setState(pJob, dg87_job_state_exit,
				atoi(pCommand->argv[4]));
			break;	
		case dg87_job_state_aborted :
			if (pCommand->argc != 5){
				dg87_connection_printf(pConnection,
					"exitCode must be specified.\n");
				return 1;
			}
			dg87_job_setState(pJob, dg87_job_state_aborted,
				atoi(pCommand->argv[4]));
			break;
		default :
			dg87_connection_printf(pConnection,
				"Incorrect state value '%s'.\n",
				pCommand->argv[3]);
			return 1;

	}
	return 0;
}
/******************************************************************************/
/* xjda
 * eXp Job Dependence Add 
 * Usage : xjda DayName TaskName JobBeforeName JobAfterName Type
 */
int cmd_xjda(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_task *pTask;
	dg87_job *pJobAfter, *pJobBefore;
	dg87_jobdep* pJobDep;
	dg87_jobdep_type type;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 5){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xjda DayName TaskName JobBeforeName JobAfterName Type"
			);
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the day*/
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;
	/* find the task */
	pTask = dg87_util_findTask(pConnection, pDay, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* control locked property */
	if (pDay->locked == FALSE){
		dg87_connection_printf(pConnection, 
			"Can't add a job dependence in task '%s' " \
			"because day '%s' is unlocked !\n",
			pTask->name, pDay->name);
		return 1;
	}
	/* find the jobs */
	pJobBefore = dg87_util_findJob(pConnection,pTask,pCommand->argv[2]);
	if (pJobBefore == NULL) return 1;
	pJobAfter = dg87_util_findJob(pConnection,pTask, pCommand->argv[3]);
	if (pJobAfter == NULL) return 1;
	/* get dependence type */
	if ((type = dg87_jobdep_asciiToType(pCommand->argv[4])) == 
			dg87_jobdep_type_unknown){
		dg87_connection_printf(pConnection,
			"Incorrect dependence type '%s' ! \n",
			pCommand->argv[4]);
		return 1;
	}
	/* create the dependence */
	pJobDep = dg87_task_jobdepAdd(pTask, pJobBefore, pJobAfter);
	if (pJobDep == NULL){
		dg87_connection_printf(pConnection,
			"Can't add a dependence between job '%s' and " \
			"job '%s' in task '%s' ! %s\n",
				pJobBefore->name, pJobAfter->name, 
				pTask->name, dg87_err_getErrorStr());
		return 1;
	}
	/* set dependence type */
	if (dg87_jobdep_setType(pJobDep, type) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set type for dependence between job '%s' and " \
			"job '%s' in task '%s' ! %s\n",
				pJobBefore->name, pJobAfter->name, 
				pTask->name, dg87_err_getErrorStr());
		dg87_task_jobdepDelete(pTask, pJobDep);
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* xjdl
 * eXploit Job Dependence List
 * Usage : xjdl [options] DayName TaskName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first taskdep to display (first index is 0)
 *  -Nnumber : number of jobdep to display (no limit if -N < 1)
 *  -a jobName : only jobdep after this job
 *  -b jobName : only jobdep before this job 
 *  -t : print dependence type
 *  -s : print dependence's state
 */
int cmd_xjdl(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "stB:S:E:I:N:b:a:";*/
	int optch, iopt;
	char* optarg;
	int start = 0; /* index of the first job to display */
	int number = 0;/* number of jobs to display */
	int count = 0; /* number of job actually display */
	dg87_item* pItem= NULL;
	int filter = 0; /* 0 = no; 1 = before; 2 = after */
	char* pJobFilterName = NULL;
	dg87_job* pJobFilter = NULL;
	dg87_task* pTask = NULL;
	dg87_day* pDay = NULL;
	dg87_jobdep* pJobDep;
	BOOL showDepType = FALSE;
	BOOL showDepState = FALSE;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xjdl [options] DayName TaskName");
		return 1;
	}

	/* find the day*/
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findTask
		(pConnection, pDay, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first job to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of job to display */
				number = atoi(optarg);
				break;
			case 'b': /* only job before */
				pJobFilterName = strdup(optarg);
				filter = 1;
				break;
			case 'a': /* only job after */
				pJobFilterName = strdup(optarg);
				filter = 2;
				break;
			case 't': /* type of dependence*/
				showDepType = TRUE;
				break;
			case 's': /* state of dependence*/
				showDepState = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option\n");
				if (pJobFilterName != NULL)
					free(pJobFilterName);
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* find the filter job */
	if (filter > 0){
		pJobFilter = dg87_util_findJob(pConnection, 
				pTask, pJobFilterName);
		if (pJobFilter == NULL){
			free(pJobFilterName);
			free(pSep); free(pBeg); free(pEnd);
			return 1;
		}
	}
	
	/* find the first JobDep to display */
	pItem = NULL;
	if (start > 0)
		pItem = dg87_list_getItem(pTask->pJobDepList, start);
	else
		pItem = pTask->pJobDepList->pFirst;

	/* create the list */
	count = 0;
	while (pItem != NULL){
		/* reach the limit of job to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pJobDep = pItem->pObject;
		if (filter == 0 ||
			(filter == 1 && pJobDep->pJobAfter == pJobFilter) ||
			(filter == 2 && pJobDep->pJobBefore == pJobFilter)){
			/* print job before and after */
			dg87_connection_printf(pConnection,"%s%s%s%s", pBeg,
					pJobDep->pJobBefore->name, pSep, 
					pJobDep->pJobAfter->name);
			/* print job dependence type */
			if (showDepType == TRUE){
				dg87_connection_printf(pConnection,
					"%s%s", pSep,
					dg87_jobdep_typeToAscii
						(pJobDep->type));
			}
			/* print task dependence's state */
			if (showDepState == TRUE){
				dg87_connection_printf(pConnection,
					"%s%s", pSep, 
					dg87_jobdep_stateToAscii(
						pJobDep->state));
			}

			dg87_connection_printf(pConnection,"%s\n", pEnd);
		}
		pItem = pItem->pNext;
	}
	/* free memory */
	if (pJobFilterName != NULL) free(pJobFilterName);
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}

/******************************************************************************/
/* xjdc
 * eXp Job Dependence Count
 * Usage : xjdc TaskName
 */
int cmd_xjdc(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_task *pTask;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xjdc DayName TaskName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the day*/
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;
	
	/* find the task */
	pTask = dg87_util_findTask(pConnection, pDay, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	dg87_connection_printf(pConnection,"%i\n", pTask->pJobDepList->count);
	return 0;
}

/******************************************************************************/
/* xjdu
 * eXp Job Dependence Update 
 * Usage : xtdu DayName TaskName JobBefore JobAfter Type NewState
 */
int cmd_xjdu(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_task* pTask;
	dg87_job* pJobBefore;
	dg87_job* pJobAfter;
	dg87_jobdep* pJobdep = NULL;
	dg87_jobdep_type type;
	dg87_jobdep_state newState;
	dg87_command* pCommand;
	dg87_item* pItem;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 6){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
		"xjdu DayName Taskname JobBefore JobAfter Type NewState");
		return 1;
	}
	/* find the day*/
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;
	/* control locked property */
	if (pDay->locked == TRUE){
		dg87_connection_printf(pConnection, 
			"Can't update task dependence because " \
			"day '%s' is locked !\n",
			pDay->name);
		return 1;
	}

	/* find the task */
	pTask = dg87_util_findTask(pConnection, pDay, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* find the job before */
	pJobBefore = dg87_util_findJob(pConnection, pTask, pCommand->argv[2]);
	if (pJobBefore == NULL) return 1;

	/* find the job after */
	pJobAfter = dg87_util_findJob(pConnection, pTask, pCommand->argv[3]);
	if (pJobAfter == NULL) return 1;

	/* find dependence type */
	type = dg87_jobdep_asciiToType(pCommand->argv[4]);
	if (type == dg87_jobdep_type_unknown){
		dg87_connection_printf(pConnection, 
			"Unknown dependence type '%s'\n", pCommand->argv[4]);
		return 1;
	}

	/* get new state */	
	newState = dg87_jobdep_asciiToState(pCommand->argv[5]);
	if (newState == dg87_jobdep_state_unknown){
		dg87_connection_printf(pConnection, 
			"Unknown dependence state '%s'\n", pCommand->argv[5]);
		return 1;
	}

	/* find the dependence */
	pItem = pTask->pJobDepList->pFirst;
	while (pItem != NULL){
		pJobdep = pItem->pObject;
		if (pJobdep->pJobBefore == pJobBefore &&
				pJobdep->pJobAfter == pJobAfter &&
				pJobdep->type == type)
			break;
		pItem = pItem->pNext;
	}
	if (pItem == NULL){
		dg87_connection_printf(pConnection, 
			"Can't find a dependence between jobs %s "\
			"and %s with type %s in the task %s of the day %s\n", 
			pJobBefore->name, pJobAfter->name, 
			dg87_jobdep_typeToAscii(type), 
			pTask->name, pDay->name);
		return 1;
	}

	/* update state */
	if (dg87_jobdep_setState(pJobdep, newState) == FALSE){
		dg87_connection_printf(pConnection, 
			"Can't change state of dependence between jobs "\
			"%s and %s in the task %s of the day %s\n", 
			pJobBefore->name, pJobAfter->name, 
			pTask->name, pDay->name);
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* End of file */
