/* file cmdtask.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "cmdtask.h"
#include "util.h"
#include "context.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h> /* prevent error for arm architecture */
#include <fcntl.h>

/******************************************************************************/
/* xta
 * eXp Task Add 
 * Usage : xta DayName TaskName TargetName QueueName description \
 * 	waitOldestTask waitAllDependences onErrorContinue
 * TargetName and QueueName can be empty string if task it not attached to
 * a queue.
 */
int cmd_xta(dg87_connection* pConnection)
{
	dg87_command* pCommand;
	dg87_day* pDay;
	dg87_task* pTask;
	dg87_target* pTarget;
	dg87_queue* pQueue;
	BOOL value;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	if (pCommand->argc != 8){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
 			"Usage : xta DayName TaskName TargetName QueueName " \
			"description waitOldestTask " \
			"waitAllDependences onErrorContinue");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;
	/* control locked property */
	if (pDay->locked == FALSE){
		dg87_connection_printf(pConnection, 
			"Can't add a task because day '%s' is unlocked !\n",
			pDay->name);
		return 1;
	}
	/* create the task */
	pTask = dg87_day_taskAdd(pDay, pCommand->argv[1]);
	if (pTask == NULL){
		dg87_connection_printf(pConnection,
				"Can't add a task '%s' ! %s\n",
				pCommand->argv[1],
				dg87_err_getErrorStr());
		return 1;
	}
	/* set description */
	if (dg87_task_setDescription(pTask, pCommand->argv[4]) == FALSE){
		dg87_connection_printf(pConnection,
				"Can't add a task '%s' ! %s\n",
				pCommand->argv[1],
				dg87_err_getErrorStr());
		dg87_day_taskDelete(pDay, pTask);
		return 1;
	}
	/* change waitOldestTask property */
	if ((value = dg87_asciiToBool(pCommand->argv[5])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s' for WaitOldestTask, "\
			"use true or false !\n",
			pCommand->argv[5]);
		return 1;
	}
	if (dg87_task_setWaitOldestTask(pTask, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set WaitOldestTask for task '%s' !\n",
			pTask->name);
		dg87_day_taskDelete(pDay, pTask);
		return 1;
	}

	/* change waitAllDependences property */
	if ((value = dg87_asciiToBool(pCommand->argv[6])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s' for WaitAllDependences, "\
		        "use true or false !\n",
			pCommand->argv[6]);
		return 1;
	}
	if (dg87_task_setWaitAllDependences(pTask, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set WaitAllDependences for task '%s' !\n",
			pTask->name);
		dg87_day_taskDelete(pDay, pTask);
		return 1;
	}

	/* change OnErrorContinue property */
	if ((value = dg87_asciiToBool(pCommand->argv[7])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s' for OnErrorContinue, "\
		        "use true or false !\n",
			pCommand->argv[7]);
		return 1;
	}
	if (dg87_task_setOnErrorContinue(pTask, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set WaitOnErrorContinue for task '%s' !\n",
			pTask->name);
		dg87_day_taskDelete(pDay, pTask);
		return 1;
	}

	/* set task queue if necessary */
	if (strlen(pCommand->argv[2]) != 0 &&
			strlen(pCommand->argv[3]) != 0){
		/* find the target */
		pTarget = dg87_util_findTarget(pConnection, pCommand->argv[2]);
		if (pTarget == NULL) {
			dg87_day_taskDelete(pDay, pTask);
			return 1;
		}
		/* find the queue */
		pQueue = dg87_util_findQueue(pConnection, pTarget, 
				pCommand->argv[3]);
		if (pQueue == NULL){
			dg87_day_taskDelete(pDay, pTask);
			return 1;
		}
		/* set target queue */
		if (dg87_task_setQueue(pTask, pQueue) == FALSE){
			dg87_connection_printf(pConnection,
				"Can't set Queue '%s' for task '%s' !\n",
				pQueue->name, pTask->name);
			dg87_day_taskDelete(pDay, pTask);
			return 1;
		}
	}
	return 0;
}
/******************************************************************************/
/* cmd_xtc
 * eXp Task Count
 * return the number of Tasks in a day.
 * Usage : xtc DayName
 */
int cmd_xtc(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", "xtc DayName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;

	dg87_connection_printf(pConnection,"%i\n", 
			pDay->pTaskVector->count);
	return 0;
}
/******************************************************************************/
/* cmd_xtl
 * eXp Task List
 * Usage : xtl [options]  DayName
 * options
 * -d : print description of task
 * -t : show target and queue
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 * -s : state of task
 */
int cmd_xtl(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "sdtB:S:E:I:N:";*/
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first task to display */
	int number = 0;/* number of TASKs to display */
	int count = 0; /* number of TASK actually display */
	dg87_element* pElement = NULL;
	dg87_task* pTask;
	BOOL showDescription = FALSE;
	BOOL showTarget = FALSE;
	BOOL showState = FALSE;
	dg87_day* pDay;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xtl [options] DayName");
		return 1;
	}
	/* find the day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first task to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of task to display */
				number = atoi(optarg);
				break;
			case 'd': /* description */
				showDescription = TRUE;
				break;
			case 't': /* show target*/
				showTarget = TRUE;
				break;
			case 's': /* show state*/
				showState = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* find the first Task to display */
	pElement = NULL;
	if (start > 0)
		pElement = dg87_vector_getElementByIndex(
			pDay->pTaskVector, start);
	else
		pElement = pDay->pTaskVector->pFirst;

	/* create the list */
	count = 0;
	while (pElement != NULL){
		/* reach the limit of task to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pTask = pElement->pObject;
		/* print taks name */
		dg87_connection_printf(pConnection, "%s%s",
			pBeg, pTask->name);
		/* print target and queue */
		if (showTarget == TRUE)
			dg87_connection_printf(pConnection, 
				"%s%s%s%s", pSep,
				(pTask->pQueue ==NULL) ? "none" :
					pTask->pQueue->pTarget->name,
				pSep,
				(pTask->pQueue ==NULL) ? "none" :
					pTask->pQueue->name);
		/* print state */
		if (showState == TRUE)
			dg87_connection_printf(pConnection, "%s%s", pSep,
				dg87_task_stateToAscii(pTask->state));

		/* print description */
		if (showDescription == TRUE)
			dg87_connection_printf(pConnection, 
					"%s%s", pSep, pTask->description);

		dg87_connection_printf(pConnection, "%s\n", pEnd);
		pElement = pElement->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}
/******************************************************************************/
/* xti
 * eXp Task Information 
 * get information for a Task 
 * Usage : xti DayName TaskName
 */
int cmd_xti(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:";*/
	int optch,iopt;
	char* optarg;
	dg87_task *pTask;
	dg87_day *pDay;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control the argument number */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xti DayName TaskName");
		return 1;
	}
	/* find the day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findTask(pConnection, pDay,
			pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");
	
	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* Task name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Task Name", pSep, pTask->name, pEnd);
	/* Day name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Day Name", pSep, pTask->pDay->name, pEnd);
	/* Task description*/
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Description", pSep, pTask->description, pEnd);
	/* day */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Day", pSep, pTask->pDay->name, pEnd);
	/* waitOldestTask */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Wait Oldest Task", pSep,
		dg87_boolToAscii(pTask->waitOldestTask), pEnd);
	/* waitAllDependences*/
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Wait All Dependences", pSep,
		dg87_boolToAscii(pTask->waitAllDependences), pEnd);
	/* onErrorContinue */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "On Error Continue", pSep,
		dg87_boolToAscii(pTask->onErrorContinue), pEnd);
	/* Enabled */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Enabled", pSep,
		dg87_boolToAscii(pTask->enabled), pEnd);

	/* Task Target */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Target", pSep, 
		(pTask->pQueue==NULL) ? "none" : pTask->pQueue->pTarget->name,
		pEnd);
	/* Target Queue */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Queue", pSep, 
		(pTask->pQueue==NULL) ? "none" : pTask->pQueue->name,
		pEnd);
	/* Task's state */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "State", pSep, 
		dg87_task_stateToAscii(pTask->state), pEnd);
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}

/******************************************************************************/
/* xtse
 * exploit task set enabled 
 * change the enabled property of the task 
 * Usage : xtse DayName TaskName BooleanValue 
 */
int cmd_xtse(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_task* pTask;
	dg87_command* pCommand;
	BOOL value;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3 ){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xtse DayName TaskName BooleanValue\n");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the day */
	if ((pDay = dg87_util_findDay(pConnection, 
				pCommand->argv[0])) == NULL){
		return 1;
	}
	/* find the task */
	if ((pTask = dg87_util_findTask(pConnection, pDay,
				pCommand->argv[1])) == NULL){
		return 1;
	}
	
	/* control locked property */
	if (pDay->locked == TRUE){
		dg87_connection_printf(pConnection, 
			"Can't update the task '%s' because " \
		        "day '%s' is locked !\n",
			pTask->name, pDay->name);
		return 1;
	}
	/* interpret boolean value */
	if ((value = dg87_asciiToBool(pCommand->argv[2])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s', use true or false !\n",
			pCommand->argv[2]);
		return 1;
	}
	if (dg87_task_setEnabled(pTask, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't change Enabled property for task '%s' !\n",
			pTask->name);
		return 1;
	}

	return 0;
}

/******************************************************************************/
/* xtu
 * exploit task update 
 * change the state of the task 
 * Usage : xtu DayName TaskName State
 */
int cmd_xtu(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_task* pTask;
	dg87_task_state state;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3 ){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xtu DayName TaskName State\n");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the day */
	if ((pDay = dg87_util_findDay(pConnection, 
				pCommand->argv[0])) == NULL){
		return 1;
	}
	/* find the task */
	if ((pTask = dg87_util_findTask(pConnection, pDay,
				pCommand->argv[1])) == NULL){
		return 1;
	}
	
	/* control locked property */
	if (pDay->locked == TRUE){
		dg87_connection_printf(pConnection, 
			"Can't update the task '%s' because " \
		        "day '%s' is locked !\n",
			pTask->name, pDay->name);
		return 1;
	}

	state = dg87_task_asciiToState(pCommand->argv[2]);
	switch (state){
		case dg87_task_state_ignore :
		case dg87_task_state_ignored :
		case dg87_task_state_running : 
		case dg87_task_state_canceled :
		case dg87_task_state_terminated : 
		case dg87_task_state_deleted : 
		case dg87_task_state_aborted :
			dg87_task_setState(pTask, state);
			break;
		default:
			dg87_connection_printf(pConnection,
				"Incorrect state value '%s'.\n", 
				pCommand->argv[2]);
			return 1;
	}
	return 0;
}
/******************************************************************************/
/* xtca
 * eXp Task Condition Add 
 * Usage : xtca DayName TaskName Type Data [Data2]
 * with type = hmin, hmax, hlate, file, manual, if_equal or if_not_equal
 * - with type=hmin, hmax, hlate, argument is an hour 
 * - with type=file, argument is a file string 
 * - with type=manual, argument is a message 
 * - with type=if_equal, if_not_equal, argument are a variable name and a value
 */
int cmd_xtca(dg87_connection* pConnection)
{
	dg87_task *pTask;
	dg87_day *pDay;
	dg87_command* pCommand;
	dg87_condition_type type;
	dg87_condition *pCondition;
	dg87_variable* pVariable;
	int minutes;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	/* control argument number */
	if (pCommand->argc < 4 || pCommand->argc > 5){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xtca DayName TaskName type data [data2]");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;
	/* find the task */
	pTask= dg87_util_findTask(pConnection, pDay, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* control locked property */
	if (pDay->locked == FALSE){
		dg87_connection_printf(pConnection, 
			"Can't add a condition to the task '%s' " \
			"because day '%s' is unlocked !\n",
			pTask->name, pDay->name);
		return 1;
	}
	/* create the task */
	/* convert condition type */
	if ((type = dg87_condition_asciiToCondition(
					pCommand->argv[2])) == FALSE){
		dg87_connection_printf(pConnection, 
			"Incorrect condition type '%s' ! \n",
			pCommand->argv[2]);
		return 1;
	}
	/* control number of arguments */
	if (type == condition_ifequal || type == condition_ifnequal){
		if (pCommand->argc == 4){
			dg87_connection_printf(pConnection,
				"With 'if_equal' and 'if_not_equal' conditions"\
				", a variable value "\
				"must be passed as optional parameter.\n");
			return 1;
		}
	} else {
		if (pCommand->argc == 5){
			dg87_connection_printf(pConnection,
			"The last optional argument is not required.\n");
			return 1;
		}
	}

	/* add the condition */
	dg87_err_clearError();
	switch(type)
	{
	case condition_hmin:
		minutes = dg87_condition_hourToMinute(pCommand->argv[3]);
		pCondition = dg87_task_conditionHminAdd(pTask, minutes);
		break;
	case condition_hmax:
		minutes = dg87_condition_hourToMinute(pCommand->argv[3]);
		pCondition = dg87_task_conditionHmaxAdd(pTask, minutes);
		break;
	case condition_hlate:
		minutes = dg87_condition_hourToMinute(pCommand->argv[3]);
		pCondition = dg87_task_conditionHlateAdd(pTask, minutes);
		break;
	case condition_file:
		pCondition = dg87_task_conditionFileAdd(pTask, 
				pCommand->argv[3]);
		break;
	case condition_manual:
		pCondition = dg87_task_conditionManualAdd(pTask, 
				pCommand->argv[3]);
	break;
	case condition_ifequal:
		pVariable = dg87_util_findVariable(pConnection, 
				pCommand->argv[3]);
		if (pVariable == NULL)
			return 1;
		pCondition = dg87_task_conditionIfEqualAdd(pTask,
				pVariable, pCommand->argv[4]);
	break;
	case condition_ifnequal:
		pVariable = dg87_util_findVariable(pConnection, 
				pCommand->argv[3]);
		if (pVariable == NULL)
			return 1;
		pCondition = dg87_task_conditionIfNotEqualAdd(pTask,
				pVariable, pCommand->argv[4]);
	break;

	default :
		dg87_connection_printf(pConnection,
				"Incorrect type of condition '%s' !\n",
				pCommand->argv[2]);
		return 1;
	}

	if (pCondition == NULL) {
		dg87_connection_printf(pConnection,
				"Can't add the condition ! %s\n",
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}


/******************************************************************************/
/* xtcl
 * eXpl Task Condition List
 * Usage : xtcl [options] DayName TaskName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first taskdep to display (first index is 0)
 *  -Nnumber : number of jobdep to display (no limit if -N < 1)
 *  -s : condition's state
 */
int cmd_xtcl(dg87_connection* pConnection)
{
	char* pHour;
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "sB:S:E:I:N:";*/
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first condition to display */
	int number = 0;/* number of conditions to display */
	int count = 0; /* number of conditions actually display */
	dg87_item* pItem= NULL;
	dg87_condition* pCondition;
	dg87_task* pTask;
	dg87_day *pDay;
	BOOL showState = FALSE;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xtcl [options] DayName TaskName");
		return 1;
	}
	/* find the day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;

	/* find the task */
	pTask= dg87_util_findTask(pConnection, 
			pDay, pCommand->argv[1]);
	if (pTask== NULL) return 1;

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first condition to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of condition to display */
				number = atoi(optarg);
				break;
			case 's': /* show conditition's state */
				showState = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* find the first condition to display */
	pItem = NULL;
	if (start > 0)
		pItem = dg87_list_getItem(pTask->pConditionList, start);
	else
		pItem = pTask->pConditionList->pFirst;

	/* create the list */
	count = 0;
	while (pItem != NULL){
		/* reach the limit of task to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pCondition = pItem->pObject;
		switch(pCondition->type){
			case condition_hmin:
				pHour = dg87_condition_minuteToHour(
				     pCondition->data.pConditionHmin->minutes);	
				dg87_connection_printf(pConnection,
					"%s%s%s%s", pBeg,
					"hmin", pSep, pHour);
				dg87_free(pHour, DG87_CHAR);
				break;
			case condition_hmax:
				pHour = dg87_condition_minuteToHour(
				     pCondition->data.pConditionHmin->minutes);	
				dg87_connection_printf(pConnection,
					"%s%s%s%s", pBeg,
					"hmax", pSep, pHour);
				dg87_free(pHour, DG87_CHAR);
				break;
			case condition_hlate:
				pHour = dg87_condition_minuteToHour(
				     pCondition->data.pConditionHmin->minutes);	
				dg87_connection_printf(pConnection,
					"%s%s%s%s", pBeg,
					"hlate", pSep, pHour);
				dg87_free(pHour, DG87_CHAR);
				break;
			case condition_file:
				dg87_connection_printf(pConnection,
					"%s%s%s%s", pBeg, "file",pSep,
					pCondition->data.pConditionFile->pPath
					);
				break;
			case condition_manual:
				dg87_connection_printf(pConnection,
					"%s%s%s%s", pBeg, "manual",pSep,
				pCondition->data.pConditionManual->pMessage
					);
				break;
			case condition_ifequal:
				dg87_connection_printf(pConnection,
					"%s%s%s%s%s%s", pBeg, 
					dg87_condition_conditionToAscii(
						pCondition->type), pSep,
					pCondition->data.pConditionIfEqual
						->pVariable->name, pSep,
					pCondition->data.pConditionIfEqual
						->pValue);
				break;
			case condition_ifnequal:
				dg87_connection_printf(pConnection,
					"%s%s%s%s%s%s", pBeg, 
					dg87_condition_conditionToAscii(
						pCondition->type), pSep,
					pCondition->data.pConditionIfEqual
						->pVariable->name, pSep,
					pCondition->data.pConditionIfEqual
						->pValue);
				break;


			default:
				dg87_connection_printf(pConnection,
					"Unknown type of condition !\n");
				break;
		}
		if (showState == TRUE)
			dg87_connection_printf(pConnection,"%s%s", pSep,
				dg87_condition_stateToAscii(
					pCondition->state));
		dg87_connection_printf(pConnection, "%s\n", pEnd);
		pItem = pItem->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}
/******************************************************************************/
/* xtcc
 * eXp Task Condition Count
 * Usage : xtcc DayName TaskName
 */
int cmd_xtcc(dg87_connection* pConnection)
{
	dg87_task *pTask;
	dg87_day *pDay;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xtcc DayName TaskName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;

	/* find the task */
	pTask= dg87_util_findTask(pConnection, pDay, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	dg87_connection_printf(pConnection,"%i\n", 
			pTask->pConditionList->count);
	return 0;
}
/******************************************************************************/
/* xtcv 
 * eXploitation Task Condition Validate
 * Usage : xtcv DayName TaskName ConditionIndex
 */
int cmd_xtcv(dg87_connection* pConnection)
{
	dg87_task *pTask;
	dg87_day *pDay;
	dg87_condition *pCondition;
	int index;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xtcv DayName TaskName ConditionIndex");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;

	/* find the task */
	pTask= dg87_util_findTask(pConnection, pDay, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* control locked property */
	if (pDay->locked == TRUE){
		dg87_connection_printf(pConnection, 
			"Can't validate condition for the task '%s' because " \
		        "day '%s' is locked !\n",
			pTask->name, pDay->name);
		return 1;
	}

	/* find the condition */
	index = atoi(pCommand->argv[2]) - 1;
	pCondition = dg87_list_getObjectByIndex(pTask->pConditionList, index);
	if (pCondition == NULL) {
		dg87_connection_printf(pConnection,
			"Incorrect index for condition for the task '%s'" \
			"in the day '%s' !",
			pTask->name, pDay->name);
		return 1;
	}

	dg87_condition_validate(pCondition);
	
	return 0;

}
/******************************************************************************/
/* xtda
 * eXp Task Dependence Add 
 * Usage : xtda DayName TaskBeforeName TaskAfterName Type
 */
int cmd_xtda(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_task *pTaskAfter, *pTaskBefore;
	dg87_taskdep* pTaskDep;
	dg87_taskdep_type type;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xtda DayName TaskBeforeName TaskAfterName Type"
			);
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the day*/
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;
	/* control locked property */
	if (pDay->locked == FALSE){
		dg87_connection_printf(pConnection, 
			"Can't add a task dependence because " \
			"day '%s' is unlocked !\n",
			pDay->name);
		return 1;
	}
	/* find the tasks */
	pTaskBefore = dg87_util_findTask(pConnection,pDay,pCommand->argv[1]);
	if (pTaskBefore == NULL) return 1;
	pTaskAfter = dg87_util_findTask(pConnection,pDay,pCommand->argv[2]);
	if (pTaskAfter == NULL) return 1;
	/* get dependence type */
	if ((type = dg87_taskdep_asciiToType(pCommand->argv[3])) == 
			dg87_taskdep_type_unknown){
		dg87_connection_printf(pConnection,
			"Incorrect dependence type '%s' ! \n",
			pCommand->argv[3]);
		return 1;
	}
	/* create the dependence */
	pTaskDep = dg87_day_taskdepAdd(pDay, pTaskBefore, pTaskAfter);
	if (pTaskDep == NULL){
		dg87_connection_printf(pConnection,
			"Can't add a dependence between task '%s' and " \
			"task '%s' ! %s\n",
				pTaskBefore->name, pTaskAfter->name, 
				dg87_err_getErrorStr());
		return 1;
	}
	/* set dependence type */
	if (dg87_taskdep_setType(pTaskDep, type) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set type for dependence between task '%s' and " \
			"task '%s' ! %s\n",
				pTaskBefore->name, pTaskAfter->name, 
				dg87_err_getErrorStr());
		dg87_day_taskdepDelete(pDay,pTaskDep);
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* xtdl
 * eXploit Task Dependence List
 * Usage : xtdl [options] DayName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first taskdep to display (first index is 0)
 *  -Nnumber : number of jobdep to display (no limit if -N < 1)
 *  -a taskName : only taskdep after this task 
 *  -b taskName : only taskdep before this task 
 *  -t : print dependence type
 *  -s : print dependence's state
 */
int cmd_xtdl(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "stB:S:E:I:N:b:a:";*/
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first taskdep to display */
	int number = 0;/* number of taskdeps to display */
	int count = 0; /* number of taskdep actually display */
	dg87_item* pItem= NULL;
	int filter = 0; /* 0 = no; 1 = before; 2 = after */
	char* pTaskFilterName = NULL;
	dg87_task* pTaskFilter = NULL;
	dg87_day* pDay = NULL;
	dg87_taskdep* pTaskDep;
	BOOL showDepType = FALSE;
	BOOL showDepState = FALSE;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xtdl [options] DayName");
		return 1;
	}

	/* find the day*/
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first job to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of job to display */
				number = atoi(optarg);
				break;
			case 'b': /* only job before */
				pTaskFilterName = strdup(optarg);
				filter = 1;
				break;
			case 'a': /* only job after */
				pTaskFilterName = strdup(optarg);
				filter = 2;
				break;
			case 't': /* type of dependence*/
				showDepType = TRUE;
				break;
			case 's': /* state of dependence*/
				showDepState = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				if (pTaskFilterName != NULL)
					free(pTaskFilterName);
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* find the task */
	if (filter > 0){
		pTaskFilter = dg87_util_findTask(pConnection, pDay,
				pTaskFilterName);
		if (pTaskFilter == NULL){

			free(pTaskFilterName);
			free(pSep); free(pBeg); free(pEnd);
			return 1;
		}
	}
	
	/* find the first TaskDep to display */
	pItem = NULL;
	if (start > 0)
		pItem = dg87_list_getItem(pDay->pTaskdepList, start);
	else
		pItem = pDay->pTaskdepList->pFirst;

	/* create the list */
	count = 0;
	while (pItem != NULL){
		/* reach the limit of job to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pTaskDep = pItem->pObject;
		if (filter == 0 ||
			(filter== 1 && pTaskDep->pTaskAfter == pTaskFilter) ||
			(filter== 2 && pTaskDep->pTaskBefore == pTaskFilter))
		{
			/* print task before and after */
			dg87_connection_printf(pConnection,"%s%s%s%s", pBeg,
					pTaskDep->pTaskBefore->name, pSep, 
					pTaskDep->pTaskAfter->name);
			/* print task dependence type */
			if (showDepType == TRUE){
				dg87_connection_printf(pConnection,
					"%s%s", pSep,
					dg87_taskdep_typeToAscii
						(pTaskDep->type));
			}
			/* print task dependence's state */
			if (showDepState == TRUE){
				dg87_connection_printf(pConnection,
					"%s%s", pSep, 
					dg87_taskdep_stateToAscii(
						pTaskDep->state));
			}

			dg87_connection_printf(pConnection,"%s\n", pEnd);
		}
		pItem = pItem->pNext;
	}
	/* free memory */
	if (pTaskFilterName != NULL) free(pTaskFilterName);
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}

/******************************************************************************/
/* xtdc
 * eXp Task Dependence Count
 * Usage : xtdc DayName
 */
int cmd_xtdc(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xtdc DayName");
		return 1;
	}
	/* find the day*/
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;
	
	dg87_connection_printf(pConnection,"%i\n", pDay->pTaskdepList->count);
	return 0;
}
/******************************************************************************/
/* xtdu
 * eXp Task Dependence Update 
 * Usage : xtdu DayName TaskBefore TaskAfter Type NewState
 */
int cmd_xtdu(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_task* pTaskBefore;
	dg87_task* pTaskAfter;
	dg87_taskdep* pTaskdep = NULL;
	dg87_taskdep_type type;
	dg87_taskdep_state newState;
	dg87_command* pCommand;
	dg87_item* pItem;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 5){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xtdu DayName TaskBefore TaskAfter Type NewState");
		return 1;
	}
	/* find the day*/
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;
	/* control locked property */
	if (pDay->locked == TRUE){
		dg87_connection_printf(pConnection, 
			"Can't update task dependence because " \
			"day '%s' is locked !\n",
			pDay->name);
		return 1;
	}

	/* find the task before */
	pTaskBefore = dg87_util_findTask(pConnection, pDay, pCommand->argv[1]);
	if (pTaskBefore == NULL) return 1;

	/* find the task before */
	pTaskAfter = dg87_util_findTask(pConnection, pDay, pCommand->argv[2]);
	if (pTaskAfter == NULL) return 1;

	/* find dependence type */
	type = dg87_taskdep_asciiToType(pCommand->argv[3]);
	if (type == dg87_taskdep_type_unknown){
		dg87_connection_printf(pConnection, 
			"Unknown dependence type '%s'", pCommand->argv[3]);
		return 1;
	}

	/* get new state */	
	newState = dg87_taskdep_asciiToState(pCommand->argv[4]);
	if (newState == dg87_taskdep_state_unknown){
		dg87_connection_printf(pConnection, 
			"Unknown dependence state '%s'", pCommand->argv[4]);
		return 1;
	}

	/* find the dependence */
	pItem = pDay->pTaskdepList->pFirst;
	while (pItem != NULL){
		pTaskdep = pItem->pObject;
		if (pTaskdep->pTaskBefore == pTaskBefore &&
				pTaskdep->pTaskAfter == pTaskAfter &&
				pTaskdep->type == type)
			break;
		pItem = pItem->pNext;
	}
	if (pItem == NULL){
		dg87_connection_printf(pConnection, 
			"Can't find a dependence between task "\
			"%s and %s with type %s in the day %s", 
			pTaskBefore->name, pTaskAfter->name, 
			dg87_taskdep_typeToAscii(type), pDay->name);
		return 1;
	}

	/* update state */
	if (dg87_taskdep_setState(pTaskdep, newState) == FALSE){
		dg87_connection_printf(pConnection, 
			"Can't change state of dependence between tasks "\
			"%s and %s in the day %s", 
			pTaskBefore->name, pTaskAfter->name, pDay->name);
		return 1;
	}
	return 0;
}

/******************************************************************************/
/* End of file */
