/* file condition.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef CONDITION_H
#define CONDITION_H 1

#include "global.h"
#include <libdg87/storage.h>

/* condition's state */
typedef enum {
	dg87_condition_waiting,
	dg87_condition_finished
} dg87_condition_state;

/* condition types */
typedef enum {
	condition_error=-1,
	condition_hmin=1,  /* hour min (can't start before)*/
	condition_hmax=2,  /* hour max (can't start after) */
	condition_hlate=3, /* To Late : don't execute the task */
	condition_file=4,  /* test if file is present */
	condition_manual=5,/* must be validate manually */
	condition_ifequal=6, /* test a variable is equal to a value */
	condition_ifnequal=7 /* test a variable is not equal to a value */
}dg87_condition_type;

/* struct for hMin, hMax and hLate Conditions */
struct dg87_condition_h {
	int minutes; /* number of minutes from 00H00 */
};
/* struct for File Condition */
struct dg87_condition_file {
	char* pPath;
};
/* struct for Manual Condition */
struct dg87_condition_manual {
	char* pMessage; /* mnemonic to identify the condition */
};
/* struct for if_equal and if_not_equal Condition */
struct dg87_condition_eq{
	struct dg87_variable_struct* pVariable;
	char* pValue;
};

/* Condition Object */
typedef struct dg87_condition_struct {
	dg87_condition_type type;
	dg87_condition_state state;
	union {
		struct dg87_condition_h* pConditionHmin;
		struct dg87_condition_h* pConditionHmax;
		struct dg87_condition_h* pConditionHlate;
		struct dg87_condition_file* pConditionFile;
		struct dg87_condition_manual* pConditionManual;
		struct dg87_condition_eq* pConditionIfEqual;
		struct dg87_condition_eq* pConditionIfNotEqual;
	} data;
} dg87_condition;

/*
 * dg87_conditionHmax_new
 * create a condition hMax and return a pointer on this object.
 * The number of minutes from 00H00 must be pass in parameter.
 */
dg87_condition* dg87_conditionHmax_new(int minutes);

/*
 * dg87_conditionHmin_new
 * create a condition hMin and return a pointer on this object.
 * The number of minutes from 00H00 must be pass in parameter.
 */
dg87_condition* dg87_conditionHmin_new(int minutes);

/*
 * dg87_conditionLate_new
 * create a condition Late and return a pointer on this object.
 * The number of minutes from 00H00 must be pass in parameter.
 */
dg87_condition* dg87_conditionHlate_new(int minutes);

/*
 * dg87_conditionFile_new
 * create a condition File and return a pointer on this object.
 * The Path of the file must be pass in parameter
 */
dg87_condition* dg87_conditionFile_new(char* pPathFile);

/*
 * dg87_conditionManual_new
 * create a condition Manual and return a pointer on this object.
 * A mnemonic must be pass in parameter to help the user to
 * identify when he must validate this condition. 
 */
dg87_condition* dg87_conditionManual_new(char* pMessage);

/*
 * dg87_conditionIfEqual_new
 * create a if_equal condition and return a pointer on this object.
 * The name of the variable and the value which the variable should
 * have must be passed as parameters.
 */
dg87_condition* dg87_conditionIfEqual_new(
		struct dg87_variable_struct* pVariable,
		char* pValue);

/*
 * dg87_conditionIfNotEqual_new
 * create a if_not_equal condition and return a pointer on this object.
 * The name of the variable and the value which the variable should
 * have must be passed as parameters.
 */
dg87_condition* dg87_conditionIfNotEqual_new(
		struct dg87_variable_struct* pVariable,
		char* pValue);

/*
 * dg87_condition_delete
 * free memory of the condition object in parameter.
 */
BOOL dg87_condition_delete(dg87_condition* pCondition);

/*
 * dg87_condition_load
 */
BOOL dg87_condition_load(dg87_condition* pCondition, dg87_storage* pStorage);

/*
 * dg87_condition_save
 */
BOOL dg87_condition_save(dg87_condition* pCondition, dg87_storage* pStorage);

/*
 * dg87_condition_getState
 */
dg87_condition_state dg87_condition_getState(dg87_condition* pCondition);

/*
 * dg87_condition_asciiToCondition 
 * convert a string into a condition type.
 * return -1 if error.
 */
dg87_condition_type dg87_condition_asciiToCondition(char* pCondition);

/*
 * dg87_condition_conditionToAscii
 * convert a condition type into a string
 */
char* dg87_condition_conditionToAscii(dg87_condition_type);

/*
 * dg87_condition_hourToMinute
 * convert a string HH:MM in a number of minutes from midnight
 */
int dg87_condition_hourToMinute(char* hour);

/*
 * dg87_condition_minuteToHour
 * convert a number of minutes from midnigh in the format HH:MM
 * return NULL on error
 * the string return must be free with dg87_free
 */
char* dg87_condition_minuteToHour(int minutes);

/*
 * dg87_condition_stateToAscii
 * convert a condition's state into a string
 */
char* dg87_condition_stateToAscii(dg87_condition_state state);

/*
 * dg87_condition_validate
 * change the state of the condition to finished even if the condition
 * is not verify.
 * This function is use to validate manual's condition.
 */
BOOL dg87_condition_validate(dg87_condition *pCondition);

#ifdef DEBUG
/*
 * dg87_condition_ckeck
 */
int dg87_condition_check(dg87_condition* pCondition, int (*print)(char*));
#endif /*DEBUG*/

#endif /*CONDITION_H*/
/* End of file */
