/* file context.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <config.h>
#include <libdg87/daemon.h>
#include <libdg87/message.h>
#include "context.h"
#include "engine.h"
#include "variable.h"

#define DG87X_DATA_VERSION 1
/*
 * dg87_context_fieldArray
 */
dg87_conf_field dg87_context_fieldArray[]=
{
	{"IPPORT", DG87X_IPPORT, NULL},
	{"PID_FILE", NULL, NULL}, 
	{"USER_ACCOUNT", NULL, NULL},
	{"DATA_FILE", DG87X_DATA_FILE, NULL},
	{"BACKUP_FILE", DG87X_BACKUP_FILE, NULL},
	{"SAVE_TEMPO", NULL, NULL},
	{"DIOGENE_PASSWORD", DG87_USER_DIOGENE_PASSWORD, NULL},
	{NULL, NULL, NULL}
};

/******************************************************************************/
dg87_context* dg87_context_new(char* pConfFile)
{
	/* allocate memory for object structure */
	dg87_context* pContext = dg87_malloc(sizeof(dg87_context), 
			DG87_CONTEXT);
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* create dg87_conf object */
	pContext->pConf = dg87_conf_new(pConfFile, dg87_context_fieldArray);
	if (pContext->pConf == NULL){
		dg87_free(pContext, DG87_CONTEXT);
		return NULL;
	}
	if (dg87_context_readConfig(pContext) == FALSE){
		printf("Error while reading configuration file :%s\n", 
				pConfFile);
		dg87_conf_delete(pContext->pConf);
		dg87_free(pContext, DG87_CONTEXT);
		return NULL;
	}
	/* create the target vector */
	pContext->pTargetVector = dg87_vector_new();
	if (pContext->pTargetVector == NULL){
		dg87_conf_delete(pContext->pConf);
		dg87_context_delete(pContext);
		return NULL;
	}

	/* create the day vector */
	pContext->pDayVector = dg87_vector_new();
	if (pContext->pDayVector == NULL){
		dg87_conf_delete(pContext->pConf);
		dg87_vector_delete(pContext->pTargetVector);
		dg87_context_delete(pContext);
		return NULL;
	}

	/* create the engine */
	pContext->pEngine = dg87_engine_new(pContext);
	if (pContext->pEngine == NULL){
		dg87_conf_delete(pContext->pConf);
		dg87_vector_delete(pContext->pTargetVector);
		dg87_vector_delete(pContext->pDayVector);
		dg87_context_delete(pContext);
		return NULL;
	}
	/* create storage */
	pContext->pStorage = dg87_storage_new( 
			dg87_conf_getFieldValue(pContext->pConf, "DATA_FILE"),
			dg87_conf_getFieldValue(pContext->pConf, "BACKUP_FILE"),
			DG87X_NAME, DG87X_DATA_VERSION);
	if (pContext->pEngine == NULL){
		dg87_engine_delete(pContext->pEngine);
		dg87_conf_delete(pContext->pConf);
		dg87_vector_delete(pContext->pTargetVector);
		dg87_vector_delete(pContext->pDayVector);
		dg87_free(pContext, DG87_CONTEXT);
		return NULL;
	}
	pContext->pServer = NULL;

	return pContext;
}
/******************************************************************************/
BOOL dg87_context_delete(dg87_context* pContext)
{
	dg87_target* pTarget;
	dg87_day* pDay;

	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}

	/* delete all days*/
	while (pContext->pDayVector->count > 0){
		pDay = dg87_vector_removeObjectByIndex
			(pContext->pDayVector,0);
		dg87_day_delete(pDay);
	}
	dg87_vector_delete(pContext->pDayVector);
	
	/* delete all targets */
	while (pContext->pTargetVector->count > 0){
		pTarget = dg87_vector_removeObjectByIndex
			(pContext->pTargetVector,0);
		dg87_target_delete(pTarget);
	}
	dg87_vector_delete(pContext->pTargetVector);

	dg87_storage_delete(pContext->pStorage);
	dg87_engine_delete(pContext->pEngine);
	dg87_conf_delete(pContext->pConf);
	dg87_free(pContext, DG87_CONTEXT);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_context_setServer(dg87_context* pContext, dg87_server* pServer)
{
	if (pContext == NULL || pServer == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pContext->pServer = pServer;
	return TRUE;
}
/******************************************************************************/
BOOL dg87_context_makePeriodicSave(dg87_context* pContext)
{
	time_t currentTime;
	currentTime = time(NULL);
	if (currentTime > pContext->saveTime + pContext->saveTempo){
		printf("periodic save\n");
		if (dg87_context_save(pContext) == FALSE)
			dg87_daemon_log("Save Error !\n");
		pContext->saveTime = time(NULL);
	}
	return TRUE;
}
/******************************************************************************/
BOOL dg87_context_readConfig(dg87_context* pContext)
{
	char* pValue;
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_conf_read(pContext->pConf) == FALSE)
		return FALSE;
	/* interpret values */
	/* TODO control values */
	pContext->ipPort = atoi(dg87_conf_getFieldValue(pContext->pConf, 
				"IPPORT"));
	pContext->pPidFile = dg87_conf_getFieldValue(pContext->pConf, 
				"PID_FILE");
	pContext->pDiogenePassword = dg87_conf_getFieldValue(pContext->pConf,
				"DIOGENE_PASSWORD");
	if (pContext->pStorage != NULL) /* pStorage is null at daemon startup */
		dg87_storage_changeDataFile(pContext->pStorage,
			dg87_conf_getFieldValue(pContext->pConf,"DATA_FILE"),
			dg87_conf_getFieldValue(pContext->pConf,"BACKUP_FILE"));
	if (pContext->pServer != NULL)
		dg87_server_reload(pContext->pServer, pContext->pPidFile,
				pContext->ipPort,pContext->pDiogenePassword);
	/* read save tempo */
	pValue = dg87_conf_getFieldValue(pContext->pConf, "SAVE_TEMPO");
	pContext->saveTempo = (pValue == NULL)?DG87_SAVE_TEMPO:atoi(pValue);
	if (pContext->saveTempo <= 0)
		pContext->saveTempo = DG87_SAVE_TEMPO;

	return TRUE;
}
/****************************************************************************/
dg87_target* dg87_context_targetAdd(dg87_context* pContext, char* pTargetName)
{
	dg87_target* pTarget;
	if (pContext == NULL || pTargetName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pTarget = dg87_target_new(pTargetName);
	if (pTarget == NULL) 
		return NULL;

	/* add the target in the TargetVector */
	/* vector generate error if an other target has the same name.*/
	/* Key must point on pTarget->name */
	if (dg87_vector_addObject(pContext->pTargetVector, 
				pTarget, pTarget->name) == FALSE)
	{
		dg87_target_delete(pTarget);
		return NULL;
	}
	return pTarget;
}
/****************************************************************************/
int dg87_context_targetCount(dg87_context* pContext)
{
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_vector_count(pContext->pTargetVector);
}
/****************************************************************************/
dg87_target* dg87_context_targetGetByName(dg87_context* pContext,char* pName)
{
	if (pContext == NULL || pName == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	return dg87_vector_getObjectByKey(pContext->pTargetVector, pName);
}
/****************************************************************************/
dg87_target* dg87_context_targetGetByIndex(dg87_context* pContext,int index)
{
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByIndex(pContext->pTargetVector, index);
}
/****************************************************************************/
BOOL dg87_context_targetDelete(dg87_context* pContext, dg87_target* pTarget)
{
	if (pContext == NULL || pTarget == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_vector_removeObject(pContext->pTargetVector, pTarget) == FALSE)
		return FALSE;
	if (dg87_target_delete(pTarget) == FALSE){
		/* add target in vector again */
		dg87_vector_addObject(pContext->pTargetVector, pTarget,
				pTarget->name);
		return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_context_targetDeleteByName(dg87_context* pContext, char* pName)
{
	dg87_target* pTarget;
	if (pContext == NULL || pName == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTarget = dg87_vector_getObjectByKey(pContext->pTargetVector, pName);
	if (pTarget == NULL) 
		return FALSE;
	return dg87_context_targetDelete(pContext, pTarget);
}
/****************************************************************************/
BOOL dg87_context_targetDeleteByIndex(dg87_context* pContext,int index)
{
	dg87_target* pTarget;
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTarget = dg87_vector_getObjectByIndex(pContext->pTargetVector, 
			index);
	if (pTarget == NULL) 
		return FALSE;
	return dg87_context_targetDelete(pContext, pTarget);
}
/****************************************************************************/
dg87_day* dg87_context_dayAdd(dg87_context* pContext, char* pDayName,
		time_t startTime)
{
	dg87_day* pDay;
	if (pContext == NULL || pDayName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pDay = dg87_day_new(pDayName, startTime);
	if (pDay == NULL) 
		return NULL;

	/* add the day in the DayVector */
	/* vector generate error if an other day has the same name.*/
	/* Key must point on pDay->name */
	if (dg87_vector_addObject(pContext->pDayVector, 
				pDay, pDay->name) == FALSE)
	{
		dg87_day_delete(pDay);
		return NULL;
	}
	return pDay;
}
/****************************************************************************/
int dg87_context_dayCount(dg87_context* pContext)
{
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_vector_count(pContext->pDayVector);
}
/****************************************************************************/
dg87_day* dg87_context_dayGetByName(dg87_context* pContext,char* pName)
{
	if (pContext == NULL || pName == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	return dg87_vector_getObjectByKey(pContext->pDayVector, pName);
}
/****************************************************************************/
dg87_day* dg87_context_dayGetByIndex(dg87_context* pContext,int index)
{
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByIndex(pContext->pDayVector, index);
}
/****************************************************************************/
BOOL dg87_context_dayDeleteByName(dg87_context* pContext, char* pName)
{
	dg87_day* pDay;
	if (pContext == NULL || pName == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pDay = dg87_vector_removeObjectByKey(pContext->pDayVector, pName);
	if (pDay == NULL) 
		return FALSE;
	dg87_day_delete(pDay);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_context_dayDeleteByIndex(dg87_context* pContext,int index)
{
	dg87_day* pDay;
	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pDay = dg87_vector_removeObjectByIndex(pContext->pDayVector, 
			index);
	if (pDay == NULL) 
		return FALSE;
	dg87_day_delete(pDay);
	return TRUE;
}
/****************************************************************************/
static BOOL dg87_context_loadConditions(dg87_context* pContext,
		dg87_task *pTask, dg87_storage *pStorage)
{
	dg87uint count;
	dg87uint minutes;
	dg87byte byte;
	dg87_condition_type type;
	dg87_variable *pVariable;
	dg87_condition *pCondition;
	char* pString;
	/* read conditions count */
	if (dg87_storage_readUInt(pStorage, &count) == FALSE)
		return FALSE;
	pCondition = NULL;
	while (count > 0){
		count --;
		/* read condition type */
		if (dg87_storage_readByte(pStorage, &byte) == FALSE)
			return FALSE;
		type = byte;
		switch(type){
		  case condition_hmin:
			if (dg87_storage_readUInt(pStorage, &minutes) == FALSE)
				return FALSE;
			pCondition = dg87_task_conditionHminAdd(pTask, minutes);
			break;
		  case condition_hmax:
			if (dg87_storage_readUInt(pStorage, &minutes) == FALSE)
				return FALSE;
			pCondition = dg87_task_conditionHmaxAdd(pTask, minutes);
			break;
		  case condition_hlate:
			if (dg87_storage_readUInt(pStorage, &minutes) == FALSE)
				return FALSE;
			pCondition= dg87_task_conditionHlateAdd(pTask, minutes);
			break;
		  case condition_file:
			if (dg87_storage_readString(pStorage, &pString)== FALSE)
				return FALSE;
			pCondition = dg87_task_conditionFileAdd(pTask, pString);
			dg87_free(pString, DG87_CHAR);
			break;
		  case condition_manual:
			if (dg87_storage_readString(pStorage, &pString)== FALSE)
				return FALSE;
			pCondition=dg87_task_conditionManualAdd(pTask, pString);
			dg87_free(pString, DG87_CHAR);
			break;
		  case condition_ifequal:
			/* find variable */
			if (dg87_storage_readString(pStorage, &pString)== FALSE)
				return FALSE;
			pVariable = dg87_engine_variableGetByName(
					pContext->pEngine, pString);
			dg87_free(pString, DG87_CHAR);
			if (pVariable == NULL)
				return FALSE;
			/* read value */
			if (dg87_storage_readString(pStorage, &pString)== FALSE)
				return FALSE;
			pCondition = dg87_task_conditionIfEqualAdd(pTask, 
						pVariable, pString);
			dg87_free(pString, DG87_CHAR);
			break;
		  case condition_ifnequal:
			/* find variable */
			if (dg87_storage_readString(pStorage, &pString)== FALSE)
				return FALSE;
			pVariable = dg87_engine_variableGetByName(
					pContext->pEngine, pString);
			dg87_free(pString, DG87_CHAR);
			if (pVariable == NULL)
				return FALSE;
			/* read value */
			if (dg87_storage_readString(pStorage, &pString)== FALSE)
				return FALSE;
			/* TODO bug ici */
			printf("string:%s\n", pString);
			pCondition = dg87_task_conditionIfNotEqualAdd(pTask, 
						pVariable, pString);
			dg87_free(pString, DG87_CHAR);
			break;
		  default:
			  break;
		}
		if (pCondition == NULL)
			return FALSE;
		if (dg87_condition_load(pCondition, pStorage) == FALSE)
			return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_context_load(dg87_context* pContext)
{
	dg87uint count, count2, count3, ipport;
	char *pString, *pString2;
	dg87byte b;
	dg87_storage *pStorage;
	dg87_target *pTarget;
	dg87_queue *pQueue;
	dg87_variable *pVariable;
	dg87_user *pUser;
	dg87_day *pDay;
	dg87_task *pTask, *pTask2;
	dg87_job *pJob, *pJob2;
	dg87_jobdep *pJobDep;
	dg87_taskdep *pTaskDep;
	time_t startTime;
	dg87long longvalue;

	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* open file */
	pStorage = pContext->pStorage;
	if (dg87_storage_open(pStorage, dg87_storage_mode_read) == FALSE)
		return FALSE;
	/* --------- Engine ---------------------- */
	if (dg87_engine_load(pContext->pEngine, pStorage) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	/* --------- Targets and queues ---------- */
	/* read target count */
	if (dg87_storage_readUInt(pStorage, &count) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	while (count > 0){
		count --;
		/* load target */
		if (dg87_storage_readString(pStorage, &pString) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if ((pTarget = dg87_context_targetAdd(pContext, pString)) 
				== FALSE){
			dg87_free(pString, DG87_CHAR);
			dg87_storage_close(pStorage);
			return FALSE;
		}
		dg87_free(pString, DG87_CHAR);
		if (dg87_target_load(pTarget, pStorage) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}

		/* --------- Queues --------- */
		/* read queue count */
		if (dg87_storage_readUInt(pStorage, &count2) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		while (count2 > 0){
			count2 --;
			/* load queue */
			if (dg87_storage_readString(pStorage, &pString) 
					== FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			if ((pQueue = dg87_target_queueAdd(pTarget, pString)) 
					== FALSE){
				dg87_free(pString, DG87_CHAR);
				dg87_storage_close(pStorage);
				return FALSE;
			}
			dg87_free(pString, DG87_CHAR);
			if (dg87_queue_load(pQueue, pStorage) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
		}
	}
	/* --------- Variables ------------------- */
	/* read variables count */
	if (dg87_storage_readUInt(pStorage, &count) == FALSE)
		return FALSE;
	while (count > 0){
		count --;
		/* load variable */
		if (dg87_storage_readString(pStorage, &pString) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if ((pVariable = dg87_engine_variableAdd(pContext->pEngine, 
						pString, "")) == FALSE){
			dg87_free(pString, DG87_CHAR);
			dg87_storage_close(pStorage);
			return FALSE;
		}
		dg87_free(pString, DG87_CHAR);
		if (dg87_variable_load(pVariable, pStorage) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
	}
	/* --------- Users ----------------------- */
	/* read users count */
	if (dg87_storage_readUInt(pStorage, &count) == FALSE)
		return FALSE;
	while (count > 0){
		count --;
		/* load user */
		if (dg87_storage_readByte(pStorage, &b) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_storage_readString(pStorage, &pString) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (b == TRUE)
			pUser = dg87_server_userGetByName(pContext->pServer, 
							pString);
		else
			pUser = dg87_server_userAdd(pContext->pServer, 
							pString);
		dg87_free(pString, DG87_CHAR);
		if (pUser == NULL){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_user_load(pUser, pStorage) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
	}

	/* --------- Days ------------------------ */
	/* read days count */
	if (dg87_storage_readUInt(pStorage, &count) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	while (count > 0){
		count --;
		/* load day */
		if (dg87_storage_readString(pStorage, &pString) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_storage_readLong(pStorage, &longvalue) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		startTime = longvalue;
		if ((pDay = dg87_context_dayAdd(pContext, 
						pString, startTime)) == FALSE){
			dg87_free(pString, DG87_CHAR);
			dg87_storage_close(pStorage);
			return FALSE;
		}
		dg87_free(pString, DG87_CHAR);
		if (dg87_day_load(pDay , pStorage) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}

		/* --------- Tasks ---------- */
		/* read task count */
		if (dg87_storage_readUInt(pStorage, &count2) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		while (count2 > 0){
			count2 --;
			/* load task */
			if (dg87_storage_readString(pStorage, &pString) 
					== FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			if ((pTask = dg87_day_taskAdd(pDay, pString)) 
					== FALSE){
				dg87_free(pString, DG87_CHAR);
				dg87_storage_close(pStorage);
				return FALSE;
			}
			dg87_free(pString, DG87_CHAR);
			if (dg87_task_load(pTask, pStorage, pContext) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}

			/* --------- Jobs ----------- */
			/* read job count */
			if (dg87_storage_readUInt(pStorage, &count3) == FALSE)
				return FALSE;
			while (count3 > 0){
				count3 --;
				/* load job */
				if (dg87_storage_readString(pStorage, &pString) 
						== FALSE){
					dg87_storage_close(pStorage);
					return FALSE;
				}
				if ((pJob = dg87_task_jobAdd(pTask, pString)) 
						== FALSE){
					dg87_free(pString, DG87_CHAR);
					dg87_storage_close(pStorage);
					return FALSE;
				}
				dg87_free(pString, DG87_CHAR);
				if (dg87_job_load(pJob, pStorage) == FALSE){
					dg87_storage_close(pStorage);
					return FALSE;
				}
			} /* next job */

			/* --------- Jobs Deps ----------- */
			/* read jobdep count */
			if (dg87_storage_readUInt(pStorage, &count3) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			while (count3 > 0){
				count3 --;
				/* find job before */
				if (dg87_storage_readString(pStorage, &pString) 
						== FALSE){
					dg87_storage_close(pStorage);
					return FALSE;
				}
				pJob = dg87_task_jobGetByName(pTask, pString);
				dg87_free(pString, DG87_CHAR);
				/* find job after */
				if (dg87_storage_readString(pStorage, &pString) 
						== FALSE){
					dg87_storage_close(pStorage);
					return FALSE;
				}
				pJob2 = dg87_task_jobGetByName(pTask, pString);
				dg87_free(pString, DG87_CHAR);
				/* load dependence */
				if ((pJobDep = dg87_task_jobdepAdd(pTask, 
						pJob, pJob2)) == NULL){
					dg87_free(pString, DG87_CHAR);
					dg87_storage_close(pStorage);
					return FALSE;
				}
				if (dg87_jobdep_load(pJobDep, pStorage)==FALSE){
					dg87_storage_close(pStorage);
					return FALSE;
				}
			} /* next jobdep */

			/* --------- Tasks conditions----- */
			if (dg87_context_loadConditions(pContext,
						pTask, pStorage) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}

		} /* next task */

		/* --------- Task Deps ----------- */
		/* read taskdep count */
		if (dg87_storage_readUInt(pStorage, &count2) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		while (count2 > 0){
			count2 --;
			/* find task before */
			if (dg87_storage_readString(pStorage, &pString) 
					== FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			pTask = dg87_day_taskGetByName(pDay, pString);
			dg87_free(pString, DG87_CHAR);
			/* find task after */
			if (dg87_storage_readString(pStorage, &pString) 
					== FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			pTask2= dg87_day_taskGetByName(pDay, pString);
			dg87_free(pString, DG87_CHAR);
			/* load dependence */
			if ((pTaskDep = dg87_day_taskdepAdd(pDay, 
					pTask, pTask2)) == NULL){
				dg87_free(pString, DG87_CHAR);
				dg87_storage_close(pStorage);
				return FALSE;
			}
			if (dg87_taskdep_load(pTaskDep, pStorage)== FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
		} /* next taskdep */
	} /* next day */

	/* --------- Messages -------------------- */
	/* read messages count */
	if (dg87_storage_readUInt(pStorage, &count) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	while (count > 0){
		count --;
		/* load host name */
		if (dg87_storage_readString(pStorage, &pString) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* load IP port */
		if (dg87_storage_readUInt(pStorage, &ipport) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* load message */
		if (dg87_storage_readString(pStorage, &pString2) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* post message */
		if (dg87_server_postMessage(pContext->pServer,
					pString, ipport, pString2) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
	}
	return TRUE;
}
/****************************************************************************/
static BOOL dg87_context_saveConditions(dg87_context* pContext,
		dg87_task *pTask, dg87_storage *pStorage)
{
	dg87_item *pItem;
	dg87_condition *pCondition;
	/* save conditions count */
	if (dg87_storage_writeUInt(pStorage, 
			pTask->pConditionList->count) == FALSE)
		return FALSE;
	/* for each condition of the task */
	pItem =  pTask->pConditionList->pFirst; 
	while (pItem != NULL){
		pCondition = pItem->pObject;
		pItem = pItem->pNext;
		/* save condition */
		if (dg87_storage_writeByte(pStorage, pCondition->type) == FALSE)
			return FALSE;
		switch (pCondition->type){
		  case condition_hmin:
			if (dg87_storage_writeUInt( pStorage, 
				pCondition->data.pConditionHmin->minutes) 
					== FALSE)
				return FALSE;
			break;
		  case condition_hmax:
			if (dg87_storage_writeUInt( pStorage, 
				pCondition->data.pConditionHmax->minutes) 
					== FALSE)
				return FALSE;
			break;
		  case condition_hlate:
			if (dg87_storage_writeUInt( pStorage, 
				pCondition->data.pConditionHlate->minutes) 
					== FALSE)
				return FALSE;
			break;
		  case condition_file:
			if (dg87_storage_writeString(pStorage,
				pCondition->data.pConditionFile->pPath)
					== FALSE)
				return FALSE;
			break;
		  case condition_manual:
			if (dg87_storage_writeString(pStorage,
				pCondition->data.pConditionManual->pMessage)
					== FALSE)
				return FALSE;
			break;
		  case condition_ifequal:
			if (dg87_storage_writeString(pStorage,
					pCondition->data.pConditionIfEqual->
					pVariable->name) == FALSE)
				return FALSE;
			if (dg87_storage_writeString(pStorage,
					pCondition->data.pConditionIfEqual->
					pValue) == FALSE)
				return FALSE;
			break;
		  case condition_ifnequal:
			if (dg87_storage_writeString(pStorage,
					pCondition->data.pConditionIfNotEqual->
					pVariable->name) == FALSE)
				return FALSE;
			if (dg87_storage_writeString(pStorage,
					pCondition->data.pConditionIfNotEqual->
					pValue) == FALSE)
				return FALSE;

			break;
		  default:
			break;
		}
		if (dg87_condition_save(pCondition, pStorage) == FALSE)
			return FALSE;
	} /* next condition */
	return TRUE;
}
/****************************************************************************/
BOOL dg87_context_save(dg87_context* pContext)
{
	dg87_element* pElement;
	dg87_element* pElement2;
	dg87_element* pElement3;
	dg87_item* pItem;
	dg87_storage *pStorage;
	dg87_target* pTarget;
	dg87_queue* pQueue;
	dg87_variable* pVariable;
	dg87_user* pUser;
	dg87_day *pDay;
	dg87_task *pTask;
	dg87_job *pJob;
	dg87_jobdep *pJobDep;
	dg87_taskdep *pTaskDep;
	dg87_message *pMessage;

	if (pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* open file */
	pStorage = pContext->pStorage;
	if (dg87_storage_open(pStorage, dg87_storage_mode_write) == FALSE)
		return FALSE;
	/* --------- Engine ---------------------- */
	if (dg87_engine_save(pContext->pEngine, pStorage) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}	
	/* --------- Targets --------------------- */
	/* save target count */
	if (dg87_storage_writeUInt(pStorage, 
				pContext->pTargetVector->count) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	/* for each target */
	pElement = pContext->pTargetVector->pFirst;
	while (pElement != NULL){
		pTarget = pElement->pObject;
		pElement = pElement->pNext;
		/* save target */
		if (dg87_storage_writeString(pStorage, pTarget->name) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_target_save(pTarget, pStorage) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}

		/* --------- Queues --------------- */
		/* save queue count */
		if (dg87_storage_writeUInt(pStorage, 
				pTarget->pQueueVector->count) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* for each queue of the target */
		pElement2 = pTarget->pQueueVector->pFirst; 
		while (pElement2 != NULL){
			pQueue = pElement2->pObject;
			pElement2 = pElement2->pNext;
			/* save queue */
			if (dg87_storage_writeString(pStorage, 
						pQueue->name) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			if (dg87_queue_save(pQueue, pStorage) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
		}
	}
	/* --------- Variables ------------------- */
	/* save variable count */
	if (dg87_storage_writeUInt(pStorage, 
			pContext->pEngine->pVariableVector->count) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	/* for each variable*/
	pElement = pContext->pEngine->pVariableVector->pFirst;
	while (pElement != NULL){
		pVariable = pElement->pObject;
		pElement = pElement->pNext;
		/* save variable */
		if (dg87_storage_writeString(pStorage, pVariable->name)==FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_variable_save(pVariable, pStorage) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
	}
	/* --------- Users ----------------------- */
	/* save user count */
	if (dg87_storage_writeUInt(pStorage, 
			pContext->pServer->pUserVector->count) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	/* for each user */
	pElement = pContext->pServer->pUserVector->pFirst;
	while (pElement != NULL){
		pUser = pElement->pObject;
		pElement = pElement->pNext;
		/* save User */
		if (dg87_storage_writeByte(pStorage, 
					pUser->systemAccount) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_storage_writeString(pStorage, pUser->name)== FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_user_save(pUser, pStorage) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
	}
	/* --------- Days ------------------------ */
	/* save days count */
	if (dg87_storage_writeUInt(pStorage, 
			pContext->pDayVector->count) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	/* for each days */
	pElement = pContext->pDayVector->pFirst;
	while (pElement != NULL){
		pDay = pElement->pObject;
		pElement = pElement->pNext;
		/* save day */
		if (dg87_storage_writeString(pStorage, pDay->name)== FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_storage_writeLong(pStorage, pDay->startTime)==FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_day_save(pDay, pStorage) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}

		/* --------- Tasks ---------------- */
		/* save tasks count */
		if (dg87_storage_writeUInt(pStorage, 
				pDay->pTaskVector->count) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* for each task of the day */
		pElement2 = pDay->pTaskVector->pFirst; 
		while (pElement2 != NULL){
			pTask = pElement2->pObject;
			pElement2 = pElement2->pNext;
			/* save task */
			if (dg87_storage_writeString(pStorage, 
						pTask->name) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			if (dg87_task_save(pTask, pStorage) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			/* --------- Jobs ---------------- */
			/* save jobs count */
			if (dg87_storage_writeUInt(pStorage, 
					pTask->pJobVector->count) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			/* for each job of the task */
			pElement3 = pTask->pJobVector->pFirst; 
			while (pElement3 != NULL){
				pJob = pElement3->pObject;
				pElement3 = pElement3->pNext;
				/* save job */
				if (dg87_storage_writeString(pStorage, 
							pJob->name) == FALSE){
					dg87_storage_close(pStorage);
					return FALSE;
				}
				if (dg87_job_save(pJob, pStorage) == FALSE){
					dg87_storage_close(pStorage);
					return FALSE;
				}
			} /* next job */

			/* --------- Jobs Dep-------------- */
			/* save job dependences count */
			if (dg87_storage_writeUInt(pStorage, 
					pTask->pJobDepList->count) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			/* for each jobdep of the task */
			pItem =  pTask->pJobDepList->pFirst; 
			while (pItem != NULL){
				pJobDep = pItem->pObject;
				pItem = pItem->pNext;
				/* save jobdep */
				if (dg87_storage_writeString(pStorage, 
					pJobDep->pJobBefore->name) == FALSE){
					dg87_storage_close(pStorage);
					return FALSE;
				}
				if (dg87_storage_writeString(pStorage, 
					pJobDep->pJobAfter->name) == FALSE){
					dg87_storage_close(pStorage);
					return FALSE;
				}
				if (dg87_jobdep_save(pJobDep, pStorage)==FALSE){
					dg87_storage_close(pStorage);
					return FALSE;
				}
			} /* next jobdep */

			/* --------- Tasks conditions ----- */
			if (dg87_context_saveConditions(pContext, pTask, 
						pStorage) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			
		} /* next task */

		/* --------- Task Dependences -------------- */
		/* save task dependences count */
		if (dg87_storage_writeUInt(pStorage, 
				pDay->pTaskdepList->count) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		/* for each taskdep of the task */
		pItem =  pDay->pTaskdepList->pFirst; 
		while (pItem != NULL){
			pTaskDep = pItem->pObject;
			pItem = pItem->pNext;
			/* save taskdep */
			if (dg87_storage_writeString(pStorage, 
					pTaskDep->pTaskBefore->name) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			if (dg87_storage_writeString(pStorage, 
					pTaskDep->pTaskAfter->name) == FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
			if (dg87_taskdep_save(pTaskDep, pStorage)== FALSE){
				dg87_storage_close(pStorage);
				return FALSE;
			}
		} /* next taskdep */


	} /* next day */

	/* --------- Messages -------------------- */
	/* save messages count */
	if (dg87_storage_writeUInt(pStorage, 
			pContext->pServer->pListMessage->count) == FALSE){
		dg87_storage_close(pStorage);
		return FALSE;
	}
	pItem = pContext->pServer->pListMessage->pFirst;
	while (pItem != NULL){
		pMessage = pItem->pObject;
		pItem = pItem->pNext;
		if (dg87_storage_writeString(pStorage, 
				pMessage->pHost) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_storage_writeUInt(pStorage, 
				pMessage->ipPort) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
		if (dg87_storage_writeString(pStorage, 
				pMessage->pMsgString) == FALSE){
			dg87_storage_close(pStorage);
			return FALSE;
		}
	}

	/* close file */
	dg87_storage_close(pStorage);
	return TRUE;
}
/****************************************************************************/

#ifdef DEBUG
int dg87_context_check(dg87_context* pContext, int (*print)(char*))
{
	dg87_element* pElement;

	/* check DayVector */
	if (pContext->pDayVector == NULL){
		print("Context : pDayVector is null !\n");
		return 1;
	}
	if (dg87_vector_check(pContext->pDayVector, print) != 0)
		return 1;
	pElement = pContext->pDayVector->pFirst;
	while (pElement != NULL){
		if (dg87_day_check(pElement->pObject, print) != 0)
			return 1;
		pElement = pElement->pNext;
	}
	
	/* check TargetVector */
	if (pContext->pTargetVector == NULL){
		print("Context : pTargetVector is null !\n");
		return 1;
	}
	if (dg87_vector_check(pContext->pTargetVector, print) != 0)
		return 1;
	pElement = pContext->pTargetVector->pFirst;
	while (pElement != NULL){
		if (dg87_target_check(pElement->pObject, print) != 0)
			return 1;
		pElement = pElement->pNext;
	}
	return 0;
}
#endif /*DEBUG*/

/****************************************************************************/
/* End of file */
