/* file day.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "day.h"
#include <libdg87/element.h>
#include "global.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/****************************************************************************/
dg87_day* dg87_day_new(char* pName, time_t startTime)
{
	dg87_day* pDay;

	if (pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}

	/* allocate memory for day */
	pDay = dg87_malloc(sizeof(dg87_day), DG87_DAY);
	if (pDay == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}

	/* allocate memory for the TaskVector */
	pDay->pTaskVector = dg87_vector_new();
	if (pDay->pTaskVector == NULL) {
		dg87_free(pDay, DG87_DAY);
		return NULL;
	}
	/* allocate memory for the TaskdepList */
	pDay->pTaskdepList = dg87_list_new();
	if (pDay->pTaskdepList == NULL) {
		dg87_vector_delete(pDay->pTaskVector);
		dg87_free(pDay, DG87_DAY);
		return NULL;
	}	
	/* initialize the structure of the object*/
	strncpy(pDay->name,pName, DG87_DAY_NAME_LENGTH);
	pDay->enabled = FALSE;
	pDay->locked = TRUE;
	pDay->startTime = startTime;
	return pDay;
}
/****************************************************************************/
BOOL dg87_day_delete(dg87_day* pDay)
{
	dg87_task *pTask;
	dg87_taskdep *pTaskDep;
	if (pDay == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* delete all taskdep objects */
	while (pDay->pTaskdepList->count > 0){
		pTaskDep = dg87_list_removeObjectByIndex(pDay->pTaskdepList, 0);
		dg87_taskdep_delete(pTaskDep);
	}
	dg87_list_delete(pDay->pTaskdepList);

	/* delete all task object */
	while (pDay->pTaskVector->count > 0){
		pTask = dg87_vector_removeObjectByIndex(pDay->pTaskVector,0);
		dg87_list_clear(pTask->pTaskDepBeforeList);
		dg87_list_clear(pTask->pTaskDepAfterList);
		dg87_task_delete(pTask);
	}
	dg87_vector_delete(pDay->pTaskVector);
	dg87_free(pDay, DG87_DAY);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_day_load(dg87_day* pDay, dg87_storage* pStorage)
{
	BOOL value;
	if (pDay == NULL || pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* 'enabled' property */
	if (dg87_storage_readByte(pStorage, &value) == FALSE)
		return FALSE;
	pDay->enabled = value;
	/* 'locked' property */
	if (dg87_storage_readByte(pStorage, &value) == FALSE)
		return FALSE;
	pDay->locked = value;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_day_save(dg87_day* pDay, dg87_storage* pStorage)
{
	if (pDay == NULL || pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* 'enabled' property */
	if (dg87_storage_writeByte(pStorage, pDay->enabled) == FALSE)
		return FALSE;
	/* 'locked' property */
	if (dg87_storage_writeByte(pStorage, pDay->locked) == FALSE)
		return FALSE;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_day_setEnabled(dg87_day* pDay, BOOL value)
{
	if (pDay == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pDay->enabled = value;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_day_validate(dg87_day* pDay)
{
	if (pDay == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pDay->locked = FALSE;
	return TRUE;
}
/****************************************************************************/
dg87_task* dg87_day_taskAdd(dg87_day* pDay, char* pTaskName)
{
	dg87_task* pTask = NULL;
	if (pDay == NULL || pTaskName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pTask = dg87_task_new(pTaskName,pDay);
	if (pTask == NULL) 
		return NULL;

	/* add the task in the TaskList */
	/* vector generate error if an other task has not the same name*/
	/* key must point on pTask->name */
	if (dg87_vector_addObject(pDay->pTaskVector, 
				pTask, pTask->name) == FALSE)
	{
		dg87_task_delete(pTask);
		return NULL;
	}
	return pTask;
}
/****************************************************************************/
int dg87_day_taskCount(dg87_day* pDay)
{
	if (pDay == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_vector_count(pDay->pTaskVector);
}
/****************************************************************************/
dg87_task* dg87_day_taskGetByName(dg87_day* pDay,char* pName)
{
	dg87_element *pElement;
	if (pDay == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}

	/* browse the list to find an item with the same name*/
	pElement = pDay->pTaskVector->pFirst;
	while (pElement != NULL){
		if (strcmp(pElement->pKey, pName) == 0)
			return pElement->pObject;
		pElement = pElement->pNext;
	}	
	dg87_err_setError(DG87_ERR_CANTFINDOBJECT);
	return NULL;
}
/****************************************************************************/
dg87_task* dg87_day_taskGetByIndex(dg87_day* pDay,int index)
{
	if (pDay == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByIndex(pDay->pTaskVector, index);
}
/****************************************************************************/
BOOL dg87_day_taskDelete(dg87_day* pDay,dg87_task* pTask)
{
	if (pDay == NULL || pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_vector_removeObject(pDay->pTaskVector, pTask) == FALSE)
		return FALSE;
	if (dg87_task_delete(pTask) == FALSE){
		/* add task in vector again */
		dg87_vector_addObject(pDay->pTaskVector, pTask, pTask->name); 
		return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_day_taskDeleteByName(dg87_day* pDay,char* pName)
{
	dg87_task* pTask;
	if (pDay == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTask = dg87_vector_getObjectByKey(pDay->pTaskVector, pName);
	if (pTask == NULL)
		return FALSE;
	return dg87_day_taskDelete(pDay, pTask);
}
/****************************************************************************/
BOOL dg87_day_taskDeleteByIndex(dg87_day* pDay,int index)
{
	dg87_task* pTask;
	if (pDay == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTask = dg87_vector_getObjectByIndex(pDay->pTaskVector, index);
	if (pTask == NULL) 
		return FALSE;
	return dg87_day_taskDelete(pDay, pTask);
}
/****************************************************************************/
dg87_taskdep* dg87_day_taskdepAdd (dg87_day* pDay, 
	 struct dg87_task_struct* pTaskBefore, 
	 struct dg87_task_struct* pTaskAfter)
{
	dg87_taskdep* pTaskDep;

	if (pDay == NULL || pTaskBefore == NULL || pTaskAfter == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}

	/* create and add the taskdep */
	pTaskDep = dg87_taskdep_new(pTaskBefore, pTaskAfter);
	if (pTaskDep == NULL) 
		return NULL;

	dg87_list_addObject(pDay->pTaskdepList, pTaskDep);
	dg87_list_addObject(pTaskBefore->pTaskDepAfterList, pTaskDep);
	dg87_list_addObject(pTaskAfter->pTaskDepBeforeList, pTaskDep);

	return pTaskDep;
}
/****************************************************************************/
int dg87_day_taskdepCount(dg87_day* pDay)
{
	if (pDay == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_list_count(pDay->pTaskdepList);
}
/****************************************************************************/
dg87_taskdep* dg87_day_taskdepGet(dg87_day* pDay, int index)
{
	if (pDay == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_list_getObjectByIndex(pDay->pTaskdepList, index);
}
/****************************************************************************/
BOOL dg87_day_taskdepDeleteByIndex(dg87_day* pDay,int index)
{
	dg87_taskdep* pTaskDep;
	if (pDay == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTaskDep = dg87_list_removeObjectByIndex(pDay->pTaskdepList, index);
	if (pTaskDep == NULL) return FALSE;
	return dg87_taskdep_delete(pTaskDep);
}
/****************************************************************************/
BOOL dg87_day_taskdepDelete(dg87_day* pDay,dg87_taskdep *pTaskDep)
{
	if (pDay == NULL || pTaskDep == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_list_removeObject(pDay->pTaskdepList, pTaskDep) == FALSE)
		return FALSE;
	return dg87_taskdep_delete(pTaskDep);
}
/****************************************************************************/
#ifdef DEBUG
int dg87_day_check(dg87_day* pDay, int (*print)(char*))
{
	dg87_element* pElement;
	dg87_item* pItem;
	dg87_taskdep* pTaskDep;

	/* check TaskVector*/
	if (pDay->pTaskVector == NULL){
		print("Day : TaskVector is null !\n");
		return 1;
	}
	if (dg87_vector_check(pDay->pTaskVector, print) != 0)
		return 1;
	pElement = pDay->pTaskVector->pFirst;
	while (pElement != NULL){
		if (dg87_task_check(pElement->pObject, print) != 0)
			return 1;
		pElement = pElement->pNext;
	}
	
	/* check TaskdepList */
	if (pDay->pTaskdepList == NULL){
		print("Day : TaskdepList is null !\n");
		return 1;
	}
	if (dg87_list_check(pDay->pTaskdepList, print) != 0)
		return 1;
	pItem = pDay->pTaskdepList->pFirst;
	while (pItem != NULL){
		pTaskDep = pItem->pObject;
		if (dg87_taskdep_check(pTaskDep, print) != 0)
			return 1;
		pItem = pItem->pNext;
	}
	return 0;
}
#endif /*DEBUG*/


/****************************************************************************/
/* End of file */
