/* file engine.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <libdg87/daemon.h>
#include "engine.h"
#include "variable.h"

/******************************************************************************/
dg87_engine* dg87_engine_new(dg87_context* pContext)
{
	dg87_engine* pEngine;

	if (pContext == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
       
	pEngine = dg87_malloc(sizeof(dg87_engine), DG87_ENGINE);
	if (pEngine == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pEngine->pVariableVector = dg87_vector_new();
	if (pEngine->pVariableVector == NULL){
		dg87_engine_delete(pEngine);
		return NULL;
	}

	pEngine->pContext = pContext;
	pEngine->enabled = FALSE;
	return pEngine;
}
/******************************************************************************/
BOOL dg87_engine_delete(dg87_engine* pEngine)
{
	dg87_variable *pVariable;

	if (pEngine == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pEngine->pVariableVector != NULL){
		while (pEngine->pVariableVector->count > 0){
			pVariable = dg87_vector_removeObjectByIndex(
					pEngine->pVariableVector, 0);
			/* force variable deletion */
			pVariable->linkedConditions = 0;
			dg87_variable_delete(pVariable);
		}
		dg87_vector_delete(pEngine->pVariableVector);
	}

	dg87_free(pEngine, DG87_ENGINE);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_engine_save(dg87_engine* pEngine, dg87_storage* pStorage)
{
	if (pEngine == NULL || pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* 'enabled' property */
	if (dg87_storage_writeByte(pStorage, pEngine->enabled) == FALSE)
		return FALSE;

	return TRUE;
}
/****************************************************************************/
BOOL dg87_engine_load(dg87_engine* pEngine, dg87_storage* pStorage)
{
	BOOL value;
	if (pEngine == NULL || pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* 'enabled' property */
	if (dg87_storage_readByte(pStorage, &value) == FALSE)
		return FALSE;
	pEngine->enabled = value;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_engine_run(dg87_engine* pEngine, dg87_server* pServer)
{
	dg87_element* pElementDay;
	dg87_element* pElementTask;
	dg87_day* pDay;
	dg87_task* pTask;
	dg87_element* pElementTarget;
	dg87_element* pElementQueue;
	dg87_queue* pQueue;
	dg87_target* pTarget;

	if (pEngine == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* save database */
	dg87_context_makePeriodicSave(pEngine->pContext);
	/* do nothing if engine is not enabled */
	if (pEngine->enabled == FALSE)
		return FALSE;

	/* for each day */
	pElementDay = pEngine->pContext->pDayVector->pFirst;
	while (pElementDay != NULL) {
		pDay = pElementDay->pObject;
		pElementDay = pElementDay->pNext;
		/* do nothing if day is locked or disabled */
		if (pDay->locked == TRUE || pDay->enabled == FALSE)
			continue;
		/* do nothing if startTime is not reached */
		if (pDay->startTime > time(0))
			continue;
		/*  update each task of the day */
		pElementTask = pDay->pTaskVector->pFirst;
		while (pElementTask != NULL){
			pTask = pElementTask->pObject;
			dg87_task_manage(pTask);
			pElementTask = pElementTask->pNext;
		}
	}
	/* for each target */
	pElementTarget = pEngine->pContext->pTargetVector->pFirst;
	while (pElementTarget != NULL){
		pTarget = pElementTarget->pObject;

		/* for each queue */
		pElementQueue = pTarget->pQueueVector->pFirst;
		while (pElementQueue != NULL){
			pQueue = pElementQueue->pObject;
			dg87_queue_manage(pEngine->pContext,pQueue);
			pElementQueue = pElementQueue->pNext;
		}

		pElementTarget = pElementTarget->pNext;
	}

	/* return FALSE to have a long time before next call */
	return FALSE;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_engine_check(dg87_engine* pEngine, int (*print)(char*))
{
	/* TODO */
	return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
dg87_variable* dg87_engine_variableAdd(dg87_engine* pEngine, 
		char* pVariableName, char* pValue)
{
	dg87_variable* pVariable;
	if (pEngine == NULL || pVariableName == NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pVariable = dg87_variable_new(pVariableName, pValue);
	if (pVariable == NULL)
		return NULL;

	/* vector generate error if an other variable has the same name */
	/* key must point on pVariable->name */
	if (dg87_vector_addObject(pEngine->pVariableVector, 
				pVariable, pVariable->name) == FALSE)
	{
		dg87_variable_delete(pVariable);
		return NULL;
	}
	return pVariable;
}
/****************************************************************************/
int dg87_engine_variableCount(dg87_engine* pEngine)
{
	if (pEngine == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_vector_count(pEngine->pVariableVector);
}
/****************************************************************************/
dg87_variable* dg87_engine_variableGetByName(dg87_engine* pEngine, char* pName)
{
	if (pEngine == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByKey(pEngine->pVariableVector, pName);
}
/****************************************************************************/
dg87_variable* dg87_engine_variableGetByIndex(dg87_engine* pEngine, int index)
{
	if (pEngine == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByIndex(pEngine->pVariableVector, index);
}

/****************************************************************************/
BOOL dg87_engine_variableDeleteByName(dg87_engine* pEngine,char* pName)
{
	dg87_variable* pVariable;
	if (pEngine == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pVariable = dg87_vector_removeObjectByKey
		(pEngine->pVariableVector, pName);
	if (pVariable == NULL) 
		return FALSE;
	dg87_variable_delete(pVariable);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_engine_variableDeleteByIndex(dg87_engine* pEngine,int index)
{
	dg87_variable* pVariable;
	if (pEngine == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pVariable = dg87_vector_removeObjectByIndex
		(pEngine->pVariableVector, index);
	if (pVariable == NULL) 
		return FALSE;
	dg87_variable_delete(pVariable);
	return TRUE;
}
/****************************************************************************/
/* End of file */
