/* file job.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "job.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/* order in this array must correspond to dg87_job_state */
/* this array must be finished by a NULL */
char* dg87_job_stateString[]={
	"uninitialize",
	"ignore",
	"wait_for_dep",
	"wait_enabled",
	"ready_to_run",
	"canceled",
	"ready_to_submit",
	"submit",
	"deleted",
	"running",
	"exit",
	"terminated",
	"ignored",
	"aborted",
	"abort_stopped",
	"delete_stopped",
	"wait_start_tempo",
	"wait_end_tempo",
	/*"ignored",*/
	NULL
};

/****************************************************************************/
dg87_job* dg87_job_new(dg87_task* pTask, char* pName)
{
	dg87_job* pJob;
	if (pTask == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	if (strlen(pName) >= DG87_JOB_NAME_LENGTH){
		dg87_err_setError(DG87_ERR_NAMETOLONG);
		return FALSE;
	}
	pJob = dg87_malloc(sizeof(dg87_job), DG87_JOB);
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	strncpy(pJob->name, pName, DG87_JOB_NAME_LENGTH); 
	pJob->pTask = pTask;
	pJob->description[0] = 0;
	pJob->pPath = NULL;
	pJob->pArgument = NULL;
	pJob->startTempo = 0; 
	pJob->endTempo = 0;
	pJob->state = dg87_job_state_uninitialize;
	pJob->exitCode = 0;
	pJob->time = 0;
	pJob->waitAllDependences = TRUE;
	pJob->onErrorContinue = TRUE;
	pJob->enabled = TRUE;
	return pJob;
}

/****************************************************************************/
BOOL dg87_job_delete(dg87_job* pJob)
{
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* ensure that job isn't in running jobs list of the queue*/
	if (pJob->pTask->pQueue != NULL)
		dg87_queue_removeRunningJob(pJob->pTask->pQueue, pJob);
	/* free memory */
	if (pJob->pPath != NULL)
		dg87_free(pJob->pPath, DG87_CHAR);
	if (pJob->pArgument != NULL)
		dg87_free(pJob->pArgument, DG87_CHAR);
	dg87_free(pJob, DG87_JOB);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_load(dg87_job* pJob, dg87_storage* pStorage)
{	
	char* pString;
	dg87long longValue;
	dg87int intValue;
	dg87byte b, b2;
	if (pJob == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read job description*/
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_job_setDescription(pJob, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);
	/* read job path */
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_job_setPath(pJob, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);
	/* read job arugment */
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_job_setArgument(pJob, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);

	/* read start tempo and end tempo */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_storage_readByte(pStorage, &b2) == FALSE)
		return FALSE;
	if (dg87_job_setTempo(pJob, b, b2) == FALSE)
		return FALSE;

	/* read waitAllDependences */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_job_setWaitAllDependences(pJob, b) == FALSE)
		return FALSE;

	/* read onErrorContinue */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_job_setOnErrorContinue(pJob, b) == FALSE)
		return FALSE;
	/* TODO enabled state time exitCode */
	/* read 'enabled' property */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	pJob->enabled = b;
	/* read 'state' property */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	pJob->state = b;
	/* read 'time' property */
	if (dg87_storage_readLong(pStorage, &longValue) == FALSE)
		return FALSE;
	pJob->time = longValue;
	/* read 'exitCode' property */
	if (dg87_storage_readInt(pStorage, &intValue) == FALSE)
		return FALSE;
	pJob->exitCode = intValue;

	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_save(dg87_job* pJob, dg87_storage* pStorage)
{	
	if (pJob == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write job description */
	if (dg87_storage_writeString(pStorage, pJob->description) == FALSE)
		return FALSE;
	/* write job path */
	if (dg87_storage_writeString(pStorage, 
			(pJob->pPath == NULL)?"":pJob->pPath) == FALSE)
		return FALSE;
	/* write job argument */
	if (dg87_storage_writeString(pStorage, 
			(pJob->pArgument == NULL)?"":pJob->pArgument) == FALSE)
		return FALSE;
	/* write start tempo */
	if (dg87_storage_writeByte(pStorage, pJob->startTempo)== FALSE)
		return FALSE;
	/* write end tempo */
	if (dg87_storage_writeByte(pStorage, pJob->endTempo)== FALSE)
		return FALSE;
	/* write waitAllDependences */
	if (dg87_storage_writeByte(pStorage, pJob->waitAllDependences)== FALSE)
		return FALSE;
	/* write onErrorContinue */
	if (dg87_storage_writeByte(pStorage, pJob->onErrorContinue)== FALSE)
		return FALSE;
	/* 'enabled' property */
	if (dg87_storage_writeByte(pStorage, pJob->enabled) == FALSE)
		return FALSE;
	/* 'state' property */
	if (dg87_storage_writeByte(pStorage, pJob->state) == FALSE)
		return FALSE;
	/* 'time' property */
	if (dg87_storage_writeLong(pStorage, pJob->time) == FALSE)
		return FALSE;
	/* 'exitCode' property */
	if (dg87_storage_writeInt(pStorage, pJob->exitCode) == FALSE)
		return FALSE;
	return TRUE;
}
/****************************************************************************/
dg87_job_status dg87_job_getStatus(dg87_job* pJob)
{
	switch (pJob->state){
	case dg87_job_state_uninitialize: 
	case dg87_job_state_ignore: 
	case dg87_job_state_waitfordep: 
	case dg87_job_state_waitenabled: 
		return dg87_job_status_waiting;

	case dg87_job_state_readytorun: 
	case dg87_job_state_abortStopped: 
	case dg87_job_state_deleteStopped: 
	case dg87_job_state_waitStartTempo: 
	case dg87_job_state_waitEndTempo:
	case dg87_job_state_readytosubmit:
	case dg87_job_state_submit: 
	case dg87_job_state_running:
	case dg87_job_state_exit: 
		return dg87_job_status_running;

	case dg87_job_state_terminated: 
	case dg87_job_state_ignored: 
	case dg87_job_state_canceled: 
		return dg87_job_status_terminated;

	case dg87_job_state_deleted: 
	case dg87_job_state_aborted: 
		return dg87_job_status_aborted;
	}
	return -1;
}
/****************************************************************************/
dg87_job_state dg87_job_manage(dg87_job* pJob)
{
	dg87_item* pItem;
	dg87_jobdep* pJobDep;
	/*int nSearch,nMax;*/
	dg87_job_state newState;

	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE; 
	}

	/* if job isn't initialize, find if it is waiting for dependences */
	if (pJob->state == dg87_job_state_uninitialize ){
		pItem = pJob->pTask->pJobDepList->pFirst;
		while (pItem != NULL){
			pJobDep = pItem->pObject;
			if (pJobDep->pJobAfter == pJob)
				break;
			pItem = pItem->pNext;
		}
		if (pItem == NULL)
			dg87_job_setState(pJob, dg87_job_state_waitenabled,-1);
		else
			dg87_job_setState(pJob, dg87_job_state_waitfordep,-1);
	}

	if (pJob->state == dg87_job_state_waitfordep ||
			pJob->state == dg87_job_state_ignore ){
		if (pJob->waitAllDependences == TRUE){
			/* Job is waiting for all dependences (AND) 
			 * 1- Job is ready to run if all dependences are
			 * validated.
			 * 2- Job is canceled if at least one dependence is
			 * canceled.
			 * 3- Job is waiting if at least one dependence is
			 * waiting */
			newState = dg87_job_state_waitenabled;
			pItem = pJob->pTask->pJobDepList->pFirst;
			while (pItem != NULL){
				pJobDep = pItem->pObject;
				pItem = pItem->pNext;
				if (pJobDep->pJobAfter != pJob) 
					continue;
				/* if a dep is canceled, then job is canceled*/
				if (pJobDep->state == 
						dg87_jobdep_state_canceled){
					newState = dg87_job_state_canceled;
					break;
				}
				/* if a dep is waiting, then job is waiting */
				if (pJobDep->state == 
						dg87_jobdep_state_waiting){
					newState = dg87_job_state_waitfordep;
					break;
				}

			}

		} else {
			/* Job is waiting for one dependence (OR). 
			 * 1- Job is ready to run if at least dependence is
			 * validated.
			 * 2- Job is canceled if all dependences are validated.
			 * 3- Job is waiting for dependence if at least one 
			 * dependence is waiting.
			 */ 
			newState = dg87_job_state_canceled;
			pItem = pJob->pTask->pJobDepList->pFirst;
			while (pItem != NULL){
				pJobDep = pItem->pObject;
				pItem = pItem->pNext;
				if (pJobDep->pJobAfter != pJob) 
					continue;
				/*if a dep is validated, then job is validated*/
				if (pJobDep->state == 
						dg87_jobdep_state_validated){
					newState = dg87_job_state_waitenabled;
					break;
				}
				/* if a dep is waiting, then job is waiting */
				if (pJobDep->state == 
						dg87_jobdep_state_waiting){
					newState = dg87_job_state_waitfordep;
					break;
				}

			}
		}

		switch (pJob->state){
			case dg87_job_state_waitfordep :
				dg87_job_setState(pJob, newState ,-1);
				break;
			case dg87_job_state_ignore:
				if (newState != dg87_job_state_waitfordep)
					dg87_job_setState(pJob,
						dg87_job_state_ignored,-1);
			default:
				/* just to avoid warnings */
				break;

		}
	}

	/* job must be enabled to be ready to run */
	if (pJob->state == dg87_job_state_waitenabled && pJob->enabled == TRUE)
		dg87_job_setState(pJob, dg87_job_state_readytorun,-1);

	/* if job is ready to run, wait start tempo*/
	if (pJob->state == dg87_job_state_readytorun){
		pJob->time = time(0);
		dg87_job_setState(pJob,dg87_job_state_waitStartTempo,-1);
	}

	/* after start tempo, launch the job */
	if (pJob->state == dg87_job_state_waitStartTempo){
		if ( time(0) >= pJob->startTempo + pJob->time)
			dg87_job_setState(pJob,dg87_job_state_readytosubmit,-1);
	}

	/* if job is exited, wait for end tempo */
	if (pJob->state == dg87_job_state_exit){
		pJob->time = time(0);
		dg87_job_setState(pJob,dg87_job_state_waitEndTempo, -1);
	}
	/* after end tempo, terminate job */
	if (pJob->state == dg87_job_state_waitEndTempo && 
		time(0) >= pJob->endTempo + pJob->time)

	{
		if (pJob->exitCode == 0)
			/* normal termination */
			dg87_job_setState(pJob,dg87_job_state_terminated,-1);
		else {
			/* stop or continue after an abort */
			if (pJob->onErrorContinue == TRUE)
				dg87_job_setState(pJob,
						dg87_job_state_aborted,-1);
			else
				dg87_job_setState(pJob,
						dg87_job_state_abortStopped,-1);

		}
	}

	return pJob->state;
}
/****************************************************************************/
BOOL dg87_job_setState(dg87_job* pJob, dg87_job_state newState,int exitCode)
{
	dg87_jobdep* pJobdep;
	dg87_item* pJobdepItem;
	dg87_queue* pQueue;

	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	printf("Job %s : %s\n", pJob->name, dg87_job_stateString[newState]);
	/* exit if there is nothing to do */
	if (pJob->state == newState && pJob->exitCode == exitCode)
		return TRUE;

	/* stop deleted job if OnErrorContinue=FALSE */
	if (newState == dg87_job_state_deleted &&
			pJob->onErrorContinue == FALSE) 
		newState = dg87_job_state_deleteStopped;
		
	pJob->state = newState;
	if (exitCode != -1)
		pJob->exitCode = exitCode;
	pQueue = pJob->pTask->pQueue;

	switch (dg87_job_getStatus(pJob)){

		case dg87_job_status_waiting:
			if (pQueue != NULL)
				dg87_queue_removeRunningJob(pQueue, pJob);
			break;
		case dg87_job_status_running:
			if (pQueue != NULL)
				dg87_queue_addRunningJob(pQueue, pJob);
			break;
		case dg87_job_status_terminated:
		case dg87_job_status_aborted:
			if (pQueue != NULL)
				dg87_queue_removeRunningJob(pQueue, pJob);
			/* update job dependences*/
			pJobdepItem = pJob->pTask->pJobDepList->pFirst;
			while (pJobdepItem != NULL){
				pJobdep = pJobdepItem->pObject;
				dg87_jobdep_evaluate(pJobdep);
				pJobdepItem = pJobdepItem->pNext;
			}
			break;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setDescription(dg87_job* pJob, char* pDescription)
{
	if (pJob == NULL || pDescription == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (strlen(pDescription) >= DG87_JOB_DESC_LENGTH){
		dg87_err_setError(DG87_ERR_DESCRIPTIONTOLONG);
		return FALSE;
	}
	strncpy(pJob->description, pDescription, DG87_JOB_DESC_LENGTH);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setTempo(dg87_job* pJob, int startTempo, int endTempo)
{
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (startTempo < 0 || startTempo > JOB_MAX_TEMPO
			|| endTempo < 0 || endTempo > JOB_MAX_TEMPO){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return FALSE;
	}
	pJob->startTempo = startTempo;
	pJob->endTempo = endTempo;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setPath(dg87_job* pJob, char* pPath)
{
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	
	if (pJob->pPath != NULL) dg87_free(pJob->pPath, DG87_CHAR);
	pJob->pPath = NULL;
	if (pPath == NULL || strlen(pPath) == 0) 
		return TRUE;

	pJob->pPath = dg87_malloc(strlen(pPath)+1, DG87_CHAR);
	if (pJob->pPath == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	strcpy(pJob->pPath, pPath); 
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setArgument(dg87_job* pJob, char* pArgument)
{
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	
	if (pJob->pArgument != NULL) dg87_free(pJob->pArgument, DG87_CHAR);
	pJob->pArgument = NULL;
	if (pArgument == NULL || strlen(pArgument) == 0) 
		return TRUE;

	pJob->pArgument = dg87_malloc(strlen(pArgument)+1, DG87_CHAR);
	if (pJob->pArgument == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	strcpy(pJob->pArgument, pArgument); 
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setWaitAllDependences(dg87_job *pJob,BOOL wait)
{
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob->waitAllDependences = wait;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setOnErrorContinue(dg87_job *pJob,BOOL value)
{
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob->onErrorContinue = value;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_setEnabled(dg87_job *pJob,BOOL value)
{
	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob->enabled = value;
	return TRUE;
}
/****************************************************************************/
dg87_job_state dg87_job_asciiToState(char* pString)
{	
	int index = 0;
	while (dg87_job_stateString[index] != NULL && 
		strcmp(pString, dg87_job_stateString[index]) != 0)
		index++;	
	return index;
}
/****************************************************************************/
char* dg87_job_stateToAscii(dg87_job_state state)
{
	return dg87_job_stateString[state];
}
/****************************************************************************/
BOOL dg87_job_findUpStreamJobs(dg87_job* pJob, dg87_list* pJobList)
{
	dg87_jobdep* pJobDep;
	dg87_item* pJobDepItem;
	if (pJob== NULL || pJobList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_list_addObject(pJobList, pJob) == FALSE)
		return FALSE;
	pJobDepItem = pJob->pTask->pJobDepList->pFirst;
	while (pJobDepItem != NULL){
		pJobDep = pJobDepItem->pObject;
		pJobDepItem = pJobDepItem->pNext;
		if (pJobDep->pJobAfter != pJob)
			continue;
		if (dg87_job_findUpStreamJobs(pJobDep->pJobBefore,
					pJobList) == FALSE)
			return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_job_findDownStreamJobs(dg87_job* pJob, dg87_list* pJobList)
{
	dg87_jobdep* pJobDep;
	dg87_item* pJobDepItem;
	if (pJob== NULL || pJobList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_list_addObject(pJobList, pJob) == FALSE)
		return FALSE;
	pJobDepItem = pJob->pTask->pJobDepList->pFirst;
	while (pJobDepItem != NULL){
		pJobDep = pJobDepItem->pObject;
		pJobDepItem = pJobDepItem->pNext;
		if (pJobDep->pJobBefore != pJob)
			continue;
		if (dg87_job_findDownStreamJobs(pJobDep->pJobAfter,
					pJobList) == FALSE)
			return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_job_check(dg87_job* pJob, int (*print)(char*))
{
	if (strlen(pJob->name) == 0){
		print("Job : name length is zero !\n");
		return 1;
	}
	if (pJob->pTask == NULL){
		print("Job : Task pointer is null !\n");
		return 1;
	}
	if (pJob->startTempo < 0 || pJob->startTempo > JOB_MAX_TEMPO){
		print("Job : incorrect value for startTempo !\n");
		return 1;
	}
	if (pJob->endTempo < 0 || pJob->endTempo > JOB_MAX_TEMPO){
		print("Job : incorrect value for endTempo\n");
		return 1;
	}
	/* pPath and pArgument could be null -> no test */

	return 0;
}
#endif /*DEBUG*/


/****************************************************************************/
/* End of file */
