/* file job.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef JOB_H 
#define JOB_H 1

#include <time.h>
#include <config.h> 
#include <libdg87/list.h>
#include <libdg87/storage.h>
#include "global.h"
#include "task.h"

#define JOB_MAX_TEMPO 3600

typedef enum
{
	dg87_job_state_uninitialize = 0,
	dg87_job_state_ignore,
	dg87_job_state_waitfordep,
	dg87_job_state_waitenabled,
	dg87_job_state_readytorun,
	dg87_job_state_canceled,
	dg87_job_state_readytosubmit,
	dg87_job_state_submit,
	dg87_job_state_deleted,
	dg87_job_state_running,
	dg87_job_state_exit,
	dg87_job_state_terminated,
	dg87_job_state_ignored,
	dg87_job_state_aborted,
	dg87_job_state_abortStopped,
	dg87_job_state_deleteStopped,
	dg87_job_state_waitStartTempo,
	dg87_job_state_waitEndTempo

	/* don't forget to update dg87_job_stateString and dg87_job_getStatus
	 * in job.c when adding a new state ! */
}dg87_job_state;

typedef enum{
	dg87_job_status_waiting,
	dg87_job_status_running, /* waittempo, running ...*/
	dg87_job_status_terminated, /* terminated, canceled */
        dg87_job_status_aborted	/* aborted, deleted */
} dg87_job_status;

typedef struct dg87_job_struct
{
	char name[DG87_JOB_NAME_LENGTH];
	char description[DG87_JOB_DESC_LENGTH];
	struct dg87_task_struct* pTask;
	char* pPath; /* path for the script */
	char* pArgument; /* argument for the script */
	int startTempo; /* temporization before execution in seconds */
	int endTempo; /* temporization after execution in seconds */
	dg87_job_state state;
	BOOL waitAllDependences;
	BOOL onErrorContinue;
	BOOL enabled;
	time_t time;
	int exitCode;
} dg87_job;

/*
 * dg87_job_new 
 * create a new job and initialize the name property
 */
dg87_job* dg87_job_new(struct dg87_task_struct* pTask, char* name);

/*
 * dg87_job_delete
 */
BOOL dg87_job_delete(dg87_job* pJob);

/*
 * dg87_job_load
 */
BOOL dg87_job_load(dg87_job* pJob, dg87_storage* pStorage);

/*
 * dg87_job_save
 */
BOOL dg87_job_save(dg87_job* pJob, dg87_storage* pStorage);

/*
 * dg87_job_setDescription
 * set the description of the job.
 */
BOOL dg87_job_setDescription(dg87_job* pJob, char* pDescription);

/*
 * dg87_job_getStatus
 */
dg87_job_status dg87_job_getStatus(dg87_job* pJob);

/* 
 * dg87_job_manage
 */
dg87_job_state dg87_job_manage(dg87_job* pJob);

/* 
 * dg87_job_setState
 * change the state of the job and update dependences after
 * this job
 */
BOOL dg87_job_setState(dg87_job* pJob, dg87_job_state newState,int exitCode);

/*
 * dg87_job_setTempo
 * set the startTempo and EndTempo for the job.
 */
BOOL dg87_job_setTempo(dg87_job* pJob, int startTempo, int endTempo);

/*
 * dg87_job_setPath
 * set the path of the job.
 * pPath could be NULL or zero length.
 */
BOOL dg87_job_setPath(dg87_job* pJob,char* pPath);

/*
 * dg87_job_setArgument
 * set the argument of the job.
 * pArgument could be NULL or zero length.
 */
BOOL dg87_job_setArgument(dg87_job* pJob,char* pArgument);

/*
 * dg87_job_setWaitAllDependences
 */
BOOL dg87_job_setWaitAllDependences(dg87_job *pJob,BOOL wait);

/*
 * dg87_job_setOnErrorContinue
 */
BOOL dg87_job_setOnErrorContinue(dg87_job *pJob,BOOL value);

/*
 * dg87_job_setEnabled
 */
BOOL dg87_job_setEnabled(dg87_job *pJob,BOOL value);

/*
 * dg87_job_findUpStreamJobs
 * This function adds to the list given in parameter all upstream jobs
 * of a job.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_job_findUpStreamJobs(dg87_job* pJob, dg87_list* pJobList);

/*
 * dg87_job_findDownStreamJobs
 * This function adds to the list given in parameter all downstream jobs
 * of a job.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_job_findDownStreamJobs(dg87_job* pJob, dg87_list* pJobList);


/*
 * dg87_job_asciiToState
 * convert a string into a job's state
 */
dg87_job_state dg87_job_asciiToState(char* pString);

/*
 * dg87_job_stateToAscii
 * convert a job's state to a string
 */
char* dg87_job_stateToAscii(dg87_job_state state);

#ifdef DEBUG
/*
 * dg87_job_ckeck
 */
int dg87_job_check(dg87_job* pJob, int (*print)(char*));
#endif /*DEBUG*/

#endif
/* End of file */
