/* file queue.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "context.h"
#include "caller.h"
#include "queue.h"
#include "job.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/****************************************************************************/
dg87_queue* dg87_queue_new(dg87_target* pTarget, char* pName)
{
	dg87_queue* pQueue;
	if (pTarget == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	if (strlen(pName) >= DG87_QUEUE_NAME_LENGTH){
		dg87_err_setError(DG87_ERR_NAMETOLONG);
		return FALSE;
	}
	pQueue = dg87_malloc(sizeof(dg87_queue), DG87_QUEUE);
	if (pQueue == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* create the TaskList */
	pQueue->pTaskList = dg87_list_new();
	if (pQueue->pTaskList == NULL){
		dg87_queue_delete(pQueue);
		return NULL;
	}
	/* create the JobList */
	pQueue->pRunningJobList = dg87_list_new();
	if (pQueue->pRunningJobList == NULL){
		dg87_list_delete(pQueue->pTaskList);
		dg87_queue_delete(pQueue);
		return NULL;
	}
	strncpy(pQueue->name, pName, DG87_QUEUE_NAME_LENGTH); 
	pQueue->pTarget = pTarget;
	pQueue->description[0] = 0;
	pQueue->open = FALSE;
	pQueue->max = 1;
	return pQueue;
}

/****************************************************************************/
BOOL dg87_queue_delete(dg87_queue* pQueue)
{
	dg87_task* pTask;

	if (pQueue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}

	/* set Task's Queue to NULL for each task */
	while (pQueue->pTaskList->count > 0){
		pTask = dg87_list_removeObjectByIndex
			(pQueue->pTaskList, 0);
		pTask->pQueue = NULL;
	}

	dg87_list_delete(pQueue->pRunningJobList);
	dg87_list_delete(pQueue->pTaskList);
	dg87_free(pQueue, DG87_QUEUE);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_queue_load(dg87_queue* pQueue, dg87_storage* pStorage)
{
	dg87uint value;
	dg87byte b;
	char* pString;
	if (pQueue == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read target description */
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_queue_setDescription(pQueue, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);
	/* read max */
	if (dg87_storage_readUInt(pStorage, &value) == FALSE)
		return FALSE;
	pQueue->max = value;
	/* read opened property */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_queue_setOpen(pQueue, b) == FALSE)
		return FALSE;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_queue_save(dg87_queue* pQueue, dg87_storage* pStorage)
{
	if (pQueue == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write target description */
	if (dg87_storage_writeString(pStorage, pQueue->description) == FALSE)
		return FALSE;
	/* write max */
	if (dg87_storage_writeUInt(pStorage, pQueue->max) == FALSE)
		return FALSE;
	/* write opened property */
	if (dg87_storage_writeByte(pStorage, pQueue->open) == FALSE)
		return FALSE;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_queue_setDescription(dg87_queue* pQueue, char* pDescription)
{
	if (pQueue == NULL || pDescription == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (strlen(pDescription) >= DG87_QUEUE_DESC_LENGTH){
		dg87_err_setError(DG87_ERR_DESCRIPTIONTOLONG);
		return FALSE;
	}
	strncpy(pQueue->description, pDescription, DG87_QUEUE_DESC_LENGTH);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_queue_setOpen(dg87_queue* pQueue, BOOL value)
{
	if (pQueue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (value != FALSE && value != TRUE){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return FALSE;
	}
	pQueue->open = value;
	return TRUE;
}

/****************************************************************************/
BOOL dg87_queue_setMax(dg87_queue* pQueue, int value)
{
	if (pQueue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (value < 1){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return FALSE;
	}
	pQueue->max = value;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_queue_addRunningJob(dg87_queue* pQueue, 
		struct dg87_job_struct* pJob)
{
	if (pQueue == NULL || pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pJob->pTask->pQueue != pQueue)
		return FALSE;
	if (dg87_list_findItem(pQueue->pRunningJobList, pJob) == NULL){
		dg87_list_addObject(pQueue->pRunningJobList, pJob);
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_queue_removeRunningJob(dg87_queue* pQueue, 
		struct dg87_job_struct* pJob)
{	
	dg87_item* pJobItem;
	if (pQueue == NULL || pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pJob->pTask->pQueue != pQueue)
		return FALSE;
	pJobItem = dg87_list_findItem(pQueue->pRunningJobList, pJob);
	if (pJobItem != NULL)
		dg87_list_removeItem(pQueue->pRunningJobList, pJobItem);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_queue_manage(struct dg87_context_struct* pContext, dg87_queue* pQueue)
{
	dg87_item* pItemJob;
	dg87_job *pJob;
	int nRunningJobs=0;

	if (pQueue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* nothing to do if queue or target is closed */
	if (pQueue->open == FALSE || pQueue->pTarget->open == FALSE)
		return TRUE;

	/*TODO : verify that number of running jobs < max */
	pItemJob = pQueue->pRunningJobList->pFirst;
	while (pItemJob != NULL){
		pJob = pItemJob->pObject;
		if (pJob->state != dg87_job_state_readytosubmit)
			nRunningJobs++;
		pItemJob = pItemJob->pNext;
	}
	if (nRunningJobs >= pQueue->max) 
		return TRUE;
	/*
	printf("queue : number of running jobs %i\n", nRunningJobs);
	printf("queue : maximum running jobs %i\n", pQueue->max);
	*/

	/* submit jobs which are waiting */
	pItemJob = pQueue->pRunningJobList->pFirst;
	while (pItemJob != NULL){
		pJob = pItemJob->pObject;
		if (pJob->state == dg87_job_state_readytosubmit){
			if (dg87_caller_submitJob(pContext, pJob) == TRUE){
				printf("queue : submit job %s\n", pJob->name);
				dg87_job_setState(pJob, 
						dg87_job_state_submit,-1);
				nRunningJobs++;
				printf("queue : number of running jobs %i\n", 
						nRunningJobs);
				printf("queue : maximum running jobs %i\n", 
						pQueue->max);
				if (nRunningJobs >= pQueue->max) {
					printf("stop !\n");
					break;
				}
			}
		}	
		pItemJob = pItemJob->pNext;
	}
	return TRUE;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_queue_check(dg87_queue* pQueue, int (*print)(char*))
{
	dg87_item *pItem;
	dg87_task* pTask;
	if (strlen(pQueue->name) == 0){
		print("Queue : name length is zero !\n");
		return 1;
	}
	if (pQueue->pTarget == NULL) {
		print("Queue : target pointer is NULL !\n");
		return 1;
	}
	/* check TaskList */
	if (pQueue->pTaskList == NULL){
		print("Queue : TaskList is NULL !\n");
		return 1;
	}
	if (dg87_list_check(pQueue->pTaskList,print) != 0)
		return 1;
	pItem = pQueue->pTaskList->pFirst;
	while (pItem != NULL){
		pTask = pItem->pObject;
		if (pTask->pQueue != pQueue){
			print("Queue : Task's queue incorrect !\n");
			return 1;
		}
		pItem = pItem->pNext;
	}

	return 0;
}
#endif /*DEBUG*/
/****************************************************************************/

/* End of file */
