/* file task.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "job.h"
#include "jobdep.h"
#include "task.h"
#include "condition.h"
#include "context.h"
#include <libdg87/list.h>
#include <libdg87/vector.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/* order in this array must correspond to dg87_job_state */
/* this array must be finished by a NULL */
char* dg87_task_stateString[]={
	"uninitialize",
	"ignore",
	"wait_for_dep",
	"wait_for_condition",
	"wait_enabled",
	"canceled",
	"running",
	"ignored",
	"aborted",
	"abort_stopped",
	"terminated",
	"deleted",
	NULL
};

/****************************************************************************/
dg87_task* dg87_task_new(char* pName, struct dg87_day_struct *pDay)
{
	dg87_task* pTask;
	if (pName == NULL || pDay == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	if (strlen(pName) >= DG87_TASK_NAME_LENGTH){
		dg87_err_setError(DG87_ERR_NAMETOLONG);
		return FALSE;
	}
	pTask = dg87_malloc(sizeof(dg87_task), DG87_TASK);
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* allocate memory for JobVector */
	pTask->pJobVector = dg87_vector_new();
	if (pTask->pJobVector == NULL){
		dg87_free(pTask, DG87_TASK);
		return NULL;
	}
	/* allocate memory for JobDepList */
	pTask->pJobDepList = dg87_list_new();
	if (pTask->pJobDepList == NULL){
		dg87_vector_delete(pTask->pJobVector);
		dg87_free(pTask, DG87_TASK);
		return NULL;
	}
	/* allocate memory for TaskDepBefore */
	pTask->pTaskDepBeforeList = dg87_list_new();
	if (pTask->pTaskDepBeforeList == NULL){
		dg87_vector_delete(pTask->pJobVector);
		dg87_list_delete(pTask->pJobDepList);
		dg87_free(pTask, DG87_TASK);
		return NULL;
	}
	/* allocate memory for TaskDepAfter */
	pTask->pTaskDepAfterList = dg87_list_new();
	if (pTask->pTaskDepAfterList == NULL){
		dg87_vector_delete(pTask->pJobVector);
		dg87_list_delete(pTask->pJobDepList);
		dg87_list_delete(pTask->pTaskDepBeforeList);
		dg87_free(pTask, DG87_TASK);
		return NULL;
	}
	/* allocate memory for ConditionList */
	pTask->pConditionList = dg87_list_new();
	if (pTask->pConditionList == NULL){
		dg87_vector_delete(pTask->pJobVector);
		dg87_list_delete(pTask->pJobDepList);
		dg87_list_delete(pTask->pTaskDepBeforeList);
		dg87_list_delete(pTask->pTaskDepAfterList);
		dg87_free(pTask, DG87_TASK);
		return NULL;
	}
	strncpy(pTask->name, pName, DG87_TASK_NAME_LENGTH); 
	pTask->description[0] = 0;
	pTask->waitOldestTask = TRUE;
	pTask->pQueue= NULL;
	pTask->pDay = pDay;
	pTask->state = dg87_task_state_uninitialize;
	pTask->waitAllDependences = TRUE;
	pTask->onErrorContinue = TRUE;
	pTask->enabled = TRUE;
	return pTask;
}

/****************************************************************************/
BOOL dg87_task_delete(dg87_task* pTask)
{
	dg87_job *pJob;
	dg87_jobdep *pJobDep;
	dg87_condition *pCondition;
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* can't delete task if task has dependances whith other task */
	if (pTask->pTaskDepBeforeList->count > 0 ||
			pTask->pTaskDepAfterList->count > 0){
		dg87_err_setError(DG87_ERR_TASKHASDEPENDENCES);
		return FALSE;
	}
	dg87_list_delete(pTask->pTaskDepBeforeList);
	dg87_list_delete(pTask->pTaskDepAfterList);

	/* remove Task from Queue */
	if (pTask->pQueue != NULL)
		dg87_list_removeObject(pTask->pQueue->pTaskList, pTask);	

	/* delete all JobDep object */
	while (pTask->pJobDepList->count > 0){
		pJobDep = dg87_list_removeObjectByIndex(pTask->pJobDepList,0);
		dg87_jobdep_delete(pJobDep);
	}
	dg87_list_delete(pTask->pJobDepList);

	/* delete all Job object */
	while (pTask->pJobVector->count > 0){
		pJob = dg87_vector_removeObjectByIndex(pTask->pJobVector,0);
		dg87_job_delete(pJob);
	}
	dg87_vector_delete(pTask->pJobVector);

	/* delete all Condition object */
	while (pTask->pConditionList->count > 0){
		pCondition = dg87_list_removeObjectByIndex
			(pTask->pConditionList,0);
		dg87_condition_delete(pCondition);
	}
	dg87_list_delete(pTask->pConditionList);

	dg87_free(pTask, DG87_TASK);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_load(dg87_task* pTask, 
		dg87_storage* pStorage, struct dg87_context_struct *pContext)
{
	dg87byte b;
	char* pString;
	dg87_target *pTarget;
	dg87_queue *pQueue;
	if (pTask == NULL || pStorage == NULL || pContext == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read task description*/
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_task_setDescription(pTask, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);

	/* read waitAllDependences */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_task_setWaitAllDependences(pTask, b) == FALSE)
		return FALSE;

	/* read onErrorContinue */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_task_setOnErrorContinue(pTask, b) == FALSE)
		return FALSE;

	/* read target and queue */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (b == TRUE){ /* queue is not NULL */
		/* find target */
		if (dg87_storage_readString(pStorage, &pString) == FALSE)
			return FALSE;
		if ((pTarget = dg87_context_targetGetByName(
				pContext, pString)) == NULL){
			dg87_free(pString, DG87_CHAR);
			return FALSE;
		}
		dg87_free(pString, DG87_CHAR);
		/* find queue */
		if (dg87_storage_readString(pStorage, &pString) == FALSE)
			return FALSE;
		if ((pQueue = dg87_target_queueGetByName(
				pTarget, pString)) == NULL){
			dg87_free(pString, DG87_CHAR);
			return FALSE;
		}
		dg87_free(pString, DG87_CHAR);
		/* set queue */
		if (dg87_task_setQueue(pTask, pQueue) == FALSE)
			return FALSE;
	}
	/* 'enabled' property */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	pTask->enabled = b;

	/* 'state' property */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	pTask->state = b;

	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_save(dg87_task* pTask, dg87_storage* pStorage)
{
	if (pTask == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write task description */
	if (dg87_storage_writeString(pStorage, pTask->description) == FALSE)
		return FALSE;
	/* write waitAllDependences */
	if (dg87_storage_writeByte(pStorage, pTask->waitAllDependences)== FALSE)
		return FALSE;
	/* write onErrorContinue */
	if (dg87_storage_writeByte(pStorage, pTask->onErrorContinue)== FALSE)
		return FALSE;
	/* save target and queue */
	if (dg87_storage_writeByte(pStorage, 
				(pTask->pQueue==NULL)?FALSE:TRUE) == FALSE)
		return FALSE;
	if (pTask->pQueue != NULL){
		if (dg87_storage_writeString(pStorage, 
					pTask->pQueue->pTarget->name) == FALSE)
			return FALSE;
		if (dg87_storage_writeString(pStorage, 
					pTask->pQueue->name) == FALSE)
			return FALSE;
	}
	/* 'enabled' property */
	if (dg87_storage_writeByte(pStorage, pTask->enabled) == FALSE)
		return FALSE;
	/* 'state' property */
	if (dg87_storage_writeByte(pStorage, pTask->state) == FALSE)
		return FALSE;
	return TRUE;
}
/****************************************************************************/
dg87_task_state dg87_task_manage(dg87_task* pTask)
{
	dg87_element *pElement;
	dg87_job *pJob;
	dg87_item *pItem;
	dg87_taskdep* pTaskDep;
	dg87_condition* pCondition;
	dg87_task_state newState;
	BOOL ready;
	int nTerminated = 0;
	int nAborted = 0;
	int nRunning = 0;
	int nWaiting = 0;

	if (pTask == NULL ){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE; /* not cool */
	}

	/* initialize state if function never been call */
	if (pTask->state == dg87_task_state_uninitialize)
		pTask->state = dg87_task_state_wait_for_dep;

	/* find if task is always waiting for dependences */
	if (pTask->state == dg87_task_state_wait_for_dep ||
			pTask->state == dg87_task_state_ignore ){
		if (pTask->waitAllDependences == TRUE){
			/* Task is waiting for all dependences (AND) 
			 * 1- Task is ready to run if all dependences are
			 * validated.
			 * 2- Task is canceled if at least one dependence is
			 * canceled.
			 * 3- Task is waiting if at least one dependence is
			 * waiting */
			newState = dg87_task_state_wait_for_condition;
			pItem = pTask->pTaskDepBeforeList->pFirst;
			while (pItem != NULL){
				pTaskDep = pItem->pObject;
				pItem = pItem->pNext;
				/* if a dep is canceled, then task is canceled*/
				if (pTaskDep->state == 
						dg87_taskdep_state_canceled){
					newState = dg87_task_state_canceled;
					break;
				}
				/* if a dep is waiting, then task is waiting */
				if (pTaskDep->state == 
						dg87_taskdep_state_waiting){
					newState = dg87_task_state_wait_for_dep;
					break;
				}

			}
		} else {
			/* Task is waiting for one dependence (OR). 
			 * 1- Task is ready to run if at least dependence is
			 * validated.
			 * 2- Task is canceled if all dependences are validated.
			 * 3- Task is waiting for dependence if at least one 
			 * dependence is waiting.
			 */ 
			newState = dg87_task_state_canceled;
			pItem = pTask->pTaskDepBeforeList->pFirst;
			while (pItem != NULL){
				pTaskDep = pItem->pObject;
				pItem = pItem->pNext;
				/*if a dep is validated,then task is validated*/
				if (pTaskDep->state == 
						dg87_taskdep_state_validated){
					newState = 
					     dg87_task_state_wait_for_condition;
					break;
				}
				/* if a dep is waiting, then task is waiting */
				if (pTaskDep->state == 
						dg87_taskdep_state_waiting){
					newState = dg87_task_state_wait_for_dep;
					break;
				}

			}
		}
		switch (pTask->state){
			case dg87_task_state_wait_for_dep:
				dg87_task_setState(pTask, newState);
				break;
			case dg87_task_state_ignore:
				if (newState != dg87_task_state_wait_for_dep)
					dg87_task_setState(pTask,
						dg87_task_state_ignored);
			default:
				/* just to avoid warnings */
				break;
		}

	}

	/* hlate : if an hlate's condition is finished then
	 * the task must be canceled */
	if (pTask->state == dg87_task_state_wait_for_condition){
		pItem = pTask->pConditionList->pFirst;
		while (pItem != NULL){
			pCondition = pItem->pObject;
			if (pCondition->type == condition_hlate){
				dg87_condition_getState(pCondition);
				if (pCondition->state ==
						dg87_condition_finished) {
					dg87_task_setState(pTask, 
						dg87_task_state_canceled);
					break;
				}
			}	
			pItem = pItem->pNext;
		}
	}


	/* hmax : if an hmax's condition is finished then
	 * the task is ready to run */
	if (pTask->state == dg87_task_state_wait_for_condition){
		pItem = pTask->pConditionList->pFirst;
		while (pItem != NULL){
			pCondition = pItem->pObject;
			if (pCondition->type == condition_hmax){
				dg87_condition_getState(pCondition);
				if (pCondition->state ==
						dg87_condition_finished) {
					dg87_task_setState(pTask, 
						dg87_task_state_waitenabled);
					break;
				}
			}	
			pItem = pItem->pNext;
		}
	}


	/* hmin, manual, file, if_equal and if_not_equal conditions */
	/* all of this condition must be finished for the task to be
	 * enabled to run */
	if (pTask->state == dg87_task_state_wait_for_condition){
		/* hmin */
		ready = TRUE;
		pItem = pTask->pConditionList->pFirst;
		while (pItem != NULL){
			pCondition = pItem->pObject;
			if (pCondition->type == condition_hmin){
				dg87_condition_getState(pCondition);
				if (pCondition->state==dg87_condition_waiting){
					ready = FALSE;
					break;
				}
			}	
			pItem = pItem->pNext;
		}

		/* manual's condition */
		if (ready == TRUE){
			pItem = pTask->pConditionList->pFirst;
			while (pItem != NULL){
				pCondition = pItem->pObject;
				if (pCondition->type == condition_manual){
					dg87_condition_getState(pCondition);
					if (pCondition->state ==
						dg87_condition_waiting) {
						ready = FALSE;
						break;
					}
				}
				pItem = pItem->pNext;
			}
		}

		/* if_equal and if_not_equal conditions */
		if (ready == TRUE){
			pItem = pTask->pConditionList->pFirst;
			while (pItem != NULL){
				pCondition = pItem->pObject;
				if (pCondition->type == condition_ifequal ||
					pCondition->type == condition_ifnequal)
				{
					dg87_condition_getState(pCondition);
					if (pCondition->state ==
						dg87_condition_waiting) {
						ready = FALSE;
						break;
					}
				}
				pItem = pItem->pNext;
			}
		}

		/* file's condition */
		if (ready == TRUE){

			pItem = pTask->pConditionList->pFirst;
			while (pItem != NULL){
				pCondition = pItem->pObject;
				if (pCondition->type == condition_file){
					dg87_condition_getState(pCondition);
					if (pCondition->state ==
						dg87_condition_waiting) {
						ready = FALSE;
						break;
					}
				}
				pItem = pItem->pNext;
			}
		}

		if (ready == TRUE)
			dg87_task_setState(pTask, dg87_task_state_waitenabled);
	}
	/* the task must be enabled to be start */
	if (pTask->state == dg87_task_state_waitenabled
			&& pTask->enabled == TRUE)
		dg87_task_setState(pTask, dg87_task_state_running);

	/* if task is running, find jobs with no dependences*/
	if (pTask->state == dg87_task_state_running /*||
		pTask->state == dg87_task_state_aborted*/){

		/* for each job of the task */
		pElement = pTask->pJobVector->pFirst;
		while (pElement != NULL){
			pJob = pElement->pObject;
			pElement = pElement->pNext;
			dg87_job_manage(pJob);
			switch (dg87_job_getStatus(pJob)){
				case dg87_job_status_running:
					nRunning++; break;
				case dg87_job_status_waiting:
					nWaiting++; break;
				case dg87_job_status_terminated:
					nTerminated++; break;
				case dg87_job_status_aborted:
					nAborted++; break;
			}
		}
		if (nRunning > 0 || nWaiting > 0)
			dg87_task_setState(pTask, dg87_task_state_running);
		else if (nAborted > 0) {
			if (pTask->onErrorContinue == TRUE)
				dg87_task_setState(pTask, 
						dg87_task_state_aborted);
			else
				dg87_task_setState(pTask, 
						dg87_task_state_abortStopped);
		}
		else
			dg87_task_setState(pTask, dg87_task_state_terminated);
	}

	return pTask->state;
}
/****************************************************************************/
BOOL dg87_task_setState(dg87_task* pTask, dg87_task_state newState)
{
	dg87_taskdep* pTaskdep;
	dg87_item* pTaskdepItem;
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* quit if there is nothing to do */
	if (pTask->state == newState)
		return TRUE;
	pTask->state = newState;

	/* TODO add other state like state_abort */
	/* update job's dependences if necessary */
	switch (pTask->state){
		case dg87_task_state_terminated:
		case dg87_task_state_aborted:
		case dg87_task_state_canceled:
		case dg87_task_state_ignored:
			pTaskdepItem = pTask->pTaskDepAfterList->pFirst;
			while (pTaskdepItem != NULL){
				pTaskdep = pTaskdepItem->pObject;
				dg87_taskdep_evaluate(pTaskdep);
				pTaskdepItem = pTaskdepItem->pNext;
			}
		default:
			/* Nothing to do */
			break;
	}
	
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_setDescription(dg87_task* pTask, char* pDescription)
{
	if (pTask == NULL || pDescription == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (strlen(pDescription) >= DG87_TASK_DESC_LENGTH){
		dg87_err_setError(DG87_ERR_DESCRIPTIONTOLONG);
		return FALSE;
	}
	strncpy(pTask->description, pDescription, DG87_TASK_DESC_LENGTH);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_setQueue(dg87_task *pTask, dg87_queue *pQueue)
{
	dg87_list *pList;

	if (pTask == NULL){  /* no test on pQueue, it can be null ! */
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}

	/* nothing to do, already done */
	if (pTask->pQueue == pQueue)
		return TRUE; 
	/* remove task of previous target */
	if (pTask->pQueue != NULL){
		pList = pTask->pQueue->pTaskList;
		dg87_list_removeObject(pList,pTask);
	}
	/* add this task to TaskList of queue */
	pTask->pQueue = pQueue;
	/* add task of actual Queue */
	if (pTask->pQueue != NULL){
		pList = pTask->pQueue->pTaskList;
		dg87_list_addObject(pList, pTask);
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_setWaitOldestTask(dg87_task *pTask,BOOL wait)
{
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTask->waitOldestTask = wait;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_setOnErrorContinue(dg87_task *pTask,BOOL value)
{
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTask->onErrorContinue = value;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_setEnabled(dg87_task *pTask,BOOL value)
{
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTask->enabled = value;
	return TRUE;
}

/****************************************************************************/
BOOL dg87_task_setWaitAllDependences(dg87_task *pTask,BOOL wait)
{
	if (pTask == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTask->waitAllDependences = wait;
	return TRUE;
}
/****************************************************************************/
dg87_job* dg87_task_jobAdd(dg87_task* pTask, char* pJobName)
{
	dg87_job* pJob;
	if (pTask == NULL || pJobName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob = dg87_job_new(pTask, pJobName);
	if (pJob == NULL) 
		return NULL;

	/* vector generate error if an other job has the same name */
	/* key must point on pJob->name */
	if (dg87_vector_addObject(pTask->pJobVector, pJob, pJob->name) == FALSE)
	{
		dg87_job_delete(pJob);
		return NULL;
	}
	return pJob;
}
/****************************************************************************/
int dg87_task_jobCount(dg87_task* pTask)
{
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_vector_count(pTask->pJobVector);
}
/****************************************************************************/
dg87_job* dg87_task_jobGetByName(dg87_task* pTask, char* pName)
{
	if (pTask == NULL || pName == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByKey(pTask->pJobVector, pName);
}
/****************************************************************************/
dg87_job* dg87_task_jobGetByIndex(dg87_task* pTask, int index)
{
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByIndex(pTask->pJobVector, index);
}
/****************************************************************************/
BOOL dg87_task_jobDelete(dg87_task* pTask, dg87_job* pJob)
{
	dg87_jobdep* pJobDep;
	dg87_item* pItem;
	if (pTask == NULL || pJob == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* remove all dependances link to this job */
	pItem = pTask->pJobDepList->pFirst;
	while (pItem != NULL){
		pJobDep = pItem->pObject;
		if (pJobDep->pJobBefore==pJob || pJobDep->pJobAfter==pJob){
			dg87_item* pTemp = pItem;
			pItem = pItem->pNext;
			dg87_list_removeItem(pTask->pJobDepList, pTemp);
			dg87_jobdep_delete(pJobDep);
		} else {
			pItem = pItem->pNext;
		}
	}
	if (dg87_vector_removeObject(pTask->pJobVector, pJob) == FALSE)
		return FALSE;
	if (dg87_job_delete(pJob) == FALSE){
		/* add job in vector again */
		/* TODO les dpendances supprimes au dbut sont perdues */
		dg87_vector_addObject(pTask->pJobVector, pJob, pJob->name);
		return FALSE;
	}
	return TRUE;
}

/****************************************************************************/
BOOL dg87_task_jobDeleteByName(dg87_task* pTask,char* pName)
{
	dg87_job* pJob;
	if (pTask == NULL || pName == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob = dg87_vector_getObjectByKey(pTask->pJobVector, pName);
	if (pJob == NULL) return FALSE;
	return dg87_task_jobDelete(pTask, pJob);
}
/****************************************************************************/
BOOL dg87_task_jobDeleteByIndex(dg87_task* pTask,int index)
{
	dg87_job* pJob;
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJob = dg87_vector_getObjectByIndex(pTask->pJobVector, index);
	if (pJob == NULL) 
		return FALSE;
	return dg87_task_jobDelete(pTask, pJob);
}
/****************************************************************************/
dg87_jobdep* dg87_task_jobdepAdd (dg87_task* pTask, 
	 struct dg87_job_struct* pJobBefore, 
	 struct dg87_job_struct* pJobAfter)
{
	dg87_jobdep* pJobDep;

	if (pTask == NULL || pJobBefore == NULL || pJobAfter == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (pJobBefore->pTask != pTask || pJobAfter->pTask != pTask){
		dg87_err_setError(DG87_ERR_JOBSNOTINSAMETASK);
		return NULL;
	}

	/* create and add the job dep */
	pJobDep = dg87_jobdep_new(pJobBefore, pJobAfter);
	if (pJobDep == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pJobDepList, pJobDep);
	
	return pJobDep;
}

/****************************************************************************/
int dg87_task_jobdepCount(dg87_task* pTask)
{
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_list_count(pTask->pJobDepList);
}
/****************************************************************************/
dg87_jobdep* dg87_task_jobdepGet(dg87_task* pTask, int index)
{
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_list_getObjectByIndex(pTask->pJobDepList, index);
}
/****************************************************************************/
BOOL dg87_task_jobdepDeleteByIndex(dg87_task* pTask,int index)
{
	dg87_jobdep* pJobDep;
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pJobDep = dg87_list_removeObjectByIndex(pTask->pJobDepList, index);
	if (pJobDep == NULL) 
		return FALSE;
	dg87_jobdep_delete(pJobDep);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_task_jobdepDelete(dg87_task* pTask, dg87_jobdep* pJobDep)
{	
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_list_removeObject(pTask->pJobDepList, pJobDep) == FALSE)
		return FALSE;
	return dg87_jobdep_delete(pJobDep);
}
/****************************************************************************/
int dg87_task_taskdepBeforeCount(dg87_task* pTask)
{
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_list_count(pTask->pTaskDepBeforeList);
}
/****************************************************************************/
dg87_taskdep* dg87_task_taskdepBeforeGet(dg87_task* pTask, int index)
{
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_list_getObjectByIndex(pTask->pTaskDepBeforeList, index);
}
/****************************************************************************/
int dg87_task_taskdepAfterCount(dg87_task* pTask)
{
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_list_count(pTask->pTaskDepAfterList);
}
/****************************************************************************/
dg87_taskdep* dg87_task_taskdepAfterGet(dg87_task* pTask, int index)
{
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_list_getObjectByIndex(pTask->pTaskDepAfterList, index);
}
/****************************************************************************/
dg87_condition* dg87_task_conditionHminAdd(dg87_task* pTask, int minutes)
{
	dg87_condition* pCondition;
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionHmin_new(minutes);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionHmaxAdd(dg87_task* pTask, int minutes)
{
	dg87_condition* pCondition;
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionHmax_new(minutes);
	if (pCondition == NULL) return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionHlateAdd(dg87_task* pTask, int minutes)
{
	dg87_condition* pCondition;
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
       	pCondition = dg87_conditionHlate_new(minutes);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionFileAdd(dg87_task* pTask, char* pFilePath)
{
	dg87_condition* pCondition;
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionFile_new(pFilePath);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionManualAdd(dg87_task* pTask, char* pMessage)
{
	dg87_condition* pCondition;
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionManual_new(pMessage);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionIfEqualAdd(
		dg87_task* pTask, 
		struct dg87_variable_struct* pVariable, 
		char* pValue)
{
	dg87_condition* pCondition;
	if (pTask == NULL || pVariable == NULL || pValue == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionIfEqual_new(pVariable, pValue);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_task_conditionIfNotEqualAdd(
		dg87_task* pTask, 
		struct dg87_variable_struct* pVariable, 
		char* pValue)
{
	dg87_condition* pCondition;
	if (pTask == NULL || pVariable == NULL || pValue == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_conditionIfNotEqual_new(pVariable, pValue);
	if (pCondition == NULL) 
		return NULL;
	dg87_list_addObject(pTask->pConditionList, pCondition);
	return pCondition;
}
/****************************************************************************/
int dg87_task_conditionCount(dg87_task* pTask)
{
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_list_count(pTask->pConditionList);
}
/****************************************************************************/
dg87_condition* dg87_task_conditionGet(dg87_task* pTask, int index)
{
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_list_getObjectByIndex(pTask->pConditionList, index);
}
/****************************************************************************/
BOOL dg87_task_conditionDelete(dg87_task* pTask,int index)
{
	dg87_condition* pCondition;
	if (pTask == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pCondition = dg87_list_removeObjectByIndex
		(pTask->pConditionList, index);
	if (pCondition == NULL) 
		return FALSE;
	dg87_condition_delete(pCondition);
	return TRUE;
}
/****************************************************************************/
dg87_task_state dg87_task_asciiToState(char* pString)
{
	int index = 0;
	while ( dg87_task_stateString[index] != NULL && 
			strcmp(pString, dg87_task_stateString[index]) != 0)
		index++;	
	return index;
}
/****************************************************************************/
char* dg87_task_stateToAscii(dg87_task_state state)
{
	return dg87_task_stateString[state];
}
/****************************************************************************/
BOOL dg87_task_findUpStreamTasks(dg87_task* pTask, dg87_list* pTaskList)
{
	dg87_taskdep* pTaskDep;
	dg87_item* pTaskDepItem;
	if (pTask== NULL || pTaskList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_list_addObject(pTaskList, pTask) == FALSE)
		return FALSE;
	pTaskDepItem = pTask->pTaskDepBeforeList->pFirst;
	while (pTaskDepItem != NULL){
		pTaskDep = pTaskDepItem->pObject;
		pTaskDepItem = pTaskDepItem->pNext;
		if (dg87_task_findUpStreamTasks(pTaskDep->pTaskBefore,
					pTaskList) == FALSE)
			return FALSE;
	}
	return TRUE;
}
/*****************************************************************************/
BOOL dg87_task_findDownStreamTasks(dg87_task* pTask, dg87_list* pTaskList)
{
	dg87_taskdep* pTaskDep;
	dg87_item* pTaskDepItem;
	if (pTask== NULL || pTaskList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (dg87_list_addObject(pTaskList, pTask) == FALSE)
		return FALSE;
	pTaskDepItem = pTask->pTaskDepAfterList->pFirst;
	while (pTaskDepItem != NULL){
		pTaskDep = pTaskDepItem->pObject;
		pTaskDepItem = pTaskDepItem->pNext;
		if (dg87_task_findDownStreamTasks(pTaskDep->pTaskAfter,
					pTaskList) == FALSE)
			return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_task_check(dg87_task* pTask, int (*print)(char*))
{
	dg87_element* pElement;
	dg87_item* pItem;
	dg87_job* pJob;

	/* check name */
	if (strlen(pTask->name) == 0){
		print("Task : name length is zero !\n");
		return 1;
	}
	/* check day */
	if (pTask->pDay == NULL){
		print("Task : Queue.pDay is NULL !\n");
		return 1;
	}
	/* check waitOldestTask */
	if (pTask->waitOldestTask != TRUE && pTask->waitOldestTask != FALSE){
		print("Task : incorrect value for waitOldestTask !\n");
		return 1;
	}
	/* check JobVector*/
	if (pTask->pJobVector == NULL){
		print("Task : JobVector is null !\n");
		return 1;
	}
	if (dg87_vector_check(pTask->pJobVector, print) != 0)
		return 1;
	pElement = pTask->pJobVector->pFirst;
	while (pElement != NULL){
		pJob = pElement->pObject;
		if (dg87_job_check(pJob, print) != 0)
			return 1;
		if (pJob->pTask != pTask){
			print("Task : Job's Task pointer incorrect !\n");
			return 1;
		}
		pElement = pElement->pNext;
	}
	
	/* check JobdepList */
	if (pTask->pJobDepList == NULL){
		print("Task : JobdepList is null !\n");
		return 1;
	}
	if (dg87_list_check(pTask->pJobDepList, print) != 0)
		return 1;
	pItem = pTask->pJobDepList->pFirst;
	while (pItem != NULL){
		if (dg87_jobdep_check(pItem->pObject, print) != 0)
			return 1;
		pItem = pItem->pNext;
	}

	/* check TaskDepBeforeList */
	if (pTask->pTaskDepBeforeList == NULL){
		print("Task : TaskDepBeforeList is null !\n");
		return 1;
	}
	if (dg87_list_check(pTask->pTaskDepBeforeList, print) != 0)
		return 1;

	/* check TaskDepAfterList */
	if (pTask->pTaskDepAfterList == NULL){
		print("Task : TaskDepAfterList is null !\n");
		return 1;
	}
	if (dg87_list_check(pTask->pTaskDepAfterList, print) != 0)
		return 1;
	
	/* check Queue */
	if (pTask->pQueue != NULL){
		if (dg87_list_findItem(pTask->pQueue->pTaskList,
					pTask) == NULL){
			print("Task : task not in Queue.TaskList !\n");
			return 1;
		}
	}

	/* check ConditionList */
	if (pTask->pConditionList == NULL){
		print("Task : ConditionList is null !\n");
		return 1;
	}
	if (dg87_list_check(pTask->pConditionList, print) != 0)
		return 1;
	pItem = pTask->pConditionList->pFirst;
	while (pItem != NULL){
		if (dg87_condition_check(pItem->pObject, print) != 0)
			return 1;
		pItem = pItem->pNext;
	}
	
	return 0;
}
#endif /*DEBUG*/


/****************************************************************************/
/* End of file */
