/* file taskdep.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "task.h"
#include "taskdep.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
/*****************************************************************************/
dg87_taskdep* dg87_taskdep_new(struct dg87_task_struct* pTaskBefore, 
		struct dg87_task_struct* pTaskAfter)
{
	dg87_taskdep* pTaskDep;
	dg87_list* pUpStreamTaskList;
	dg87_list* pDownStreamTaskList;
	if (pTaskBefore == NULL || pTaskAfter == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pTaskDep = dg87_malloc(sizeof(dg87_taskdep),DG87_TASKDEP);
	if (pTaskDep == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* find all upstream tasks */
	pUpStreamTaskList = dg87_list_new();
	if (pUpStreamTaskList == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	dg87_task_findUpStreamTasks(pTaskBefore, pUpStreamTaskList);
	/* find all downstream tasks */
	pDownStreamTaskList = dg87_list_new();
	if (pDownStreamTaskList == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	dg87_task_findDownStreamTasks(pTaskAfter, pDownStreamTaskList);
	/* detect circular dependences*/ 
	if (dg87_list_intersection(pDownStreamTaskList, 
				pUpStreamTaskList) == TRUE){
		dg87_list_delete(pDownStreamTaskList);
		dg87_list_delete(pUpStreamTaskList);
		dg87_err_setError(DG87_ERR_CIRCULARDEPENDENCE);
		return NULL;
	}
	dg87_list_delete(pDownStreamTaskList);
	dg87_list_delete(pUpStreamTaskList);

	pTaskDep->pTaskBefore = pTaskBefore;
	pTaskDep->pTaskAfter = pTaskAfter;
	pTaskDep->type = dg87_taskdep_type_always;
	pTaskDep->state = dg87_taskdep_state_waiting;
	return pTaskDep;
}
/*****************************************************************************/
BOOL dg87_taskdep_delete(dg87_taskdep* pTaskDep)
{
	if (pTaskDep == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* remove taskDep from vectors */
	dg87_list_removeObject(
			pTaskDep->pTaskBefore->pTaskDepAfterList, pTaskDep);
	dg87_list_removeObject(
			pTaskDep->pTaskAfter->pTaskDepBeforeList, pTaskDep);

	dg87_free(pTaskDep, DG87_TASKDEP);
	return TRUE;
}
/*****************************************************************************/
BOOL dg87_taskdep_load(dg87_taskdep* pTaskDep, dg87_storage* pStorage)
{
	dg87byte b;
	if (pTaskDep == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read dependence type */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	if (dg87_taskdep_setType(pTaskDep, b) == FALSE)
		return FALSE;
	/* read dependence state */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	pTaskDep->state = b;
	return TRUE;
}
/*****************************************************************************/
BOOL dg87_taskdep_save(dg87_taskdep* pTaskDep, dg87_storage* pStorage)
{
	if (pTaskDep == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write dependence type */
	if (dg87_storage_writeByte(pStorage, pTaskDep->type)== FALSE)
		return FALSE;
	/* write dependence state */
	if (dg87_storage_writeByte(pStorage, pTaskDep->state)== FALSE)
		return FALSE;
	return TRUE;
}
/*****************************************************************************/
BOOL dg87_taskdep_setType(dg87_taskdep* pTaskDep, dg87_taskdep_type type)
{
	if (pTaskDep == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTaskDep->type = type;
	return TRUE;
}
/*****************************************************************************/
BOOL dg87_taskdep_setState(dg87_taskdep* pTaskDep, dg87_taskdep_state state)
{
	if (pTaskDep == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pTaskDep->state = state;
	return TRUE;
}
/*****************************************************************************/
BOOL dg87_taskdep_evaluate(dg87_taskdep* pTaskdep)
{
	dg87_task_state taskState;
	if (pTaskdep == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}

	/* return if already done */
	if (pTaskdep->state != dg87_taskdep_state_waiting)
		return TRUE;
	taskState = pTaskdep->pTaskBefore->state;
	if (taskState == dg87_task_state_abortStopped)
		return TRUE;

	/* if task is canceled the taskdep is canceled */
	if (taskState == dg87_task_state_canceled){
		pTaskdep->state = dg87_taskdep_state_canceled;
		return TRUE;
	}

	/* return if task is not terminated or aborted or ignored */
	if (taskState != dg87_task_state_terminated
		&& taskState != dg87_task_state_aborted 
		&& taskState != dg87_task_state_ignored)
		return TRUE;

	switch(pTaskdep->type){
		case dg87_taskdep_type_always:
			pTaskdep->state = dg87_taskdep_state_validated;
			break;
		case dg87_taskdep_type_onSuccess:
			if (taskState == dg87_task_state_terminated ||
				taskState == dg87_task_state_ignored)
				pTaskdep->state = dg87_taskdep_state_validated;
			else
				pTaskdep->state = dg87_taskdep_state_canceled;
			break;
		case dg87_taskdep_type_onError:
			if (taskState == 
					dg87_task_state_aborted)
				pTaskdep->state = dg87_taskdep_state_validated;
			else
				pTaskdep->state = dg87_taskdep_state_canceled;
			break;
		default:
			/* never come here */
			break;
	}

	return TRUE;
}
/*****************************************************************************/
dg87_taskdep_type dg87_taskdep_asciiToType(char* pStrType)
{
	if (pStrType == NULL)
		return dg87_taskdep_type_unknown;
	if (strcasecmp(pStrType, "always") == 0)
		return dg87_taskdep_type_always;
	else if (strcasecmp(pStrType, "onsuccess") == 0)
		return dg87_taskdep_type_onSuccess;
	else if (strcasecmp(pStrType, "onerror") == 0)
		return dg87_taskdep_type_onError;
	return dg87_taskdep_type_unknown;
}
/*****************************************************************************/
char* dg87_taskdep_typeToAscii(dg87_taskdep_type deptype)
{
	switch (deptype){
		case dg87_taskdep_type_always: return "always";
		case dg87_taskdep_type_onSuccess: return "onsuccess";
		case dg87_taskdep_type_onError: return "onerror";
		default: return "unknown"; /* to remove warning */
	}
	return NULL;
}
/*****************************************************************************/
dg87_taskdep_state dg87_taskdep_asciiToState(char* pStrState)
{
	if (pStrState == NULL)
		return dg87_taskdep_state_unknown;
	if (strcasecmp(pStrState, "waiting") == 0)
		return dg87_taskdep_state_waiting;
	if (strcasecmp(pStrState, "validated") == 0)
		return dg87_taskdep_state_validated;
	if (strcasecmp(pStrState, "canceled") == 0)
		return dg87_taskdep_state_canceled;
	return dg87_taskdep_state_unknown;
}
/*****************************************************************************/
char* dg87_taskdep_stateToAscii(dg87_taskdep_state depstate)
{
	switch (depstate){
		case dg87_taskdep_state_waiting: return "waiting";
		case dg87_taskdep_state_validated: return "validated";
		case dg87_taskdep_state_canceled: return "canceled";
		default: return "unknown"; /* to remove warning */
	}
	return NULL;
}

/*****************************************************************************/
#ifdef DEBUG
int dg87_taskdep_check(dg87_taskdep* pTaskdep, int (*print)(char*))
{
	/* check TaskBefore */
	if (pTaskdep->pTaskBefore == NULL){
		print("TaskDep : TaskBefore is NULL !\n");
		return 1;
	}
	if (dg87_list_findItem(pTaskdep->pTaskBefore->pTaskDepAfterList,
			pTaskdep) == NULL) {
		print("Taskdep : not found in TaskBefore->TaskdepAfter !\n");
		return 1;
	}
	/* check TaskAfter */
	if (pTaskdep->pTaskAfter == NULL){
		print("TaskDep : TaskAfter is NULL !\n");
		return 1;
	}
	if (dg87_list_findItem(pTaskdep->pTaskAfter->pTaskDepBeforeList,
			pTaskdep) == NULL) {
		print("Taskdep : not found in TaskAfter->TaskdepBefore !\n");
		return 1;
	}
	return 0;
}
#endif /*DEBUG*/

/*****************************************************************************/
/* End of file */
