/* file command.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "command.h"

#define WITHOUTARG 0
#define WITHARG 1

/* TODO replace global variable by function arguments */
static char* c; /* pointer on the current character */
static char* p; /* pointer on the begining of argument */
static char currentOption;  /* last option which was found */

/* local function to skip to end of argument */
static BOOL skipArgument(dg87_command* pCommand)
{
	assert(pCommand != NULL);
	if (*c == '\"'){
		/* skip to end of quoted string*/
		p = ++c;
		while (*c != '\0'){
			if (*(c-1) != '\\' && *c == '\"')
				break;
			c++;
		}
		if (*c == '\0'){
			pCommand->error = DG87_COMMAND_ERR_STRNOTCLOSED;
			return FALSE;
		}
	} else {
		/* go to end of word */
		p = c;
		while (*c != ' ' && *c != '\0')
			c++;
	}
	*c = 0;
	return TRUE;
}
	
/******************************************************************************/
				
/* local function to add an argument to the function */
static BOOL addArgument(dg87_command *pCommand)
{
	char **newBuffer;
	int newSize;
	assert(pCommand != NULL);
	if (skipArgument(pCommand) == FALSE)
		return FALSE;
	if (pCommand->argv == NULL){
		newSize = 1;
		newBuffer = dg87_malloc(sizeof(char*),DG87_CHAR);
	} else {
		newSize = pCommand->argc + 1;
		newBuffer = realloc(pCommand->argv,
				sizeof(char*) * newSize);
	}
	if (newBuffer == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	pCommand->argc = newSize;
	pCommand->argv = newBuffer; 
	pCommand->argv[pCommand->argc - 1] = p;
		
	return TRUE;
}

/******************************************************************************/
/* local function to add an option to the function */
static BOOL addOption(dg87_command *pCommand, int needArg)
{
	char *newBuffer;
	char **newArgBuffer;
	int newSize;
	assert(pCommand != NULL);
	if (pCommand->optc == 0){
		newSize = 1;
		newBuffer = dg87_malloc(sizeof(char),DG87_CHAR);
		newArgBuffer = dg87_malloc(sizeof(char*),DG87_CHAR);
	} else {
		newSize = pCommand->optc + 1;
		newBuffer = realloc(pCommand->optv,
				sizeof(char) * newSize);
		newArgBuffer = realloc(pCommand->optargv,
				sizeof(char*) * newSize);
	}
	if (newBuffer == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	if (newArgBuffer == NULL){
		dg87_free(newBuffer,DG87_CHAR);
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	pCommand->optc = newSize;
	pCommand->optv = newBuffer; 
	pCommand->optargv = newArgBuffer; 
	pCommand->optv[pCommand->optc - 1] = currentOption;
	if (needArg == WITHARG){
		if (skipArgument(pCommand)== FALSE)
			return FALSE;
		pCommand->optargv[pCommand->optc - 1] = p;
	} else {
		pCommand->optargv[pCommand->optc - 1] = NULL;
	}
	return TRUE;	
}


/******************************************************************************/
static void dg87_command_splitString(dg87_command* pCommand)
{
	char* co; /* pointer on optstr */
	int option = 1; /* zero if option disabled by -- */
	char* pend; /* pointer on the end of the string '\0' */
	
	assert(pCommand != NULL);

	/* replace separators by space */
	for (c = pCommand->buffer; *c != '\0'; c++)
		if (*c == '\t') *c = ' ';
	pend = c;

	/* for each characters */
	c = pCommand->buffer;
	while (c < pend){

		/* skip separators */
		while (*c == ' ') c++;
		if (*c == '\0') 
			break;

		/* quoted string as function argument */
		if (*c == '\"'){
			if (addArgument(pCommand) == FALSE)
				return;
			c++;
			continue;
		}

		/* dash dot */
		if (*c == '-' && option == 1){
			/* dash dot a normal argument */
			c++;
			if (*c == ' ' || *c == ':' || *c == '\0'){
				c--;
				if (addArgument(pCommand) == FALSE)
					return;
				c++;
				continue;
			}
			/* detect -- as end of option */
			if (*c == '-'){
				option = 0;
				c++;
				continue;
			}
			/* interpret one or more options */
			while (*c != ' ' && *c != '\0'){

				currentOption = *c;
				/* find the option */
				co = pCommand->optstr; 
				while (*co != '\0' && *co != *c) co++;
				if (*co == '\0'){
					pCommand->error = 
						DG87_COMMAND_ERR_UNKNOWNOPTION;
					pCommand->errorOption = currentOption;
					return;
				}

				/* if the option need an argument */
				if (*(co+1) == ':'){
					c++;
					/* skip separators before argument */
					while (*c == ' ' && *c != '\0') c++;
					if (*c == '\0'){
						pCommand->error = 
						DG87_COMMAND_ERR_NOARGUMENT;
						pCommand->errorOption =
							currentOption;
						return;
					}
					if (addOption(pCommand, WITHARG)== FALSE)
						return;
					/* stop to search other options */
					break;
				} else {
					c++;
					if (addOption(pCommand, WITHOUTARG)== FALSE)
						return;
				}
			}
			c++;
			continue;
		}

		/* skip to end of word argument */
		if (addArgument(pCommand) == FALSE)
			return;
		c++;
	}
}

/******************************************************************************/
dg87_command* dg87_command_new(char* pCmdString)
{
	dg87_command* pCommand;
	char* pArg;
	assert(pCmdString != NULL);
	if (pCmdString == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCommand = dg87_malloc(sizeof(dg87_command),DG87_COMMAND);
	if (pCommand == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pCommand->argc = 0; pCommand->argv = NULL;
	pCommand->optc = 0; pCommand->optv = NULL; pCommand->optargv = NULL;
	pCommand->error = 0;
	pCommand->errorOption = '\0';
	pCommand->optstr = NULL;

	/* extract function name */
	pArg = pCmdString;
	while (*pArg != '\0' && *pArg != '\t' && *pArg != ' ')
		pArg++;
	if (*pArg == '\0'){
		/* function whithout arguments */
		pCommand->buffer = dg87_strdup("",DG87_CHAR);
	} else {
		/* function with arguments */
		*pArg = '\0';
		pCommand->buffer = dg87_strdup(pArg+1,DG87_CHAR);
	}
	if (pCommand->buffer == NULL){
		dg87_free(pCommand,DG87_COMMAND);
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pCommand->pFunctionName = dg87_strdup(pCmdString,DG87_CHAR);
	if (pCommand->pFunctionName == NULL){
		dg87_free(pCommand->buffer,DG87_CHAR);
		dg87_free(pCommand,DG87_COMMAND);
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	return pCommand;
}
/******************************************************************************/
BOOL dg87_command_interpretOptions(dg87_command* pCommand, char* pOptString)
{	
	assert(pCommand != NULL && pOptString != NULL);
	if (pCommand == NULL || pOptString == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}

	pCommand->optstr = dg87_strdup(pOptString,DG87_CHAR);
	if (pCommand->optstr == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	
	dg87_command_splitString(pCommand);
	return TRUE;
}
/******************************************************************************/
BOOL dg87_command_delete(dg87_command* pCommand)
{
	assert(pCommand != NULL);
	if (pCommand == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pCommand->buffer != NULL) 
		dg87_free(pCommand->buffer, DG87_CHAR);
	if (pCommand->pFunctionName != NULL) 
		dg87_free(pCommand->pFunctionName, DG87_CHAR);
	if (pCommand->optstr != NULL) 
		dg87_free(pCommand->optstr, DG87_CHAR);
	if (pCommand->argv != NULL) 
		dg87_free(pCommand->argv, DG87_CHAR);
	if (pCommand->optv != NULL) 
		dg87_free(pCommand->optv, DG87_CHAR);
	if (pCommand->optargv != NULL) 
		dg87_free(pCommand->optargv, DG87_CHAR);
	dg87_free(pCommand, DG87_COMMAND);
	return TRUE;
}
/******************************************************************************/
dg87_command_family dg87_command_asciiToFamily(char* pString)
{
	assert(pString != NULL);
	if (pString == NULL) return -1;

	if ( strcasecmp(pString, "generic") == 0)
		return dg87_command_generic;
	if ( strcasecmp(pString, "communication") == 0)
		return dg87_command_communication;
	if ( strcasecmp(pString, "user") == 0)
		return dg87_command_user;
	if ( strcasecmp(pString, "engine") == 0)
		return dg87_command_engine;
	if ( strcasecmp(pString, "target") == 0)
		return dg87_command_target;
	if ( strcasecmp(pString, "referential") == 0)
		return dg87_command_ref;
	if ( strcasecmp(pString, "job") == 0)
		return dg87_command_job;
	if ( strcasecmp(pString, "day") == 0)
		return dg87_command_day;
	if ( strcasecmp(pString, "variable") == 0)
		return dg87_command_variable;
	return -1;
}
/******************************************************************************/
char* dg87_command_familyToAscii(dg87_command_family family)
{
	switch(family)
	{
		case dg87_command_generic:  return "generic";
		case dg87_command_communication:  return "communication";
		case dg87_command_user:  return "user";
		case dg87_command_engine:  return "engine";
		case dg87_command_target:  return "target";
		case dg87_command_ref:  return "referential";
		case dg87_command_day:  return "day";
		case dg87_command_job:  return "job";
		case dg87_command_variable:  return "variable";
	}
	return "?";
}
/******************************************************************************/
dg87_command_privilege dg87_command_asciiToPrivilege(char* pString)
{
	assert(pString != NULL);
	if (pString == NULL) return -1;

	if ( strcasecmp(pString, "none") == 0)
		return dg87_command_access_none;
	if ( strcasecmp(pString, "consult") == 0)
		return dg87_command_access_consult;
	if ( strcasecmp(pString, "manage") == 0)
		return dg87_command_access_manage;
	if ( strcasecmp(pString, "admin") == 0)
		return dg87_command_access_admin;
	return -1;
}
/******************************************************************************/
char* dg87_command_privilegeToAscii(dg87_command_privilege privilege)
{

	switch(privilege)
	{
		case dg87_command_access_none:  return "none";
		case dg87_command_access_consult:  return "consult";
		case dg87_command_access_manage:  return "manage";
		case dg87_command_access_admin:  return "admin";
	}
	return "?";
}
/******************************************************************************/
/* End of file */
