/* file conf.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "conf.h"

#define DG87_CONF_MAXLINELENGTH 256 

/******************************************************************************/
dg87_conf *dg87_conf_new(char* pConfigFile, dg87_conf_field *pFieldArray)
{
	dg87_conf *pConf;
	assert(pConfigFile != NULL && pFieldArray != NULL);
	if (pConfigFile == NULL || pFieldArray == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (strlen(pConfigFile) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pConf = dg87_malloc(sizeof(dg87_conf), DG87_OTHER);
	if (pConf == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pConf->pConfigFile = dg87_strdup(pConfigFile, DG87_CHAR);
	if (pConf->pConfigFile == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pConf, DG87_OTHER);
	}
	pConf->pFieldArray = pFieldArray;
	return pConf;
}
/******************************************************************************/
BOOL dg87_conf_delete(dg87_conf *pConf)
{
	assert(pConf != NULL);
	if (pConf == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pConf->pConfigFile != NULL)
		dg87_free(pConf->pConfigFile, DG87_OTHER);
	dg87_free(pConf, DG87_OTHER);
	return TRUE;
}
/******************************************************************************/
BOOL dg87_conf_read(dg87_conf *pConf)
{
	FILE* fp;
	char buffer[DG87_CONF_MAXLINELENGTH];
	char* pValue;
	int i;
	dg87_conf_field *pField;

	assert(pConf != NULL);
	if (pConf == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* initialize each field value */
	pField = pConf->pFieldArray;
	while (pField->pName != NULL){
		if (pField->pValue != NULL)
			dg87_free(pField->pValue, DG87_CHAR);
		pField->pValue = NULL;
		/* copy initial value */
		if (pField->pDefaultValue != NULL){
			if ((pField->pValue = dg87_strdup(pField->pDefaultValue,
						DG87_CHAR)) == NULL){
				dg87_err_setError(DG87_ERR_OUTOFMEMORY);
				return FALSE;
			}
		}
		pField++; /* next field */
	}

	/* open configuration file */
	fp = fopen(pConf->pConfigFile, "r");
	if (fp == NULL){
		/* TODO log error message */
		printf("Can't open configuration file '%s'\n", 
				pConf->pConfigFile);
		dg87_err_setError(DG87_ERR_CANTOPENCONFIGFILE);
		return FALSE;
	}

	/* read each lines */
	while (fgets(buffer, DG87_CONF_MAXLINELENGTH, fp) != NULL){

		/* skip comment */
		if (buffer[0] == '#')
			continue;

		/* remove \n at end of line */
		if ((pValue = strrchr(buffer, '\n')) != NULL)
			*pValue = '\0';

		/* remove trailling spaces */
		for (i = strlen(buffer) - 1; i > 0; i--){
			if (buffer[i] != '\t' && buffer[i] != ' ')
				break;
			buffer[i] = '\0';
		}

		/* ignore empty lines */
		if (strlen(buffer) == 0)
			continue;

		/* search separator */
		if ((pValue= strchr(buffer, '=')) == NULL){
			fclose(fp);
			/* TODO log error message */
			printf("can't find separator '=' in line \n '%s'\n",
					buffer);
			dg87_err_setError(DG87_ERR_CONFSYNTAXERROR);
			return FALSE;
		}

		/* replace separator by '\0' */
		*(pValue++) = '\0';

		/* printf("name:%s\n",buffer); printf("value:%s\n",pValue);*/

		/* find field in array */
		pField = pConf->pFieldArray;
		while (pField->pName != NULL){
			/* compare field name */
			if (strcmp(buffer, pField->pName) != 0){
				pField++; /* next field */
				continue;
			}
			/* copy value */
			if (pField->pValue != NULL)
				dg87_free(pField->pValue, DG87_CHAR);
			if ((pField->pValue = dg87_strdup(pValue, 
							DG87_CHAR)) == NULL){
				fclose(fp);
				dg87_err_setError(DG87_ERR_OUTOFMEMORY);
				return FALSE;
			}
			break;
		}
		/* unknown field name */
		if (pField->pName == NULL){
			fclose(fp);
			/* TODO log error message */
			printf("Unknown field name '%s'\n", buffer);
			dg87_err_setError(DG87_ERR_CONFUNKNOWNFIELDNAME);
			return FALSE;
		}
	}
	fclose(fp);
	return TRUE;
}
/******************************************************************************/
char* dg87_conf_getFieldValue(dg87_conf *pConf, char* pFieldName)
{
	dg87_conf_field *pField;
	assert(pConf != NULL && pFieldName != NULL);
	if (pConf == NULL || pFieldName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* find field in array */
	pField = pConf->pFieldArray;
	while (pField->pName != NULL){
		/* compare field name */
		if (strcmp(pFieldName, pField->pName) == 0)
			return pField->pValue;
		pField++;
	}
	dg87_err_setError(DG87_ERR_CONFUNKNOWNFIELDNAME);
	return NULL;
}

/******************************************************************************/
/* End of file */
