/* file connection.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdarg.h>
#include <stdio.h>
#include <sys/time.h> /* prevent error for arm architecture */
#include <assert.h>

#include "connection.h"

/******************************************************************************/
dg87_connection* dg87_connection_in_new(int fd, char* pHost, int ipPort,
		struct dg87_server_struct* pServer)
{
	dg87_connection* pConnection;
	assert(pHost != NULL && pServer != NULL);
	if (pHost == NULL || pServer == NULL || fd == -1) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}	
	pConnection = dg87_malloc(sizeof(dg87_connection), DG87_CONNECTION);
	if (pConnection == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pConnection->pHost = dg87_strdup(pHost,DG87_CHAR);
	if (pConnection->pHost == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pConnection, DG87_CONNECTION);
		return NULL;
	}
	pConnection->ipPort = ipPort;
	pConnection->fd = fd;
	pConnection->pBuffer = NULL;
	pConnection->bufferSize = 0;
	pConnection->messageSize = 0;
	pConnection->type = dg87_connection_type_in;
	pConnection->state = dg87_connection_state_waitMessage;
	pConnection->pServer = pServer;
	pConnection->pCommand = NULL;
	pConnection->close = FALSE;
	pConnection->pUser = NULL;
	return pConnection;
}
/******************************************************************************/
dg87_connection* dg87_connection_out_new(char* pHost, int ipPort)
{
	dg87_connection* pConnection;
	assert(pHost != NULL);
	if (pHost == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pConnection = dg87_malloc(sizeof(dg87_connection),DG87_CONNECTION);
	if (pConnection == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pConnection->pHost = dg87_strdup(pHost,DG87_CHAR);
	if (pConnection->pHost == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pConnection,DG87_CONNECTION);
		return NULL;
	}
	pConnection->ipPort = ipPort;
	pConnection->fd = -1;
	pConnection->pBuffer = NULL;
	pConnection->bufferSize = 0;
	pConnection->messageSize = 0;
	pConnection->type = dg87_connection_type_out;
	pConnection->state= dg87_connection_state_readingMessage;
	pConnection->pServer = NULL;
	pConnection->pCommand = NULL;
	pConnection->close = FALSE;
	pConnection->pUser = NULL;
	return pConnection;
}
/******************************************************************************/
BOOL dg87_connection_delete(dg87_connection* pConnection)
{
	assert(pConnection != NULL);
	if (pConnection == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pConnection->pUser != NULL){
		dg87_user_updateConnectionCount(pConnection->pUser, -1);
		pConnection->pUser = NULL;
	}
	if (pConnection->pCommand != NULL)
		dg87_command_delete(pConnection->pCommand);
	if (pConnection->pBuffer != NULL)
		dg87_free(pConnection->pBuffer, DG87_CHAR);
	if (pConnection->pHost != NULL)
		dg87_free(pConnection->pHost, DG87_CHAR);
	if (pConnection->fd != -1) close(pConnection->fd);
	dg87_free(pConnection, DG87_CONNECTION);
	return TRUE;
}

/******************************************************************************/
BOOL dg87_connection_bufferConcat(dg87_connection *pConnection, char* pString)
{
	char* pNewBuffer;
	int newSize; 
	int stringSize;

	assert(pConnection != NULL && pString != NULL);
	if (pConnection == NULL || pString == NULL){ 
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	stringSize = strlen(pString);
	/* initialize the buffer */
	if (pConnection->pBuffer == NULL){
		newSize = DG87_CONNECTION_BUFFERSIZE;
		pNewBuffer = dg87_malloc(newSize, DG87_CHAR);
		if (pNewBuffer == NULL){
			dg87_err_setError(DG87_ERR_OUTOFMEMORY);
			return FALSE;
		}
		pConnection->pBuffer = pNewBuffer; 
		pConnection->bufferSize = newSize; 
		pConnection->messageSize = 0;
		pConnection->pBuffer[0] = 0;
	}
	/* if the buffer is to small to concat the string */
	while (pConnection->messageSize+stringSize >= pConnection->bufferSize)
	{
		/* then expand the buffer */
		newSize = pConnection->bufferSize + DG87_CONNECTION_BUFFERSIZE;
		pNewBuffer = realloc(pConnection->pBuffer, newSize);
		if (pNewBuffer == NULL){
			dg87_err_setError(DG87_ERR_OUTOFMEMORY);
			return FALSE;
		}
		pConnection->pBuffer = pNewBuffer;
		pConnection->bufferSize = newSize;
	}
	/* append the string to the buffer */
	memcpy(pConnection->pBuffer + pConnection->messageSize,
			pString, stringSize + 1);
	pConnection->messageSize += stringSize;

	return TRUE;
}

/******************************************************************************/
char* dg87_connection_read(dg87_connection* pConnection)
{
	char *pBuffer = NULL;
	assert(pConnection != NULL);
	if (pConnection == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (pConnection->state != dg87_connection_state_messageAvailable){
		dg87_err_setError(DG87_ERR_CNXMESSAGENOTAVAILABLE);
		return NULL;
	}
	pBuffer = pConnection->pBuffer;
	pConnection->pBuffer = NULL;
	pConnection->bufferSize = 0;
	pConnection->messageSize = 0;
	pConnection->state = dg87_connection_state_waitMessage;
	return pBuffer;
}
/******************************************************************************/
BOOL dg87_connection_close(dg87_connection* pConnection)
{	
	assert(pConnection != NULL);
	if (pConnection == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pConnection->close = TRUE;
	return TRUE;
}
/******************************************************************************/
BOOL dg87_connection_readCommand(dg87_connection* pConnection)
{	
	assert(pConnection != NULL);
	if (pConnection == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pConnection->state != dg87_connection_state_messageAvailable){
		dg87_err_setError(DG87_ERR_CNXMESSAGENOTAVAILABLE);
		return FALSE;
	}

	if (pConnection->pCommand != NULL){
		dg87_command_delete(pConnection->pCommand);
		pConnection->pCommand = NULL;
	}
	pConnection->pCommand = dg87_command_new(pConnection->pBuffer);
	if (pConnection->pCommand == NULL)
		return FALSE;

	if (pConnection->pBuffer != NULL){
		dg87_free(pConnection->pBuffer, DG87_CHAR);
		pConnection->pBuffer = NULL;
	}
	pConnection->bufferSize = 0;
	pConnection->messageSize = 0;
	pConnection->state = dg87_connection_state_waitMessage;
	return TRUE;
}
/******************************************************************************/
BOOL dg87_connection_printf
	(dg87_connection* pConnection, char* strformat, ...)
{
	char buffer[60];
	char c[2];
	va_list ap;
	int ival;
	long lval;
	char* pString;
	
	assert(pConnection != NULL && strformat != NULL);
	if (pConnection == NULL || strformat == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pConnection->state != dg87_connection_state_waitMessage){
		dg87_err_setError(DG87_ERR_CNXNOTWAITINGMESSAGE);
		return FALSE;
	}

	va_start(ap,strformat); /* ap points on the first optional argument */
	while (*strformat)
	{
		if (*strformat == '\n'){
			/* print CR LF */
			dg87_connection_bufferConcat(pConnection,"\n\r");
		} else if (*strformat != '%'){
			/* not a format character (%), print it */
			c[0] = *strformat; c[1] = 0;
			dg87_connection_bufferConcat(pConnection, c);
		}
		/* interpret character after % */
		else switch(*++strformat)
		{
			case 's': /* replace '%s' by a string */
				pString = va_arg(ap, char*);
				if (pString != NULL)
				dg87_connection_bufferConcat(
						pConnection, pString);
				break;
			case 'c': /* replace '%c' by a character */
				ival = va_arg(ap, int);
				sprintf(buffer,"%c", ival);
				dg87_connection_bufferConcat(
						pConnection, buffer);
				break;
			case 'i': /* replace '%i' by a integer */
				ival = va_arg(ap, int);
				sprintf(buffer,"%i", ival);
				dg87_connection_bufferConcat(
						pConnection, buffer);
				break;
			case 'l': /* replace '%l' by a long */
				lval = va_arg(ap, long);
				sprintf(buffer,"%li", lval);
				dg87_connection_bufferConcat(
						pConnection, buffer);
				break;
			default:
				c[0] = *strformat; c[1] = 0;
				dg87_connection_bufferConcat(pConnection, c);
		} /* end case */
		strformat++; /* next character */

	} /* end while */
	va_end(ap);	
	return TRUE;

}
/******************************************************************************/
BOOL dg87_connection_send(dg87_connection* pConnection)
{
	assert(pConnection != NULL);
	if (pConnection == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pConnection->state != dg87_connection_state_waitMessage){
		dg87_err_setError(DG87_ERR_CNXNOTWAITINGMESSAGE);
		return FALSE;
	}
	/* add the terminator to the message */
	if (pConnection->type == dg87_connection_type_in)
		dg87_connection_bufferConcat(pConnection,
				DG87_CONNECTION_OUT_TERMINATOR);
	else
		dg87_connection_bufferConcat(pConnection,
				DG87_CONNECTION_IN_TERMINATOR);
	pConnection->state = dg87_connection_state_sendingMessage;
	return TRUE;
}
/******************************************************************************/
BOOL dg87_connection_getMessage(dg87_connection* pConnection)
{
	/* private function : don't test parameters */
	struct timeval tv;
	fd_set readfds;
	int nread, ntoread;
	int delta;
	char* pNewBuffer;
	int newSize;
	char* pTerminator;
	char* pString;

	assert(pConnection != NULL);
	/* return if there is nothing to read */
	tv.tv_sec = 0; tv.tv_usec = 0;
	FD_ZERO(&readfds);
	FD_SET(pConnection->fd, &readfds);
	if (select(pConnection->fd + 1, &readfds, NULL , NULL, &tv) == 0){
		/* close connection if necessary */
		if (pConnection->close == TRUE){
			close(pConnection->fd);
			pConnection->state = 
				dg87_connection_state_connectionClose;
			return TRUE;
		}
		return FALSE;
	}

	/* get the number of bytes to read */	
	ioctl(pConnection->fd, FIONREAD, &ntoread);
	if (ntoread == 0){
		/* connection close by remote */
		pConnection->state = dg87_connection_state_connectionClose;
		if (pConnection->pBuffer != NULL){
			dg87_free(pConnection->pBuffer, DG87_CHAR);
			pConnection->pBuffer = NULL;
		}
		return FALSE;
	}

	/* allocate enough memory for the buffer */
	if (pConnection->pBuffer == NULL){
		delta = 0;
		pConnection->bufferSize = ntoread;
		pConnection->pBuffer = dg87_malloc(pConnection->bufferSize+1,
				DG87_CHAR);
		if (pConnection->pBuffer == NULL)
			return FALSE;
		memset(pConnection->pBuffer,0,pConnection->bufferSize+1);
	}else{
		delta = pConnection->bufferSize;
		newSize = pConnection->bufferSize + ntoread;
		pNewBuffer = realloc(pConnection->pBuffer, newSize+1);
		if (pNewBuffer == NULL)
			return FALSE;
		/* if realloc fails, pConnection->pBuffer point on the 
		 * previous buffer */
		pConnection->pBuffer = pNewBuffer;
		pConnection->bufferSize = newSize;
	}

	/* the start reading position is pBuffer + delta */
	nread =  read(pConnection->fd, pConnection->pBuffer + delta, ntoread);
	if (nread != ntoread){
		/* connection is close */
		pConnection->state = dg87_connection_state_connectionClose;
		if (pConnection->pBuffer != NULL){
			dg87_free(pConnection->pBuffer, DG87_CHAR);
			pConnection->pBuffer = NULL;
		}
		return FALSE;
	}
	*(pConnection->pBuffer + pConnection->bufferSize) = '\0';

	/* detect terminator */
	if (pConnection->bufferSize < 1)
		return TRUE;

	/* the type of connection determine the terminator */
	if (pConnection->type == dg87_connection_type_in)
		pTerminator = DG87_CONNECTION_IN_TERMINATOR;
	else
		pTerminator = DG87_CONNECTION_OUT_TERMINATOR;

	/* try to find the terminator at the end of the buffer */
	pString = pConnection->pBuffer + pConnection->bufferSize 
		 - strlen(pTerminator); 
	if (strcmp(pString, pTerminator) == 0){
		pConnection->pBuffer[pConnection->bufferSize 
			- strlen(pTerminator)] = '\0';
		pConnection->state = dg87_connection_state_messageAvailable;
	}
	return TRUE;
}
/******************************************************************************/
BOOL dg87_connection_sendMessage(dg87_connection* pConnection)
{
	/* private function : don't test parameters */
	struct timeval tv;
	fd_set writefds;
	ssize_t nwrite;

	assert(pConnection != NULL);
	/* return if fd is not ready to be writing */ 
	tv.tv_sec = 0; tv.tv_usec = 0;
	FD_ZERO(&writefds);
	FD_SET(pConnection->fd, &writefds);
	if (select(pConnection->fd + 1, NULL, &writefds, NULL, &tv) == 0)
		return FALSE;

	/* get the size of the output buffer */
	/*
	size = sizeof(size_t);
	getsockopt(pConnection->fd, SOL_SOCKET, SO_SNDBUF, &sndBufSize, &size);
	printf("output buffer size : %i\n", sndBufSize);
	size = sizeof(size_t);
	getsockopt(pConnection->fd, SOL_SOCKET, SO_RCVBUF, &sndBufSize, &size);
	printf("input buffer size : %i\n", sndBufSize);
	*/

	/* send buffer */
	/* TODO timeout if remote can't receive all the buffer ? 
	 * we can just fill the output buffer and send the remaining
	 * in next updates */
	nwrite = write(pConnection->fd, pConnection->pBuffer,
			pConnection->messageSize);
	if (nwrite != pConnection->messageSize){
		pConnection->state = dg87_connection_state_error;
		return FALSE;
	}
	dg87_free(pConnection->pBuffer, DG87_CHAR);
	pConnection->pBuffer = NULL;
	if (pConnection->close == TRUE){
		close(pConnection->fd);
		pConnection->state = dg87_connection_state_connectionClose;
		return TRUE;
	}
	pConnection->state = dg87_connection_state_readingMessage;
	return TRUE;
}
/******************************************************************************/
BOOL dg87_connection_update(dg87_connection* pConnection)
{
	struct sockaddr_in address;
	struct hostent *pHostEnt;
	int sockfd,length,result;

	assert(pConnection != NULL);
	if (pConnection == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* open the connection if necessary*/
	if (pConnection->fd == -1){
		/* convert hostname to IP address */
		pHostEnt = gethostbyname(pConnection->pHost);
		if (!pHostEnt){
			pConnection->state = dg87_connection_state_error;
			dg87_err_setError(DG87_ERR_CNXCANTGETHOSTBYNAME);
			return FALSE;
		}
		/* create the socket */
		sockfd = socket(AF_INET, SOCK_STREAM, 0);
		if (sockfd == -1){
			dg87_err_setError(DG87_ERR_CNXSOCKET);
			return FALSE;
		}
		/* connect to the remote */
		address.sin_family = AF_INET;
		address.sin_addr = *(struct in_addr *)*pHostEnt->h_addr_list;
		address.sin_port = htons(pConnection->ipPort);
		length = sizeof(address);
		result = connect(sockfd, (struct sockaddr *)&address, length);
		if (result == -1){
			pConnection->state = dg87_connection_state_error;
			dg87_err_setError(DG87_ERR_CNXCANTCONNECT);
			return FALSE;
		}
		pConnection->fd = sockfd;
	}
	/* read or write message */
	switch (pConnection->state){
		case dg87_connection_state_readingMessage:
			return dg87_connection_getMessage(pConnection);
		case dg87_connection_state_sendingMessage:
			return dg87_connection_sendMessage(pConnection);
		default:
			/* nothing to do */
			break;
	}
	return FALSE;
}
/******************************************************************************/
BOOL dg87_connection_setUser(dg87_connection* pConnection, dg87_user* pUser)
{
	assert(pConnection != NULL);
	if (pConnection == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}

	/* remove previous user */
	if (pConnection->pUser != NULL)
		dg87_user_updateConnectionCount(pConnection->pUser, -1);

	/* add current user */
	pConnection->pUser = pUser;
	if (pConnection->pUser != NULL)
		dg87_user_updateConnectionCount(pConnection->pUser, 1);

	return TRUE;
}

/******************************************************************************/
/* End of file */
