/* file connection.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef DG87_CONNECTION_H
#define DG87_CONNECTION_H 1

#include "global.h"
#include "server.h"
#include "command.h"

#define DG87_CONNECTION_BUFFERSIZE 1024
#define DG87_CONNECTION_IN_TERMINATOR "\r\n"
#define DG87_CONNECTION_OUT_TERMINATOR "\r\nDiogene87>"

/*
 * dg87_connection_type
 * The type of a connection can be "in" or "out".
 * An "in" connection is established by a remote client.
 * 	Initially the program read a request message from the remote,
 *	and then the program send a response message to the remote.
 * An "out" connection is established by the program.
 * 	Initially the program send a message to the remote,
 * 	and after the program read a response message from the remote. 
 */
typedef enum dg87_connection_type_enum
{
	dg87_connection_type_in,
	dg87_connection_type_out
} dg87_connection_type;

/*
 * dg87_connection_state
 */
typedef enum dg87_connection_state_enum
{
	/* readingMessage : connection is reading a message from the remote */
	dg87_connection_state_readingMessage,

	/* messageAvailable : a message from the remote is available and can 
	 * be read with a call to dg87_connection_getMessage */
	dg87_connection_state_messageAvailable,

	/* waitMessage : connection is waiting from a message to send to the
	 * remote. The message can be create by multiple call to
	 * dg87_connection_write and dg87_connection_printf.
	 * The message will be send by a call to dg87_connection_postMessage */
	dg87_connection_state_waitMessage,

	/* sendingMessage : connection is sending the message to the remote */
	dg87_connection_state_sendingMessage,

	/* connectionClose : the connection has been close by the remote */
	dg87_connection_state_connectionClose,

	/* error : the connection can't be open */
	dg87_connection_state_error
} dg87_connection_state;

/*
 * dg87_connection object
 */
typedef struct dg87_connection_struct
{
	struct dg87_server_struct* pServer;
	dg87_connection_type type;
	dg87_connection_state state;
	int fd;
	char* pHost;
	int ipPort;
	char* pBuffer;
	int bufferSize;
	int messageSize;
	struct dg87_command_struct* pCommand;
	BOOL close;
	struct dg87_user_struct* pUser; /* user connected (can be null) */
} dg87_connection;	

/*
 * dg87_connection_in_new
 * create a connection object which type is dg87_connection_type_in
 */
dg87_connection* dg87_connection_in_new(int fd, char* pHost, int ipPort,
		struct dg87_server_struct* pServer);

/*
 * dg87_connection_out_new
 * create a connection object which type is dg87_connection_type_out
 */
dg87_connection* dg87_connection_out_new(char* pHost, int ipPort);

/*
 * dg87_connection_delete
 * close the connection and delete the memory for this object
 */
BOOL dg87_connection_delete(dg87_connection* pConnection);

/*
 * dg87_connection_read
 * read a message from the remote.
 * the function return NULL if the message is not available
 */
char* dg87_connection_read(dg87_connection* pConnection);

/*
 * dg87_connection_close
 */
BOOL dg87_connection_close(dg87_connection* pConnection);

/*
 * dg87_connection_readCommand
 */
BOOL dg87_connection_readCommand(dg87_connection* pConnection);

/*
 * dg87_connection_printf
 * append a string to the message which will be send to the remote.
 */
BOOL dg87_connection_printf
	(dg87_connection* pConnection, char* strformat, ...);

/* 
 * dg87_connection_send
 * transmit the message to the remote.
 */
BOOL dg87_connection_send(dg87_connection* pConnection);

/*
 * dg87_connection_update
 * transmit a message to the remote or receive a message from the remote.
 */
BOOL dg87_connection_update(dg87_connection* pConnection);

/* 
 * dg87_connection_setUser
 * associate a user to identify who is connected.
 * user can be NULL if user isn't known.
 */
BOOL dg87_connection_setUser(dg87_connection* pConnection, 
		struct dg87_user_struct* pUser);

#endif /*DG87_CONNECTION_H*/

