/* file list.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "list.h"

/**************************************************************************/
dg87_list* dg87_list_new()
{
	dg87_list* pList = dg87_malloc(sizeof(dg87_list), DG87_LIST);
	if (pList == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pList->pFirst = NULL;
	pList->pLast = NULL;
	pList->count = 0;
	return pList;
}
/**************************************************************************/
BOOL dg87_list_delete(dg87_list* pList)
{
	assert(pList != NULL);
	if (pList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	dg87_list_clear(pList);
	dg87_free(pList, DG87_LIST);
	return TRUE;
}
/**************************************************************************/
BOOL dg87_list_clear(dg87_list* pList)
{
	assert(pList != NULL);
	if (pList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pList->count > 0)
	{
		dg87_item *pCurrent = pList->pFirst;
		dg87_item *pNext = pCurrent->pNext;
		while (pCurrent != NULL){
			dg87_item_delete(pCurrent);
			pCurrent = pNext;
			if (pCurrent != NULL)
				pNext = pCurrent->pNext;
		}
	}
	pList->pFirst = NULL;
	pList->pLast = NULL;
	pList->count = 0;
	return TRUE;
}


/**************************************************************************/
BOOL dg87_list_addObject(dg87_list* pList, void* pObject)
{
	dg87_item* pItem;
	dg87_item *pTemp;
	assert(pList != NULL && pObject != NULL);
	if (pList == NULL || pObject == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pItem = dg87_item_new(pObject);
	if (pItem == NULL)
		return FALSE;
	if (pList->pLast == NULL){
		/* the list is empty, add the first element */
		pList->pFirst = pItem;
		pList->pLast = pItem;
		pList->count=1;
	}else{
		/* add the element at the end */
		pTemp = pList->pLast;
		pTemp->pNext = pItem;
		pItem->pPrev = pTemp;
		pList->pLast = pItem;
		pList->count++;
	}
	return TRUE;
}
/**************************************************************************/
BOOL dg87_list_insertObject(dg87_list* pList, void* pObject, dg87uint index)
{
	dg87_item *pItem, *pTemp;
	dg87uint i = 0;
	
	assert(pList != NULL && pObject != NULL);
	if (pList == NULL || pObject == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (index > pList->count) {
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return FALSE;
	}

	pItem = dg87_item_new(pObject);
	if (pItem == NULL) return FALSE;

	/* if the list is empty, we add at the begin of the list */
	if (pList->pFirst == NULL){
		/* the list is empty */
		pList->pFirst = pItem;
		pList->pLast = pItem;
		pList->count = 1;
	}else if (index == pList->count){
		/* add at the end of the list */
		pTemp = pList->pLast;
		pTemp->pNext = pItem;
		pItem->pPrev = pTemp;
		pList->pLast = pItem;
		pList->count++;
	}else{
		/* the list is not empty */
		i = 0;
		pTemp = pList->pFirst;
		/* find the item at the given index */
		while (pTemp->pNext != NULL && i < index){
			pTemp = pTemp->pNext;
			i++;
		}
		/* taskdate the links */
		pItem->pNext = pItem;
		if (pTemp->pPrev == NULL)
			pList->pFirst = pItem;
		else
			pTemp->pPrev->pNext = pItem;
		pItem->pPrev = pTemp->pPrev;
		pTemp->pPrev = pItem;
		pList->count++;
	}
	
	return TRUE;
}
/**************************************************************************/
dg87uint dg87_list_count(dg87_list* pList)
{
	assert(pList != NULL);
	if (pList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return pList->count;
}
/**************************************************************************/
dg87_item* dg87_list_getItem(dg87_list* pList, dg87uint index)
{
	dg87uint i;
	assert(pList != NULL);
	if (pList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	} 
	if (index >= pList->count){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	} 

	if (index == 0)
		return pList->pFirst;
	else if (index == pList->count - 1)
		return pList->pLast;
	else{
		dg87_item* pItem = pList->pFirst;
		for (i = 0; i < index; i++)
			pItem = pItem->pNext;
		return pItem; 
	}
	return NULL;
}
/**************************************************************************/
BOOL dg87_list_moveItem(dg87_list* pList, dg87uint index, int delta)
{
	dg87_item* pFirstItem;
	dg87_item* pNextItem;
	void* pObject;
	assert(pList != NULL);
	if (pList == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	} 
	if (index >= pList->count){
		dg87_err_setError(DG87_ERR_INDEXOUTOFBOUND);
		return FALSE;
	} 
	switch (delta){
		case 1 :
			if (index == pList->count - 1){
				/* can't move next element */
				dg87_err_setError(DG87_ERR_INDEXOUTOFBOUND);
				return FALSE;
			}
			pFirstItem = dg87_list_getItem(pList, index);
			pNextItem = pFirstItem->pNext;
			break;
		case -1 :
			if (index == 0){
				/* can't move first element */
				dg87_err_setError(DG87_ERR_INDEXOUTOFBOUND);
				return FALSE;
			}
			pFirstItem = dg87_list_getItem(pList, index - 1);
			pNextItem = pFirstItem->pNext;
			break;
		default:
			dg87_err_setError(DG87_ERR_INCORRECTVALUE);
			return FALSE;
	}
	pObject = pFirstItem->pObject;
	pFirstItem->pObject = pNextItem->pObject;
	pNextItem->pObject = pObject;
	return TRUE;
}
/**************************************************************************/
void* dg87_list_getObjectByIndex(dg87_list* pList, dg87uint index)
{
	dg87_item* pItem;
	assert(pList != NULL);
	if (pList == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
       	pItem = dg87_list_getItem(pList, index);
	if (pItem == NULL) 
		return NULL;
	return pItem->pObject;
}
/**************************************************************************/
BOOL dg87_list_removeObject(dg87_list* pList, void* pObject)
{
	dg87_item* pItem;
	assert(pList != NULL && pObject != NULL);
	if (pList == NULL || pObject == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pItem = dg87_list_findItem(pList, pObject);
	if (pItem == NULL)
		return FALSE;

	dg87_list_removeItem(pList, pItem);
	return TRUE;
}
/**************************************************************************/
void* dg87_list_removeObjectByIndex(dg87_list* pList, dg87uint index)
{
	dg87_item* pItem;
	assert(pList != NULL);
	if (pList == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pItem = dg87_list_getItem(pList, index);
	if (pItem == NULL) return NULL;

	return dg87_list_removeItem(pList, pItem);
}
/**************************************************************************/
void* dg87_list_removeItem(dg87_list* pList, dg87_item* pItem)
{
	void* pObject;
	assert(pList != NULL && pItem != NULL);
	if (pList == NULL || pItem == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}

	pObject = pItem->pObject;

	/* change the link of the previous item in the list */
	if (pItem->pPrev == NULL)
		pList->pFirst = pItem->pNext;
	else
		pItem->pPrev->pNext = pItem->pNext;

	/* change the link of the next item in the list */
	if (pItem->pNext == NULL)
		pList->pLast = pItem->pPrev;
	else
		pItem->pNext->pPrev = pItem->pPrev;

	/* delete the item */
	pList->count--;
	dg87_item_delete(pItem);
	return pObject;
}
/***************************************************************************/
dg87_item* dg87_list_findItem(dg87_list* pList, void* pObject)
{
	dg87_item* pItem;
	assert(pList != NULL && pObject != NULL);
	if (pList == NULL || pObject == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	/* browse the list to find the item associate to the object */
	pItem = pList->pFirst;
	while (pItem != NULL && pItem->pObject != pObject)
		pItem = pItem->pNext;
	return pItem;
}

/***************************************************************************/
BOOL dg87_list_intersection(dg87_list* pList1, dg87_list* pList2)
{
	dg87_item *pItem1, *pItem2;
	assert(pList1 != NULL && pList2 != NULL);
	if (pList1 == NULL || pList2 == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* for each object of the first list*/
	pItem1 = pList1->pFirst;
	while (pItem1 != NULL){
		/* for each object of the first list*/
		pItem2 = pList2->pFirst;
		while (pItem2 != NULL){
			/* if an object is in the two lists */ 
			if (pItem1->pObject == pItem2->pObject)
				return TRUE;
			/* next object*/
			pItem2= pItem2->pNext;
		}
		/* next object*/
		pItem1= pItem1->pNext;
	}
	return FALSE;
}
/***************************************************************************/
#ifdef DEBUG
int dg87_list_check(dg87_list* pList, int (*print)(char*))
{
	dg87_item* pItem;
	dg87uint count = 0;
	pItem = pList->pFirst;
	while (pItem != NULL){

		/* check object */
		if (pItem->pObject == NULL){
			print("List : Object pointer is NULL !\n");
			return 1;
		}

		/* check next element */
		if (pItem->pNext == NULL){
			/* last element */
			if (pList->pLast != pItem){
				print("List : Last element incorrect !\n");
				return 1;
			}
		}
		else if (pItem->pNext->pPrev != pItem){
			print("List : Previous pointer incorrect !\n");
			return 1;
		}

		/* check previous element */
		if (pItem->pPrev == NULL){
			/* first element */
			if (pList->pFirst != pItem){
				print("List : First element incorrect !\n");
				return 1;
			}
		}
		else if (pItem->pPrev->pNext != pItem){
			print("List : Previous pointer incorrect !\n");
			return 1;
		}

		/* next element */
		count ++;
		pItem = pItem->pNext;

	}
	/* check object's count */
	if (count != pList->count){
		print("List : incorrect count !\n");
		return 1;
	}
	/* check empty list */
	if (pList->count == 0 || pList->pFirst == NULL || pList->pLast == NULL)
	{
		if (pList->count != 0 
				|| pList->pFirst != NULL 
				|| pList->pLast != NULL){
			print("List : incorrect empty list\n");
			return 1;
		}
	}
	return 0;
}
#endif /*DEBUG*/

/***************************************************************************/
/* End of file */
