/* file message.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <config.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <assert.h>

#include "global.h"
#include "message.h"
#include "daemon.h"

/******************************************************************************/
BOOL dg87_message_verifyBanner(char* pBanner)
{
	char *pChar1, *pChar2;
	int size;
	assert(pBanner != NULL);
	if (pBanner == NULL)
		return FALSE;

	/* banner sample : diogene87-dg87d-0.4.0-(PXX) 
	 * this function must verify that PXX between parentheses
	 * correspond to DG87_PROTOCOL_VERSION */

	/* find parentheses */
	pChar1 = strchr(pBanner, '(');
	pChar2 = strchr(pBanner, ')');
	if (pChar1 == NULL || pChar2 == NULL)
		return FALSE;
	
	/* at least on character between parentheses */
	if (pChar1+1 >= pChar2)
		return FALSE;
	
	pChar1++; size = pChar2 - pChar1 - 1;
	if (strncmp(DG87_PROTOCOL_VERSION, pChar1 , size) != 0)
		return FALSE;

	return TRUE;
}
/******************************************************************************/
BOOL dg87_message_send(dg87_message* pMessage, dg87_server *pServer)
{	
	dg87_connection* pConnection;
	char* pResponse;
	char* pBanner;
	assert(pMessage != NULL && pServer != NULL);
	if (pMessage == NULL || pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pMessage->state == dg87_message_state_cancel)
		return FALSE;

	if (pMessage->pConnection == NULL){
		/* create a connection object */
		pConnection = dg87_connection_out_new(pMessage->pHost, 
				pMessage->ipPort);
		if (pConnection == NULL)
			return FALSE;
		pMessage->pConnection = pConnection; 
	}
	pConnection = pMessage->pConnection;

	dg87_connection_update(pConnection);

	switch (pMessage->state) {

		case dg87_message_state_notConnected:
			if(pConnection->state == dg87_connection_state_error){
				/* temporization between two connection retry */
				time_t currentTime = time(NULL);
				if (pMessage->lastConnectionRetry != 0 &&
						pMessage->lastConnectionRetry + 
						DG87_CONNECTION_RETRY_TEMPO > 
						currentTime)
					return FALSE;
				/* try again */
				pConnection->state = 
					dg87_connection_state_readingMessage;
				pMessage->lastConnectionRetry = currentTime;
			} else {
				printf("connection ready \n");
				pMessage->state = dg87_message_state_readBanner;
			}
			break;

		case dg87_message_state_readBanner:
			/* read the banner */
			if (pConnection->state ==
				dg87_connection_state_messageAvailable){
				pBanner = dg87_connection_read(pConnection);
				/*  verify protocol */
				if (pBanner == NULL) {
					pMessage->state = 
						dg87_message_state_bannerError;
					return FALSE;
				}
				
				if (dg87_message_verifyBanner(pBanner) == FALSE)
				{
					pMessage->state = 
						dg87_message_state_bannerError;
					dg87_free(pBanner, DG87_CHAR);
					return FALSE;
				}
				dg87_free(pBanner, DG87_CHAR);
				pMessage->state = 
					dg87_message_state_sendUser;
			}
			return FALSE;

		case dg87_message_state_sendUser:
			/* send user name and password */
			dg87_connection_printf(pMessage->pConnection,
					"user \"%s\" \"%s\"",
					DG87_USER_DIOGENE_NAME, 
					pServer->pDiogenePassword);
			dg87_connection_send(pMessage->pConnection);
			pMessage->state = dg87_message_state_readUserResponse;
			return FALSE;

		case dg87_message_state_readUserResponse:
			if (pConnection->state ==
				dg87_connection_state_messageAvailable){
				pResponse = dg87_connection_read(pConnection);
				if (pResponse == NULL){
					pMessage->state = 
						dg87_message_state_bannerError;
					dg87_free(pResponse, DG87_CHAR);
					return FALSE;
				}
				if (strcmp(pResponse, "OK") != 0){
					pMessage->state = 
						dg87_message_state_bannerError;
					dg87_free(pResponse, DG87_CHAR);
					return FALSE;
				}
				dg87_free(pResponse, DG87_CHAR);
				pMessage->state = 
					dg87_message_state_sendMessage;
			}
			return FALSE;

		case dg87_message_state_sendMessage:
			/* send the message */
			dg87_connection_printf(pMessage->pConnection,
					pMessage->pMsgString);
			dg87_connection_send(pMessage->pConnection);
			pMessage->state = dg87_message_state_readResponse;
			return FALSE;

		case dg87_message_state_readResponse:
			/* try to read the response */
			pResponse = dg87_connection_read(pMessage->pConnection);
			/* TODO interpret response */
			if (pResponse == NULL)
				return FALSE;
			dg87_free(pResponse, DG87_CHAR);
			return TRUE;

		case dg87_message_state_cancel:
		case dg87_message_state_bannerError:
		case dg87_message_state_authError:
			/* Nothing to do */
			break;
	}

	return FALSE;
}
/******************************************************************************/
dg87_message* dg87_message_new(char* pHost, int ipPort, char* pMsgString)
{
	dg87_message* pMessage;
	assert(pHost != NULL && pMsgString != NULL);
	if (pHost == NULL || pMsgString == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pMessage = dg87_malloc(sizeof(dg87_message), DG87_MESSAGE);
	if (pMessage == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pMessage->pHost = dg87_strdup(pHost, DG87_CHAR);
	if (pMessage->pHost == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pMessage, DG87_MESSAGE);
		return NULL;
	}
	pMessage->pMsgString = dg87_strdup(pMsgString, DG87_CHAR);
	if (pMessage->pMsgString == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pMessage->pHost, DG87_CHAR);
		dg87_free(pMessage, DG87_MESSAGE);
		return NULL;
	}
	pMessage->ipPort = ipPort;
	pMessage->pConnection = NULL;
	pMessage->state = dg87_message_state_notConnected;
	pMessage->lastConnectionRetry = 0;

	return pMessage;
}
/******************************************************************************/
BOOL dg87_message_delete(dg87_message* pMessage)
{
	assert(pMessage != NULL);
	if (pMessage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pMessage->pConnection != NULL)
		dg87_connection_delete(pMessage->pConnection);
	dg87_free(pMessage->pHost, DG87_CHAR);
	dg87_free(pMessage->pMsgString, DG87_CHAR);
	dg87_free(pMessage, DG87_MESSAGE);
	return TRUE;
}
/******************************************************************************/
BOOL dg87_message_cancel(dg87_message* pMessage)
{	
	char* pString;
	char *pMsg = "Cancel message '%s'";
	assert(pMessage != NULL);
	if (pMessage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pMessage->state == dg87_message_state_cancel)
		return TRUE;

	pString = dg87_malloc(sizeof(char) * (strlen(pMsg) 
			+ strlen(pMessage->pMsgString)+1), DG87_CHAR);
	if (pString == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	sprintf(pString, pMsg, pMessage->pMsgString);
	dg87_daemon_log(pString);
	dg87_free(pString, DG87_CHAR);
	
	pMessage->state = dg87_message_state_cancel;
	return TRUE;
}
/******************************************************************************/
char* dg87_message_stateToAscii(dg87_message_state state)
{
	switch(state){
	case dg87_message_state_notConnected:
		return "not connected";
	case dg87_message_state_readBanner:
		return "read banner";
	case dg87_message_state_sendUser:
		return "send user name";
	case dg87_message_state_readUserResponse:
		return "get user response";
	case dg87_message_state_sendMessage:
		return "send message";
	case dg87_message_state_readResponse:
		return "get message response";
	case dg87_message_state_cancel:
		return "canceld";
	case dg87_message_state_bannerError:
		return "error";
	case dg87_message_state_authError:
		return "authentification error";
	default:
		return "Unknown state";
	}

}
/******************************************************************************/
/* End of file */
