/* file server.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <assert.h>

#include <config.h>
#include "server.h"
#include "daemon.h"
#include "message.h"
#include "user.h"

#define CNX_MAX 10 /* max number of sockets connection */

/*****************************************************************************/
/* dg87_server_closePidFile
 * Close and delete actual PID file.
 */
static BOOL dg87_server_closePidFile(dg87_server *pServer)
{
	assert(pServer != NULL);
	if (pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* return if no PID file or if PID file is already closed */
	if (pServer->pPidFile == NULL || pServer->fdPidFile == 0)
		return TRUE;
	close(pServer->fdPidFile); 
	unlink(pServer->pPidFile);
	pServer->fdPidFile = 0;
	return TRUE;
}
/*****************************************************************************/
/* dg87_server_openPidFile
 * Open PID file and write process ID in it, and lock it.
 * pServer->pPidFile can be NULL if no PID file is used.
 */
static BOOL dg87_server_openPidFile(dg87_server *pServer)
{
	char str[12];
	assert(pServer != NULL);
	if (pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* close previous lock file */
	dg87_server_closePidFile(pServer);

	/* return if no pid file */
	if (pServer->pPidFile == NULL)
		return TRUE;
	/* try to open the lock file */
	pServer->fdPidFile = open(pServer->pPidFile, O_RDWR | O_CREAT, 0640);
	if (pServer->fdPidFile < 0){
		dg87_daemon_log("Can't open PID file !");
		return FALSE;
	}
	/* try to lock the file */
	if (lockf(pServer->fdPidFile, F_TLOCK,0) < 0){
		dg87_daemon_log("Can't lock PID file !");
		return FALSE;
	}
	/* write the pid of the deamon in the file */
	snprintf(str, 12, "%d\n", getpid());
	if (write(pServer->fdPidFile, str, strlen(str)) == -1){
		dg87_daemon_log("Can't write in PID file !");
		return FALSE;
	}
	/* don't close the file to keep the file locked */
	return TRUE;
}
/******************************************************************************/
static BOOL dg87_server_createUsers(dg87_server* pServer)
{
	dg87_user* pUser;
	assert(pServer != NULL);
	if (pServer == NULL)
		return FALSE;
	/* create diogene */
	pUser = dg87_server_userAdd(pServer , DG87_USER_DIOGENE_NAME);
	if (pUser == FALSE)
		return FALSE;
	dg87_user_setPassword(pUser, pServer->pDiogenePassword);
	dg87_user_setPrivilege(pUser, 
			dg87_command_day, dg87_command_access_admin);
	dg87_user_setPrivilege(pUser, 
			dg87_command_job, dg87_command_access_admin);
	pUser->systemAccount = TRUE;
	/* create administrator */
	pUser = dg87_server_userAdd(pServer , DG87_USER_ADMIN_NAME);
	if (pUser == FALSE)
		return FALSE;
	dg87_user_setAdministrator(pUser);
	return TRUE;
}
/******************************************************************************/
dg87_server* dg87_server_new(
	char* pPackageName,
	char* pProgramName,
	char* pProgramVersion,
	int ipPort,
	BOOL (*work_function)(dg87_server*),
	int (*signal_function)(dg87_server*, int signo),
	void* pData,
	struct dg87_interpret_cmd_struct *pCmdArray,
	char* pPidFile,
	char* pDiogenePassword)
{
	dg87_server* pServer;

	assert(pProgramName != NULL && pProgramVersion != NULL &&
			work_function != NULL && signal_function != NULL &&
			pData != NULL && pCmdArray != NULL &&
			pDiogenePassword != NULL);
	if (pPackageName == NULL || 
			pProgramName == NULL || 
			pProgramVersion ==NULL ||
			work_function == NULL ||
			signal_function == NULL ||
			pData == NULL || pCmdArray == NULL ||
			pDiogenePassword == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}

	pServer = dg87_malloc(sizeof(dg87_server), DG87_SERVER);
	if (pServer == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pServer->pListConnection = dg87_list_new();
	if (pServer->pListConnection == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pServer,DG87_SERVER);
		return NULL;
	}
	pServer->pListMessage = dg87_list_new();
	if (pServer->pListMessage == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_list_delete(pServer->pListConnection);
		dg87_free(pServer,DG87_SERVER);
		return NULL;
	}

	pServer->pUserVector = dg87_vector_new();
	if (pServer->pUserVector == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_list_delete(pServer->pListConnection);
		dg87_list_delete(pServer->pListMessage);
		dg87_free(pServer,DG87_SERVER);
		return NULL;
	}

	pServer->pDiogenePassword = pDiogenePassword;
	if (dg87_server_createUsers(pServer) == FALSE){
		dg87_list_delete(pServer->pListConnection);
		dg87_list_delete(pServer->pListMessage);
		dg87_vector_delete(pServer->pUserVector);
		dg87_free(pServer,DG87_SERVER);
		return NULL;
	}

	pServer->pInterpret = dg87_interpret_new(pServer,pCmdArray);
	if (pServer->pInterpret == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_list_delete(pServer->pListConnection);
		dg87_list_delete(pServer->pListMessage);
		dg87_vector_delete(pServer->pUserVector);
		dg87_free(pServer,DG87_SERVER);
		return NULL;
	}


	if (pPidFile == NULL)
		pServer->pPidFile = NULL;
	else {
		pServer->pPidFile = dg87_strdup(pPidFile, DG87_CHAR);
		if (pServer->pPidFile == NULL){
			dg87_err_setError(DG87_ERR_OUTOFMEMORY);
			dg87_list_delete(pServer->pListConnection);
			dg87_list_delete(pServer->pListMessage);
			dg87_vector_delete(pServer->pUserVector);
			dg87_interpret_delete(pServer->pInterpret);
			dg87_free(pServer->pDiogenePassword,DG87_CHAR);
			dg87_free(pServer,DG87_SERVER);
			return NULL;
		}
	}
	pServer->fdPidFile = 0;

	pServer->pPackageName = pPackageName;
	pServer->pProgramName = pProgramName;
	pServer->pProgramVersion = pProgramVersion;
	pServer->ipPort = ipPort;
	pServer->work_function = work_function;
	pServer->signal_function = signal_function;
	pServer->pData = pData;
	pServer->quit = FALSE;
	pServer->shutdown = FALSE;

	return pServer;
}
/******************************************************************************/
BOOL dg87_server_delete(dg87_server* pServer)
{
	dg87_item* pItem;
	dg87_connection* pConnection;
	dg87_element* pElement;
	dg87_user* pUser;
	dg87_message* pMessage;
	assert(pServer != NULL);
	if (pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* delete all remaining connection */
	if (pServer->pListConnection->count > 0){
		pItem = pServer->pListConnection->pFirst;
		while (pItem != NULL){
			pConnection = pItem->pObject;
			pItem = pItem->pNext;
			dg87_connection_close(pConnection);
		}
	} 
	dg87_list_delete(pServer->pListConnection);

	/* cancel all message to send */
	if (pServer->pListMessage->count > 0){
		pItem = pServer->pListMessage->pFirst;
		while (pItem != NULL){
			pMessage = pItem->pObject;
			pItem = pItem->pNext;
			dg87_message_cancel(pMessage);
		}
	}
	dg87_list_delete(pServer->pListMessage);

	/* delete users */
	if (pServer->pUserVector->count > 0){
		pElement = pServer->pUserVector->pFirst;
		while (pElement != NULL){
			pUser = pElement->pObject;
			pElement= pElement->pNext;
			dg87_user_delete(pUser);
		}
	} 
	dg87_vector_delete(pServer->pUserVector);

	dg87_interpret_delete(pServer->pInterpret);
	if (pServer->pPidFile != NULL)
		dg87_free(pServer->pPidFile, DG87_CHAR);
	dg87_free(pServer, DG87_SERVER);
	return TRUE;
}

/******************************************************************************/
dg87_connection* dg87_server_addConnection(dg87_server* pServer,int fd, 
		char* pHost, int ipPort)
{
	dg87_connection* pConnection;
	assert(pServer != NULL && pHost != NULL);
	if (pServer == NULL || pHost == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pConnection = dg87_connection_in_new(fd, pHost, ipPort, pServer);
	if (pConnection == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	if (dg87_list_addObject(pServer->pListConnection, pConnection) == FALSE)
	{
		dg87_connection_delete(pConnection);
		return NULL;
	}
	/* send the banner */
	dg87_connection_printf(pConnection,"%s-%s-%s-(%s)\n",
			pServer->pPackageName, 
			pServer->pProgramName, 
			pServer->pProgramVersion,
			DG87_PROTOCOL_VERSION);
	dg87_connection_send(pConnection);
	return pConnection;
}

/******************************************************************************/
BOOL dg87_server_removeConnection(dg87_server* pServer,
		dg87_connection* pConnection)
{
	assert(pServer != NULL && pConnection != NULL);
	if (pServer == NULL || pConnection == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	dg87_list_removeObject(pServer->pListConnection, pConnection);
	dg87_connection_delete(pConnection);
	return TRUE;
}

/******************************************************************************/
/* dg87_server_prepare
 * This functions creates a new socket (pServer->ipPort)
 */
static int dg87_server_prepare(dg87_server* pServer)
{
	struct sockaddr_in address_server;
	int optval;
	int sockfd_new;
	assert(pServer != NULL);
	/* open a new socket */
	if ((sockfd_new = socket(AF_INET, SOCK_STREAM, 0)) < 0){
		dg87_daemon_log("Can't initialize socket !");
		return -1;	
	}
	if (setsockopt(sockfd_new , SOL_SOCKET, SO_REUSEADDR,
				&optval, sizeof(int)) == -1){
		dg87_daemon_log("Can't set socket option !");
		return -1;
	}
	memset((char*)&address_server,0,
			sizeof(struct sockaddr_in));
	address_server.sin_family = AF_INET;
	address_server.sin_addr.s_addr = htonl(INADDR_ANY);
	address_server.sin_port = htons(pServer->ipPort);

	if (bind(sockfd_new, (struct sockaddr*)&address_server, 
			sizeof(address_server)) == -1) {
		dg87_daemon_log("Can't bind socket !");
		return -1;
	}

	if (listen(sockfd_new,CNX_MAX) == -1){
		dg87_daemon_log("Can't listen socket !");
		return -1;
	}

	return sockfd_new;
}
/******************************************************************************/
/* TODO find a way to remove this static variable */
static dg87_server *pLocalServer;
static void signal_function(int signo)
{
	pLocalServer->signal_function(pLocalServer, signo);
	if (signo == SIGINT || signo == SIGTERM){
		dg87_daemon_log("Signal received : shutdown in progress");
		printf("Shutdown in progress...\n");
		pLocalServer->shutdown = TRUE;
	}
}
/******************************************************************************/
BOOL dg87_server_run(dg87_server* pServer)
{
	int sockfd_server, sockfd_client, sockfd_new;
	socklen_t lng_client;
	struct sockaddr_in address_client;
	struct timeval tv;
	int result;
	int ipPort;
	fd_set testreadfds;
	fd_set testwritefds;
	struct sigaction sa_old;
	struct sigaction sa_new;
	dg87_item* pItem;
	dg87_connection* pConnection;
	dg87_message* pMessage;

	assert(pServer != NULL);
	if (pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pLocalServer = pServer;

	if (dg87_server_openPidFile(pServer) == FALSE)
		return FALSE;

	/* catch signals */
	sa_new.sa_handler= signal_function;
	sigemptyset(&sa_new.sa_mask);
	sa_new.sa_flags = 0;
	sigaction(SIGTERM, &sa_new, &sa_old);
	sigaction(SIGHUP, &sa_new, &sa_old);
	sigaction(SIGINT, &sa_new, &sa_old);
	sigaction(SIGPIPE, &sa_new, &sa_old);
	sigaction(SIGUSR1, &sa_new, &sa_old);

	ipPort = -1;
	sockfd_server = -1;
	pServer->quit = FALSE;
	pServer->shutdown = FALSE;
	while(pServer->quit == FALSE){

		/* if daemon starts of SIGHUP signal changes IP port */
		if (ipPort != pServer->ipPort){

			/* open a new socket */
			if ((sockfd_new = dg87_server_prepare(pServer)) < 0){
				/* restore previous IP port value */
				pServer->ipPort = ipPort;
			} else {
				/* close previous socket */
				if (sockfd_server != -1){
					dg87_daemon_log("Change IP Port !");
					close(sockfd_server);
				}
				/* select new socket */
				sockfd_server = sockfd_new;
			}

			/* memorize actual IP port value */
			ipPort = pServer->ipPort;
		}

		/* quit if socket can't be opened */ 
		if (sockfd_server == -1)
			return FALSE;

		/* call work function */
		if (pServer->work_function(pServer) == TRUE){
			/* short timeout : workfunction has work to do */ 
			tv.tv_sec = 0; tv.tv_usec = 1000;
		}else{
			/* long timeout : workfunction has nothing to do */
			tv.tv_sec = 0; tv.tv_usec = 50000;
		}

		/* for each message to send */
		pItem = pServer->pListMessage->pFirst;
		while (pItem != NULL){
			pMessage = pItem->pObject;
			pItem = pItem->pNext;
			/* delete message if it was canceled */
			if (pMessage->state == dg87_message_state_cancel){
				dg87_list_removeObject(pServer->pListMessage, 
						pMessage);
				dg87_message_delete(pMessage);
			}
			/* try to send the message */
			else if (dg87_message_send(pMessage, pServer) == TRUE){
				dg87_list_removeObject(pServer->pListMessage, 
						pMessage);
				dg87_message_delete(pMessage);
			}
		}

		/* if shutdown in progress */
		if (pServer->shutdown == TRUE){
			/* close all remaining connection */
			if (pServer->pListConnection->count > 0){
				pItem = pServer->pListConnection->pFirst;
				while (pItem != NULL){
					pConnection = pItem->pObject;
					pItem = pItem->pNext;
					dg87_connection_close(pConnection);
				}
			} 
			pServer->quit = TRUE;
		}

		/* construct fds with connection */
		FD_ZERO(&testreadfds);
		FD_ZERO(&testwritefds);
		FD_SET(sockfd_server, &testreadfds);
		pItem = pServer->pListConnection->pFirst;
		while (pItem != NULL){
			pConnection = pItem->pObject;
			pItem = pItem->pNext;
			switch (pConnection->state){
				case dg87_connection_state_readingMessage:
					FD_SET(pConnection->fd, &testreadfds);
					break;
				case dg87_connection_state_sendingMessage:
					FD_SET(pConnection->fd, &testwritefds);
					break;
				default:
					/* nothing to do */
				break;
			}
		}
		
		/* FD_SETSIZE as first parameter for select */
		result = select(FD_SETSIZE , 
				&testreadfds, &testwritefds, NULL, &tv);

		/* continue if there is no request */
		if (result == 0)
			continue;

		if (result < 1) {
			/* come here if signal is sended */
			continue;
		}

		/* request for new connection */
		if (FD_ISSET(sockfd_server, &testreadfds)) {
			lng_client = sizeof(address_client);
			sockfd_client = accept(sockfd_server,
				(struct sockaddr*)
				&address_client,
				&lng_client);
			if (sockfd_client == -1)
				dg87_daemon_log("accept error");
			else if (pServer->shutdown == TRUE)
				/* serveruse new connection */
				close (sockfd_client);
			else{
				dg87_server_addConnection(pServer, 
					sockfd_client,
					inet_ntoa(address_client.sin_addr),
					ntohs(address_client.sin_port));
			}
		}

		/* for each Connection */
		pItem = pServer->pListConnection->pFirst;
		while (pItem != NULL){
			pConnection = pItem->pObject;
			pItem = pItem->pNext;

			/* if there is nothing to read or to write, continue*/ 
			if (! FD_ISSET(pConnection->fd, &testreadfds)
				&& ! FD_ISSET(pConnection->fd, &testwritefds))
				continue;	

			dg87_connection_update(pConnection);

			if (pConnection->state == 
				dg87_connection_state_messageAvailable){
				if (dg87_interpret_execute(pServer->pInterpret,
						       pConnection) == -1){
					printf("Shutdown in progress...\n");
					pServer->shutdown = TRUE;
				}
			}

			if (pConnection->state ==
				dg87_connection_state_connectionClose){
				dg87_server_removeConnection(pServer,
						pConnection);
				continue;
			}
		} /* next connection */

	 }

	dg87_server_closePidFile(pServer);
	return TRUE;
}
/******************************************************************************/
BOOL dg87_server_postMessage(dg87_server* pServer, 
		char* pHost, int ipPort, char* pMsgString)
{
	dg87_message* pMessage;
	assert(pServer != NULL && pHost != NULL && pMsgString != NULL);
	if (pServer == NULL || pHost == NULL || pMsgString == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pMessage = dg87_message_new(pHost, ipPort, pMsgString);
	if (pMessage == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	if (dg87_list_addObject(pServer->pListMessage, pMessage) == FALSE){
		dg87_message_delete(pMessage);
		return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
dg87_user* dg87_server_userAdd(dg87_server* pServer, char* pUserName)
{
	dg87_user* pUser = NULL;
	assert(pServer != NULL && pUserName != NULL);
	if (pServer == NULL || pUserName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pUser = dg87_user_new(pUserName);
	if (pUser == NULL) 
		return NULL;

	/* add the user in the UserList */
	/* vector generate error if an other user has not the same name*/
	/* key must point on pUser->name */
	if (dg87_vector_addObject(pServer->pUserVector, 
				pUser, pUser->name) == FALSE)
	{
		dg87_user_delete(pUser);
		return NULL;
	}

	return pUser;
}
/****************************************************************************/
int dg87_server_userCount(dg87_server* pServer)
{
	assert(pServer != NULL);
	if (pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return dg87_vector_count(pServer->pUserVector);
}
/****************************************************************************/
dg87_user* dg87_server_userGetByName(dg87_server* pServer,char* pName)
{
	dg87_element *pElement;
	assert(pServer != NULL && pName != NULL);
	if (pServer == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}

	/* browse the list to find an item with the same name*/
	pElement = pServer->pUserVector->pFirst;
	while (pElement != NULL){
		if (strcmp(pElement->pKey, pName) == 0)
			return pElement->pObject;
		pElement = pElement->pNext;
	}	
	dg87_err_setError(DG87_ERR_CANTFINDOBJECT);
	return NULL;
}
/****************************************************************************/
dg87_user* dg87_server_userGetByIndex(dg87_server* pServer,int index)
{
	assert(pServer != NULL);
	if (pServer == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	return dg87_vector_getObjectByIndex(pServer->pUserVector, index);
}
/****************************************************************************/
BOOL dg87_server_userDeleteByName(dg87_server* pServer,char* pName)
{
	dg87_user* pUser;
	assert(pServer != NULL && pName != NULL);
	if (pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pUser = dg87_vector_removeObjectByKey(pServer->pUserVector, pName);
	if (pUser == NULL)
		return FALSE;
	if (dg87_user_delete(pUser) == FALSE){
		/* add the user in the vector again */
		dg87_vector_addObject(pServer->pUserVector, pUser, pUser->name); 
		return FALSE;
	}
	return TRUE;
}
/****************************************************************************/
BOOL dg87_server_userDeleteByIndex(dg87_server* pServer,int index)
{
	dg87_user* pUser;
	assert(pServer != NULL);
	if (pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pUser = dg87_vector_removeObjectByIndex(pServer->pUserVector, index);
	if (pUser == NULL) 
		return FALSE;
	dg87_user_delete(pUser);
	return TRUE;
}
/******************************************************************************/
BOOL dg87_server_reload(dg87_server* pServer, char* pNewPidFile, int ipPort,
		char* pDiogenePassword)
{
	dg87_user* pUser;
	char* pPreviousPidFile;
	int fdPreviousPidFile;
	assert(pServer != NULL);
	if (pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pServer->ipPort = ipPort;
	/* change diogene password */
	pServer->pDiogenePassword = pDiogenePassword;
	pUser = dg87_server_userGetByName(pServer, DG87_USER_DIOGENE_NAME);
	if (pUser == NULL)
		return FALSE;
	dg87_user_setPassword(pUser, pServer->pDiogenePassword);
	/* quit if pid files are the same */
	if (pServer->pPidFile == NULL && pNewPidFile == NULL)
		return TRUE;
	if (pServer->pPidFile != NULL && pNewPidFile != NULL && 
			strcmp(pServer->pPidFile, pNewPidFile) == 0)
		return TRUE;
	/* save previous PID file */
	pPreviousPidFile = pServer->pPidFile;
	fdPreviousPidFile = pServer->fdPidFile;
	pServer->fdPidFile = 0;
	pServer->pPidFile = NULL;
	/* initialize new PID file */
	if (pNewPidFile != NULL){
		pServer->pPidFile = dg87_strdup(pNewPidFile, DG87_CHAR);
		if (pServer->pPidFile == NULL){
			dg87_err_setError(DG87_ERR_OUTOFMEMORY);
			return FALSE;
		}
		/* open new PID file */
		if (dg87_server_openPidFile(pServer) == FALSE){
			/* restore previous PID file */
			pServer->pPidFile = pPreviousPidFile;
			pServer->fdPidFile = fdPreviousPidFile;
			return FALSE;
		}
	}
	/* free previous PID file */
	if (pPreviousPidFile != NULL){
		close(fdPreviousPidFile); 
		unlink(pPreviousPidFile);
		dg87_free(pPreviousPidFile, DG87_CHAR);
	}

	return TRUE;
}
/******************************************************************************/
/* End of file */
