/* file storage.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "storage.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <netinet/in.h> /* uint16_t- uint32_t */
#include <assert.h>

/***************************************************************************/
dg87_storage *dg87_storage_new(char* pDataFile, char* pBackupFile,
		char* pSignature, int version)
{
	dg87_storage* pStorage;
	assert(pDataFile != NULL && pBackupFile != NULL && pSignature != NULL);
	if (pDataFile == NULL || pBackupFile == NULL || pSignature == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pStorage = dg87_malloc(sizeof(dg87_storage), DG87_STORAGE);
	if (pStorage == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pStorage->pDataFile = dg87_strdup(pDataFile, DG87_CHAR);
	if (pStorage->pDataFile == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pStorage, DG87_STORAGE);
		return NULL;
	}
	pStorage->pBackupFile = dg87_strdup(pBackupFile, DG87_CHAR);
	if (pStorage->pBackupFile == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pStorage->pDataFile, DG87_CHAR);
		dg87_free(pStorage, DG87_STORAGE);
		return NULL;
	}
	pStorage->mode = dg87_storage_mode_closed;
	pStorage->pSignature = pSignature;
	pStorage->version = version;
	pStorage->fd = -1;
	return pStorage;
}
/***************************************************************************/
BOOL dg87_storage_delete(dg87_storage* pStorage)
{
	assert(pStorage != NULL);
	if (pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	dg87_free(pStorage->pDataFile, DG87_CHAR);
	dg87_free(pStorage->pBackupFile, DG87_CHAR);
	dg87_free(pStorage, DG87_STORAGE);
	return TRUE;
}
/***************************************************************************/
BOOL dg87_storage_changeDataFile(dg87_storage* pStorage, 
		char* pDataFile, char* pBackupFile)
{
	char* pNewFile;
	assert(pStorage != NULL && pDataFile != NULL && pBackupFile != NULL);
	if (pStorage == NULL || pDataFile == NULL || pBackupFile == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* copy data file */
	pNewFile = dg87_strdup(pDataFile, DG87_CHAR);
	if (pNewFile == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	dg87_free(pStorage->pDataFile, DG87_CHAR);
	pStorage->pDataFile = pNewFile;

	/* copy backup file */
	pNewFile = dg87_strdup(pBackupFile, DG87_CHAR);
	if (pNewFile == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	dg87_free(pStorage->pBackupFile, DG87_CHAR);
	pStorage->pBackupFile = pNewFile;

	return TRUE;
}
/***************************************************************************/
static BOOL dg87_storage_checkHeader(dg87_storage* pStorage)
{
	char* pString;
	int version;
	uint16_t tmpvalue;
	int l;
	assert(pStorage != NULL);
	/* verify signature */
	l = strlen(pStorage->pSignature);
	pString = dg87_malloc((l+1)*sizeof(char), DG87_CHAR);
	if (read(pStorage->fd, pString, l*sizeof(char)) == -1){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	pString[l+1] = '\0';
	if (strncmp(pString, pStorage->pSignature, l) != 0){
		dg87_free(pString, DG87_CHAR);
		dg87_err_setError(DG87_ERR_STORAGEWRONGSIGN);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);

	/* verify version */
	if (read(pStorage->fd, &tmpvalue, sizeof(uint16_t)) == -1)
		return FALSE;
	version = ntohs(tmpvalue);
	if (version != pStorage->version){
		dg87_err_setError(DG87_ERR_STORAGEWRONGVERSION);
		return FALSE;
	}
	return TRUE;
}
/***************************************************************************/
static BOOL dg87_storage_writeHeader(dg87_storage* pStorage)
{
	uint16_t value;
	int l;
	assert(pStorage != NULL);
	/* write signature */
	l = strlen(pStorage->pSignature);
	if (write(pStorage->fd, pStorage->pSignature, l*sizeof(char)) == -1)
		return FALSE;
	/* write version */
	value = htons(pStorage->version);
	if (write(pStorage->fd, &value, sizeof(uint16_t)) == -1)
		return FALSE;
	return TRUE;
}
/***************************************************************************/
static BOOL dg87_storage_open_write(dg87_storage* pStorage)
{
	/* writing on backup file */
	assert(pStorage != NULL);
	fprintf(stderr, "Open '%s' for writing\n", pStorage->pBackupFile);
	if ((pStorage->fd = open(pStorage->pBackupFile, 
				O_WRONLY | O_CREAT,
				S_IRUSR | S_IWUSR | O_TRUNC)) == -1){
		dg87_err_setError(DG87_ERR_STORAGECANTOPEN);
		return FALSE;
	}
	pStorage->mode = dg87_storage_mode_write;
	if (dg87_storage_writeHeader(pStorage) == FALSE)
		return FALSE;
	return TRUE;
}
/***************************************************************************/
static BOOL dg87_storage_open_read(dg87_storage* pStorage)
{
	assert(pStorage != NULL);
	/* reading data file */
	fprintf(stderr, "Open '%s' for reading\n", pStorage->pDataFile);
	if ((pStorage->fd = open(pStorage->pDataFile, 
				O_RDONLY,
				S_IRUSR | S_IWUSR | O_TRUNC)) == -1){
		dg87_err_setError(DG87_ERR_STORAGECANTOPEN);
		return FALSE;
	}
	pStorage->mode = dg87_storage_mode_read;
	if (dg87_storage_checkHeader(pStorage) == FALSE)
		return FALSE;

	return TRUE;
}
/***************************************************************************/
BOOL dg87_storage_open(dg87_storage* pStorage, dg87_storage_mode mode)
{
	assert(pStorage != NULL);
	if (pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	switch(mode){
		case dg87_storage_mode_write:
			return dg87_storage_open_write(pStorage);
		case dg87_storage_mode_read:
			return dg87_storage_open_read(pStorage);
		default:
			dg87_err_setError(DG87_ERR_STORAGEWRONGMODE);
			return FALSE;
	}
	return FALSE;
}
/***************************************************************************/
BOOL dg87_storage_close(dg87_storage* pStorage)
{
	assert(pStorage != NULL);
	if (pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pStorage->mode == dg87_storage_mode_closed)
		return TRUE;
	/* close file */
	if (close(pStorage->fd) == -1){
		dg87_err_setError(DG87_ERR_STORAGECANTCLOSE);
		return FALSE;
	}
	/* if write mode, rename backup file to data file */
	if (pStorage->mode == dg87_storage_mode_write){
		fprintf(stderr, "Rename data file : %s \n", 
				pStorage->pDataFile);
		if (rename(pStorage->pBackupFile, pStorage->pDataFile) == -1){
			fprintf(stderr, "Can't rename data file !\n");
			return FALSE;
		}
	}
	pStorage->mode = dg87_storage_mode_closed;
	return TRUE;
}
/***************************************************************************/
BOOL dg87_storage_writeString(dg87_storage* pStorage, char* pString)
{
	uint16_t tmpvalue;
	size_t size;
	assert(pStorage != NULL);
	if (pStorage == NULL || pString == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* TODO conversion */
	size = strlen(pString);
	tmpvalue = htons(size);
	if (write(pStorage->fd, &tmpvalue, sizeof(uint16_t)) == -1)
		return FALSE;

	if (write(pStorage->fd, pString, size * sizeof(char)) == -1)
		return FALSE;
	return TRUE;
}
/***************************************************************************/
BOOL dg87_storage_readString(dg87_storage* pStorage, char** ppString)
{
	size_t size;
	uint16_t tmpvalue;
	char* pString;
	assert(pStorage != NULL && ppString != NULL);
	if (pStorage == NULL || ppString == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	*ppString = NULL;
	/* read size */
	if (read(pStorage->fd, &tmpvalue, sizeof(uint16_t)) == -1)
		return FALSE;
	size = ntohs(tmpvalue);
	/* allocate memory for string */
	pString = dg87_malloc((size + 1) * sizeof(char), DG87_CHAR);
	if (pString == NULL)
		return FALSE;
	/* read string */
	if (size > 0){
		if (read(pStorage->fd, pString , size) == -1){
			dg87_free(pString, DG87_CHAR);
			*ppString = NULL;
			return FALSE;
		}
	}
	pString[size] = '\0';
	*ppString = pString;
	return TRUE;
}
/***************************************************************************/
BOOL dg87_storage_writeLong(dg87_storage* pStorage, dg87long value)
{
	uint32_t tmpvalue;
	assert(pStorage != NULL);
	if (pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	tmpvalue = htonl(value);
	if (write(pStorage->fd, &tmpvalue, sizeof(uint32_t)) == -1)
		return FALSE;
	return TRUE;
}
/***************************************************************************/
BOOL dg87_storage_readLong(dg87_storage* pStorage, dg87long *pValue)
{
	uint32_t tmpvalue;
	assert(pStorage != NULL && pValue != NULL);
	if (pStorage == NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (read(pStorage->fd, &tmpvalue, sizeof(uint32_t)) == -1)
		return FALSE;
	*pValue = ntohl(tmpvalue);
	return TRUE;
}
/***************************************************************************/
BOOL dg87_storage_writeUInt(dg87_storage* pStorage, dg87uint value)
{
	uint16_t tmpvalue;
	assert(pStorage != NULL);
	if (pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	tmpvalue = htons(value);
	if (write(pStorage->fd, &tmpvalue, sizeof(uint16_t)) == -1)
		return FALSE;
	return TRUE;
}
/***************************************************************************/
BOOL dg87_storage_readUInt(dg87_storage* pStorage, dg87uint *pValue)
{
	uint16_t tmpvalue;
	assert(pStorage != NULL && pValue != NULL);
	if (pStorage == NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (read(pStorage->fd, &tmpvalue, sizeof(uint16_t)) == -1)
		return FALSE;
	*pValue = ntohs(tmpvalue);
	return TRUE;
}
/***************************************************************************/
BOOL dg87_storage_writeInt(dg87_storage* pStorage, dg87int value)
{
	int16_t tmpvalue;
	assert(pStorage != NULL);
	if (pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	tmpvalue = htons(value);
	if (write(pStorage->fd, &tmpvalue, sizeof(int16_t)) == -1)
		return FALSE;
	return TRUE;
}
/***************************************************************************/
BOOL dg87_storage_readInt(dg87_storage* pStorage, dg87int *pValue)
{
	int16_t tmpvalue;
	assert(pStorage != NULL && pValue != NULL);
	if (pStorage == NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (read(pStorage->fd, &tmpvalue, sizeof(int16_t)) == -1)
		return FALSE;
	*pValue = ntohs(tmpvalue);
	return TRUE;
}

/***************************************************************************/
BOOL dg87_storage_writeByte(dg87_storage* pStorage, dg87byte value)
{
	assert(pStorage != NULL);
	if (pStorage == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (write(pStorage->fd, &value, sizeof(dg87byte)) == -1)
		return FALSE;
	return TRUE;
}
/***************************************************************************/
BOOL dg87_storage_readByte(dg87_storage* pStorage, dg87byte *pValue)
{
	assert(pStorage != NULL && pValue != NULL);
	if (pStorage == NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (read(pStorage->fd, pValue, sizeof(dg87byte)) == -1)
		return FALSE;
	return TRUE;
}

/***************************************************************************/
/* End of file */
