/* file user.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "user.h"

/******************************************************************************/
dg87_user* dg87_user_new(char* pName)
{
	int i;
	dg87_user* pUser;
	assert(pName != NULL);
	if (pName == NULL){ 
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}	
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pUser = dg87_malloc(sizeof(dg87_user), DG87_USER);
	if (pUser== NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	strncpy(pUser->name, pName, DG87_USER_NAME_LENGTH); 
	pUser->password[0] = '\0';
	pUser->connectionCount = 0;
	pUser->systemAccount = FALSE;
	for (i = 1; i < DG87_COMMAND_FAM_MAX; i++)
		pUser->privileges[i] = dg87_command_access_none ;
	pUser->privileges[dg87_command_generic] = dg87_command_access_consult;
	return pUser;
}
/******************************************************************************/
BOOL dg87_user_delete(dg87_user* pUser)
{	
	assert(pUser != NULL);
	if (pUser == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pUser->systemAccount == TRUE){
		dg87_err_setError(DG87_ERR_CANTDELETESYSTEMACCOUNT);
		return FALSE;
	}
	if (pUser->connectionCount > 0){
		dg87_err_setError(DG87_ERR_USERCONNECTED);
		return FALSE;
	}
	dg87_free(pUser, DG87_USER);
	return TRUE;
}
/******************************************************************************/
BOOL dg87_user_load(dg87_user* pUser, dg87_storage* pStorage)
{
	char* pString;
	int i;
	dg87byte byte;
	assert(pUser != NULL && pStorage != NULL);
	if (pUser == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read user password */
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_user_setPassword(pUser, pString) == FALSE)
		return FALSE;
	/* read privileges */
	for (i = 0; i < DG87_COMMAND_FAM_MAX; i++){
		if (dg87_storage_readByte(pStorage, &byte) == FALSE)
			return FALSE;
		pUser->privileges[i] = byte;
	}
	return TRUE;
}
/******************************************************************************/
BOOL dg87_user_save(dg87_user* pUser, dg87_storage* pStorage)
{
	int i;
	assert(pUser != NULL && pStorage != NULL);
	if (pUser == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write user password */
	if (dg87_storage_writeString(pStorage, pUser->password) == FALSE)
		return FALSE;
	/* write privileges */
	for (i = 0; i < DG87_COMMAND_FAM_MAX; i++){
		if (dg87_storage_writeByte(pStorage, pUser->privileges[i]) 
				== FALSE)
			return FALSE;
	}

	return TRUE;
}
/******************************************************************************/
BOOL dg87_user_updateConnectionCount(dg87_user* pUser, int delta)
{
	assert(pUser != NULL);
	if (pUser == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pUser->connectionCount += delta;
	if (pUser->connectionCount < 0)
		pUser->connectionCount = 0;
	return TRUE;
}
/******************************************************************************/
BOOL dg87_user_setPassword(dg87_user* pUser, char* pPassword)
{
	assert(pUser != NULL && pPassword != NULL);
	if (pUser == NULL || pPassword == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (strlen(pPassword) >= DG87_USER_PASSWORD_LENGTH){
		dg87_err_setError(DG87_ERR_PASSWORDTOLONG);
		return FALSE;
	}
	strncpy(pUser->password, pPassword, DG87_USER_PASSWORD_LENGTH);
	return TRUE;
}
/******************************************************************************/
BOOL dg87_user_verifyPassword(dg87_user* pUser, char* pPassword)
{
	assert(pUser != NULL && pPassword != NULL);
	if (pUser == NULL || pPassword == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (strlen(pPassword) >= DG87_USER_PASSWORD_LENGTH){
		dg87_err_setError(DG87_ERR_PASSWORDTOLONG);
		return FALSE;
	}
	if (strncmp(pUser->password, pPassword, DG87_USER_PASSWORD_LENGTH) != 0)
		return FALSE;

	return TRUE;
}
/******************************************************************************/
BOOL dg87_user_setPrivilege(dg87_user* pUser, 
		dg87_command_family family,
		dg87_command_privilege privilege)
{
	assert(pUser != NULL);
	if (pUser == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pUser->systemAccount == TRUE){
		dg87_err_setError(DG87_ERR_CANTCHANGESYSTEMACCOUNT);
		return FALSE;
	}
	pUser->privileges[family] = privilege;
	return TRUE;
}
/******************************************************************************/
BOOL dg87_user_verifyPrivilege(dg87_user* pUser,
		dg87_command_family family,
		dg87_command_privilege privilege)
{
	assert(pUser != NULL);
	if (pUser == NULL && privilege == dg87_command_access_none)
		return TRUE;
	if (pUser != NULL && pUser->privileges[family] >= privilege)
		return TRUE;
	return FALSE;
}
/******************************************************************************/
BOOL dg87_user_setAdministrator(dg87_user* pUser)
{
	int i;
	assert(pUser != NULL);
	if (pUser == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pUser->systemAccount = TRUE;
	for (i = 0; i < DG87_COMMAND_FAM_MAX; i++)
		pUser->privileges[i] = dg87_command_access_admin;
	return TRUE;
}
/******************************************************************************/
/* End of file */
