/* file vector.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "vector.h"

/***************************************************************************/
dg87_vector* dg87_vector_new(void)
{
	dg87_vector* pVector = dg87_malloc(sizeof(dg87_vector), DG87_VECTOR);
	if (pVector == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pVector->pFirst = NULL;
	pVector->pLast = NULL;
	pVector->count = 0;
	return pVector;
}
/***************************************************************************/
BOOL dg87_vector_delete(dg87_vector *pVector)
{
	dg87_element* pElement,*pTemp;
	assert(pVector != NULL);
	if (pVector == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* delete each element in the vector */
	pElement = pVector->pFirst;
	while (pElement != NULL){
		pTemp = pElement;
		pElement = pElement->pNext;
		dg87_element_delete(pTemp);
	}
	dg87_free(pVector, DG87_VECTOR);
	return TRUE;
}
/***************************************************************************/
BOOL dg87_vector_addObject(dg87_vector *pVector, void *pObject, char *pKey)
{
	dg87_element* pElement;
	dg87_element* pTemp;
	dg87int compar=0;
	assert(pVector != NULL && pKey != NULL);
	if (pVector == NULL || pObject == NULL || pKey == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (strlen(pKey) == 0) {
		dg87_err_setError(DG87_ERR_INCORRECTKEY);
		return FALSE;
	}
	
	/* create the associate element */
	pElement = dg87_element_new(pObject, pKey);
	if (pElement == NULL) 
		return FALSE;

	/* add the element in the list */
	if (pVector->count == 0){
		/* list is empty */
		pVector->pFirst = pElement;
		pVector->pLast = pElement;
		pVector->count = 1;
		return TRUE;
	}
	/* optimisation if key is greater than last element */
	if (strcmp(pKey, pVector->pLast->pKey) > 0){
		pTemp = pVector->pLast;
		pTemp->pNext = pElement;
		pElement->pNext = NULL;
		pElement->pPrev = pTemp;
		pVector->pLast = pElement;
		pVector->count++;
		return TRUE;
	}

	/* find the position for the object in the vector */
	pTemp = pVector->pFirst;
	while (pTemp != NULL) {
		compar = strcmp(pKey, pTemp->pKey);
		if (compar <= 0)
			break;
		pTemp = pTemp->pNext;
	}	

	/* add at the end of the list */
	if (pTemp == NULL){
		pElement->pPrev = pVector->pLast;
		pVector->pLast->pNext = pElement;
		pVector->pLast = pElement;
		pElement->pNext = NULL;
		pVector->count++;
		return TRUE;
	}

	/* add just before pTemp */
	if (compar != 0)
	{
		pElement->pNext = pTemp;
		if (pTemp->pPrev == NULL)
			pVector->pFirst = pElement;
		else
			pTemp->pPrev->pNext = pElement;
		pElement->pPrev = pTemp->pPrev;
		pTemp->pPrev = pElement;
		pVector->count++;
		return TRUE;
	}
	/* an object as the same key in the vector */
	dg87_err_setError(DG87_ERR_KEYNOTUNIQUE);
	dg87_element_delete(pElement);
	return FALSE;
}
/***************************************************************************/
void* dg87_vector_deleteElement(dg87_vector *pVector, dg87_element *pElement)
{
	void* pObject;
	assert(pVector != NULL && pElement != NULL);
	if (pVector == NULL || pElement == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}

	pObject = pElement->pObject;

	/* change the link of the previous element in the vector */
	if (pElement->pPrev == NULL)
		pVector->pFirst = pElement->pNext;
	else
		pElement->pPrev->pNext = pElement->pNext;

	/* change the link of the next element in the vector */
	if (pElement->pNext == NULL)
		pVector->pLast = pElement->pPrev;
	else
		pElement->pNext->pPrev = pElement->pPrev;

	/* delete the element object */
	pVector->count--;
	dg87_element_delete(pElement);
	return pObject;
}

/***************************************************************************/
BOOL dg87_vector_removeObject(dg87_vector *pVector, void* pObject)
{
	dg87_element *pElement;
	assert(pVector != NULL && pObject != NULL);
	if (pVector == NULL || pObject == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pElement = pVector->pFirst;
	while (pElement != NULL){
		if (pElement->pObject == pObject){
			dg87_vector_deleteElement(pVector, pElement);
			return TRUE;
		}
		pElement = pElement->pNext;
	}
	dg87_err_setError(DG87_ERR_CANNOTFINDOBJECT);
	return FALSE;
}
/***************************************************************************/
void* dg87_vector_removeObjectByKey(dg87_vector *pVector, char* pKey)
{
	dg87_element *pElement;
	assert(pVector != NULL && pKey != NULL);
	if (pVector == NULL || pKey == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pElement = dg87_vector_getElementByKey(pVector, pKey);
	if (pElement == NULL){
		dg87_err_setError(DG87_ERR_CANNOTFINDOBJECT);
		return NULL;
	}
	return dg87_vector_deleteElement(pVector, pElement);
}
/***************************************************************************/
void* dg87_vector_removeObjectByIndex(dg87_vector *pVector, dg87uint index)
{
	dg87_element *pElement;
	assert(pVector != NULL);
	if (pVector == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pElement = dg87_vector_getElementByIndex(pVector, index);
	return dg87_vector_deleteElement(pVector, pElement);
}
/***************************************************************************/
dg87uint dg87_vector_count(dg87_vector *pVector)
{
	assert(pVector != NULL);
	if (pVector == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}
	return pVector->count;
}
/***************************************************************************/
void* dg87_vector_getObjectByIndex(dg87_vector *pVector, dg87uint index)
{
	dg87_element* pElement; 
	assert(pVector != NULL);
	if (pVector == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pElement = dg87_vector_getElementByIndex(pVector, index);
	return (pElement==NULL)?NULL:pElement->pObject;
}
/***************************************************************************/
void* dg87_vector_getObjectByKey(dg87_vector *pVector, char* pKey)
{
	dg87_element* pElement;
	assert(pVector != NULL  && pKey != NULL);
	if (pVector == NULL || pKey == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pElement = dg87_vector_getElementByKey(pVector, pKey);
	if (pElement == NULL) {
		dg87_err_setError(DG87_ERR_CANNOTFINDOBJECT);
		return NULL;
	}
	return pElement->pObject;
}
/***************************************************************************/
dg87_element* dg87_vector_getElementByIndex(dg87_vector *pVector, dg87uint index)
{
	dg87_element* pElement;
	dg87uint i = 0;
	assert(pVector != NULL);
	if (pVector == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (index >= pVector->count){
		dg87_err_setError(DG87_ERR_INDEXOUTOFBOUND);
		return NULL;
	}
	if (pVector->count == 0) 
		return NULL;
	if (index == 0)
		return pVector->pFirst;
	if (index == pVector->count - 1)
		return pVector->pLast;
	pElement = pVector->pFirst;
	while (pElement != NULL && i++ < index)
		pElement = pElement->pNext;
	return pElement;
}
/***************************************************************************/
dg87_element* dg87_vector_getElementByKey(dg87_vector *pVector, char* pKey)
{
	dg87_element* pElement;
	int length;

	assert(pVector != NULL && pKey != NULL);
	if (pVector == NULL || pKey == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pKey) == 0) {
		dg87_err_setError(DG87_ERR_INCORRECTKEY);
		return NULL;
	}

	pElement = pVector->pFirst;
	while (pElement != NULL){
		length = strcmp(pKey,pElement->pKey);
		if (length == 0)
		       /* we found the exact key */
		       return pElement;
		else if (length < 0) 
		       /* actual key is greater than search key */
		       return NULL;
		pElement = pElement->pNext;
	}
	dg87_err_setError(DG87_ERR_CANTFINDOBJECT);
	return NULL;
}
/***************************************************************************/
void dg87_vector_dump(dg87_vector* pVector)
{
	dg87_element* pElement;
	assert(pVector != NULL);
	if (pVector == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return;
	}
	pElement = pVector->pFirst;
	printf("Vecteur[%i]={", pVector->count);
	while (pElement != NULL){
		printf("%s,", pElement->pKey);
		pElement = pElement->pNext;
	}
	printf("}\n");
}
/***************************************************************************/
dg87_element* dg87_vector_findElement(dg87_vector* pVector, void* pObject)
{
	dg87_element* pElement;
	assert(pVector != NULL && pObject != NULL);
	if (pVector == NULL || pObject == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	/* browse the vector to find the item associate to the object */
	pElement = pVector->pFirst;
	while (pElement != NULL && pElement->pObject != pObject)
		pElement = pElement->pNext;
	return pElement;
}

/***************************************************************************/
#ifdef DEBUG
int dg87_vector_check(dg87_vector* pVector, int (*print)(char*))
{
	dg87_element* pElement;
	dg87uint count = 0;
	pElement = pVector->pFirst;
	while (pElement != NULL){

		/* check object */
		if (pElement->pObject == NULL){
			print("Vector : Object pointer is NULL !\n");
			return 1;
		}
		/* check key's pointer */
		if (pElement->pKey == NULL){
			print("Vector : Key pointer is NULL !\n");
			return 1;
		}
		/* check key length */
		if (strlen(pElement->pKey) == 0){
			print("Vector : Key is zero length !\n");
			return 1;
		}

		/* check next element */
		if (pElement->pNext == NULL){
			/* last element */
			if (pVector->pLast != pElement){
				print("Vector : Last element incorrect !\n");
				return 1;
			}
		}
		else if (pElement->pNext->pPrev != pElement){
			print("Vector : Previous pointer incorrect !\n");
			return 1;
		}

		/* check previous element */
		if (pElement->pPrev == NULL){
			/* first element */
			if (pVector->pFirst != pElement){
				print("Vector : First element incorrect !\n");
				return 1;
			}
		}
		else if (pElement->pPrev->pNext != pElement){
			print("Vector : Previous pointer incorrect !\n");
			return 1;
		}

		/* check key order */
		if (pElement->pPrev != NULL){
			if (strcmp(pElement->pPrev->pKey, pElement->pKey) >= 0){
				print("Vector : sort problem !\n");
				return 1;
			}
		}

		/* next element */
		count ++;
		pElement = pElement->pNext;

	}
	/* check object's count */
	if (count != pVector->count){
		print("Vector : incorrect count !\n");
		return 1;
	}

	/* TODO check empty vector */
	if (count == 0 || pVector->pFirst == NULL || pVector->pLast == NULL){

	}
	return 0;
}
#endif /*DEBUG*/

/***************************************************************************/
/* End of file */
